@@ -19,12 +19,6 @@
 
 /* command-line client program for libgretl, mpi version */
 
-#include <stdio.h>
-#include <stdlib.h>
-
-#include <dirent.h>
-#include <mpi.h>
-
 #include "libgretl.h"
 #include "version.h"
 #include "monte_carlo.h"
@@ -46,8 +40,11 @@
 # include "gretl_www.h"
 #endif
 
+#include <dirent.h>
+#include <mpi.h>
+
 #ifdef WIN32
-# include <windows.h>
+# include "gretl_win32.h"
 #else
 # include <sys/stat.h>
 # include <sys/types.h>
@@ -403,7 +400,11 @@ int main (int argc, char *argv[])
     }
 
     runit = 0;
-    sprintf(line, "run %s\n", runfile);
+    if (strchr(runfile, ' ')) {
+	sprintf(line, "run \"%s\"", runfile);
+    } else {
+	sprintf(line, "run %s", runfile);
+    }
     state.flags |= INIT_EXEC;
     err = cli_exec_line(&state, id, dset, progopt, cmdprn);
     state.flags ^= INIT_EXEC;
@@ -421,7 +422,7 @@ int main (int argc, char *argv[])
 	}
 
 	if (gretl_execute_loop()) { 
-	    err = gretl_loop_exec(&state, dset);
+	    err = gretl_loop_exec(&state, dset, NULL);
 	    if (err) {
 		break;
 	    }
@@ -498,7 +499,8 @@ static void cli_exec_callback (ExecState *s, void *ptr,
     int ci = s->cmd->ci;
 
     if (ci == MODELTAB || ci == GRAPHPG) {
-	pprintf(s->prn, _("%s: command not available\n"), s->cmd->word);
+	pprintf(s->prn, _("%s: command not available\n"), 
+		gretl_command_word(ci));
     } else if (ci == OPEN) {
 	char *fname = (char *) ptr;
 
@@ -528,10 +530,6 @@ static int cli_try_http (const char *s, char *fname, int *http)
 {
     int err = 0;
 
-    /* skip past command word */
-    s += strcspn(s, " ");
-    s += strspn(s, " ");
-
     if (strncmp(s, "http://", 7) == 0) {
 #ifdef USE_CURL
 	err = retrieve_public_file(s, fname);
@@ -547,9 +545,8 @@ static int cli_try_http (const char *s, char *fname, int *http)
     return err;
 }
 
-static int cli_open_append (CMD *cmd, const char *line, 
-			    DATASET *dset, MODEL *model,
-			    PRN *prn)
+static int cli_open_append (CMD *cmd, DATASET *dset, 
+			    MODEL *model, PRN *prn)
 {
     gretlopt opt = cmd->opt;
     PRN *vprn = prn;
@@ -558,19 +555,28 @@ static int cli_open_append (CMD *cmd, const char *line,
     int ftype;
     int err = 0;
 
-    err = cli_try_http(line, newfile, &http);
-    if (err) {
-	errmsg(err, prn);
-	return err;
-    }
-
-    if (!http && !(opt & OPT_O)) {
-	/* not using http or ODBC */
-	err = getopenfile(line, newfile, (opt & OPT_W)?
-			  OPT_W : OPT_NONE);
+    if (opt & OPT_K) {
+	/* --frompkg=whatever */
+	err = get_package_data_path(cmd->param, newfile);
 	if (err) {
 	    errmsg(err, prn);
 	    return err;
+	}    
+    } else {
+	err = cli_try_http(cmd->param, newfile, &http);
+	if (err) {
+	    errmsg(err, prn);
+	    return err;
+	}
+
+	if (!http && !(opt & OPT_O)) {
+	    /* not using http or ODBC */
+	    err = get_full_filename(cmd->param, newfile, (opt & OPT_W)?
+				    OPT_W : OPT_NONE);
+	    if (err) {
+		errmsg(err, prn);
+		return err;
+	    }
 	}
     }
     
@@ -605,7 +611,7 @@ static int cli_open_append (CMD *cmd, const char *line,
     } else if (OTHER_IMPORT(ftype)) {
 	err = import_other(newfile, ftype, dset, opt, vprn);
     } else if (ftype == GRETL_ODBC) {
-	err = set_odbc_dsn(line, vprn);
+	err = set_odbc_dsn(cmd->param, vprn);
     } else if (dbdata) {
 	err = set_db_name(newfile, ftype, vprn);
     } else {
@@ -702,6 +708,7 @@ static int cli_exec_line (ExecState *s, int id, DATASET *dset,
     PRN *prn = s->prn;
     MODEL *model = s->model;
     char runfile[MAXLEN];
+    int renumber = 0;
     int err = 0;
 
     if (gretl_compiling_function()) {
@@ -746,7 +753,7 @@ static int cli_exec_line (ExecState *s, int id, DATASET *dset,
 	    set_gretl_errno(err);
 	    catch = 1;
 	}
-	echo_command(cmd, dset, line, prn);
+	gretl_echo_command(cmd, line, prn);
         errmsg(err, prn);
 	return (catch)? 0 : err;
     }
@@ -776,7 +783,7 @@ static int cli_exec_line (ExecState *s, int id, DATASET *dset,
 	/* accumulating loop commands */
 	if (gretl_echo_on()) {
 	    /* straight visual echo */
-	    echo_command(cmd, dset, line, prn);
+	    gretl_echo_command(cmd, line, prn);
 	}
 	err = gretl_loop_append_line(s, dset);
 	if (err) {
@@ -790,7 +797,7 @@ static int cli_exec_line (ExecState *s, int id, DATASET *dset,
 	if (cmd->ci == FUNC && runit > 1) {
 	    ; /* don't echo */
 	} else {
-	    echo_command(cmd, dset, line, prn);
+	    gretl_echo_command(cmd, line, prn);
 	}
     }
 
@@ -801,41 +808,21 @@ static int cli_exec_line (ExecState *s, int id, DATASET *dset,
     switch (cmd->ci) {
 
     case DELEET:
-	if (cmd->opt & OPT_D) {
-	    /* delete from database */
-	    err = db_delete_series_by_name(cmd->param, prn);
-	} else if (cmd->opt & OPT_T) {
-	    /* delete all by type (not series) */
-	    err = gretl_cmd_exec(s, dset);
-	} else if (*cmd->param != '\0') {
-	    /* got a named non-series variable */
-	    err = gretl_delete_var_by_name(cmd->param, prn);
-	} else if (*cmd->parm2 != '\0') {
-	    /* "extra" means we got "list <listname> delete" */
-	    if (get_list_by_name(cmd->parm2)) {
-		err = user_var_delete_by_name(cmd->parm2, prn);
-	    } else {
-		err = E_UNKVAR;
-	    }
-	} else {
-	    /* list of series? */
-	    int nv = 0;
-
-	    err = dataset_drop_listed_variables(cmd->list, dset, 
-						&nv, prn);
-	}
+	err = gretl_delete_variables(cmd->list, cmd->param,
+				     cmd->opt, dset, &renumber,
+				     prn);
 	if (err) {
 	    errmsg(err, prn);
-	} 
+	}
 	break;
 
     case HELP:
-	cli_help(cmd->param, cmd->opt, prn);
+	cli_help(cmd->param, cmd->parm2, cmd->opt, prn);
 	break;
 
     case OPEN:
     case APPEND:
-	err = cli_open_append(cmd, line, dset, model, prn);
+	err = cli_open_append(cmd, dset, model, prn);
 	break;
 
     case NULLDATA:
@@ -879,9 +866,9 @@ static int cli_exec_line (ExecState *s, int id, DATASET *dset,
     case RUN:
     case INCLUDE:
 	if (cmd->ci == INCLUDE) {
-	    err = getopenfile(line, runfile, OPT_I);
+	    err = get_full_filename(cmd->param, runfile, OPT_I);
 	} else {
-	    err = getopenfile(line, runfile, OPT_S);
+	    err = get_full_filename(cmd->param, runfile, OPT_S);
 	}
 	if (err) { 
 	    errmsg(err, prn);
@@ -928,11 +915,11 @@ static int cli_exec_line (ExecState *s, int id, DATASET *dset,
 	break;
 
     case DATAMOD:
-	if (cmd->aux == DS_CLEAR) {
+	if (cmd->auxint == DS_CLEAR) {
 	    err = cli_clear_data(cmd, dset, model);
 	    pputs(prn, _("Dataset cleared\n"));
 	    break;
-	} else if (cmd->aux == DS_RENUMBER) {
+	} else if (cmd->auxint == DS_RENUMBER) {
 	    err = cli_renumber_series(cmd->param, dset, prn);
 	    break;
 	}
