@@ -19,11 +19,6 @@
 
 /* command-line client program for libgretl */
 
-#include <stdio.h>
-#include <stdlib.h>
-
-#include <dirent.h>
-
 #include "libgretl.h"
 #include "version.h"
 #include "monte_carlo.h"
@@ -45,8 +40,10 @@
 # include "gretl_www.h"
 #endif
 
+#include <dirent.h>
+
 #ifdef WIN32
-# include <windows.h>
+# include "gretl_win32.h"
 #else
 # include <sys/stat.h>
 # include <sys/types.h>
@@ -363,7 +360,7 @@ static int maybe_get_input_line_continuation (char *line)
 	    strcpy(tmp, line_read);
 #else
 	    fgets(tmp, MAXLINE, stdin); 
-#endif /* HAVE_READLINE */
+#endif
 	}
 
 	if (*tmp != '\0') {
@@ -646,8 +643,12 @@ int main (int argc, char *argv[])
 	runit = 0;
 	if (makepkg) {
 	    set_gretl_echo(0);
-	} 
-	sprintf(line, "run %s\n", runfile);
+	}
+	if (strchr(runfile, ' ')) {
+	    sprintf(line, "run \"%s\"", runfile);
+	} else {
+	    sprintf(line, "run %s", runfile);
+	}
 	err = cli_exec_line(&state, dset, cmdprn);
 	if (err && fb == NULL) {
 	    exit(EXIT_FAILURE);
@@ -663,7 +664,7 @@ int main (int argc, char *argv[])
 	}
 
 	if (gretl_execute_loop()) { 
-	    if (gretl_loop_exec(&state, dset)) {
+	    if (gretl_loop_exec(&state, dset, NULL)) {
 		return 1;
 	    }
 	} else {
@@ -675,7 +676,8 @@ int main (int argc, char *argv[])
 	}
 
 	if (!state.in_comment) {
-	    if (cmd.context == FOREIGN || gretl_compiling_python(line)) {
+	    if (cmd.context == FOREIGN || cmd.context == MPI ||
+		gretl_compiling_python(line)) {
 		tailstrip(line);
 	    } else {
 		err = maybe_get_input_line_continuation(line); 
@@ -737,7 +739,8 @@ static void cli_exec_callback (ExecState *s, void *ptr,
     int ci = s->cmd->ci;
 
     if (ci == MODELTAB || ci == GRAPHPG) {
-	pprintf(s->prn, _("%s: command not available\n"), s->cmd->word);
+	pprintf(s->prn, _("%s: command not available\n"), 
+		gretl_command_word(ci));
     } else if (ci == OPEN) {
 	char *fname = (char *) ptr;
 
@@ -767,10 +770,6 @@ static int cli_try_http (const char *s, char *fname, int *http)
 {
     int err = 0;
 
-    /* skip past command word */
-    s += strcspn(s, " ");
-    s += strspn(s, " ");
-
     if (strncmp(s, "http://", 7) == 0) {
 #ifdef USE_CURL
 	err = retrieve_public_file(s, fname);
@@ -786,9 +785,8 @@ static int cli_try_http (const char *s, char *fname, int *http)
     return err;
 }
 
-static int cli_open_append (CMD *cmd, const char *line, 
-			    DATASET *dset, MODEL *model,
-			    PRN *prn)
+static int cli_open_append (CMD *cmd, DATASET *dset, 
+			    MODEL *model, PRN *prn)
 {
     gretlopt opt = cmd->opt;
     PRN *vprn = prn;
@@ -798,20 +796,29 @@ static int cli_open_append (CMD *cmd, const char *line,
     int ftype;
     int err = 0;
 
-    err = cli_try_http(line, newfile, &http);
-    if (err) {
-	errmsg(err, prn);
-	return err;
-    }
-
-    if (!http && !(opt & OPT_O)) {
-	/* not using http or ODBC */
-	err = getopenfile(line, newfile, (opt & OPT_W)?
-			  OPT_W : OPT_NONE);
+    if (opt & OPT_K) {
+	/* --frompkg=whatever */
+	err = get_package_data_path(cmd->param, newfile);
 	if (err) {
 	    errmsg(err, prn);
 	    return err;
 	}
+    } else {
+	err = cli_try_http(cmd->param, newfile, &http);
+	if (err) {
+	    errmsg(err, prn);
+	    return err;
+	}
+
+	if (!http && !(opt & OPT_O)) {
+	    /* not using http or ODBC */
+	    err = get_full_filename(cmd->param, newfile, (opt & OPT_W)?
+				    OPT_W : OPT_NONE);
+	    if (err) {
+		errmsg(err, prn);
+		return err;
+	    }
+	}
     }
 
     if (opt & OPT_W) {
@@ -856,7 +863,7 @@ static int cli_open_append (CMD *cmd, const char *line,
     } else if (OTHER_IMPORT(ftype)) {
 	err = import_other(newfile, ftype, dset, opt, vprn);
     } else if (ftype == GRETL_ODBC) {
-	err = set_odbc_dsn(line, vprn);
+	err = set_odbc_dsn(cmd->param, vprn);
     } else if (dbdata) {
 	err = set_db_name(newfile, ftype, vprn);
     } else {
@@ -953,6 +960,7 @@ static int cli_exec_line (ExecState *s, DATASET *dset, PRN *cmdprn)
     MODEL *model = s->model;
     int old_runit = runit;
     char runfile[MAXLEN];
+    int renumber = 0;
     int err = 0;
 
 #if 0
@@ -970,6 +978,9 @@ static int cli_exec_line (ExecState *s, DATASET *dset, PRN *cmdprn)
     }
 
     if (string_is_blank(line)) {
+	if (gretl_echo_space()) {
+	    pputc(prn, '\n');
+	}
 	return 0;
     }
 
@@ -1003,7 +1014,7 @@ static int cli_exec_line (ExecState *s, DATASET *dset, PRN *cmdprn)
 	    set_gretl_errno(err);
 	    catch = 1;
 	}
-	echo_command(cmd, dset, line, prn);
+	gretl_echo_command(cmd, line, prn);
         errmsg(err, prn);
 	return (catch)? 0 : err;
     }
@@ -1038,13 +1049,13 @@ static int cli_exec_line (ExecState *s, DATASET *dset, PRN *cmdprn)
 	/* accumulating loop commands */
 	if (gretl_echo_on() && (!gretl_compiling_loop() || batch || runit)) {
 	    /* straight visual echo */
-	    echo_command(cmd, dset, line, prn);
+	    gretl_echo_command(cmd, line, prn);
 	}
 	err = gretl_loop_append_line(s, dset);
 	if (err) {
 	    errmsg(err, prn);
 	} else if (!batch && !runit) {
-	    gretl_record_command(cmd, dset, line, cmdprn);
+	    gretl_record_command(cmd, line, cmdprn);
 	}
 	return err;
     }
@@ -1054,7 +1065,7 @@ static int cli_exec_line (ExecState *s, DATASET *dset, PRN *cmdprn)
 	if (cmd->ci == FUNC && runit > 1) {
 	    ; /* don't echo */
 	} else if (batch || runit) {
-	    echo_command(cmd, dset, line, prn);
+	    gretl_echo_command(cmd, line, prn);
 	}
     }
 
@@ -1065,46 +1076,25 @@ static int cli_exec_line (ExecState *s, DATASET *dset, PRN *cmdprn)
     switch (cmd->ci) {
 
     case DELEET:
-	if (cmd->opt & OPT_D) {
-	    /* delete from database */
-	    err = db_delete_series_by_name(cmd->param, prn);
-	} else if (cmd->opt & OPT_T) {
-	    /* delete all by type (not series) */
-	    err = gretl_cmd_exec(s, dset);
-	} else if (*cmd->param != '\0') {
-	    /* got a named non-series variable */
-	    err = gretl_delete_var_by_name(cmd->param, prn);
-	} else if (*cmd->parm2 != '\0') {
-	    /* "extra" means we got "list <listname> delete" */
-	    if (get_list_by_name(cmd->parm2)) {
-		err = user_var_delete_by_name(cmd->parm2, prn);
-	    } else {
-		err = E_UNKVAR;
-	    }
-	} else {
-	    /* list of series? */
-	    int nv = 0;
-
-	    err = dataset_drop_listed_variables(cmd->list, dset, 
-						&nv, prn);
-	    if (!err && nv > 0 && !batch) {
-		pputs(prn, _("Take note: variables have been renumbered"));
-		pputc(prn, '\n');
-		maybe_list_vars(dset, prn);
-	    }
-	}
+	err = gretl_delete_variables(cmd->list, cmd->param,
+				     cmd->opt, dset, &renumber,
+				     prn);
 	if (err) {
 	    errmsg(err, prn);
-	} 
+	} else if (renumber && !batch) {
+	    pputs(prn, _("Take note: variables have been renumbered"));
+	    pputc(prn, '\n');
+	    maybe_list_vars(dset, prn);
+	}
 	break;
 
     case HELP:
-	cli_help(cmd->param, cmd->opt, prn);
+	cli_help(cmd->param, cmd->parm2, cmd->opt, prn);
 	break;
 
     case OPEN:
     case APPEND:
-	err = cli_open_append(cmd, line, dset, model, prn);
+	err = cli_open_append(cmd, dset, model, prn);
 	break;
 
     case NULLDATA:
@@ -1146,9 +1136,9 @@ static int cli_exec_line (ExecState *s, DATASET *dset, PRN *cmdprn)
     case RUN:
     case INCLUDE:
 	if (cmd->ci == INCLUDE) {
-	    err = getopenfile(line, runfile, OPT_I);
+	    err = get_full_filename(cmd->param, runfile, OPT_I);
 	} else {
-	    err = getopenfile(line, runfile, OPT_S);
+	    err = get_full_filename(cmd->param, runfile, OPT_S);
 	}
 	if (err) { 
 	    errmsg(err, prn);
@@ -1195,11 +1185,11 @@ static int cli_exec_line (ExecState *s, DATASET *dset, PRN *cmdprn)
 	break;
 
     case DATAMOD:
-	if (cmd->aux == DS_CLEAR) {
+	if (cmd->auxint == DS_CLEAR) {
 	    err = cli_clear_data(cmd, dset, model);
 	    pputs(prn, _("Dataset cleared\n"));
 	    break;
-	} else if (cmd->aux == DS_RENUMBER) {
+	} else if (cmd->auxint == DS_RENUMBER) {
 	    err = cli_renumber_series(cmd->param, dset, prn);
 	    break;
 	}
@@ -1212,7 +1202,7 @@ static int cli_exec_line (ExecState *s, DATASET *dset, PRN *cmdprn)
 
     if (!err && cmd->ci != QUIT && gretl_echo_on() && !batch && !old_runit) {
 	/* record a successful interactive command */
-	gretl_record_command(cmd, dset, line, cmdprn);
+	gretl_record_command(cmd, line, cmdprn);
     }
 
     if (err) {
