@@ -33,8 +33,6 @@
 #include "datawiz.h"
 #include "winstack.h"
 
-#include "boxplots.h"
-
 static void doubleclick_action (windata_t *vwin)
 {
     switch (vwin->role) {
@@ -192,6 +190,10 @@ void file_save (windata_t *vwin, int ci)
     case SAVE_SCRIPT:
     case SAVE_GP_CMDS:
     case SAVE_R_CMDS:
+    case SAVE_OX_CMDS:
+    case SAVE_OCTAVE_CMDS:
+    case SAVE_PYTHON_CMDS:
+    case SAVE_STATA_CMDS:
     case SAVE_DATA:
     case SAVE_DATA_AS:
 	file_selector(ci, FSEL_DATA_VWIN, vwin);
@@ -201,7 +203,7 @@ void file_save (windata_t *vwin, int ci)
 	data_export_selection_wrapper(ci);
 	break;
     case SAVE_FUNCTIONS:
-	functions_selection_wrapper();
+	functions_selection_wrapper(vwin_toplevel(vwin));
 	break;
     case SAVE_TEX:
     case SAVE_TEXT:
@@ -366,6 +368,8 @@ static int selector_callback_code (const gchar *s)
 	return SCATTERS;
     if (!strcmp(s, "MultiTS"))
 	return TSPLOTS;
+    if (!strcmp(s, "GR_BOX"))
+	return GR_BOX;
     if (!strcmp(s, "GR_FBOX"))
 	return GR_FBOX;
     if (!strcmp(s, "VLAGSEL"))
@@ -443,6 +447,8 @@ void selector_callback (GtkAction *action, gpointer data)
 			 NULL);
     } else if (ci == TSPLOTS) {
 	simple_selection(ci, _("gretl: define graph"), do_scatters, NULL);
+    } else if (ci == GR_BOX) {
+	simple_selection(ci, _("gretl: define graph"), do_regular_boxplot, NULL);
     } else if (ci == SPEARMAN) {
 	char title[64];
 	
@@ -470,8 +476,6 @@ static int gretl_callback_code (const gchar *s)
 	return VSETMISS;
     if (!strcmp(s, "GSETMISS")) 
 	return GSETMISS;
-    if (!strcmp(s, "GR_BOX")) 
-	return GR_BOX;
     if (!strcmp(s, "gmm")) 
 	return GMM;
     if (!strcmp(s, "mle")) 
@@ -528,13 +532,6 @@ void gretl_callback (GtkAction *action, gpointer data)
 	query = N_("Enter value to be read as \"missing\":");
 	okfunc = do_global_setmiss;
 	break;
-    case GR_BOX:
-	title = N_("gretl: boxplots");
-	query = N_("Specify variables to plot:");
-	okfunc = do_box_graph;
-	click = VARCLICK_INSERT_NAME;
-	defstr = get_last_boxplots_string();
-	break;
     case GMM:
 	title = N_("gretl: GMM");
 	query = N_("GMM: Specify function and orthogonality conditions:");
