@@ -313,10 +313,10 @@ static int real_console_exec (ExecState *state)
     push_history_line(state->line);
 
     state->flags = CONSOLE_EXEC;
-    err = gui_exec_line(state, dataset);
+    err = gui_exec_line(state, dataset, console_main);
 
     while (!err && gretl_execute_loop()) {
-	err = gretl_loop_exec(state, dataset);
+	err = gretl_loop_exec(state, dataset, NULL);
     }
 
 #if CDEBUG
