@@ -23,16 +23,17 @@
 #include "boxplots.h"
 #include "database.h"
 #include "datafiles.h"
+#include "gretl_xml.h"
 #include "gretl_www.h"
 #include "gretl_untar.h"
 #include "gretl_zip.h"
-#include "gretl_xml.h"
 #include "menustate.h"
 #include "treeutils.h"
 #include "winstack.h"
 #include "toolbar.h"
 #include "dlgutils.h"
 #include "fncall.h"
+#include "dbread.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -210,17 +211,17 @@ static int expand_data_dialog (int src_pd, int targ_pd, int *interpol,
 	(targ_pd == 12 && src_pd == 4)) {
 	/* interpolation is an option */
 	const char *opts[] = {
-	    N_("Interpolate higher frequency values"),
-	    N_("Repeat the lower frequency values")
+	    N_("Repeat the lower frequency values"),
+	    N_("Interpolate higher frequency values")
 	};
 
 	resp = radio_dialog("gretl", _("Adding a lower frequency series to a\n"
 				       "higher frequency dataset"),
 			    opts, 2, 0, EXPAND, parent);
-	if (resp == 0) {
+	if (resp == 1) {
 	    *interpol = 1;
 	    resp = GRETL_YES;
-	} else if (resp == 1) {
+	} else if (resp == 0) {
 	    resp = GRETL_YES;
 	}
     } else {
@@ -232,7 +233,7 @@ static int expand_data_dialog (int src_pd, int targ_pd, int *interpol,
 			      "repeating each value %d times.  In general, this is\n"
 			      "not a valid thing to do."),
 			    mult);
-	resp = yes_no_dialog("gretl", msg, 0);
+	resp = yes_no_dialog("gretl", msg, parent);
 	g_free(msg);
     }
 
@@ -302,35 +303,49 @@ static const char *trimmed_db_name (const char *fname)
 static void record_db_open_command (dbwrapper *dw)
 {
     if (dw->fname != NULL) {
-	int quotes = (strchr(dw->fname, ' ') != NULL);
+	const char *current_db = get_db_name();
+	char *dbpath;
 
 	if (dw->dbtype == GRETL_PCGIVE_DB) {
-	    if (quotes) {
-		lib_command_sprintf("open \"%s.bn7\"", dw->fname);
-	    } else {
-		lib_command_sprintf("open %s.bn7", dw->fname);
-	    }
+	    dbpath = g_strdup_printf("%s.bn7", dw->fname);
 	} else if (dw->dbtype == GRETL_NATIVE_DB) {
-	    const char *tmp = trimmed_db_name(dw->fname);
-
-	    if (tmp != NULL) {
-		lib_command_sprintf("open %s.bin", tmp);
-	    } else if (quotes) {
-		lib_command_sprintf("open \"%s.bin\"", dw->fname);
-	    } else {
-		lib_command_sprintf("open %s.bin", dw->fname);
-	    }
-	} else if (dw->dbtype == GRETL_NATIVE_DB_WWW) {
-	    lib_command_sprintf("open %s --www", dw->fname);
+	    dbpath = g_strdup_printf("%s.bin", dw->fname);
 	} else {
-	    if (quotes) {
-		lib_command_sprintf("open \"%s\"", dw->fname);
-	    } else {
-		lib_command_sprintf("open %s", dw->fname);
+	    dbpath = g_strdup(dw->fname);
+	}
+
+	/* record the "open" command if the database in
+	   question is not already open */
+
+	if (strcmp(current_db, dbpath)) {
+	    int err = set_db_name(dbpath, dw->dbtype, NULL);
+	    int done = 0;
+
+	    if (!err && dw->dbtype == GRETL_NATIVE_DB) {
+		const char *s = trimmed_db_name(dbpath);
+
+		if (s != NULL) {
+		    lib_command_sprintf("open %s", s);
+		    done = 1;
+		}
+	    }
+
+	    if (!err && !done) {
+		if (dw->dbtype == GRETL_NATIVE_DB_WWW) {
+		    lib_command_sprintf("open %s --www", dbpath);
+		} else if (strchr(dbpath, ' ') != NULL) {
+		    lib_command_sprintf("open \"%s\"", dbpath);
+		} else {
+		    lib_command_sprintf("open %s", dbpath);
+		}
+	    }
+
+	    if (!err) {
+		record_command_verbatim();
 	    }
 	}
 
-	record_command_verbatim();
+	g_free(dbpath);
     }
 }
 
@@ -377,9 +392,10 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 	dbv = series_index(dataset, sinfo->varname);
 	if (dbv < dataset->v) {
 	    if (dw->nv == 1) {
-		resp = yes_no_dialog ("gretl",                      
-				      _("There is already a variable of this name\n"
-					"in the dataset.  OK to overwrite it?"), 0);
+		resp = yes_no_dialog("gretl",                      
+				     _("There is already a variable of this name\n"
+				       "in the dataset.  OK to overwrite it?"),
+				     vwin_toplevel(vwin));
 		if (resp != GRETL_YES) {
 		    return 0;
 		}
@@ -435,10 +451,11 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 
 	/* record successful importation in command log */
 	if (compact && (cstr = compact_method_string(method)) != NULL) {
-	    lib_command_sprintf("data (compact=%s) %s", cstr,
-				sinfo->varname);
+	    lib_command_sprintf("data %s --compact=%s", sinfo->varname,
+				cstr);
+	} else if (interpol) {
+	    lib_command_sprintf("data %s --interpolate", sinfo->varname);
 	} else {
-	    /* FIXME: handle expand/interpolate option */
 	    lib_command_sprintf("data %s", sinfo->varname);
 	}
 
@@ -561,7 +578,7 @@ static void db_delete_callback (GtkWidget *w, windata_t *vwin)
 			      "from the database '%s'?"), 
 			    gtk_window_get_title(GTK_WINDOW(vwin->main)));
 
-    resp = yes_no_dialog ("gretl", query, 0);
+    resp = yes_no_dialog("gretl", query, vwin_toplevel(vwin));
 
     g_free(query);
 
@@ -763,6 +780,11 @@ static void db_view_codebook (GtkWidget *w, windata_t *vwin)
     view_file(cbname, 0, 0, 78, 350, VIEW_CODEBOOK);
 }
 
+static void db_show_index (GtkWidget *w, windata_t *vwin)
+{
+    show_native_dbs();
+}
+
 static void build_db_popup (windata_t *vwin, int cb, int del)
 {
     if (vwin->popup != NULL) {
@@ -793,29 +815,25 @@ static void build_db_popup (windata_t *vwin, int cb, int del)
     }
 }
 
-static void db_close (GtkWidget *w, windata_t *vwin)
-{
-    gtk_widget_destroy(vwin->main);
-}
-
 enum {
     DEL_BTN = 1,
-    CB_BTN
+    CB_BTN,
+    IDX_BTN
 };
 
 static GretlToolItem db_items[] = {
-    { N_("Display values"), GTK_STOCK_OPEN,   G_CALLBACK(db_display_series), 0 },
+    { N_("Display values"), GTK_STOCK_MEDIA_PLAY, G_CALLBACK(db_display_series), 0 },
     { N_("Graph"),          GRETL_STOCK_TS,   G_CALLBACK(db_graph_series), 0 },
     { N_("Add to dataset"), GTK_STOCK_ADD,    G_CALLBACK(db_import_series), 0 },
+    { N_("List databases"), GTK_STOCK_INDEX,  G_CALLBACK(db_show_index), IDX_BTN },
     { N_("Delete"),         GTK_STOCK_DELETE, G_CALLBACK(db_delete_callback), DEL_BTN },
-    { N_("Codebook"),       GRETL_STOCK_BOOK, G_CALLBACK(db_view_codebook), CB_BTN },
-    { N_("Windows"),        GRETL_STOCK_WINLIST, GNULL, 0 },
-    { N_("Close"),          GTK_STOCK_CLOSE,  G_CALLBACK(db_close), 0 }
+    { N_("Codebook"),       GRETL_STOCK_BOOK, G_CALLBACK(db_view_codebook), CB_BTN }
 };
 
 static int n_db_items = G_N_ELEMENTS(db_items);
 
-static void make_db_toolbar (windata_t *vwin, int cb, int del)
+static void make_db_toolbar (windata_t *vwin, int cb, int del,
+			     int index_button)
 {
     GtkWidget *hbox;
     GretlToolItem *item;
@@ -830,17 +848,16 @@ static void make_db_toolbar (windata_t *vwin, int cb, int del)
 	item = &db_items[i];
 	if (!del && item->flag == DEL_BTN) {
 	    continue;
-	} if (!cb && item->flag == CB_BTN) {
+	} else if (!cb && item->flag == CB_BTN) {
+	    continue;
+	} else if (!index_button && item->flag == IDX_BTN) {
 	    continue;
 	}
-	if (winlist_item(item)) {
-	    vwin_toolbar_insert_winlist(vwin);
-	} else {
-	    gretl_toolbar_insert(vwin->mbar, item, item->func, vwin, -1);
-	}
+	gretl_toolbar_insert(vwin->mbar, item, item->func, vwin, -1);
     }
 
     gtk_box_pack_start(GTK_BOX(hbox), vwin->mbar, FALSE, FALSE, 0);
+    vwin_add_winlist(vwin);
     vwin_add_finder(vwin);
     gtk_widget_show_all(hbox);
 }
@@ -927,7 +944,8 @@ maybe_adjust_descrip_column (windata_t *vwin)
 }
 
 static int 
-make_db_index_window (int action, char *fname, char *buf)
+make_db_index_window (int action, char *fname, char *buf,
+		      int index_button)
 {
     GtkWidget *listbox;
     gchar *title;
@@ -951,7 +969,7 @@ make_db_index_window (int action, char *fname, char *buf)
 	title = strrchr(fname, SLASH) + 1;
     } else {
 	title = fname;
-    } 
+    }
 
     vwin = gretl_browser_new(action, title);
     if (vwin == NULL) {
@@ -976,7 +994,7 @@ make_db_index_window (int action, char *fname, char *buf)
     cb = db_has_codebook(fname);
     del = db_is_writable(action, fname);
 
-    make_db_toolbar(vwin, cb, del);
+    make_db_toolbar(vwin, cb, del, index_button);
     build_db_popup(vwin, cb, del);
 
     listbox = database_window(vwin);
@@ -992,7 +1010,7 @@ make_db_index_window (int action, char *fname, char *buf)
 	gtk_box_pack_start(GTK_BOX(hbox), vwin->status, FALSE, FALSE, 0);
     }
 
-    if (action == NATIVE_SERIES) { 
+    if (action == NATIVE_SERIES) {
 	err = add_local_db_series_list(vwin);
     } else if (action == REMOTE_SERIES) { 
 	err = add_remote_db_series_list(vwin, buf);
@@ -1015,12 +1033,12 @@ make_db_index_window (int action, char *fname, char *buf)
 
 void open_rats_window (char *fname)
 {
-    make_db_index_window(RATS_SERIES, fname, NULL);
+    make_db_index_window(RATS_SERIES, fname, NULL, 0);
 }
 
 void open_bn7_window (char *fname)
 {
-    make_db_index_window(PCGIVE_SERIES, fname, NULL);
+    make_db_index_window(PCGIVE_SERIES, fname, NULL, 0);
 }
 
 static int check_serinfo (char *str, char *sername, int *nobs)
@@ -1039,7 +1057,7 @@ static int check_serinfo (char *str, char *sername, int *nobs)
 	!isdigit((unsigned char) *endobs) ||
 	(pdc != 'M' && pdc != 'A' && pdc != 'Q' && pdc != 'U' &&
 	 pdc != 'D' && pdc != 'B')) {
-	errbox(_("Database parse error at variable '%s'"), sername);
+	errbox_printf(_("Database parse error at variable '%s'"), sername);
 	fprintf(stderr, "%s: stobs='%s', endobs='%s', pdc='%c', nobs = %d\n",
 		sername, stobs, endobs, pdc, *nobs);
 	err = 1;
@@ -1116,8 +1134,8 @@ static int add_local_db_series_list (windata_t *vwin)
 
 	len = strlen(line1);
 	if (line1[len-1] != '\n') {
-	    errbox("Database index line is too long: max is %d characters",
-		   DB_LINELEN - 1);
+	    errbox_printf("Database index line is too long: max is %d characters",
+			  DB_LINELEN - 1);
 	    break;
 	}
 
@@ -1388,7 +1406,7 @@ static GtkWidget *database_window (windata_t *vwin)
        least part of the observation-info column */
 
     box = gtk_vbox_new(FALSE, 0);
-    vwin_add_list_box(vwin, GTK_BOX(box), 4, TRUE, types, titles, 0);
+    vwin_add_list_box(vwin, GTK_BOX(box), 4, 1, types, titles, 0);
     g_signal_connect(G_OBJECT(vwin->listbox), "button-press-event",
 		     G_CALLBACK(popup_menu_handler), 
 		     vwin->popup);
@@ -1599,7 +1617,7 @@ gboolean open_named_db_index (char *dbname)
 	file_read_errbox(dbname);
     } else {
 	fclose(fp);
-	make_db_index_window(action, dbname, NULL);
+	make_db_index_window(action, dbname, NULL, 0);
 	ret = TRUE;
     }
     
@@ -1619,7 +1637,7 @@ gboolean open_named_remote_db_index (char *dbname)
     } else if (getbuf != NULL && !strncmp(getbuf, "Couldn't open", 13)) {
 	errbox(getbuf);
     } else {
-	err = make_db_index_window(REMOTE_SERIES, dbname, getbuf);
+	err = make_db_index_window(REMOTE_SERIES, dbname, getbuf, 0);
 	if (!err) {
 	    ret = TRUE;
 	}
@@ -1632,10 +1650,11 @@ gboolean open_named_remote_db_index (char *dbname)
 
 void open_db_index (GtkWidget *w, gpointer data)
 {
+    windata_t *vwin = (windata_t *) data;
     gchar *fname = NULL, *dbdir = NULL;
     char dbfile[MAXLEN];
     int action = NATIVE_SERIES;
-    windata_t *vwin = (windata_t *) data;
+    int idx = 0;
 
     tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), 
 			 vwin->active_var, 0, &fname);
@@ -1652,7 +1671,19 @@ void open_db_index (GtkWidget *w, gpointer data)
     g_free(fname);
     g_free(dbdir);
 
-    make_db_index_window(action, dbfile, NULL); 
+    if (action == NATIVE_SERIES) {
+	GtkTreeModel *mod;
+	
+	mod = gtk_tree_view_get_model(GTK_TREE_VIEW(vwin->listbox));
+	idx = tree_model_count_rows(mod) > 1;
+    }
+
+    make_db_index_window(action, dbfile, NULL, idx);
+
+    if (action == NATIVE_SERIES) {
+	/* close the window from which this db was selected */
+	gtk_widget_destroy(vwin->main);
+    }
 }
 
 void open_remote_db_index (GtkWidget *w, gpointer data)
@@ -1683,7 +1714,7 @@ void open_remote_db_index (GtkWidget *w, gpointer data)
 	show_network_error(vwin);
     } else {
 	update_statusline(vwin, "OK");
-	make_db_index_window(REMOTE_SERIES, fname, getbuf);
+	make_db_index_window(REMOTE_SERIES, fname, getbuf, 0);
     }
 
     g_free(fname);
@@ -1861,11 +1892,12 @@ static int ggz_extract (char *ggzname)
     return err;
 }
 
-static void offer_db_open (char *target)
+static void offer_db_open (char *target, windata_t *vwin)
 {
-    int resp = yes_no_dialog ("gretl",                      
-			      _("Database installed.\n"
-				"Open it now?"), 0);
+    int resp = yes_no_dialog("gretl",                      
+			     _("Database installed.\n"
+			       "Open it now?"),
+			     vwin_toplevel(vwin));
 
     if (resp == GRETL_YES) { 
 	char dbpath[MAXLEN];
@@ -1918,6 +1950,8 @@ static int get_target_in_home (char *targ, int code,
     return err;
 }
 
+#ifndef OS_OSX
+
 static void get_system_target (char *targ, int code, 
 			       const char *objname,
 			       const char *ext)
@@ -1934,6 +1968,8 @@ static void get_system_target (char *targ, int code,
     strcat(targ, ext);
 }
 
+#endif
+
 enum {
     REAL_INSTALL,
     TMP_INSTALL
@@ -1944,8 +1980,7 @@ enum {
    data files, or function package 
 */
 
-static char *get_writable_target (int code, char *objname,
-				  gboolean zipfile)
+static char *get_writable_target (int code, char *objname)
 {
     const char *ext;
     char *targ;
@@ -1966,7 +2001,7 @@ static char *get_writable_target (int code, char *objname,
     if (code == REMOTE_DB) {
 	ext = ".ggz";
     } else if (code == REMOTE_FUNC_FILES) {
-	ext = zipfile ? ".zip" : ".gfn";
+	ext = ".gfn";
     } else {
 	ext = ".tar.gz";
     }
@@ -2033,64 +2068,26 @@ static int unpack_book_data (const char *fname)
     return err;
 }
 
-int unzip_package_file (const char *zipname, const char *path)
+static gchar *make_gfn_path (const char *pkgname)
 {
-    char *p, dirname[FILENAME_MAX];
-    GError *gerr = NULL;
-    int err = 0;
+    const char *fpath = gretl_function_package_path();
 
-    strcpy(dirname, path);
-    p = strrchr(dirname, SLASH);
-    if (p != NULL) {
-	*p = '\0';
-    }
-
-    err = gretl_chdir(dirname);
-    if (err) {
-	return err;
-    }
-
-#ifdef G_OS_WIN32
-    if (1) {
-	char *test;
-
-	fprintf(stderr, "unzip_package_file: unzipping in '%s'\n", dirname);
-	*dirname = '\0';
-	test = getcwd(dirname, FILENAME_MAX - 1);
-	fprintf(stderr, " check: getcwd() gives '%s'\n", test);
-    }
-#endif
-
-    err = gretl_unzip_file(zipname, &gerr);
-    if (gerr != NULL) {
-	gretl_errmsg_set(gerr->message);
-	if (!err) {
-	    err = 1;
-	}
-	g_error_free(gerr);
-    }
-
-    gretl_remove(zipname);
-
-    if (err) {
-	fprintf(stderr, "gretl_unzip_file: err = %d\n", err);
-    }
-
-    return err;
+    return g_strdup_printf("%s%s%c%s.gfn", fpath,
+			   pkgname, SLASH, pkgname);
 }
 
 #define STATUS_COLUMN  4
 #define ZIPFILE_COLUMN 5
 
 /* note : @vwin here is the source viewer window displaying the
-   remote file (database or datafiles package or function package)
-   that is being installed onto the local machine.
+   remote file (database, or datafiles package, or function package)
+   that is to be installed onto the local machine.
 */
 
 void install_file_from_server (GtkWidget *w, windata_t *vwin)
 {
     gchar *objname = NULL;
-    char *path = NULL;
+    char *targ = NULL;
     gboolean zipfile = FALSE;
     int err = 0;
 
@@ -2099,6 +2096,7 @@ void install_file_from_server (GtkWidget *w, windata_t *vwin)
     */
 
     if (vwin->role == REMOTE_DB) {
+	/* database files */
 	GtkTreeSelection *sel;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
@@ -2108,7 +2106,7 @@ void install_file_from_server (GtkWidget *w, windata_t *vwin)
 	    gtk_tree_model_get(model, &iter, 0, &objname, -1);
 	}
     } else {
-	/* remote datafiles or function package */
+	/* datafiles or function package */
 	tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), 
 			     vwin->active_var, 0, &objname);
 	if (vwin->role == REMOTE_FUNC_FILES) {
@@ -2120,36 +2118,43 @@ void install_file_from_server (GtkWidget *w, windata_t *vwin)
     if (objname == NULL || *objname == '\0') {
 	g_free(objname);
 	return;
-    } 
+    }
 
-    path = get_writable_target(vwin->role, objname, zipfile);
-    if (path == NULL) {
-	g_free(objname);
-	return;
+    if (!zipfile) {
+	targ = get_writable_target(vwin->role, objname);
+	if (targ == NULL) {
+	    g_free(objname);
+	    return;
+	}
     }
 
     if (vwin->role == REMOTE_FUNC_FILES) {
 	if (zipfile) {
-	    gchar *zipname = g_strdup_printf("%s.zip", objname);
+	    const char *path = gretl_function_package_path();
+	    gchar *basename = g_strdup_printf("%s.zip", objname);
+	    gchar *fullname;
 
-	    err = retrieve_remote_function_package(zipname, path);
+	    fullname = g_strdup_printf("%s%s", path, basename);
+	    err = retrieve_remote_function_package(basename, fullname);
 	    if (!err) {
-		err = unzip_package_file(zipname, path);
+		err = gretl_unzip_into(fullname, path);
+		gretl_remove(fullname);
 	    }
-	    g_free(zipname);
+	    g_free(fullname);
+	    g_free(basename);
 	} else {
-	    err = retrieve_remote_function_package(objname, path);
+	    err = retrieve_remote_function_package(objname, targ);
 	}
     } else if (vwin->role == REMOTE_DATA_PKGS) {
 	gchar *tarname = g_strdup_printf("%s.tar.gz", objname);
 
-	err = retrieve_remote_datafiles_package(tarname, path);
+	err = retrieve_remote_datafiles_package(tarname, targ);
 	g_free(tarname);
     } else if (vwin->role == REMOTE_DB) {
 #if G_BYTE_ORDER == G_BIG_ENDIAN
-	err = retrieve_remote_db(objname, path, GRAB_NBO_DATA);
+	err = retrieve_remote_db(objname, targ, GRAB_NBO_DATA);
 #else
-	err = retrieve_remote_db(objname, path, GRAB_DATA);
+	err = retrieve_remote_db(objname, targ, GRAB_DATA);
 #endif
     }
 
@@ -2159,7 +2164,17 @@ void install_file_from_server (GtkWidget *w, windata_t *vwin)
 	windata_t *local = get_local_viewer(vwin->role);
 	
 	if (vwin->role == REMOTE_FUNC_FILES) {
-	    if (!maybe_handle_pkg_menu_option(path, vwin->main)) {
+	    int notified = 0;
+
+	    if (zipfile) {
+		gchar *gfnpath = make_gfn_path(objname);
+
+		notified = maybe_handle_pkg_menu_option(gfnpath, vwin->main);
+		g_free(gfnpath);
+	    } else {
+		notified = maybe_handle_pkg_menu_option(targ, vwin->main);
+	    }
+	    if (!notified) {
 		infobox(_("Installed"));
 	    }
 	    list_store_set_string(GTK_TREE_VIEW(vwin->listbox),
@@ -2169,9 +2184,9 @@ void install_file_from_server (GtkWidget *w, windata_t *vwin)
 		populate_filelist(local, NULL);
 	    }
 	} else if (vwin->role == REMOTE_DATA_PKGS) {
-	    fprintf(stderr, "downloaded '%s'\n", path);
-	    err = unpack_book_data(path);
-	    remove(path);
+	    fprintf(stderr, "downloaded '%s'\n", targ);
+	    err = unpack_book_data(targ);
+	    remove(targ);
 	    if (err) {
 		errbox(_("Error unzipping compressed data"));
 	    } else {
@@ -2179,8 +2194,8 @@ void install_file_from_server (GtkWidget *w, windata_t *vwin)
 	    }
 	} else {
 	    /* gretl-zipped database package */
-	    fprintf(stderr, "downloaded '%s'\n", path);
-	    err = ggz_extract(path);
+	    fprintf(stderr, "downloaded '%s'\n", targ);
+	    err = ggz_extract(targ);
 	    if (err) {
 		if (err != E_FOPEN) {
 		    errbox(_("Error unzipping compressed data"));
@@ -2192,26 +2207,78 @@ void install_file_from_server (GtkWidget *w, windata_t *vwin)
 		if (local != NULL) {
 		    populate_filelist(local, NULL);
 		} else {
-		    offer_db_open(path);
+		    offer_db_open(targ, vwin);
 		}
 	    }
 	}
     }
 
     g_free(objname);
-    free(path);
+    free(targ);
+}
+
+/* Called when the "install" command is used to install a function
+   package via console or script: try to sync the local and/or remote
+   function-package windows, if they're open. Also present the
+   package's menu-attachment option, if any.
+*/
+
+void maybe_update_pkgview (const char *filename,
+			   const char *pkgname,
+			   int zipfile,
+			   GtkWidget *parent)
+{
+    windata_t *vwin;
+
+    /* update local package browser? */
+    vwin = get_browser_for_role(FUNC_FILES); 
+    if (vwin != NULL) {
+	populate_filelist(vwin, NULL);
+    }
+
+   /* update remote package browser? */ 
+    vwin = get_browser_for_role(REMOTE_FUNC_FILES); 
+    if (vwin != NULL && find_package_in_viewer(vwin, pkgname)) {
+	list_store_set_string(GTK_TREE_VIEW(vwin->listbox),
+			      vwin->active_var, STATUS_COLUMN,
+			      _("Up to date"));
+    }
+
+    if (parent != NULL) {
+	/* offer menu attachment if applicable */
+	if (zipfile) {
+	    gchar *gfnpath = make_gfn_path(pkgname);
+
+	    maybe_handle_pkg_menu_option(gfnpath, parent);
+	    g_free(gfnpath);
+	} else {
+	    maybe_handle_pkg_menu_option(filename, parent);
+	}
+    }
 }
 
 void pkg_info_from_server (GtkWidget *w, windata_t *vwin)
 {
     static int idx;
     gchar *path, *objname = NULL;
+    int zipfile = 0;
     int err = 0;
 
     tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), 
 			 vwin->active_var, 0, &objname);
+    tree_view_get_bool(GTK_TREE_VIEW(vwin->listbox), 
+		       vwin->active_var, ZIPFILE_COLUMN, &zipfile);
+
     path = g_strdup_printf("%sdltmp.%d", gretl_dotdir(), idx++);
-    err = retrieve_remote_function_package(objname, path);
+
+    if (zipfile) {
+	gchar *zipname = g_strdup_printf("%s.zip", objname);
+
+	err = retrieve_remote_gfn_content(zipname, path);
+	g_free(zipname);
+    } else {
+	err = retrieve_remote_function_package(objname, path);
+    }
 
     if (err) {
 	show_network_error(NULL);
@@ -2355,7 +2422,8 @@ read_db_files_in_dir (DIR *dir, int dbtype, const char *path,
     return ndb;
 }
 
-static void get_local_object_status (const char *fname, int role, 
+static void get_local_object_status (const char *fname,
+				     int role, 
 				     const char **status, 
 				     time_t remtime)
 {
@@ -2791,9 +2859,9 @@ gint populate_remote_func_list (windata_t *vwin)
     GtkTreeIter iter;  
     char *getbuf = NULL;
     char line[1024];
-    char fname[32];
+    char fname[64];
+    char basename[32];
     const char *status;
-    char *basename;
     time_t remtime;
     int n, err = 0;
 
@@ -2826,11 +2894,18 @@ gint populate_remote_func_list (windata_t *vwin)
 	    continue;
 	}
 
+	strcpy(basename, fname);
+	strip_extension(basename);
+
+	zipfile = has_suffix(fname, ".zip");
+	if (zipfile) {
+	    /* local status: look for PKG/PKG.gfn, not PKG.zip */
+	    sprintf(fname, "%s%c%s.gfn", basename, SLASH, basename);
+	}
+
 	status = "";
 	get_local_object_status(fname, vwin->role, &status, remtime);
-	zipfile = has_suffix(fname, ".zip");
-	basename = strip_extension(fname);
-
+	
 	if (bufgets(line, sizeof line, getbuf)) {
 	    tailstrip(line);
 	    utf8_correct(line);
@@ -2898,7 +2973,6 @@ gint populate_remote_data_pkg_list (windata_t *vwin)
     char line[256];
     char fname[32];
     char tstr[16];
-    char *basename;
     char *descrip;
     int n, err = 0;
 
@@ -2926,7 +3000,7 @@ gint populate_remote_data_pkg_list (windata_t *vwin)
 	    continue;
 	}
 
-	basename = strip_extension(fname);
+	strip_extension(fname);
 
 	if (bufgets(line, sizeof line, getbuf)) {
 	    tailstrip(line);
@@ -2940,7 +3014,7 @@ gint populate_remote_data_pkg_list (windata_t *vwin)
 
 	gtk_list_store_append(store, &iter);
 	gtk_list_store_set(store, &iter, 
-			   0, basename, 
+			   0, fname, 
 			   1, descrip, 
 			   2, tstr,
 			   -1);
