@@ -13,7 +13,10 @@ void install_file_from_server (GtkWidget *w, windata_t *vwin);
 
 void pkg_info_from_server (GtkWidget *w, windata_t *vwin);
 
-int unzip_package_file (const char *zipname, const char *path);
+void maybe_update_pkgview (const char *filename,
+			   const char *pkgname,
+			   int zipfile,
+			   GtkWidget *parent);
 
 gint populate_dbfilelist (windata_t *vwin, int *pndb);
 
