@@ -35,6 +35,7 @@
 #include "winstack.h"
 #include "fileselect.h"
 #include "dlgutils.h"
+#include "textbuf.h"
 
 #include "gretl_xml.h"
 #include "gretl_func.h"
@@ -77,6 +78,11 @@ enum {
     COLL_PS
 };
 
+enum {
+    PKG_ATTR_RES = 1 << 0,
+    PKG_ATTR_DOC = 1 << 1
+};
+
 struct fpkg_response {
     int col1_width;
     int try_server;
@@ -847,15 +853,23 @@ windata_t *display_function_package_data (const char *pkgname,
 	    pprintf(prn, "File: %s\n", path);
 	}
 	err = print_function_package_info(path, prn);
+    } else if (role == VIEW_PKG_SAMPLE) {
+	err = print_function_package_sample(path, tabwidth, prn);
     } else {
-	err = print_function_package_code(path, prn);
+	err = print_function_package_code(path, tabwidth, prn);
     }
 	
     if (err) {
 	gretl_print_destroy(prn);
 	gui_errmsg(err);
     } else {
-	gchar *title = g_strdup_printf("gretl: %s", pkgname);
+	gchar *title;
+
+	if (role == VIEW_PKG_SAMPLE) {
+	    title = g_strdup_printf("gretl: %s sample", pkgname);
+	} else {
+	    title = g_strdup_printf("gretl: %s", pkgname);
+	}
 
 	vwin = view_buffer(prn, 78, 350, title, role, NULL);
 	strcpy(vwin->fname, path);
@@ -934,7 +948,7 @@ void set_funcs_dir_callback (windata_t *vwin, char *path)
 
 	    sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(vwin->listbox));
 	    gtk_tree_selection_selected_foreach(sel, fix_selected_row, vwin);
-	    infobox(_("Found %d function file(s)"), nfn - nfn0);
+	    infobox_printf(_("Found %d function file(s)"), nfn - nfn0);
 	    g_object_set_data(G_OBJECT(vwin->listbox), "nfn", GINT_TO_POINTER(nfn));
 	    if (resp > 0) {
 		g_object_set_data(G_OBJECT(vwin->listbox), "keepdir", GINT_TO_POINTER(1));
@@ -968,7 +982,13 @@ static void browser_functions_handler (windata_t *vwin, int task)
     if (dircol != 0) {
 	tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), vwin->active_var, 
 			     dircol, &dir);
-	build_path(path, dir, pkgname, ".gfn");
+	if (task == VIEW_PKG_RESOURCES) {
+	    build_path(path, dir, "examples", NULL);
+	} else if (task == VIEW_PKG_DOC) {
+	    build_path(path, dir, pkgname, ".pdf");
+	} else {
+	    build_path(path, dir, pkgname, ".gfn");
+	}
 	g_free(dir);
     } else {
 	strcpy(path, pkgname);
@@ -985,19 +1005,21 @@ static void browser_functions_handler (windata_t *vwin, int task)
 	err = gui_delete_fn_pkg(pkgname, path, vwin);
     } else if (task == VIEW_FN_PKG_INFO) {
 	display_function_package_data(pkgname, path, VIEW_PKG_INFO);
+    } else if (task == VIEW_FN_PKG_SAMPLE) {
+	display_function_package_data(pkgname, path, VIEW_PKG_SAMPLE);
     } else if (task == VIEW_FN_PKG_CODE) {
 	display_function_package_data(pkgname, path, VIEW_PKG_CODE);
     } else if (task == EDIT_FN_PKG) {
 	edit_function_package(path);
     } else if (task == MENU_ADD_FN_PKG) {
-	gui_add_package_to_menu(path, TRUE, vwin->main);
+	gui_add_package_to_menu(path, vwin->main, NULL);
+    } else if (task == VIEW_PKG_RESOURCES) {
+	file_selector_with_startdir(OPEN_ANY, path, vwin_toplevel(vwin));
+    } else if (task == VIEW_PKG_DOC) {
+	gretl_show_pdf(path);
     } else if (task == CALL_FN_PKG) {
 	/* note: this is the double-click default */
-	call_function_package(path, vwin, &err);
-	if (err == FN_NO_DATA) {
-	    display_function_package_data(pkgname, path, VIEW_PKG_INFO);
-	    err = 0;
-	}
+	err = open_function_package(pkgname, path, vwin);
     }
 
     g_free(pkgname);
@@ -1046,10 +1068,10 @@ static void show_addon_info (GtkWidget *w, gpointer data)
 	}
 
 	if (local != NULL) {
-	    infobox("%s:\n%s\n%s", pkgname, descrip, local);
+	    infobox_printf("%s:\n%s\n%s", pkgname, descrip, local);
 	    g_free(local);
 	} else {
-	    infobox("%s:\n%s", pkgname, descrip);
+	    infobox_printf("%s:\n%s", pkgname, descrip);
 	}
     }
 
@@ -1123,6 +1145,27 @@ static void show_function_code (GtkWidget *w, gpointer data)
     browser_functions_handler(vwin, VIEW_FN_PKG_CODE);
 }
 
+static void show_function_sample (GtkWidget *w, gpointer data)
+{
+    windata_t *vwin = (windata_t *) data;
+
+    browser_functions_handler(vwin, VIEW_FN_PKG_SAMPLE);
+}
+
+static void show_package_resources (GtkWidget *w, gpointer data)
+{
+    windata_t *vwin = (windata_t *) data;
+
+    browser_functions_handler(vwin, VIEW_PKG_RESOURCES);
+}
+
+static void show_package_doc (GtkWidget *w, gpointer data)
+{
+    windata_t *vwin = (windata_t *) data;
+
+    browser_functions_handler(vwin, VIEW_PKG_DOC);
+}
+
 static void browser_del_func (GtkWidget *w, gpointer data)
 {
     windata_t *vwin = (windata_t *) data;
@@ -1150,16 +1193,13 @@ windata_t *get_local_viewer (int remote_role)
     return vwin;
 }
 
-static void new_package_callback (GtkWidget *w, gpointer p)
+static void new_package_callback (GtkWidget *w, gpointer data)
 {
-    functions_selection_wrapper();
+    windata_t *vwin = (windata_t *) data;
+    
+    functions_selection_wrapper(vwin_toplevel(vwin));
 }
 
-static void close_files_viewer (GtkWidget *w, windata_t *vwin)
-{
-    gtk_widget_destroy(vwin->main);
-} 
-
 static void build_datafiles_popup (windata_t *vwin)
 {
     if (vwin->popup == NULL) {
@@ -1198,24 +1238,42 @@ static int get_menu_add_ok (windata_t *vwin)
     return ret;
 }
 
-static void check_add_button_state (GtkTreeSelection *sel, windata_t *vwin)
+static void check_extra_buttons_state (GtkTreeSelection *sel, windata_t *vwin)
 {
     GtkWidget *button;
+    gint flags;
 
     button = g_object_get_data(G_OBJECT(vwin->mbar), "add-button");
     if (button != NULL) {
 	gtk_widget_set_sensitive(button, get_menu_add_ok(vwin));
     }
+
+    /* Get flags from last, hidden int column, to determine
+       whether we can offer links to a package's "examples"
+       directory and/or its documentation in PDF format.
+    */
+    tree_view_get_int(GTK_TREE_VIEW(vwin->listbox),
+		      vwin->active_var, 5, &flags);
+
+    button = g_object_get_data(G_OBJECT(vwin->mbar), "res-button");
+    if (button != NULL) {
+	gtk_widget_set_sensitive(button, flags & PKG_ATTR_RES);
+    }
+
+    button = g_object_get_data(G_OBJECT(vwin->mbar), "doc-button");
+    if (button != NULL) {
+	gtk_widget_set_sensitive(button, flags & PKG_ATTR_DOC);
+    }    
 }
 
-static void connect_menu_add_signal (windata_t *vwin)
+static void connect_menu_adjust_signal (windata_t *vwin)
 {
     GtkTreeSelection *sel;
 
     sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(vwin->listbox));
 
     g_signal_connect(G_OBJECT(sel), "changed",
-		     G_CALLBACK(check_add_button_state), 
+		     G_CALLBACK(check_extra_buttons_state), 
 		     vwin);
 }
 
@@ -1225,31 +1283,56 @@ static void build_funcfiles_popup (windata_t *vwin)
 
     if (vwin->role == FUNC_FILES) {
 	/* local function files: full menu */
-	int menu_add_ok = 0;
-	GtkWidget *add;
+	int add_ok = 0;
+	int res_ok = 0;
+	int doc_ok = 0;
+	GtkWidget *b;
 
-	add = g_object_get_data(G_OBJECT(vwin->mbar), "add-button");
-	if (add != NULL && gtk_widget_is_sensitive(add)) {
-	    menu_add_ok = 1;
+	b = g_object_get_data(G_OBJECT(vwin->mbar), "add-button");
+	if (b != NULL && gtk_widget_is_sensitive(b)) {
+	    add_ok = 1;
 	}
 
+	b = g_object_get_data(G_OBJECT(vwin->mbar), "res-button");
+	if (b != NULL && gtk_widget_is_sensitive(b)) {
+	    res_ok = 1;
+	}
+
+	b = g_object_get_data(G_OBJECT(vwin->mbar), "doc-button");
+	if (b != NULL && gtk_widget_is_sensitive(b)) {
+	    doc_ok = 1;
+	}	
+
 	add_popup_item(_("Edit"), vwin->popup, 
 		       G_CALLBACK(browser_edit_func), 
 		       vwin);
 	add_popup_item(_("Info"), vwin->popup, 
 		       G_CALLBACK(show_function_info), 
 		       vwin);
+	add_popup_item(_("Sample script"), vwin->popup, 
+		       G_CALLBACK(show_function_sample), 
+		       vwin);
 	add_popup_item(_("View code"), vwin->popup, 
 		       G_CALLBACK(show_function_code), 
 		       vwin);
 	add_popup_item(_("Execute"), vwin->popup, 
 		       G_CALLBACK(browser_call_func), 
 		       vwin);
-	if (menu_add_ok) {
+	if (res_ok) {
+	    add_popup_item(_("Resources..."), vwin->popup, 
+			   G_CALLBACK(show_package_resources), 
+			   vwin);
+	}
+	if (add_ok) {
 	    add_popup_item(_("Add to menu"), vwin->popup, 
 			   G_CALLBACK(add_func_to_menu), 
 			   vwin);
 	}
+	if (doc_ok) {
+	    add_popup_item(_("Help"), vwin->popup, 
+			   G_CALLBACK(show_package_doc), 
+			   vwin);
+	}	
 	add_popup_item(_("Delete"), vwin->popup, 
 		       G_CALLBACK(browser_del_func), 
 		       vwin);
@@ -1388,33 +1471,36 @@ enum {
     BTN_FIND,
     BTN_OPEN,
     BTN_DIR,
-    BTN_CLOSE
+    BTN_RES,
+    BTN_DOC
 };
 
 static GretlToolItem files_items[] = {
-    { N_("Open"),           GTK_STOCK_OK, NULL, BTN_OPEN },
+    { N_("Open"),           GTK_STOCK_OPEN, NULL, BTN_OPEN },
     { N_("Select directory"), GTK_STOCK_DIRECTORY, NULL, BTN_DIR },
     { N_("Edit"),           GTK_STOCK_EDIT,       G_CALLBACK(browser_edit_func), BTN_EDIT },
     { N_("Info"),           GTK_STOCK_INFO,       NULL,                          BTN_INFO },
+    { N_("Sample script"),  GTK_STOCK_JUSTIFY_LEFT, G_CALLBACK(show_function_sample), BTN_CODE },
     { N_("View code"),      GTK_STOCK_PROPERTIES, G_CALLBACK(show_function_code), BTN_CODE },
     { N_("List series"),    GTK_STOCK_INDEX,      NULL,                           BTN_INDX },
     { N_("Install"),        GTK_STOCK_SAVE,       NULL,                           BTN_INST },
     { N_("Execute"),        GTK_STOCK_EXECUTE,    G_CALLBACK(browser_call_func), BTN_EXEC },
+    { N_("Resources..."),   GTK_STOCK_OPEN,       G_CALLBACK(show_package_resources), BTN_RES },
     { N_("Add to menu"),    GTK_STOCK_ADD,        G_CALLBACK(add_func_to_menu),  BTN_ADD },
+    { N_("Help"),           GRETL_STOCK_PDF,      G_CALLBACK(show_package_doc),  BTN_DOC },
     { N_("Delete"),         GTK_STOCK_DELETE,     G_CALLBACK(browser_del_func),  BTN_DEL },
     { N_("Look on server"), GTK_STOCK_NETWORK,    NULL,                          BTN_WWW },
     { N_("Local machine"),  GTK_STOCK_HOME,       NULL,                          BTN_HOME },
-    { N_("New"),            GTK_STOCK_NEW,        G_CALLBACK(new_package_callback), BTN_NEW },
-    { N_("Windows"),        GRETL_STOCK_WINLIST,  GNULL, 0 },
-    { N_("Close"),          GTK_STOCK_CLOSE,      G_CALLBACK(close_files_viewer), BTN_CLOSE }
+    { N_("New"),            GTK_STOCK_NEW,        G_CALLBACK(new_package_callback), BTN_NEW }
 };
 
 static int n_files_items = G_N_ELEMENTS(files_items);
 
-#define common_item(f) (f == 0 || f == BTN_FIND || f == BTN_CLOSE)
+#define common_item(f) (f == 0 || f == BTN_FIND)
 
 #define local_funcs_item(f) (f == BTN_EDIT || f == BTN_NEW || \
-			     f == BTN_DEL || f == BTN_CODE)
+			     f == BTN_DEL || f == BTN_CODE || \
+			     f == BTN_RES || f == BTN_DOC)
 
 static int files_item_get_callback (GretlToolItem *item, int role)
 {
@@ -1444,7 +1530,7 @@ static int files_item_get_callback (GretlToolItem *item, int role)
 	    item->func = G_CALLBACK(browser_open_data);
 	} else if (role == PS_FILES) {
 	    item->func = G_CALLBACK(browser_open_ps);
-	} 
+	}
     } else if (item->flag == BTN_INFO) {
 	if (role == TEXTBOOK_DATA) {
 	    item->func = G_CALLBACK(show_datafile_info);
@@ -1501,18 +1587,23 @@ static void make_files_toolbar (windata_t *vwin)
 
     for (i=0; i<n_files_items; i++) {
 	item = &files_items[i];
-	if (winlist_item(item)) {
-	    vwin_toolbar_insert_winlist(vwin);
-	} else if (files_item_get_callback(item, vwin->role)) {
+	if (files_item_get_callback(item, vwin->role)) {
 	    button = gretl_toolbar_insert(vwin->mbar, item, item->func, vwin, -1);
 	    if (item->flag == BTN_ADD) {
 		g_object_set_data(G_OBJECT(vwin->mbar), "add-button", button);
 		gtk_widget_set_sensitive(button, FALSE);
-	    }
+	    } else if (item->flag == BTN_RES) {
+		g_object_set_data(G_OBJECT(vwin->mbar), "res-button", button);
+		gtk_widget_set_sensitive(button, FALSE);
+	    } else if (item->flag == BTN_DOC) {
+		g_object_set_data(G_OBJECT(vwin->mbar), "doc-button", button);
+		gtk_widget_set_sensitive(button, FALSE);
+	    }		
 	}
     }
 
     gtk_box_pack_start(GTK_BOX(hbox), vwin->mbar, FALSE, FALSE, 0);
+    vwin_add_winlist(vwin);
     vwin_add_finder(vwin);
     gtk_widget_show_all(hbox);
 }
@@ -1719,7 +1810,7 @@ void display_files (int role, gpointer data)
 			 G_CALLBACK(funcfiles_popup_handler), 
 			 vwin);
 	if (role == FUNC_FILES) {
-	    connect_menu_add_signal(vwin);
+	    connect_menu_adjust_signal(vwin);
 	}
     } else if (role == NATIVE_DB || role == REMOTE_DB) {
 	build_db_popup(vwin);
@@ -1829,10 +1920,11 @@ void show_native_dbs (void)
 }
 
 static int get_func_info (const char *path, char **pdesc, 
-			  char **pver)
+			  char **pver, int *pdfdoc)
 {
-    int err = get_function_file_header(path, pdesc, pver);
+    int err;
 
+    err = get_function_file_header(path, pdesc, pver, pdfdoc);
     if (err) {
 	gui_errmsg(err);
     }
@@ -1885,35 +1977,73 @@ static int is_functions_dir (const char *path)
     return n > 0 && !strcmp(path + n, "functions");
 }
 
+/* For a function package that lives in its own directory,
+   see if it has an "examples" subdir.
+*/
+
+static int have_examples (const char *dirname)
+{
+    struct stat sbuf;
+    gchar *test;
+    int ret = 0;
+
+    test = g_strdup_printf("%s%cexamples", dirname, SLASH);
+
+    if (stat(test, &sbuf) == 0 &&
+	(sbuf.st_mode & S_IFDIR)) {
+	ret = 1;
+    }
+
+    g_free(test);
+
+    return ret;
+}
+
 static int ok_gfn_path (const char *fullname, 
 			const char *shortname,
 			const char *dirname,
 			GtkListStore *store, 
 			GtkTreeIter *iter,
 			int imax, 
-			int *maxlen)
+			int *maxlen,
+			int subdir)
 {
     char *descrip = NULL, *version = NULL;
+    int pdfdoc = 0;
     int err, ok = 0;
 
-    err = get_func_info(fullname, &descrip, &version);
+    err = get_func_info(fullname, &descrip, &version, &pdfdoc);
 
     if (!err && !fn_file_is_duplicate(shortname, version, store, imax)) {
 	if (iter != NULL) {
 	    gchar *fname = g_strdup(shortname);
 	    int n = strlen(fname) - 4;
+	    gint flags = 0;
 
-	    fname[n] = '\0'; /* chop off ".gfn" */
+	    /* chop off ".gfn" for display */
+	    fname[n] = '\0';
 	    if (n > *maxlen) {
 		*maxlen = n;
 	    }
+
+	    if (subdir) {
+		if (have_examples(dirname)) {
+		    flags |= PKG_ATTR_RES;
+		}
+		if (pdfdoc) {
+		    flags |= PKG_ATTR_DOC;
+		}
+	    }
+	    
 	    gtk_list_store_append(store, iter);
 	    gtk_list_store_set(store, iter, 
 			       0, fname, 
 			       1, version,
 			       2, descrip, 
 			       3, function_package_is_loaded(fullname), 
-			       4, dirname, -1);
+			       4, dirname,
+			       5, flags,
+			       -1);
 	    g_free(fname);
 	}
 	ok = 1;
@@ -1966,7 +2096,7 @@ read_fn_files_in_dir (DIR *dir, const char *path,
 		    realbase = g_strdup_printf("%s.gfn", basename);
 		    realpath = g_strdup_printf("%s%c%s", path, SLASH, basename);
 		    *nfn += ok_gfn_path(fullname, realbase, realpath,
-					store, iter, imax, maxlen);
+					store, iter, imax, maxlen, 1);
 		    g_free(realbase);
 		    g_free(realpath);
 		} else {
@@ -1992,7 +2122,7 @@ read_fn_files_in_dir (DIR *dir, const char *path,
 	if (has_suffix(basename, ".gfn")) {
 	    build_path(fullname, path, basename, NULL);
 	    *nfn += ok_gfn_path(fullname, basename, path,
-				store, iter, imax, maxlen);
+				store, iter, imax, maxlen, 0);
 	}
     }
 }
@@ -2076,7 +2206,7 @@ gint populate_func_list (windata_t *vwin, struct fpkg_response *fresp)
 	    resp = yes_no_dialog(_("gretl: function packages"),
 				 _("No gretl function packages were found on this computer.\n"
 				   "Do you want to take a look on the gretl server?"),
-				 0);
+				 vwin_toplevel(vwin));
 	    if (resp == GRETL_YES) {
 		fresp->try_server = 1;
 	    }
@@ -2218,7 +2348,8 @@ static GtkWidget *files_vbox (windata_t *vwin)
 	G_TYPE_STRING,
 	G_TYPE_STRING,
 	G_TYPE_BOOLEAN,
-	G_TYPE_STRING   /* hidden string: directory */
+	G_TYPE_STRING,   /* hidden string: directory */
+	G_TYPE_INT       /* hidden flags: has examples dir? doc? */
     };
     GType remote_func_types[] = {
 	G_TYPE_STRING,
@@ -2238,7 +2369,7 @@ static GtkWidget *files_vbox (windata_t *vwin)
     const char **titles = data_titles;
     GType *types = types_2;
     int full_width = 500, file_height = 260;
-    int hidden_col = 0;
+    int hidden_cols = 0;
     int use_tree = 0;
     GtkWidget *vbox;
     int cols = 2;
@@ -2247,7 +2378,7 @@ static GtkWidget *files_vbox (windata_t *vwin)
     case NATIVE_DB:
 	titles = db_titles;
 	cols = 3;
-	hidden_col = TRUE;
+	hidden_cols = 1;
 	break;
     case REMOTE_DB:
 	titles = remote_db_titles;
@@ -2270,15 +2401,15 @@ static GtkWidget *files_vbox (windata_t *vwin)
 	titles = func_titles;
 	types = func_types;
 	cols = G_N_ELEMENTS(func_types);
-	hidden_col = TRUE;
-	full_width = 560;
+	hidden_cols = 2;
+	full_width = 750;
 	file_height = 320;
 	break;
     case REMOTE_FUNC_FILES:
 	titles = remote_func_titles;
 	types = remote_func_types;
 	cols = G_N_ELEMENTS(remote_func_types);
-	hidden_col = TRUE;
+	hidden_cols = 1;
 	full_width = 580;
 	file_height = 340;
 	break;
@@ -2286,7 +2417,7 @@ static GtkWidget *files_vbox (windata_t *vwin)
 	titles = addons_titles;
 	types = addons_types;
 	cols = G_N_ELEMENTS(addons_types);
-	hidden_col = TRUE;
+	hidden_cols = 1;
 	full_width = 400;
 	break;	
     default:
@@ -2303,7 +2434,7 @@ static GtkWidget *files_vbox (windata_t *vwin)
     vbox = gtk_vbox_new(FALSE, 0);
     gtk_widget_set_size_request(vbox, full_width, file_height);
     /* note: the following packs and attaches vwin->listbox */
-    vwin_add_list_box(vwin, GTK_BOX(vbox), cols, hidden_col, 
+    vwin_add_list_box(vwin, GTK_BOX(vbox), cols, hidden_cols, 
 		      types, titles, use_tree);
     gtk_widget_show(vbox);
 
