@@ -149,17 +149,18 @@ static void dwinfo_init (DATASET *dwinfo)
 
 static void prep_spreadsheet (GtkWidget *widget, dialog_t *dlg) 
 {
-    const gchar *buf;
+    GtkWidget *parent = edit_dialog_get_window(dlg);
+    const gchar *buf = edit_dialog_get_text(dlg);
     int t;
 
-    buf = edit_dialog_get_text(dlg);
-
-    if (buf == NULL || gui_validate_varname(buf, GRETL_TYPE_SERIES)) {
+    if (buf == NULL || gui_validate_varname(buf,
+					    GRETL_TYPE_SERIES,
+					    parent)) {
 	edit_dialog_reset(dlg);
 	return;
     }
 
-    dataset->varname[1][0] = 0;
+    dataset->varname[1][0] = '\0';
     strncat(dataset->varname[1], buf, VNAMELEN - 1);
 
     edit_dialog_close(dlg);
@@ -261,8 +262,7 @@ static void maybe_unrestrict_dataset (void)
 static int dwiz_make_changes (DATASET *dwinfo, dw_opts *opts,
 			      GtkWidget *dlg)
 {
-    char setline[64];
-    char record[128];
+    gchar *setobs_cmd = NULL;
     gretlopt opt = OPT_NONE;
     int create = (opts->flags & DW_CREATE);
     int delmiss = (opts->flags & DW_DROPMISS);
@@ -273,8 +273,6 @@ static int dwiz_make_changes (DATASET *dwinfo, dw_opts *opts,
     fprintf(stderr, "dwiz_make_changes: create = %d\n", create);
 #endif
 
-    *record = '\0';
-
     /* preliminaries */
     if (time_series(dwinfo)) {
 	ntodate(dwinfo->stobs, dwinfo->t1, dwinfo);
@@ -295,8 +293,9 @@ static int dwiz_make_changes (DATASET *dwinfo, dw_opts *opts,
 	    err = set_panel_structure_from_vars(uv, tv, dataset);
 	}
 	if (!err) {
-	    sprintf(record, "setobs %s %s --panel-vars",
-		    dataset->varname[uv], dataset->varname[tv]);
+	    setobs_cmd = g_strdup_printf("setobs %s %s --panel-vars",
+					 dataset->varname[uv],
+					 dataset->varname[tv]);
 	}
 	goto finalize;
     }
@@ -337,8 +336,6 @@ static int dwiz_make_changes (DATASET *dwinfo, dw_opts *opts,
 	strcpy(dwinfo->stobs, "1");
     }
 
-    sprintf(setline, "setobs %d %s", dwinfo->pd, dwinfo->stobs);
-
     if (dwinfo->structure == TIME_SERIES) {
 	opt = OPT_T;
     } else if (dwinfo->structure == STACKED_TIME_SERIES) {
@@ -351,15 +348,17 @@ static int dwiz_make_changes (DATASET *dwinfo, dw_opts *opts,
 	opt = OPT_N;
     }
 
-    err = set_obs(setline, dataset, opt);
+    err = simple_set_obs(dataset, dwinfo->pd, dwinfo->stobs, opt);
 
 #if DWDEBUG
-    fprintf(stderr, "setline = '%s', opt = %d; set_obs returned %d\n", 
-	    setline, opt, err);
+    fprintf(stderr, "pd=%d, stobs='%s', opt=%d; set_obs returned %d\n", 
+	    dwinfo->pd, dwinfo->stobs, opt, err);
 #endif
 
-    if (!err) {
-	sprintf(record, "%s%s", setline, print_flags(opt, SETOBS));
+    if (!err && setobs_cmd == NULL) {
+	setobs_cmd = g_strdup_printf("setobs %d %s%s",
+				     dwinfo->pd, dwinfo->stobs, 
+				     print_flags(opt, SETOBS));
     }
 
  finalize:
@@ -386,11 +385,13 @@ static int dwiz_make_changes (DATASET *dwinfo, dw_opts *opts,
 	mark_dataset_as_modified();
     }
 
-    if (!err && *record != '\0') {
-	lib_command_strcpy(record);
+    if (!err && setobs_cmd != NULL) {
+	lib_command_strcpy(setobs_cmd);
 	record_command_verbatim();
     }
 
+    g_free(setobs_cmd);
+
 #if DWDEBUG
     fprintf(stderr, "dwiz_make_changes: returning %d\n", err);
 #endif
@@ -649,7 +650,7 @@ static void compute_default_ts_info (DATASET *dwinfo)
 
     if (dwinfo->structure == SPECIAL_TIME_SERIES) {
 	/* non-standard time series */
-	dwinfo->n = 500;
+	dwinfo->n = 9999 * dwinfo->pd;
 	dwinfo->t1 = 0;
 	if (dwinfo->pd > 1) {
 	    int p = dwinfo->pd;
@@ -703,7 +704,7 @@ static void compute_default_ts_info (DATASET *dwinfo)
 	       dwinfo->pd == 6 ||
 	       dwinfo->pd == 7) {
 	strcpy(dwinfo->stobs, "1900-01-01");
-	dwinfo->n = 40000;
+	dwinfo->n = 50000;
 	/* set default start to 1960-01-01 (a Friday) */
 	if (dwinfo->pd == 5) {
 	    dwinfo->t1 = 15654;
@@ -1115,7 +1116,6 @@ static void dw_set_t1 (GtkWidget *w, DATASET *dwinfo)
 static void add_startobs_spinner (GtkWidget *vbox,
 				  DATASET *dwinfo,
 				  int direction)
-
 {
     GtkWidget *hbox, *label, *spin;
     GtkAdjustment *adj;
