@@ -67,6 +67,7 @@ static struct extmap action_map[] = {
     { SAVE_OX_CMDS,      ".ox" },
     { SAVE_OCTAVE_CMDS,  ".m" }, 
     { SAVE_PYTHON_CMDS,  ".py" },
+    { SAVE_STATA_CMDS,   ".do" },
     { SAVE_FUNCTIONS,    ".gfn" },
     { SAVE_MARKERS,      ".txt" },
     { SAVE_LABELS,       ".txt" },
@@ -98,7 +99,7 @@ static const char *get_extension_for_action (int action, gpointer data)
 	return ".gdtb";
     }
 
-    if (action == EXPORT_GDT) {
+    if (action == EXPORT_GDT || action == SAVE_BOOT_DATA) {
 	return ".gdt";
     } else if (action == EXPORT_GDTB) {
 	return ".gdtb";
@@ -220,8 +221,9 @@ static void script_window_update (windata_t *vwin, const char *fname)
 
     g_free(trfname);
 
-    if (vwin->role == VIEW_LOG || vwin->role == VIEW_SCRIPT ||
-	vwin->role == VIEW_PKG_CODE) {
+    if (vwin->role == VIEW_SCRIPT ||
+	vwin->role == VIEW_PKG_CODE ||
+	vwin->role == VIEW_PKG_SAMPLE) {
 	/* change role of window for editing */
 	vwin->role = EDIT_SCRIPT;
     } else if (vwin->role == EDIT_GP) {
@@ -237,6 +239,49 @@ static void script_window_update (windata_t *vwin, const char *fname)
     }
 }
 
+static void log_view_save_as (const char *fname, windata_t *vwin)
+{
+    char basename[128];
+    gchar *msg;
+    int resp;
+
+    gretl_basename(basename, fname, 0);
+    msg = g_strdup_printf("Open %s in the script editor?", basename);
+
+    resp = yes_no_dialog(NULL, msg, vwin_toplevel(vwin));
+    g_free(msg);
+
+    if (resp == GRETL_YES) {
+	strcpy(tryfile, fname);
+	if (view_script(tryfile, 1, EDIT_SCRIPT) != NULL) {
+	    strcpy(scriptfile, tryfile);
+	    mkfilelist(FILE_LIST_SCRIPT, scriptfile);
+	    gretl_set_current_dir(scriptfile);
+	}
+    }
+}
+
+gchar *pre_trim_buffer (gchar *s)
+{
+    gchar *ret = s;
+    int n = 0;
+
+    if (*s == '#') {
+	while (*s) {
+	    if (*s == '#') {
+		n++;
+	    }
+	    if (n == 3 && *s == '\n') {
+		ret = s + 1;
+		break;
+	    }
+	    s++;
+	}
+    }
+
+    return ret;
+}
+
 static void 
 save_editable_content (int action, const char *fname, windata_t *vwin)
 {
@@ -255,43 +300,31 @@ save_editable_content (int action, const char *fname, windata_t *vwin)
 	return;
     }
 
-#if 1 /* 2014-09-11: just keep any UTF-8 */
-    system_print_buf(buf, fp);
-#else /* what we used to do */
-    if (action == SAVE_SCRIPT) {
-	/* don't mess with encoding */
-	system_print_buf(buf, fp);
+    if (vwin->role == VIEW_LOG) {
+	gchar *lbuf = pre_trim_buffer(buf);
+
+	system_print_buf(lbuf, fp);
     } else {
-	gchar *trbuf;
-
-	/* FIXME which cases come here and do
-	   they want this treatment? */
-
-	if (!g_get_charset(&cset)) {
-	    /* UTF-8 minuses not wanted for locale */
-	    strip_unicode_minus(buf);
-	}
-
-	trbuf = my_locale_from_utf8(buf);
-	if (trbuf != NULL) {
-	    system_print_buf(trbuf, fp);
-	    g_free(trbuf);
-	}
+	system_print_buf(buf, fp);
     }
-#endif
 
     g_free(buf);
     fclose(fp);
-    
+
     if (action == SAVE_SCRIPT) {
-	strcpy(scriptfile, fname);
-	mkfilelist(FILE_LIST_SCRIPT, scriptfile);
-	script_window_update(vwin, fname);
+	if (vwin->role == VIEW_LOG) {
+	    log_view_save_as(fname, vwin);
+	} else {
+	    strcpy(scriptfile, fname);
+	    mkfilelist(FILE_LIST_SCRIPT, scriptfile);
+	    script_window_update(vwin, fname);
+	}
     } else if (action == SAVE_GP_CMDS || 
 	       action == SAVE_R_CMDS ||
 	       action == SAVE_OX_CMDS ||
 	       action == SAVE_OCTAVE_CMDS ||
-	       action == SAVE_PYTHON_CMDS) {
+	       action == SAVE_PYTHON_CMDS ||
+	       action == SAVE_STATA_CMDS) {
 	script_window_update(vwin, fname);
     } 
 }
@@ -332,12 +365,14 @@ static void filesel_open_script (const char *fname, windata_t *vwin)
 	view_script(fname, 1, EDIT_R);
     } else if (has_suffix(fname, ".plt")) {
 	view_script(fname, 1, EDIT_GP);
-    } else if (ox_support && has_suffix(fname, ".ox")) {
+    } else if (has_suffix(fname, ".ox")) {
 	view_script(fname, 1, EDIT_OX);
     } else if (has_suffix(fname, ".m")) {
 	view_script(fname, 1, EDIT_OCTAVE);
     } else if (has_suffix(fname, ".py")) {
 	view_script(fname, 1, EDIT_PYTHON);
+    } else if (has_suffix(fname, ".do")) {
+	view_script(fname, 1, EDIT_STATA);
     } else {
 	strcpy(tryfile, fname);
 	if (view_script(tryfile, 1, EDIT_SCRIPT) != NULL) {
@@ -475,7 +510,7 @@ static int overwrite_stop (const char *fname)
 	if (yes_no_dialog(NULL, 
 			  _("There is already a file of this name.\n"
 			    "OK to overwrite it?"), 
-			  0) == GRETL_NO) {
+			  NULL) == GRETL_NO) {
 	    return 1;
 	}
     }
@@ -483,6 +518,17 @@ static int overwrite_stop (const char *fname)
     return 0;
 }
 
+static void os_open_other (const char *fname)
+{
+#if defined(G_OS_WIN32)
+    win32_open_file(fname);
+#elif defined(OS_OSX)
+    osx_open_file(fname);
+#else
+    gretl_fork("xdg-open", fname);
+#endif
+}
+
 static void
 file_selector_process_result (const char *in_fname, int action, 
 			      FselDataSrc src, gpointer data)
@@ -491,7 +537,7 @@ file_selector_process_result (const char *in_fname, int action,
     int quit = 0;
     int err = 0;
 
-    *fname = 0;
+    *fname = '\0';
     strncat(fname, in_fname, FILENAME_MAX - 1);
 
     if (action < END_OPEN) {
@@ -505,7 +551,19 @@ file_selector_process_result (const char *in_fname, int action,
 	} else {
 	    fclose(fp);
 	}
-    } 
+    }
+
+    if (action == OPEN_ANY) {
+	if (has_suffix(fname, ".inp")) {
+	    action = OPEN_SCRIPT;
+	} else if (has_suffix(fname, ".gdt") ||
+		   has_suffix(fname, ".gdtb")) {
+	    action = OPEN_DATA;
+	} else {
+	    os_open_other(fname);
+	    return;
+	}
+    }
 
     if (action == OPEN_DATA) {
 	strcpy(tryfile, fname);
@@ -823,9 +881,8 @@ static void filesel_set_filters (GtkWidget *filesel, int action,
 	filesel_add_filter(filesel, N_("gnuplot files (*.plt)"), "*.plt");
 	filesel_add_filter(filesel, N_("GNU Octave files (*.m)"), "*.m");
 	filesel_add_filter(filesel, N_("Python files (*.py)"), "*.py");
-	if (ox_support) {
-	    filesel_add_filter(filesel, N_("Ox files (*.ox)"), "*.ox");
-	}
+	filesel_add_filter(filesel, N_("Ox files (*.ox)"), "*.ox");
+	filesel_add_filter(filesel, N_("Stata files (*.do)"), "*.do");
     } else if (action == OPEN_LABELS || action == OPEN_BARS) {
 	filesel_add_filter(filesel, N_("ASCII files (*.txt)"), "*.txt");
 	filesel_add_filter(filesel, N_("all files (*.*)"), "*");
@@ -833,6 +890,11 @@ static void filesel_set_filters (GtkWidget *filesel, int action,
 	       action == SAVE_BOOT_DATA) {
 	filesel_add_filter(filesel, N_("Gretl datafiles (*.gdt)"), "*.gdt");
 	filesel_add_filter(filesel, N_("Gretl binary datafiles (*.gdtb)"), "*.gdtb");
+    } else if (action == OPEN_ANY) {
+	filesel_add_filter(filesel, N_("gretl script files (*.inp)"), "*.inp");
+	filesel_add_filter(filesel, N_("Gretl datafiles (*.gdt)"), "*.gdt");
+	filesel_add_filter(filesel, N_("Gretl binary datafiles (*.gdtb)"), "*.gdtb");
+	filesel_add_filter(filesel, N_("all files (*.*)"), "*");
     } else {
 	GtkFileFilter *filter = get_file_filter(action, data);
 
@@ -844,8 +906,10 @@ static void remember_folder (GtkFileChooser *chooser, char *savedir)
 {
     gchar *folder = gtk_file_chooser_get_current_folder(chooser);
 
-    strcpy(savedir, folder);
-    g_free(folder);
+    if (folder != NULL) {
+	strcpy(savedir, folder);
+	g_free(folder);
+    }
 }
 
 static void record_compress_level (GtkWidget *b, gpointer p)
@@ -899,7 +963,8 @@ static void check_native_save_filter (GtkWidget *filesel)
 }
 
 static void gtk_file_selector (int action, FselDataSrc src, 
-			       gpointer data, GtkWidget *parent) 
+			       gpointer data, GtkWidget *parent,
+			       const char *dirname) 
 {
     static char savedir[MAXLEN];
     char startdir[MAXLEN];
@@ -932,7 +997,9 @@ static void gtk_file_selector (int action, FselDataSrc src,
 	}
     }
 
-    if (remember && *savedir != '\0') {
+    if (dirname != NULL) {
+	strcpy(startdir, dirname);
+    } else if (remember && *savedir != '\0') {
 	strcpy(startdir, savedir);
     } else {
 	get_default_dir(startdir, action);
@@ -948,13 +1015,18 @@ static void gtk_file_selector (int action, FselDataSrc src,
 	parent = mdata->main;
     }
 
-    filesel = gtk_file_chooser_dialog_new(NULL, GTK_WINDOW(parent), fsel_action,
-					  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					  okstr, GTK_RESPONSE_ACCEPT,
+    filesel = gtk_file_chooser_dialog_new(NULL,
+					  GTK_WINDOW(parent),
+					  fsel_action,
+					  GTK_STOCK_CANCEL,
+					  GTK_RESPONSE_CANCEL,
+					  okstr,
+					  GTK_RESPONSE_ACCEPT,
 					  NULL);
 
     if (action == SAVE_DATA || 
-	action == SAVE_DATA_AS || 
+	action == SAVE_DATA_AS ||
+	action == SAVE_BOOT_DATA ||
 	action == EXPORT_GDT ||
 	action == EXPORT_GDTB) {
 	add_compression_level_option(filesel);
@@ -1006,7 +1078,9 @@ static void gtk_file_selector (int action, FselDataSrc src,
     }
 }
 
-void file_selector (int action, FselDataSrc src, gpointer data)
+void file_selector (int action,
+		    FselDataSrc src,
+		    gpointer data)
 {
     GtkWidget *w = NULL;
 
@@ -1016,11 +1090,20 @@ void file_selector (int action, FselDataSrc src, gpointer data)
 	w = vwin_toplevel(vwin);
     }
 
-    gtk_file_selector(action, src, data, w);
+    gtk_file_selector(action, src, data, w, NULL);
 }
 
-void file_selector_with_parent (int action, FselDataSrc src, 
-				gpointer data, GtkWidget *w)
+void file_selector_with_parent (int action,
+				FselDataSrc src, 
+				gpointer data,
+				GtkWidget *parent)
 {
-    gtk_file_selector(action, src, data, w);
+    gtk_file_selector(action, src, data, parent, NULL);
+}
+
+void file_selector_with_startdir (int action,
+				  const char *startdir,
+				  GtkWidget *parent)
+{
+    gtk_file_selector(action, 0, NULL, parent, startdir);
 }
