@@ -290,7 +290,7 @@ static int varname_error (filter_info *finfo, int i)
     if (*s == 0) {
 	errbox(_("Variable name is missing"));
 	return 1;
-    } else if (gui_validate_varname(s, GRETL_TYPE_SERIES)) {
+    } else if (gui_validate_varname(s, GRETL_TYPE_SERIES, finfo->dlg)) {
 	return 1;
     } else if (i == 2 && (finfo->save_opt & FILTER_SAVE_TREND)) {
 	if (!strcmp(s, finfo->save_t)) {
@@ -1203,10 +1203,10 @@ static void record_filter_command (filter_info *finfo)
 
     if (trend) {
 	s = fcmd + strlen(fcmd);
-	sprintf(s, "setinfo %s -d \"%s\"\n", finfo->save_t, finfo->label_t);
+	sprintf(s, "setinfo %s --description=\"%s\"\n", finfo->save_t, finfo->label_t);
     } else if (cycle) {
 	s = fcmd + strlen(fcmd);
-	sprintf(s, "setinfo %s -d \"%s\"\n", finfo->save_c, finfo->label_c);
+	sprintf(s, "setinfo %s --description=\"%s\"\n", finfo->save_c, finfo->label_c);
     }	
 
     if (trend && cycle) {
@@ -1214,10 +1214,10 @@ static void record_filter_command (filter_info *finfo)
 	sprintf(s, "series %s = %s - %s\n", finfo->save_c, finfo->vname, 
 		finfo->save_t);
 	s = fcmd + strlen(fcmd);
-	sprintf(s, "setinfo %s -d \"%s\"\n", finfo->save_c, finfo->label_c);
+	sprintf(s, "setinfo %s --description=\"%s\"\n", finfo->save_c, finfo->label_c);
     }
 
-    add_command_to_stack(fcmd);
+    add_command_to_stack(fcmd, 0);
 }
 
 static int calculate_filter (filter_info *finfo)
