@@ -25,6 +25,7 @@
 #include "textbuf.h"
 #include "fileselect.h"
 #include "gretl_www.h"
+#include "gretl_zip.h"
 #include "winstack.h"
 #include "selector.h"
 #include "textutil.h"
@@ -42,7 +43,8 @@
 #include "gretl_typemap.h"
 
 #define PKG_DEBUG 0
-#define NENTRIES 4
+#define NENTRIES 5
+#define N_FILE_ENTRIES 4
 
 enum {
     NO_WINDOW,
@@ -50,6 +52,13 @@ enum {
     MODEL_WINDOW
 };
 
+enum {
+    APPEND_SAMPLE  = 1 << 0, /* write functions as .inp: append sample? */
+    WRITE_SAMPFILE = 1 << 1, /* write .spec file: also write sample script? */
+    WRITE_HELPFILE = 1 << 2, /* write .spec file: also write help file? */
+    WRITE_GUI_HELP = 1 << 3  /* write .spec file: also write gui help? */
+} PkgSaveFlags;
+
 typedef struct function_info_ function_info;
 typedef struct login_info_ login_info;
 
@@ -57,7 +66,10 @@ typedef struct login_info_ login_info;
 
 struct function_info_ {
     GtkWidget *dlg;        /* editing dialog box */
-    GtkWidget *entries[NENTRIES]; /* entry boxes for author, etc. */
+    /* entry boxes for author, etc. */
+    GtkWidget *entries[NENTRIES];
+    /* entry boxes for data file names */
+    GtkWidget *file_entries[N_FILE_ENTRIES];
     GtkWidget *text;       /* text box for help */
     GtkWidget *codesel;    /* code-editing selector */
     GtkWidget *save;       /* Save button in dialog */
@@ -70,22 +82,30 @@ struct function_info_ {
     GtkWidget *currtree;   /* currently displayed menu treeview */
     GtkWidget *alttree;    /* currently undisplayed menu treeview */
     GtkWidget *treewin;    /* scrolled window to hold menu trees */
+    GtkWidget *specdlg;    /* pkg spec save dialog */
     windata_t *samplewin;  /* window for editing sample script */
     GList *codewins;       /* list of windows editing function code */
     fnpkg *pkg;            /* pointer to package being edited */
     gchar *fname;          /* package filename */
     gchar *author;         /* package author */
+    gchar *email;          /* author's email address */
     gchar *version;        /* package version number */
     gchar *date;           /* package last-revised date */
     gchar *pkgdesc;        /* package description */
     gchar *sample;         /* sample script for package */
     gchar *help;           /* package help text */
+    gchar *gui_help;       /* GUI-specific help text */
+    gchar *sample_fname;   /* filename: sample script */
+    gchar *help_fname;     /* filename: help text */
+    gchar *gui_help_fname; /* filename: GUI-specific help */
     char **pubnames;       /* names of public functions */
     char **privnames;      /* names of private functions */
     char **specials;       /* names of special functions */
+    char **datafiles;      /* names of included data files */
     int n_pub;             /* number of public functions */
     int n_priv;            /* number of private functions */
     int n_special;         /* (max) number of special functions */
+    int n_files;           /* number of include data files */
     gboolean uses_subdir;  /* the package has its own subdir (0/1) */
     gchar *menupath;       /* path for menu attachment, if any */
     gchar *menulabel;      /* label for menu attachment, if any */
@@ -95,6 +115,7 @@ struct function_info_ {
     int minver;            /* minimum gretl version, package */
     gboolean upload;       /* upload to server on save? */
     gboolean modified;     /* anything changed in package? */
+    int save_flags;        /* see PkgSaveFlags */
 };
 
 /* info relating to login to server for upload */
@@ -136,6 +157,7 @@ function_info *finfo_new (void)
     finfo->pkg = NULL;
     finfo->fname = NULL;
     finfo->author = NULL;
+    finfo->email = NULL;
     finfo->version = NULL;
     finfo->date = NULL;
     finfo->pkgdesc = NULL;
@@ -150,6 +172,8 @@ function_info *finfo_new (void)
 
     finfo->upload = FALSE;
     finfo->modified = FALSE;
+    finfo->save_flags = WRITE_SAMPFILE |
+	WRITE_HELPFILE | WRITE_GUI_HELP;
 
     finfo->active = NULL;
     finfo->samplewin = NULL;
@@ -157,13 +181,22 @@ function_info *finfo_new (void)
     finfo->codesel = NULL;
     finfo->popup = NULL;
     finfo->extra = NULL;
+    finfo->specdlg = NULL;
 
     finfo->help = NULL;
+    finfo->gui_help = NULL;
+
+    finfo->sample_fname = NULL;
+    finfo->help_fname = NULL;
+    finfo->gui_help_fname = NULL;
+    
     finfo->pubnames = NULL;
     finfo->privnames = NULL;
+    finfo->datafiles = NULL;
 
     finfo->n_pub = 0;
     finfo->n_priv = 0;
+    finfo->n_files = 0;
 
     finfo->dreq = 0;
     finfo->minver = 10804;
@@ -197,11 +230,18 @@ static void finfo_free (function_info *finfo)
 {
     g_free(finfo->fname);
     g_free(finfo->author);
+    g_free(finfo->email);
     g_free(finfo->version);
     g_free(finfo->date);
     g_free(finfo->pkgdesc);
     g_free(finfo->sample);
     g_free(finfo->help);
+    g_free(finfo->gui_help);
+
+    g_free(finfo->sample_fname);
+    g_free(finfo->help_fname);
+    g_free(finfo->gui_help_fname);
+    
     g_free(finfo->menupath);
     g_free(finfo->menulabel);
 
@@ -215,6 +255,10 @@ static void finfo_free (function_info *finfo)
 
     if (finfo->specials != NULL) {
 	strings_array_free(finfo->specials, finfo->n_special);
+    }
+
+    if (finfo->datafiles != NULL) {
+	strings_array_free(finfo->datafiles, finfo->n_files);
     }    
 
     if (finfo->samplewin != NULL) {
@@ -358,6 +402,7 @@ static int finfo_save (function_info *finfo, int saveas)
     const char *missing = "???";
     char **fields[] = {
 	&finfo->author,
+	&finfo->email,
 	&finfo->version,
 	&finfo->date,
 	&finfo->pkgdesc
@@ -442,8 +487,9 @@ static gboolean update_active_func (GtkComboBox *menu,
    offset of 9 bytes into the line, skipping "function ".
 */
 
-static int extract_funcname (char *name, const char *s)
+static int extract_funcname (const char *s, const char *origname)
 {
+    char newname[FN_NAMELEN];
     char word[FN_NAMELEN];
     int n, type, err = 0;
 
@@ -454,7 +500,7 @@ static int extract_funcname (char *name, const char *s)
 	return E_DATA;
     }
 
-    *word = '\0';
+    *newname = *word = '\0';
     strncat(word, s, n);
 
     if (!strcmp(word, "void")) {
@@ -463,10 +509,8 @@ static int extract_funcname (char *name, const char *s)
 	type = gretl_type_from_string(word);
     }
 
-    if (type == 0) {
-	/* old-style */
-	strcpy(name, word);
-    } else if (!ok_function_return_type(type)) {
+    if (!ok_function_return_type(type)) {
+	gretl_errmsg_sprintf("%s: bad or missing return type", origname);
 	err = E_DATA;
     } else {
 	s += n;
@@ -475,8 +519,11 @@ static int extract_funcname (char *name, const char *s)
 	if (n == 0 || n > FN_NAMELEN - 1) {
 	    err = E_DATA;
 	} else {
-	    *name = '\0';
-	    strncat(name, s, n);
+	    strncat(newname, s, n);
+	    if (strcmp(newname, origname)) {
+		gretl_errmsg_set(_("You can't change the name of a function here"));
+		err = E_DATA;
+	    }
 	}
     }
 
@@ -485,7 +532,7 @@ static int extract_funcname (char *name, const char *s)
 
 static int pretest_funcname (char *buf, const char *origname)
 {
-    char *s, line[MAXLINE], newname[FN_NAMELEN];
+    char *s, line[MAXLINE];
     int err = 0;
 
     bufgets_init(buf);
@@ -493,12 +540,7 @@ static int pretest_funcname (char *buf, const char *origname)
     while (bufgets(line, sizeof line, buf) && !err) {
 	s = line + strspn(line, " \t");
 	if (!strncmp(s, "function ", 9)) {
-	    if (extract_funcname(newname, s + 9)) {
-		err = E_DATA;
-	    } else if (strcmp(newname, origname)) {
-		gretl_errmsg_set(_("You can't change the name of a function here"));
-		err = E_DATA;
-	    }
+	    err = extract_funcname(s + 9, origname);
 	    break;
 	}
     }
@@ -527,7 +569,7 @@ int update_func_code (windata_t *vwin)
 	int save_batch = gretl_in_batch_mode();
 
 	set_current_function_package(finfo->pkg);
-	err = execute_script(NULL, text, NULL, INCLUDE_EXEC);
+	err = execute_script(NULL, text, NULL, INCLUDE_EXEC, NULL);
 	set_current_function_package(NULL);
 	gretl_set_batch_mode(save_batch);
     }
@@ -672,7 +714,7 @@ static void edit_code_callback (GtkWidget *w, function_info *finfo)
 	/* the package may not be saved yet */
 	fun = get_user_function_by_name(funname);
 	if (fun == NULL) {
-	    errbox(_("Can't find the function '%s'"), funname);
+	    errbox_printf(_("Can't find the function '%s'"), funname);
 	}
     }
 
@@ -680,7 +722,7 @@ static void edit_code_callback (GtkWidget *w, function_info *finfo)
 	return;
     }    
 
-    gretl_function_print_code(fun, prn);
+    gretl_function_print_code(fun, tabwidth, prn);
 
     vwin = view_buffer(prn, SCRIPT_WIDTH, SCRIPT_HEIGHT, 
 		       finfo->active, EDIT_PKG_CODE, finfo);
@@ -827,24 +869,247 @@ static void add_remove_callback (GtkWidget *w, function_info *finfo)
 
 static void gfn_to_script_callback (GtkWidget *w, function_info *finfo)
 {
+    gint resp;
+    
     if (finfo->n_pub + finfo->n_priv == 0) {
 	warnbox("No code to save");
 	return;
     }
 
+    if (finfo->sample != NULL) {
+	resp = yes_no_cancel_dialog("gretl",
+				    _("Saving packaged functions as script:\n"
+				      "include the sample script?"),
+				    finfo->dlg);
+	
+	if (canceled(resp)) {
+	    return;
+	}
+
+	if (resp == GRETL_YES) {
+	    finfo->save_flags |= APPEND_SAMPLE;
+	} else {
+	    finfo->save_flags &= ~APPEND_SAMPLE;
+	}
+    }
+
     file_selector_with_parent(SAVE_FUNCTIONS_AS, FSEL_DATA_MISC, 
 			      finfo, finfo->dlg);
 }
 
+struct spec_info {
+    GtkWidget *dialog;
+    GtkWidget *checks[3];
+    GtkWidget *entries[3];
+    int *flags;
+    function_info *finfo;
+    int retval;
+};
+
+static void reset_finfo_filename (function_info *finfo, int i, gchar *src)
+{
+    if (i == 0) {
+	g_free(finfo->sample_fname);
+	finfo->sample_fname = src;
+    } else if (i == 1) {
+	g_free(finfo->help_fname);
+	finfo->help_fname = src;
+    } else {
+	g_free(finfo->gui_help_fname);
+	finfo->gui_help_fname = src;
+    }
+}
+
+static void spec_save_ok (GtkWidget *button, gpointer data)
+{
+    struct spec_info *sinfo = data;
+    function_info *finfo = sinfo->finfo;
+    gchar *fname;
+    int i, flag;
+
+    for (i=0; i<3; i++) {
+	if (sinfo->checks[i] != NULL) {
+	    flag = sinfo->flags[i];
+	    finfo->save_flags &= ~flag;
+	    if (button_is_active(sinfo->checks[i])) {
+		fname = entry_box_get_trimmed_text(sinfo->entries[i]);
+		if (fname != NULL) {
+		    finfo->save_flags |= flag;
+		    reset_finfo_filename(finfo, i, fname);
+		}
+	    }
+	}
+    }
+
+    sinfo->retval = 0;
+    gtk_widget_destroy(sinfo->dialog);
+}
+
+static gchar *get_pkg_text_filename (function_info *finfo,
+				     const char *pkgname,
+				     const char **ids,
+				     int i)
+{
+    const char *s;
+    gchar *fname = NULL;
+
+    s = function_package_get_string(finfo->pkg, ids[i]);
+
+    if (s != NULL) {
+	fname = g_strdup(s);
+    } else if (i == 0) {
+	fname = g_strdup_printf("%s_sample.inp", pkgname);
+    } else if (i == 1) {
+	fname = g_strdup_printf("%s_help.txt", pkgname);
+    } else {
+	fname = g_strdup_printf("%s_gui_help.txt", pkgname);
+    }
+
+    return fname;
+}
+
+static void sensitize_auxname_entry (GtkToggleButton *button,
+				     GtkWidget *w)
+{
+    gboolean s = gtk_toggle_button_get_active(button);
+	
+    gtk_widget_set_sensitive(w, s);
+}
+
+static void nullify_spec_dialog (GtkWidget *w, function_info *finfo)
+{
+    finfo->specdlg = NULL;
+}
+
+static int gfn_spec_save_dialog (function_info *finfo,
+				 const char **texts)
+{
+    const gchar *msgs[] = {
+	N_("Save sample script as"),
+	N_("Save help text as"),
+	N_("Save GUI help as")
+    };
+    const char *ids[] = {
+	"sample-fname",
+	"help-fname",
+	"gui-help-fname"
+    };
+    int flags[] = {
+	WRITE_SAMPFILE,
+	WRITE_HELPFILE,
+	WRITE_GUI_HELP
+    };    
+    struct spec_info sinfo;
+    GtkWidget *dialog, *entry;
+    GtkWidget *vbox, *hbox, *w;
+    GtkWidget *table;
+    const char *pkgname;
+    gchar *tmp;
+    int i, j, n = 0;
+
+    sinfo.retval = GRETL_CANCEL;
+    sinfo.finfo = finfo;
+    sinfo.flags = flags;
+
+    finfo->specdlg = sinfo.dialog = dialog =
+	gretl_dialog_new(NULL, finfo->dlg, GRETL_DLG_BLOCK);
+    g_signal_connect(G_OBJECT(dialog), "destroy",
+		     G_CALLBACK(nullify_spec_dialog), finfo);
+    
+    vbox = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
+
+    pkgname = function_package_get_name(finfo->pkg);
+
+    hbox = gtk_hbox_new(FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 5);
+    tmp = g_strdup_printf(_("Saving %s.spec: also save ancillary file(s)?"),
+			  pkgname);
+    w = gtk_label_new(tmp);
+    gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 5);
+
+    for (i=0; i<3; i++) {
+	n += (texts[i] != NULL);
+    }
+
+    table = gtk_table_new(n, 2, FALSE);
+    gtk_table_set_row_spacings(GTK_TABLE(table), 5);
+    gtk_table_set_col_spacings(GTK_TABLE(table), 5);
+    gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 5);
+
+    j = 0;
+    for (i=0; i<3; i++) {
+	if (texts[i] == NULL) {
+	    sinfo.checks[i] = sinfo.entries[i] = NULL;
+	    continue;
+	}
+	tmp = get_pkg_text_filename(finfo, pkgname, ids, i);
+	w = sinfo.checks[i] = gtk_check_button_new_with_label(_(msgs[i]));
+	if (finfo->save_flags & flags[i]) {
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
+	}
+	gtk_table_attach_defaults(GTK_TABLE(table), w, 0, 1, j, j+1);
+	entry = sinfo.entries[i] = gtk_entry_new();
+	gtk_entry_set_max_length(GTK_ENTRY(entry), 64);
+	gtk_entry_set_width_chars(GTK_ENTRY(entry), 28);
+	gtk_entry_set_text(GTK_ENTRY(entry), tmp);
+	g_signal_connect(G_OBJECT(w), "toggled", 
+			 G_CALLBACK(sensitize_auxname_entry),
+			 entry);
+	gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, j, j+1);
+	g_free(tmp);
+	j++;
+    }
+
+    hbox = gtk_dialog_get_action_area(GTK_DIALOG(dialog));
+
+    cancel_delete_button(hbox, dialog);
+    w = ok_button(hbox);
+    g_signal_connect(G_OBJECT(w), "clicked", 
+		     G_CALLBACK(spec_save_ok), &sinfo);
+    gtk_widget_grab_default(w);
+
+    gtk_widget_show_all(dialog);
+
+    return sinfo.retval;
+}
+
 static void gfn_to_spec_callback (GtkWidget *w, function_info *finfo)
 {
+    const char *texts[] = {
+	finfo->sample,
+	finfo->help,
+	finfo->gui_help
+    };
+    int resp = 0;
+
+    if (finfo->specdlg != NULL) {
+	gtk_window_present(GTK_WINDOW(finfo->specdlg));
+	return;
+    }    
+    
     if (finfo->pkg == NULL) {
 	warnbox(_("Please save your package first"));
 	return;
     }
 
-    file_selector_with_parent(SAVE_GFN_SPEC, FSEL_DATA_MISC, 
-			      finfo, finfo->dlg);
+    if (texts[0] == NULL) {
+	texts[0] = function_package_get_string(finfo->pkg, "sample-script");
+    }
+    if (texts[1] != NULL && !strncmp(texts[1], "pdfdoc", 6)) {
+	texts[1] = NULL;
+    }    
+    if (texts[2] == NULL) {
+	texts[2] = function_package_get_string(finfo->pkg, "gui-help");
+    }
+
+    if (texts[0] != NULL || texts[1] != NULL || texts[2] != NULL) {
+	resp = gfn_spec_save_dialog(finfo, texts);
+    }
+
+    if (!canceled(resp)) {
+	file_selector_with_parent(SAVE_GFN_SPEC, FSEL_DATA_MISC, 
+				  finfo, finfo->dlg);
+    }
 }
 
 static GtkWidget *label_hbox (GtkWidget *w, const char *txt)
@@ -1195,7 +1460,8 @@ static gint query_save_package (GtkWidget *w, GdkEvent *event,
 				function_info *finfo)
 {
     if (finfo->modified) {
-	int resp = yes_no_dialog("gretl", _("Save changes?"), 1);
+	int resp =
+	    yes_no_cancel_dialog("gretl", _("Save changes?"), w);
 
 	if (resp == GRETL_CANCEL) {
 	    return TRUE;
@@ -1410,6 +1676,32 @@ static void switch_menu_view (GtkComboBox *combo,
     }	
 }
 
+static void add_data_files_entries (GtkWidget *holder,
+				    function_info *finfo)
+{
+    const char *msg = N_("You may add or delete names of data"
+			 "files to be included in the package.");
+    GtkWidget *w, *hbox, *entry;
+    int i;
+
+    w = gtk_label_new(_(msg));
+    gtk_label_set_line_wrap(GTK_LABEL(w), TRUE);
+    hbox = gtk_hbox_new(FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(holder), hbox, FALSE, FALSE, 5);
+
+    for (i=0; i<N_FILE_ENTRIES; i++) {
+	hbox = gtk_hbox_new(FALSE, 5);
+	finfo->file_entries[i] = entry = gtk_entry_new();
+	gtk_entry_set_width_chars(GTK_ENTRY(entry), 32);
+	if (i < finfo->n_files) {
+	    gtk_entry_set_text(GTK_ENTRY(entry), finfo->datafiles[i]);
+	}
+	gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(holder), hbox, FALSE, FALSE, 5);
+    }
+}
+
 static void add_menu_attach_top (GtkWidget *holder,
 				 function_info *finfo)
 {
@@ -1573,6 +1865,57 @@ static int process_special_functions (function_info *finfo)
     return n_changed;
 }
 
+static int process_data_file_names (function_info *finfo)
+{
+    gchar *fname;
+    int i, nf = 0;
+    int changed = 0;
+
+    for (i=0; i<N_FILE_ENTRIES; i++) {
+	fname = entry_box_get_trimmed_text(finfo->file_entries[i]);
+	if (fname != NULL) {
+	    nf++;
+	    if (i < finfo->n_files &&
+		strcmp(fname, finfo->datafiles[i])) {
+		changed = 1;
+	    }
+	}
+	g_free(fname);
+    }
+
+    if (!changed && nf != finfo->n_files) {
+	/* added or deleted */
+	changed = 1;
+    }
+
+    if (changed) {
+	strings_array_free(finfo->datafiles, finfo->n_files);
+	if (nf == 0) {
+	    finfo->datafiles = NULL;
+	    finfo->n_files = 0;
+	} else {
+	    finfo->datafiles = strings_array_new(nf);
+	    if (finfo->datafiles != NULL) {
+		finfo->n_files = nf;
+	    }
+	}
+
+	if (finfo->datafiles != NULL) {
+	    int j = 0;
+	    
+	    for (i=0; i<N_FILE_ENTRIES; i++) {
+		fname = entry_box_get_trimmed_text(finfo->file_entries[i]);
+		if (fname != NULL) {
+		    finfo->datafiles[j++] = gretl_strdup(fname);
+		}
+		g_free(fname);
+	    }
+	}	
+    }
+
+    return changed;
+}
+
 static void extra_properties_callback (GtkWidget *w, function_info *finfo)
 {
     GtkWidget *notebook;
@@ -1584,7 +1927,7 @@ static void extra_properties_callback (GtkWidget *w, function_info *finfo)
     
     if (page == 0) {
 	changed = process_special_functions(finfo);
-    } else {
+    } else if (page == 1) {
 	int focus_label = 0;
 
 	changed = process_menu_attachment(finfo, &focus_label);
@@ -1593,6 +1936,8 @@ static void extra_properties_callback (GtkWidget *w, function_info *finfo)
 
 	    gtk_widget_grab_focus(w);
 	}
+    } else {
+	changed = process_data_file_names(finfo);
     }
 
     if (changed) {
@@ -1643,8 +1988,10 @@ static void finfo_extra_help (GtkWidget *w, function_info *finfo)
     
     if (page == 0) {
 	show_gui_help(GUI_FUNCS);
-    } else {
+    } else if (page == 1) {
 	show_gui_help(MENU_ATTACH);
+    } else {
+	show_gui_help(PKG_FILES);
     }
 }
 
@@ -1769,13 +2116,14 @@ static void extra_properties_dialog (GtkWidget *w, function_info *finfo)
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
     gtk_box_pack_start(GTK_BOX(hbox), table, FALSE, FALSE, 5);
 
+    /* the menu attachment page */
+
     vbox = gtk_vbox_new(FALSE, 0);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
     tmp = gtk_label_new(_("Menu attachment"));
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, tmp);
 
     add_menu_attach_top(vbox, finfo);
-
     finfo->maintree = add_menu_navigator(vbox, finfo, 0);
     finfo->modeltree = add_menu_navigator(vbox, finfo, 1);
 
@@ -1783,6 +2131,16 @@ static void extra_properties_dialog (GtkWidget *w, function_info *finfo)
 	gtk_widget_set_sensitive(finfo->currtree, FALSE);
     }
 
+    /* the data files page */
+    
+    vbox = gtk_vbox_new(FALSE, 0);
+    gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
+    tmp = gtk_label_new(_("Data files"));
+    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, tmp);
+    add_data_files_entries(vbox, finfo);
+
+    /* the common buttons area */
+    
     hbox = gtk_dialog_get_action_area(GTK_DIALOG(dlg));
 
     /* Apply button */
@@ -1809,6 +2167,20 @@ static void extra_properties_dialog (GtkWidget *w, function_info *finfo)
     gtk_widget_show_all(dlg);
 }
 
+int package_editor_exit_check (GtkWidget *w)
+{
+    function_info *finfo;
+
+    finfo = g_object_get_data(G_OBJECT(w), "finfo");
+    
+    if (finfo != NULL && finfo->modified) {
+	gtk_window_present(GTK_WINDOW(w));
+	return query_save_package(w, NULL, finfo);
+    }
+
+    return FALSE;
+}
+
 /* Dialog for editing function package.  The user can get here in
    either of two ways: after selecting functions to put into a
    newly created package, or upon selecting an existing package
@@ -1822,12 +2194,14 @@ static void finfo_dialog (function_info *finfo)
     GtkTextBuffer *hbuf = NULL;
     const char *entry_labels[] = {
 	N_("Author"),
+	N_("Email"),
 	N_("Version"),
 	N_("Date (YYYY-MM-DD)"),
 	N_("Package description")
     };
     char *entry_texts[] = {
 	finfo->author,
+	finfo->email,
 	finfo->version,
 	finfo->date,
 	finfo->pkgdesc
@@ -1839,7 +2213,7 @@ static void finfo_dialog (function_info *finfo)
     gtk_window_set_default_size(GTK_WINDOW(finfo->dlg), 640, 500);
 
     if (finfo->fname != NULL) {
-	gchar *title = title_from_filename(finfo->fname, TRUE);
+	gchar *title = title_from_filename(finfo->fname, EDIT_PKG_CODE, TRUE);
 
 	gtk_window_set_title(GTK_WINDOW(finfo->dlg), title);
 	g_free(title);
@@ -1848,6 +2222,7 @@ static void finfo_dialog (function_info *finfo)
 			     _("gretl: function package editor"));
     } 
 
+    g_object_set_data(G_OBJECT(finfo->dlg), "finfo", finfo);
     gtk_widget_set_name(finfo->dlg, "pkg-editor");
     g_signal_connect(G_OBJECT(finfo->dlg), "delete-event",
 		     G_CALLBACK(query_save_package), finfo);
@@ -1878,7 +2253,7 @@ static void finfo_dialog (function_info *finfo)
 
 	if (entry_texts[i] != NULL) {
 	    gtk_entry_set_text(GTK_ENTRY(entry), entry_texts[i]);
-	    if (i == 2) {
+	    if (i == 3) {
 		g_signal_connect(G_OBJECT(entry), "button-press-event",
 				 G_CALLBACK(today_popup), &finfo->popup);
 	    }
@@ -1888,16 +2263,23 @@ static void finfo_dialog (function_info *finfo)
 	    if (s != NULL) {
 		gtk_entry_set_text(GTK_ENTRY(entry), s);
 	    }
-	} else if (i == 1) {
-	    gtk_entry_set_text(GTK_ENTRY(entry), "1.0");
 	} else if (i == 2) {
+	    gtk_entry_set_text(GTK_ENTRY(entry), "1.0");
+	} else if (i == 3) {
 	    gtk_entry_set_text(GTK_ENTRY(entry), print_today());
 	}
 
 	if (i == 0 && entry_texts[i] == NULL) {
+	    /* no author's name */
 	    gtk_widget_grab_focus(entry);
 	    focused = 1;
-	} else if (i == 1 && !focused) {
+	} else if (i == 1 && !focused &&
+		   (entry_texts[i] == NULL || *entry_texts[i] == '\0')) {
+	    /* no email address */
+	    gtk_widget_grab_focus(entry);
+	    focused = 1;
+	} else if (i == 2 && !focused) {
+	    /* version number */
 	    gtk_widget_grab_focus(entry);
 	}
 
@@ -2155,7 +2537,7 @@ static int validate_package_file (const char *fname, int verbose)
 	    errbox(buf);
 	    err = 1;
 	} else if (verbose) {
-	    infobox(_("%s: validated against DTD OK"), pkgname);
+	    infobox_printf(_("%s: validated against DTD OK"), pkgname);
 	} else {
 	    fprintf(stderr, "%s: validated against DTD OK\n", pkgname);
 	}
@@ -2170,27 +2552,227 @@ static int validate_package_file (const char *fname, int verbose)
     return err;
 }
 
-static void do_pkg_upload (function_info *finfo, const char *fname)
+static void zip_report (int err, int nf, PRN *prn)
 {
+    if (err && nf) {
+	pprintf(prn, "<@fail> (%s)\n", _("not found"));	
+    } else if (err) {
+	pputs(prn, "<@fail>\n");
+    } else {
+	pputs(prn, "<@ok>\n");
+    }
+}
+
+static int pkg_zipfile_add (const char *fname,
+			    const char *dotpath,
+			    PRN *prn)
+{
+    gchar *dest = NULL;
+    struct stat sbuf;
+    int nf = 0;
+    int err = 0;
+
+    pprintf(prn, "Copying %s... ", fname);
+
+    if (stat(fname, &sbuf) != 0) {
+	nf = err = E_DATA;
+    } else if (sbuf.st_mode & S_IFDIR) {
+	/* aha, we've got a subdir */
+	gchar *ziptmp;
+
+	ziptmp = g_strdup_printf("%s%cpkgtmp.zip", dotpath, SLASH);
+	err = gretl_make_zipfile(ziptmp, fname);
+	if (!err) {
+	    err = gretl_unzip_into(ziptmp, dotpath);
+	    gretl_remove(ziptmp);
+	}
+	g_free(ziptmp);
+    } else {
+	/* a regular file, we hope */
+	dest = g_strdup_printf("%s%c%s", dotpath, SLASH, fname);
+	err = gretl_copy_file(fname, dest);
+    }
+    
+    zip_report(err, nf, prn);
+    g_free(dest);
+
+    return err;
+}
+
+/* Collect pkg.gfn and pkg.pdf into a temporary dir under
+   the user's dotdir, and make a zipfile containing these
+   two files.
+*/
+
+static int pkg_make_zipfile (function_info *finfo, int pdfdoc,
+			     gchar **zipname)
+{
+    char origdir[FILENAME_MAX];
+    char pkgbase[FILENAME_MAX];
+    const char *pkgname;
+    gchar *tmp, *dotpath = NULL;
+    windata_t *vwin;
+    PRN *prn = NULL;
+    int dir_made = 0;
+    int err = 0;
+
+    /* record where we are now */
+    *origdir = '\0';
+    if (getcwd(origdir, FILENAME_MAX - 1) == NULL) {
+	*origdir = '\0';
+    }
+    fprintf(stderr, "origdir: '%s'\n", origdir);
+
+    /* determine the common path to files for packaging */
+    strcpy(pkgbase, finfo->fname);
+    tmp = strrchr(pkgbase, SLASH);
+    if (tmp != NULL) {
+	*(tmp + 1) = '\0';
+    } else {
+	*pkgbase = '\0';
+    }
+
+    fprintf(stderr, "pkgbase: '%s'\n", pkgbase);
+
+    /* get the basename of the package */
+    pkgname = function_package_get_name(finfo->pkg);
+    fprintf(stderr, "pkgname: '%s'\n", pkgname);
+
+    /* open recorder for possible errors */
+    bufopen(&prn);
+    
+    if (*pkgbase != '\0') {
+	/* get into place for copying */
+	pputs(prn, "Getting in place... ");
+	err = gretl_chdir(pkgbase);
+	zip_report(err, 0, prn);
+    }
+
+    if (!err) {
+	/* path to temporary dir for zipping */
+	dotpath = g_strdup_printf("%s%s", gretl_dotdir(), pkgname);
+	pputs(prn, "Making temporary directory... ");
+	err = gretl_mkdir(dotpath);
+	zip_report(err, 0, prn);
+    }
+    
+    if (!err) {
+	dir_made = 1;
+    }
+
+    if (!err) {
+	/* copy the gfn file into place */
+	tmp = g_strdup_printf("%s.gfn", pkgname);
+	err = pkg_zipfile_add(tmp, dotpath, prn);
+	g_free(tmp);
+    }
+
+    if (!err && pdfdoc) {
+	/* copy PDF file into place */
+	tmp = g_strdup_printf("%s.pdf", pkgname);
+	err = pkg_zipfile_add(tmp, dotpath, prn);
+	g_free(tmp);
+    }
+
+    if (!err && finfo->datafiles != NULL) {
+	/* copy data files into place, if any */
+	int i;
+
+	for (i=0; i<finfo->n_files && !err; i++) {
+	    err = pkg_zipfile_add(finfo->datafiles[i],
+				  dotpath, prn);
+	}
+    }	
+
+    if (!err) {
+	/* get into place for making zipfile */
+	err = gretl_chdir(gretl_dotdir());
+	if (!err) {
+	    tmp = g_strdup_printf("%s.zip", pkgname);
+	    pprintf(prn, "Making %s... ", tmp);
+	    err = gretl_make_zipfile(tmp, pkgname);
+	    zip_report(err, 0, prn);
+	    g_free(tmp);
+	    if (!err) {
+		*zipname = g_strdup_printf("%s%s.zip", gretl_dotdir(),
+					   pkgname);
+	    }
+	}
+    }
+
+    if (err) {
+	/* show details on failure */
+	vwin = view_buffer(prn, 78, 300, _("build zip file"), ZIPBUILD, NULL);
+	gtk_window_set_transient_for(GTK_WINDOW(vwin->main),
+				     GTK_WINDOW(finfo->dlg));
+	gtk_window_set_destroy_with_parent(GTK_WINDOW(vwin->main), TRUE);
+    } else {
+	gretl_print_destroy(prn);
+    }
+
+    if (*origdir != '\0') {
+	/* get back to where we came from */
+	gretl_chdir(origdir);
+    }
+
+    if (dir_made) {
+	/* delete the temporary zip directory */
+	gretl_deltree(dotpath);
+    }
+
+    g_free(dotpath);
+
+    return err;
+}
+
+static int pkg_has_pdf_help (function_info *finfo)
+{
+    if (finfo->help != NULL &&
+	!strncmp(finfo->help, "pdfdoc", 6)) {
+	return 1;
+    } else {
+	return 0;
+    }
+}
+
+static void do_pkg_upload (function_info *finfo, const char *gfnpath)
+{
+    const char *fname;
     gchar *buf = NULL;
     char *retbuf = NULL;
+    gchar *zipname = NULL;
     login_info linfo;
     GdkDisplay *disp;
     GdkCursor *cursor;
     GdkWindow *w1;
     gint x, y;
+    gsize buflen;
     int error_printed = 0;
+    int pdfdoc;
     int err;
 
-    err = validate_package_file(fname, 0);
+    err = validate_package_file(gfnpath, 0);
     if (err) {
 	return;
     }
 
+    pdfdoc = pkg_has_pdf_help(finfo);
+
+    if (pdfdoc || finfo->datafiles != NULL) {
+	err = pkg_make_zipfile(finfo, pdfdoc, &zipname);
+	if (err) {
+	    /* the error message will have been handled abive */
+	    return;
+	}
+    }
+
+    fname = zipname != NULL ? zipname : gfnpath;
+
     login_dialog(&linfo, finfo->dlg);
 
     if (linfo.canceled) {
 	linfo_free(&linfo);
+	g_free(zipname);
 	return;
     }
 
@@ -2204,14 +2786,14 @@ static void do_pkg_upload (function_info *finfo, const char *fname)
 	gdk_cursor_unref(cursor);
     }
 
-    err = gretl_file_get_contents(fname, &buf, NULL);
+    err = gretl_file_get_contents(fname, &buf, &buflen);
 
     if (err) {
 	error_printed = 1;
     } else {
 	err = upload_function_package(linfo.login, linfo.pass, 
 				      path_last_element(fname),
-				      buf, &retbuf);
+				      buf, buflen, &retbuf);
 	fprintf(stderr, "upload_function_package: err = %d\n", err);
     }
 
@@ -2228,6 +2810,12 @@ static void do_pkg_upload (function_info *finfo, const char *fname)
 	infobox(retbuf);
     }
 
+    if (zipname != NULL) {
+	/* delete the upload zipfile */
+	gretl_remove(zipname);
+	g_free(zipname);
+    }
+    
     g_free(buf);
     free(retbuf);
 
@@ -2272,13 +2860,15 @@ static int check_package_filename (function_info *finfo, const char *fname)
     return err;
 }
 
-static int dont_overwrite_pkg (const char *fname)
+static int dont_overwrite_pkg (function_info *finfo, const char *fname)
 {
     FILE *fp = gretl_fopen(fname, "r");
     int ret = 0;
 
     if (fp != NULL) {
-	int resp = yes_no_dialog("gretl", _("OK to overwrite?"), 0);
+	int resp = yes_no_dialog("gretl",
+				 _("OK to overwrite?"),
+				 finfo->dlg);
 
 	ret = (resp == GRETL_NO);
 	fclose(fp);
@@ -2315,7 +2905,7 @@ int save_function_package (const char *fname, gpointer p)
 
     if (finfo->fname == NULL || strcmp(finfo->fname, fname)) {
 	/* new or save as */
-	if (dont_overwrite_pkg(fname)) {
+	if (dont_overwrite_pkg(finfo, fname)) {
 	    return 1;
 	}
 	err = check_package_filename(finfo, fname);
@@ -2352,6 +2942,7 @@ int save_function_package (const char *fname, gpointer p)
     if (!err) {
 	err = function_package_set_properties(finfo->pkg,
 					      "author",  finfo->author,
+					      "email",   finfo->email,
 					      "version", finfo->version,
 					      "date",    finfo->date,
 					      "description", finfo->pkgdesc,
@@ -2369,6 +2960,9 @@ int save_function_package (const char *fname, gpointer p)
 
     if (!err) {
 	pkg_save_special_functions(finfo);
+	function_package_set_data_files(finfo->pkg,
+					finfo->datafiles,
+					finfo->n_files);
     }
 
     if (!err) {
@@ -2395,12 +2989,13 @@ int save_function_package (const char *fname, gpointer p)
 	    do_pkg_upload(finfo, fname);
 	}
 	/* revise packages.xml in accordance with any
-	   changes above */
-	revise_package_status(pkgname, 
-			      finfo->menulabel,
-			      finfo->menupath,
-			      finfo->uses_subdir,
-			      TRUE, FALSE);
+	   changes above, as needed */
+	maybe_update_packages_xml(pkgname, 
+				  finfo->menulabel,
+				  finfo->menupath,
+				  finfo->uses_subdir,
+				  NULL, /* notification not needed */
+				  finfo->dlg);
     }
 
     return err;
@@ -2422,29 +3017,37 @@ int save_function_package_as_script (const char *fname, gpointer p)
 	return err;
     }
 
+    /* write basic package info */
     pprintf(prn, "# author='%s'\n", finfo->author);
+    if (finfo->email != NULL && *finfo->email != '\0') {
+	pprintf(prn, "# email='%s'\n", finfo->email);
+    }
     pprintf(prn, "# version='%s'\n", finfo->version);
     pprintf(prn, "# date='%s'\n", finfo->date);
 
+    /* write private functions, if any */
     for (i=0; i<finfo->n_priv; i++) {
 	fun = get_function_from_package(finfo->privnames[i],
 					finfo->pkg);
 	if (fun != NULL) {
 	    pputc(prn, '\n');
-	    gretl_function_print_code(fun, prn);
+	    gretl_function_print_code(fun, tabwidth, prn);
 	}
     }
 
+    /* write public functions */
     for (i=0; i<finfo->n_pub; i++) {
 	fun = get_function_from_package(finfo->pubnames[i],
 					finfo->pkg);
 	if (fun != NULL) {
 	    pputc(prn, '\n');
-	    gretl_function_print_code(fun, prn);
+	    gretl_function_print_code(fun, tabwidth, prn);
 	}
     }
 
-    if (finfo->sample != NULL) {
+    /* append sample script? */
+    if ((finfo->save_flags & APPEND_SAMPLE) &&
+	finfo->sample != NULL) {
 	int n = strlen(finfo->sample);
 
 	pputs(prn, "\n# sample function call\n");
@@ -2460,51 +3063,66 @@ int save_function_package_as_script (const char *fname, gpointer p)
 }
 
 static void maybe_print (PRN *prn, const char *key, 
-			       const char *arg)
+			 const char *arg)
 {
-    if (arg != NULL) {
+    if (arg != NULL && *arg != '\0') {
 	pprintf(prn, "%s = %s\n", key, arg);
     } else {
 	pprintf(prn, "%s = \n", key);
     }
 }
 
-static const char *aux_basename (const char *fname)
-{
-    const char *p = strrchr(fname, SLASH);
-
-    if (p != NULL) {
-	/* take last part of filename */
-	return p + 1;
-    } else {
-	return fname;
-    }
-}
-
-static int maybe_write_aux_file (const char *fname, 
+static int maybe_write_aux_file (function_info *finfo,
+				 const char *fname,
+				 const char *id,
 				 const gchar *content,
-				 const char *sfx,
 				 PRN *prn)
 {
     int ret = 0;
 
     if (content != NULL && *content != '\0') {
-	char *s, tmp[FILENAME_MAX];
-	FILE *fp;
+	const char *auxname = NULL;
+	int flag;
 
-	strcpy(tmp, fname);
-	s = strstr(tmp, ".spec");
-	if (s != NULL) {
-	    *s = '\0';
-	    strncat(s, sfx, strlen(sfx));
-	    fp = gretl_fopen(tmp, "w");
+	if (!strcmp(id, "sample-script")) {
+	    auxname = finfo->sample_fname;
+	    flag = WRITE_SAMPFILE;
+	} else if (!strcmp(id, "help")) {
+	    auxname = finfo->help_fname;
+	    flag = WRITE_HELPFILE;
+	} else {
+	    auxname = finfo->gui_help_fname;
+	    flag = WRITE_GUI_HELP;
+	}
+
+	if (auxname != NULL && (finfo->save_flags & flag)) {
+	    /* we'll write out the actual file */
+	    FILE *fp = NULL;
+
+	    if (strrchr(fname, SLASH)) {
+		/* package fname has directory component */
+		char *s, tmp[FILENAME_MAX];
+		
+		strcpy(tmp, fname);
+		s = strrchr(tmp, SLASH);
+		*(s + 1) = '\0';
+		strcat(tmp, auxname);
+		fp = gretl_fopen(tmp, "w");
+	    } else {
+		fp = gretl_fopen(auxname, "w");
+	    }
+	    
 	    if (fp != NULL) {
 		fputs(content, fp);
 		fclose(fp);
-		pputs(prn, aux_basename(tmp));
 		ret = 1;
 	    }
 	}
+
+	if (auxname != NULL) {
+	    /* record filename in spec file */
+	    pputs(prn, auxname);
+	}
     }
 
     return ret;
@@ -2529,13 +3147,16 @@ int save_function_package_spec (const char *fname, gpointer p)
 	GUI_PRECHECK,
 	NULL
     };
+    const char *gui_help;
+    const char *sample;
     gchar *strval;
     int intval;
     function_info *finfo = p;
     PRN *prn;
     const char *reqstr = NULL;
     int v1, v2, v3;
-    int i, len, nnp = 0;
+    int nnp = 0, nmo = 0;
+    int i, len;
     int err = 0;
 
     prn = gretl_print_new_with_filename(fname, &err);
@@ -2545,6 +3166,7 @@ int save_function_package_spec (const char *fname, gpointer p)
     }
 
     maybe_print(prn, "author", finfo->author);
+    maybe_print(prn, "email", finfo->email);
     maybe_print(prn, "version", finfo->version);
     maybe_print(prn, "date", finfo->date);
     maybe_print(prn, "description", finfo->pkgdesc);
@@ -2608,6 +3230,9 @@ int save_function_package_spec (const char *fname, gpointer p)
 	if (user_func_is_noprint(fun)) {
 	    nnp++;
 	}
+	if (user_func_is_menu_only(fun)) {
+	    nmo++;
+	}	
     }
     pputc(prn, '\n');
 
@@ -2625,25 +3250,55 @@ int save_function_package_spec (const char *fname, gpointer p)
 	pputc(prn, '\n');
     }
 
-    /* write out help text file and/or sample script? */
+    if (nmo > 0) {
+	/* menu-only interface names */
+	pputs(prn, "menu-only = ");
+	for (i=0; i<finfo->n_pub; i++) {
+	    const char *s = finfo->pubnames[i];
+	    ufunc *fun = get_function_from_package(s, finfo->pkg);
 
+	    if (user_func_is_menu_only(fun)) {
+		pprintf(prn, "%s ", s);
+	    }
+	}
+	pputc(prn, '\n');
+    }
+
+    /* write out help text? */
     pputs(prn, "help = ");
     if (finfo->text != NULL) {
 	gchar *help = textview_get_trimmed_text(finfo->text);
 
 	if (help != NULL) {
-	    if (!strncmp(help, "pdfdoc:", 7)) {
+	    if (!strncmp(help, "pdfdoc", 6)) {
 		pputs(prn, help + 7);
 	    } else {
-		maybe_write_aux_file(fname, help, "_help.txt", prn);
+		maybe_write_aux_file(finfo, fname, "help",
+				     help, prn);
 	    }
 	    g_free(help);
 	}
 	pputc(prn, '\n');
     }
 
+    gui_help = (finfo->gui_help != NULL)? finfo->gui_help :
+	function_package_get_string(finfo->pkg, "gui-help");
+
+    /* write out GUI-specific help text? */
+    if (gui_help != NULL) {
+	pputs(prn, "gui-help = ");
+	maybe_write_aux_file(finfo, fname, "gui-help",
+			     gui_help, prn);
+	pputc(prn, '\n');
+    }
+
+    sample = (finfo->sample != NULL)? finfo->sample :
+	function_package_get_string(finfo->pkg, "sample-script");
+
+    /* write out sample script? */
     pputs(prn, "sample-script = ");
-    maybe_write_aux_file(fname, finfo->sample, "_sample.inp", prn);
+    maybe_write_aux_file(finfo, fname, "sample-script",
+			 sample, prn);
     pputc(prn, '\n');
 
     gretl_print_destroy(prn);
@@ -2781,6 +3436,21 @@ static int finfo_set_special_names (function_info *finfo)
     return err;
 }
 
+static int finfo_set_data_files (function_info *finfo)
+{
+    char **S;
+    int n = 0;
+
+    S = function_package_get_data_files(finfo->pkg, &n);
+
+    if (S != NULL) {
+	finfo->datafiles = S;
+	finfo->n_files = n;
+    }
+
+    return 0;
+}
+
 void edit_function_package (const char *fname)
 {
     function_info *finfo;
@@ -2807,6 +3477,7 @@ void edit_function_package (const char *fname)
 					  "publist",  &publist,
 					  "privlist", &privlist,
 					  "author",   &finfo->author,
+					  "email",    &finfo->email,
 					  "version",  &finfo->version,
 					  "date",     &finfo->date,
 					  "description", &finfo->pkgdesc,
@@ -2831,6 +3502,10 @@ void edit_function_package (const char *fname)
 	finfo_set_menuwin(finfo);
     }
 
+    if (!err) {
+	err = finfo_set_data_files(finfo);
+    }
+
 #if PKG_DEBUG
     printlist(publist, "publist");
     printlist(privlist, "privlist");
@@ -2876,20 +3551,21 @@ void edit_package_at_startup (const char *fname)
     }
 }
 
-int no_user_functions_check (void)
+int no_user_functions_check (GtkWidget *parent)
 {
     int err = 0;
 
     if (n_free_functions() == 0) {
+	const gchar *query =
+	    N_("No functions are available for packaging at present.\n"
+	       "Do you want to write a function now?");
 	int resp;
 
 	err = 1;
 	resp = yes_no_dialog(_("gretl: function packages"),
-			     _("No functions are available for packaging at present.\n"
-			       "Do you want to write a function now?"),
-			     0);
+			     _(query), parent);
 	if (resp == GRETL_YES) {
-	    do_new_script(FUNC);
+	    do_new_script(FUNC, NULL);
 	}
     } 
 
