@@ -163,7 +163,6 @@ int main_x = -1;
 int main_y = -1;
 int mainwin_width = 520;
 int mainwin_height = 420;
-int ox_support = FALSE;
 
 #if defined(G_OS_WIN32)
 char calculator[MAXSTR] = "calc.exe";
@@ -224,9 +223,14 @@ static void new_matrix_callback (GtkAction *action, gpointer p)
     gui_new_matrix(mdata->main);
 }
 
+static void pc_change_callback (GtkAction *action, gpointer p)
+{
+    percent_change_dialog(mdata->active_var);
+}
+
 static void new_package_callback (GtkAction *action, gpointer p)
 {
-    if (no_user_functions_check()) {
+    if (no_user_functions_check(NULL)) {
 	return;
     } else {
 	fsave_callback(action, p);
@@ -306,12 +310,14 @@ static int script_type (const char *fname)
     } else if (has_suffix(fname, ".plt") ||
 	       has_suffix(fname, ".gp")) {
 	return EDIT_GP;
-    } else if (ox_support && has_suffix(fname, ".ox")) {
+    } else if (has_suffix(fname, ".ox")) {
 	return EDIT_OX;
     } else if (has_suffix(fname, ".m")) {
 	return EDIT_OCTAVE;
     } else if (has_suffix(fname, ".py")) {
 	return EDIT_PYTHON;
+    } else if (has_suffix(fname, ".do")) {
+	return EDIT_STATA;
     } else {
 	return 0;
     }
@@ -1320,7 +1326,6 @@ static void make_main_window (void)
 #endif
     GtkWidget *box, *dlabel;
     GtkWidget *align;
-    GtkWidget *menu;
     const char *titles[] = {
 	N_("ID #"), 
 	N_("Variable name"), 
@@ -1356,13 +1361,15 @@ static void make_main_window (void)
     gtk_window_set_default_size(GTK_WINDOW(mdata->main), 
 				mainwin_width, mainwin_height);
 
-    menu = make_main_menu();
-    if (menu == NULL) {
+    mdata->mbar = make_main_menu();
+    if (mdata->mbar == NULL) {
 	exit(EXIT_FAILURE);
     }
 
     /* put the main menu bar in place */
-    gtk_box_pack_start(GTK_BOX(mdata->vbox), menu, FALSE, TRUE, 0);
+    box = gtk_hbox_new(FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(box), mdata->mbar, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(mdata->vbox), box, FALSE, FALSE, 0);
 
 #ifdef MAC_INTEGRATION
     mac_mgr = add_mac_menu();
@@ -1383,7 +1390,7 @@ static void make_main_window (void)
     fprintf(stderr, " adding main-window listbox...\n");
 #endif
    
-    vwin_add_list_box(mdata, GTK_BOX(box), 3, FALSE, types, titles, 1);
+    vwin_add_list_box(mdata, GTK_BOX(box), 3, 0, types, titles, 1);
 
     gtk_drag_dest_set(mdata->listbox,
 		      GTK_DEST_DEFAULT_ALL,
@@ -1427,6 +1434,7 @@ static void make_main_window (void)
     fprintf(stderr, "  set_app_font done\n");
 #endif
 
+    vwin_add_winlist(mdata);
     add_mainwin_toolbar(mdata->vbox);
 
 #if GUI_DEBUG
@@ -1485,6 +1493,7 @@ GtkActionEntry main_entries[] = {
     { "OctaveScript", NULL, N_("Octave script"), NULL, NULL, G_CALLBACK(new_script_callback) },
     { "PyScript", NULL, N_("Python script"), NULL, NULL, G_CALLBACK(new_script_callback) },
     { "OxScript", NULL, N_("Ox program"), NULL, NULL, G_CALLBACK(new_script_callback) },
+    { "StataScript", NULL, N_("Stata program"), NULL, NULL, G_CALLBACK(new_script_callback) },
 
     { "SessionFiles", NULL, N_("_Session files"), NULL, NULL, NULL },
     { "OpenSession", GTK_STOCK_OPEN, N_("_Open session..."), "", NULL, G_CALLBACK(open_session_callback) },
@@ -1560,7 +1569,7 @@ GtkActionEntry main_entries[] = {
     { "ImpulsePlot", NULL, N_("X-Y with _impulses..."), NULL, NULL, G_CALLBACK(selector_callback) },
     { "FactorPlot", NULL, N_("X-Y with _factor separation..."), NULL, NULL, G_CALLBACK(selector_callback) },
     { "FrischPlot", NULL, N_("X-Y with _control..."), NULL, NULL, G_CALLBACK(selector_callback) },
-    { "GR_BOX", NULL, N_("_Boxplots..."), NULL, NULL, G_CALLBACK(gretl_callback) },
+    { "GR_BOX", NULL, N_("_Boxplots..."), NULL, NULL, G_CALLBACK(selector_callback) },
     { "GR_FBOX", NULL, N_("_Factorized boxplot..."), NULL, NULL, G_CALLBACK(selector_callback) },
     { "GR_QQ", NULL, N_("_Q-Q plot..."), NULL, NULL, G_CALLBACK(menu_op_action) },
     { "ThreeDPlot", NULL, N_("_3D plot..."), NULL, NULL, G_CALLBACK(selector_callback) },
@@ -1585,6 +1594,8 @@ GtkActionEntry main_entries[] = {
       G_CALLBACK(logs_etc_callback) },
     { "sdiff", NULL, N_("_Seasonal differences of selected variables"), NULL, NULL, 
       G_CALLBACK(logs_etc_callback) },
+    { "pcdiff", NULL, N_("_Percentage change of selected variable"), NULL, NULL, 
+      G_CALLBACK(pc_change_callback) },
     { "AddIndex", NULL, N_("_Index variable"), NULL, NULL, G_CALLBACK(add_index) },
     { "AddTime", NULL, N_("_Time trend"), NULL, NULL, G_CALLBACK(add_index) },
     { "AddUnit", NULL, N_("_Panel unit index"), NULL, NULL, G_CALLBACK(add_index) },
@@ -1593,8 +1604,8 @@ GtkActionEntry main_entries[] = {
     { "UnitDums", NULL, N_("_Unit dummies"), NULL, NULL, G_CALLBACK(add_dummies) },
     { "TimeDums", NULL, N_("_Time dummies"), NULL, NULL, G_CALLBACK(add_dummies) },
     { "RangeDum", NULL, N_("_Observation range dummy"), NULL, NULL, G_CALLBACK(range_dummy_dialog) },
-    { "dummify", NULL, N_("Dummies for selected _discrete variables"), NULL, NULL, 
-      G_CALLBACK(logs_etc_callback) },
+    { "dummify", NULL, N_("Dummies for _discrete variable..."), NULL, NULL, 
+      G_CALLBACK(add_dummies) },
     { "NewMatrix", NULL, N_("_Define matrix..."), NULL, NULL, G_CALLBACK(new_matrix_callback) },
 
     /* Sample */
@@ -1843,15 +1854,6 @@ static void add_conditional_items (windata_t *vwin)
 			  FALSE);
 #endif
 
-    if (ox_support) {
-	gtk_ui_manager_add_ui(ui, gtk_ui_manager_new_merge_id(ui),
-			      "/menubar/File/ScriptFiles/NewScript/OxScript",
-			      N_("Ox program"),
-			      "OxScript",
-			      GTK_UI_MANAGER_MENUITEM, 
-			      FALSE);
-    }
-
     maybe_add_packages_to_menus(vwin);
 }
 
@@ -2008,7 +2010,7 @@ static void name_new_list (GtkWidget *widget, dialog_t *dlg)
     char *lname = (char *) edit_dialog_get_data(dlg);
     const gchar *buf = edit_dialog_get_text(dlg);
 
-    if (buf == NULL || gui_validate_varname(buf, GRETL_TYPE_LIST)) {
+    if (buf == NULL || gui_validate_varname(buf, GRETL_TYPE_LIST, NULL)) {
 	return;
     }
 
@@ -2025,7 +2027,7 @@ static void real_make_mainwin_list (const int *list,
     if (err) {
 	gui_errmsg(err);
     } else {
-	lstr = gretl_list_to_string(list);
+	lstr = gretl_list_to_string(list, dataset, &err);
     }
 
     if (lstr != NULL) {
@@ -2145,11 +2147,13 @@ int gretl_fork (const char *progvar, const char *arg)
 	prog = viewpdf;
     } else if (!strcmp(progvar, "viewps")) {
 	prog = viewps;
+    } else {
+	prog = progvar;
     }
 #endif
 
     if (prog == NULL) {
-	errbox("Internal error: variable %s is undefined", progvar);
+	errbox_printf("Internal error: variable %s is undefined", progvar);
 	return 1;
     }
     
@@ -2388,7 +2392,8 @@ main_popup_handler (GtkWidget *w, GdkEventButton *event, gpointer data)
 {
     if (right_click(event)) {
 	/* ignore all but right-clicks */
-	int selcount = vwin_selection_count(mdata, NULL);
+	int selvar = 0;
+	int selcount = vwin_selection_count(mdata, &selvar);
 
 	if (mdata->popup) {
 	    gtk_widget_destroy(mdata->popup);
@@ -2396,7 +2401,7 @@ main_popup_handler (GtkWidget *w, GdkEventButton *event, gpointer data)
 	}
 
 	if (selcount == 1) {
-	    mdata->popup = build_var_popup();
+	    mdata->popup = build_var_popup(selvar);
 	} else if (selcount > 1) {
 	    mdata->popup = build_selection_popup();
 	}
@@ -2436,6 +2441,8 @@ static int gui_query_stop (void)
     return script_stopper(0);
 }
 
+/* callback from "stop" button in script output viewer */
+
 void do_stop_script (GtkWidget *w, windata_t *vwin)
 {
     script_stopper(1);
