@@ -43,6 +43,7 @@ enum extra_cmds {
     VIEW_MODELTABLE,
     VIEW_PKG_INFO,
     VIEW_PKG_CODE,
+    VIEW_PKG_SAMPLE,
     VIEW_BUNDLE,
     VIEW_FILE,
     VIEW_DOC,
@@ -60,6 +61,7 @@ enum extra_cmds {
     EDIT_OX,
     EDIT_OCTAVE,
     EDIT_PYTHON,
+    EDIT_STATA,
     EDIT_X12A,
     EDIT_MAX,
     CLI_HELP,
@@ -118,6 +120,8 @@ enum extra_cmds {
     GUI_FUNCS,
     MENU_ATTACH,
     DAILY_PURGE,
+    ZIPBUILD,
+    PKG_FILES,
     GUI_CMD_MAX
 };
 
@@ -134,6 +138,7 @@ enum file_ops {
     OPEN_LABELS,
     OPEN_BARS,
     OPEN_GFN,
+    OPEN_ANY,
     END_OPEN,        /* marker for end of file open section */
     AUTO_SAVE_DATA,
     SAVE_DATA,
@@ -161,6 +166,7 @@ enum file_ops {
     SAVE_OX_CMDS,
     SAVE_OCTAVE_CMDS,
     SAVE_PYTHON_CMDS,
+    SAVE_STATA_CMDS,
     SAVE_CONSOLE,
     SAVE_CMD_LOG,
     SAVE_FUNCTIONS,
@@ -218,7 +224,12 @@ enum clipstuff {
     TARGET_STRING,
     TARGET_TEXT,
     TARGET_COMPOUND_TEXT,
-    TARGET_RTF
+    TARGET_RTF,
+    TARGET_SVG,
+    TARGET_EMF,
+    TARGET_EPS,
+    TARGET_PDF,
+    TARGET_PNG
 };
 
 enum data_status_flags {
@@ -277,13 +288,15 @@ enum calc_functions {
 enum auto_dummies {
     TS_DUMMIES,
     PANEL_UNIT_DUMMIES,
-    PANEL_TIME_DUMMIES
+    PANEL_TIME_DUMMIES,
+    DISCRETE_DUMMIES
 };
 
 enum script_output_policies {
-    OUTPUT_NEW_WINDOW,
-    OUTPUT_REPLACE,
-    OUTPUT_APPEND
+    OUTPUT_POLICY_UNSET,
+    OUTPUT_POLICY_REPLACE,
+    OUTPUT_POLICY_APPEND,
+    OUTPUT_POLICY_NEW_WINDOW
 };
 
 #endif /* GRETL_ENUMS_H */
