@@ -19,11 +19,6 @@
 
 #undef CHILD_DEBUG
 
-#ifdef _WIN64
-# include <pango/pangowin32.h>
-# include <gdk/gdkwin32.h>
-#endif
-
 #include "gretl.h"
 #include "gretlwin32.h"
 #include "textutil.h"
@@ -34,10 +29,8 @@
 
 #include <dirent.h>
 
-#ifndef _WIN64
-# include <pango/pangowin32.h>
-# include <gdk/gdkwin32.h>
-#endif
+#include <pango/pangowin32.h>
+#include <gdk/gdkwin32.h>
 
 /* extra Windows headers */
 #include <mapi.h>
@@ -326,6 +319,7 @@ int prn_to_clipboard (PRN *prn, int fmt)
 {
     char *buf = gretl_print_steal_buffer(prn);
     char *modbuf = NULL;
+    int rtf_format = 0;
     int err = 0;
 
     if (buf == NULL || *buf == '\0') {
@@ -338,6 +332,10 @@ int prn_to_clipboard (PRN *prn, int fmt)
 	return 1;
     }
 
+    if (fmt == GRETL_FORMAT_RTF || fmt == GRETL_FORMAT_RTF_TXT) {
+	rtf_format = 1;
+    }
+
     EmptyClipboard();
 
     err = maybe_post_process_buffer(buf, fmt, W_COPY, &modbuf);
@@ -353,26 +351,29 @@ int prn_to_clipboard (PRN *prn, int fmt)
 
 	winbuf = modbuf != NULL ? modbuf : buf;
 
-	if (!rtf_format(prn) && !gretl_is_ascii(winbuf)) {
+	if (!rtf_format && !gretl_is_ascii(winbuf)) {
 	    /* for Windows clipboard, recode UTF-8 to UTF-16 */
 	    ubuf = g_utf8_to_utf16(winbuf, -1, NULL, &wrote, NULL);
 	}
 
 	if (ubuf != NULL) {
-	    winbuf = ubuf;
 	    sz = (wrote + 1) * sizeof(gunichar2);
 	} else {
 	    sz = strlen(winbuf) + 1;
 	}
-
-	winclip = GlobalAlloc(GMEM_MOVEABLE, sz);	
+	
+	winclip = GlobalAlloc(GMEM_MOVEABLE, sz);
 	ptr = GlobalLock(winclip);
-	memcpy(ptr, winbuf, sz);
-	GlobalUnlock(winclip);
+	if (ubuf != NULL) {
+	    memcpy(ptr, ubuf, sz);
+	} else {
+	    memcpy(ptr, winbuf, sz);
+	}
+	GlobalUnlock(winclip);    
 
 	if (ubuf != NULL) {
 	    clip_format = CF_UNICODETEXT;
-	} else if (rtf_format(prn)) { 
+	} else if (rtf_format) {
 	    clip_format = RegisterClipboardFormat("Rich Text Format");
 	} else if (fmt == GRETL_FORMAT_CSV) {
 	    clip_format = RegisterClipboardFormat("CSV");
@@ -563,8 +564,9 @@ static int win32_open_arg (const char *arg, char *ext)
 
     if ((ptrcast) ShellExecute(NULL, "open", arg, NULL, NULL, SW_SHOW) <= 32) {
 	/* if the above fails, get the appropriate fileext regkey and 
-	   look up the program */
-	if (GetRegKey(HKEY_CLASSES_ROOT, ext, key) == ERROR_SUCCESS) {
+	   look up the program 
+	*/
+	if (*ext && GetRegKey(HKEY_CLASSES_ROOT, ext, key) == ERROR_SUCCESS) {
 	    lstrcat(key,"\\shell\\open\\command");
 	    if (GetRegKey(HKEY_CLASSES_ROOT, key, key) == ERROR_SUCCESS) {
 		char *p;
@@ -577,10 +579,10 @@ static int win32_open_arg (const char *arg, char *ext)
 			/* if no parameter */
 			p = key + lstrlen(key) - 1;
 		    } else {
-			*p = '\0';    /* remove the param */
+			*p = '\0'; /* remove the param */
 		    }
 		} else {
-		    *p = '\0';        /* remove the param */
+		    *p = '\0'; /* remove the param */
 		}
 
 		lstrcat(p, " ");
@@ -609,8 +611,11 @@ int win32_open_file (const char *fname)
 
     if (has_suffix(fname, ".pdf")) {
 	strcpy(sfx, ".pdf");
-    } else {
+    } else if (has_suffix(fname, ".ps") ||
+	       has_suffix(fname, ".eps")) {
 	strcpy(sfx, ".ps");
+    } else {
+	*sfx = '\0';
     }
 
     err = win32_open_arg(fname, sfx);
@@ -687,58 +692,6 @@ void win32_font_selector (char *fontname, int flag)
     g_free(cf.lpLogFont); 
 }
 
-static BOOL running_as_admin (void)
-{
-    SID_IDENTIFIER_AUTHORITY auth = {SECURITY_NT_AUTHORITY};
-    PSID admin_group = NULL;
-    BOOL ok, ret = FALSE;
-
-    ok = AllocateAndInitializeSid(&auth, 2,
-				  SECURITY_BUILTIN_DOMAIN_RID, 
-				  DOMAIN_ALIAS_RID_ADMINS, 
-				  0, 0, 0, 0, 0, 0, 
-				  &admin_group);
-
-    if (ok) {
-	ok = CheckTokenMembership(NULL, admin_group, &ret);
-    }
-
-    if (!ok) {
-	fprintf(stderr, "running_as_admin: the check failed\n");
-    }
- 
-    if (admin_group != NULL) {
-	FreeSid(admin_group);
-    }
- 
-    return ret;
-}
-
-/* Note: at some point MS will scrap the virtual store
-   apparatus (supposedly), in which case we'll want to
-   add a further condition in this function -- that is,
-   dwMajorVersion < something.
-*/
-
-int windows_uses_virtual_store (void)
-{
-    OSVERSIONINFO osvi;
-    int ret;
-
-    ZeroMemory(&osvi, sizeof osvi);
-    osvi.dwOSVersionInfoSize = sizeof osvi;
-
-    GetVersionEx(&osvi);
-    /* VirtualStore came in with Vista */
-    ret = osvi.dwMajorVersion > 5;
-    
-    if (ret && running_as_admin()) {
-	ret = 0;
-    }
-
-    return ret;
-}
-
 int win32_rename_dir (const char *oldname, const char *newname)
 {
     char *oldtmp = NULL, *newtmp = NULL;
