@@ -26,6 +26,7 @@
 				r == EDIT_OX ||     \
                                 r == EDIT_OCTAVE || \
 				r == EDIT_PYTHON || \
+				r == EDIT_STATA ||  \
 				r == EDIT_X12A)
 
 #define vwin_editing_buffer(r) (r == EDIT_HEADER || \
@@ -105,7 +106,7 @@ void set_reuseable_output_window (int policy, windata_t *vwin);
 
 gchar *gretl_window_title (const char *s);
 
-gchar *title_from_filename (const char *fname, gboolean prepend);
+gchar *title_from_filename (const char *fname, int role, gboolean prepend);
 
 windata_t *console_window (int hsize, int vsize);
 
@@ -128,9 +129,11 @@ void set_model_save_state (windata_t *vwin, gboolean s);
 
 void viewer_set_editable (windata_t *vwin);
 
-int gui_validate_varname (const char *varname, GretlType t);
+int gui_validate_varname_strict (const char *name, GretlType type,
+				 GtkWidget *parent);
 
-int gui_validate_varname_strict (const char *name, GretlType t);
+int gui_validate_varname (const char *varname, GretlType type,
+			  GtkWidget *parent);
 
 gretlopt get_tex_eqn_opt (void);
 
@@ -153,6 +156,8 @@ void *gui_get_plugin_function (const char *funcname);
 
 int get_imported_data (char *fname, int ftype, int append);
 
+char *gretl_basename (char *dest, const char *src, int addscore);
+
 char *double_underscores (char *targ, const char *src);
 
 void verbose_gerror_report (GError *gerr, const char *src);
