@@ -46,7 +46,6 @@ static char *en_cli_helpfile;
 
 static windata_t *real_do_help (int idx, int pos, int role);
 static void en_help_callback (GtkWidget *w, windata_t *hwin);
-static void delete_help_viewer (GtkWidget *w, windata_t *hwin);
 static char *funcs_helpfile (void);
 static void helpwin_set_topic_index (windata_t *hwin, int idx);
 
@@ -74,9 +73,7 @@ enum {
 static GretlToolItem help_tools[] = {
     { N_("Larger"), GTK_STOCK_ZOOM_IN, G_CALLBACK(text_larger), 0},
     { N_("Smaller"), GTK_STOCK_ZOOM_OUT, G_CALLBACK(text_smaller), 0},
-    { N_("Show English help"), GRETL_STOCK_EN, G_CALLBACK(en_help_callback), EN_ITEM },
-    { N_("Windows"), GRETL_STOCK_WINLIST, GNULL, 0 },
-    { N_("Close"), GTK_STOCK_CLOSE, G_CALLBACK(delete_help_viewer), 0 }
+    { N_("Show English help"), GRETL_STOCK_EN, G_CALLBACK(en_help_callback), EN_ITEM }
 };
 
 static gint n_help_tools = G_N_ELEMENTS(help_tools);
@@ -148,6 +145,7 @@ static struct gui_help_item gui_help_items[] = {
     { MENU_ATTACH,    "menu-attach" },
     { REPROBIT,       "reprobit" },
     { DAILY_PURGE,    "daily-purge" },
+    { PKG_FILES,      "data-files" },
     { -1,             NULL },
 };
 
@@ -684,11 +682,6 @@ static void en_help_callback (GtkWidget *w, windata_t *hwin)
     real_do_help(idx, pos, role);
 }
 
-static void delete_help_viewer (GtkWidget *w, windata_t *hwin) 
-{
-    gtk_widget_destroy(hwin->main); 
-}
-
 static void normalize_base (GtkWidget *w, gpointer p)
 {
     gtk_widget_modify_base(w, GTK_STATE_SELECTED, NULL);
@@ -993,18 +986,15 @@ void set_up_helpview_menu (windata_t *hwin)
 	if (!SHOW_EN_BUTTON(hwin->role) && item->flag == EN_ITEM) {
 	    continue;
 	}
-	if (winlist_item(item)) {
-	    vwin_toolbar_insert_winlist(hwin);
-	} else {
-	    gretl_toolbar_insert(hwin->mbar, item, item->func,
-				 hwin, -1);
-	}
+	gretl_toolbar_insert(hwin->mbar, item, item->func, hwin, -1);
     }	
 
     gtk_box_pack_start(GTK_BOX(hwin->vbox), hbox, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(hbox), hwin->mbar, FALSE, FALSE, 0);
 
-     if (SHOW_FINDER(hwin->role)) {
+    vwin_add_winlist(hwin);
+
+    if (SHOW_FINDER(hwin->role)) {
 	vwin_add_finder(hwin);
     }
 
@@ -1701,6 +1691,47 @@ static gboolean real_find_in_listbox (windata_t *vwin,
     return (pos >= 0);
 }
 
+/* Given @targ the name of a function package, try to find
+   it in @vwin's listbox, and if found focus that row.
+   Return TRUE if found, FALSE otherwise.
+*/
+
+gboolean find_package_in_viewer (windata_t *vwin, 
+				 const gchar *targ)
+{
+    char haystack[MAXLEN];
+    GtkTreeModel *model;
+    GtkTreeIter iter;
+    int pos = -1;
+
+    model = gtk_tree_view_get_model(GTK_TREE_VIEW(vwin->listbox));
+
+    if (model == NULL || !gtk_tree_model_get_iter_first(model, &iter)) {
+	return FALSE;
+    }
+
+    while (pos < 0) {
+	get_tree_model_haystack(model, &iter, 0, haystack);
+	pos = string_match_pos(haystack, targ, TRUE, 0);
+	if (pos >= 0 || !gtk_tree_model_iter_next(model, &iter)) {
+	    break;
+	}
+    }
+
+    if (pos >= 0) {
+	GtkTreePath *path = gtk_tree_model_get_path(model, &iter);
+
+	gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(vwin->listbox),
+				     path, NULL, FALSE, 0, 0);
+	gtk_tree_view_set_cursor(GTK_TREE_VIEW(vwin->listbox),
+				 path, NULL, FALSE);
+	vwin->active_var = tree_path_get_row_number(path);
+	gtk_tree_path_free(path);
+    } 
+
+    return (pos >= 0);
+}
+
 /* used for windows that do not have a built-in search entry,
    but which call the function find_string_dialog() */
 
@@ -1882,7 +1913,7 @@ static int get_writable_doc_path (char *path, const char *fname)
     }
 
 #ifdef G_OS_WIN32
-    if (sysdoc_writable < 0 && windows_uses_virtual_store()) {
+    if (sysdoc_writable < 0 && win32_uses_virtual_store()) {
 	/* don't write to virtualized location */
 	sysdoc_writable = 0;
     }
