@@ -53,12 +53,13 @@ int lib_command_strcat (const char *s);
 
 int record_command_verbatim (void);
 
-int parse_lib_command (void);
+int record_model_command_verbatim (int model_ID);
 
-int record_lib_command (void);
+int *command_list_from_string (const char *s, int *err);
 
 int execute_script (const char *runfile, const char *buf,
-		    PRN *prn, int exec_code);
+		    PRN *prn, int exec_code,
+		    GtkWidget *parent);
 
 int user_fopen (const char *fname, char *fullname, PRN **pprn);
 
@@ -82,9 +83,7 @@ void gui_graph_handler (int err);
 
 /* sample-related functions */
 
-int bool_subsample (gretlopt opt);
-
-int do_set_sample (void);
+int bool_subsample (const char *param, gretlopt opt);
 
 void drop_missing_data (void);
 
@@ -214,9 +213,10 @@ void do_variable_setmiss (GtkWidget *w, dialog_t *dlg);
 
 void do_edit_label (GtkWidget *w, dialog_t *dlg);
 
-int do_rename_variable (int v, const char *newname);
+int do_rename_variable (int v, const char *newname,
+			GtkWidget *parent);
 
-int record_varlabel_change (int v);
+int record_varlabel_change (int v, int desc, int gname);
 
 void do_resid_freq (GtkAction *action, gpointer p);
 
@@ -256,6 +256,8 @@ void do_remove_obs (void);
 
 void add_logs_etc (int ci, int varnum);
 
+void add_discrete_dummies (int varnum);
+
 void logs_etc_callback (GtkAction *action);
 
 void add_system_resid (GtkAction *action, gpointer p);
@@ -279,6 +281,8 @@ void do_graph_var (int varnum);
 
 void do_boxplot_var (int varnum, gretlopt opt);
 
+int do_regular_boxplot (selector *sr);
+
 int do_factorized_boxplot (selector *sr);
 
 void ts_plot_callback (void);
@@ -291,8 +295,6 @@ int do_splot_from_selector (selector *sr);
 
 void plot_from_selection (int code);
 
-void do_box_graph (GtkWidget *w, dialog_t *dlg);
-
 int do_dummy_graph (selector *sr);
 
 int do_xyz_graph (selector *sr);
@@ -313,7 +315,7 @@ gboolean do_open_script (int action);
 
 void dataset_info (void);
 
-void do_new_script (int code);
+void do_new_script (int code, const char *buf);
 
 void new_script_callback (GtkAction *action);
 
@@ -325,7 +327,8 @@ gboolean get_csv_exclude_obs (void);
 
 void do_save_text (char *fname, MODEL *pmod);
 
-int gui_exec_line (ExecState *s, DATASET *dset);
+int gui_exec_line (ExecState *s, DATASET *dset,
+		   GtkWidget *parent);
 
 int exec_line_with_output_handler (ExecState *s, 
 				   DATASET *dset,
@@ -334,7 +337,7 @@ int exec_line_with_output_handler (ExecState *s,
 
 void finalize_script_output_window (int role, gpointer data);
 
-void start_wait_for_output (GtkWidget *w);
+void start_wait_for_output (windata_t *vwin, GtkWidget *w);
 
 int waiting_for_output (void);
 
