@@ -267,6 +267,7 @@ void ts_or_panel_menu_state (gboolean s)
     flip(mdata->ui, "/menubar/Add/lags", s);
     flip(mdata->ui, "/menubar/Add/diff", s);
     flip(mdata->ui, "/menubar/Add/ldiff", s);
+    flip(mdata->ui, "/menubar/Add/pcdiff", s);
 
     s = dataset_is_seasonal(dataset);
     flip(mdata->ui, "/menubar/Add/sdiff", s);
@@ -311,6 +312,14 @@ void compact_data_state (gboolean s)
     }
 }
 
+void single_var_menu_state (int nsel)
+{
+    gboolean s = dataset_is_time_series(dataset) ||
+	dataset_is_panel(dataset);
+	
+    flip(mdata->ui, "/menubar/Add/pcdiff", s && nsel == 1);
+}
+
 void main_menus_enable (gboolean s)
 {
     if (mdata->ui != NULL) {
@@ -380,6 +389,10 @@ static gint var_popup_click (GtkWidget *w, gpointer p)
 	add_logs_etc(LOGS, v);
     } else if (!strcmp(item, _("Add difference"))) {
 	add_logs_etc(DIFF, v);
+    } else if (!strcmp(item, _("Add percent change..."))) {
+	percent_change_dialog(v);
+    } else if (!strcmp(item, _("Dummify..."))) {
+	add_discrete_dummies(v);
     } else if (!strcmp(item, _("Define new variable..."))) { 
 	genr_callback();
     }
@@ -493,7 +506,18 @@ static gint selection_popup_click (GtkWidget *w, gpointer p)
     return FALSE;
 }
 
-GtkWidget *build_var_popup (void)
+static int series_is_dummifiable (int v)
+{
+    if (!series_is_discrete(dataset, v)) {
+	return 0;
+    } else if (gretl_isdummy(0, dataset->n-1, dataset->Z[v])) {
+	return 0;
+    } else {
+	return 1;
+    }
+}
+
+GtkWidget *build_var_popup (int selvar)
 {
     const char *items[] = {
 	N_("Display values"),
@@ -511,6 +535,9 @@ GtkWidget *build_var_popup (void)
 	NULL,
 	N_("Add log"),
 	N_("Add difference"),
+	N_("Add percent change..."),
+	N_("Dummify..."),
+	NULL,
 	N_("Define new variable...")
     };
     GtkWidget *menu;
@@ -536,15 +563,21 @@ GtkWidget *build_var_popup (void)
 	    continue;
 	}	
 	if ((i == 6 || i == 7) && !dataset_is_time_series(dataset)) {
+	    /* correlogram, periodogram */ 
 	    continue;
 	}
-	if ((i == 2 || i == 14) && !extended_ts(dataset)) {
+	if ((i == 2 || i == 14 || i == 15) && !extended_ts(dataset)) {
+	    /* time-series plot, difference, percent change */
 	    continue;
 	}
 	if (i == 5 && dataset_is_time_series(dataset)) {
 	    /* skip boxplot option */
 	    continue;
 	}
+	if (i == 16 && !series_is_dummifiable(selvar)) {
+	    /* skip dummify option */
+	    continue;
+	}	
 	item = gtk_menu_item_new_with_label(_(items[i]));
 	g_signal_connect(G_OBJECT(item), "activate",
 			 G_CALLBACK(var_popup_click),
