@@ -30,11 +30,12 @@ static MODEL **table_models;
 static int n_models;
 static char **pnames;
 static int n_params;
-static int use_tstats;
-static int do_pvals;
-static int colheads;
 static int depvarnum;
 
+/* may be governed by, and saved in, rc file */
+static int colheads;
+static int use_tstats;
+static int do_pvals;
 static int do_asts = 1;
 static int mt_figs = 4;   /* figures for printing */
 static char mt_fmt = 'g'; /* floating-point format ('g' or 'f') */
@@ -191,8 +192,7 @@ static int model_table_precheck (MODEL *pmod, int add_mode)
     /* various sorts of models that will not work */
     if (pmod->ci == NLS || pmod->ci == MLE || pmod->ci == GMM ||
 	pmod->ci == ARBOND || pmod->ci == DPANEL ||
-	pmod->ci == INTREG || pmod->ci == IVREG ||
-	pmod->ci == BIPROBIT) {
+	pmod->ci == INTREG || pmod->ci == BIPROBIT) {
 	mtable_errmsg(_("Sorry, this model can't be put in the model table"),
 		      gui);
 	return 1;
@@ -1080,6 +1080,13 @@ int display_model_table (int gui)
 	return 1;
     }
 
+    get_model_table_prefs(&colheads,
+			  &use_tstats,
+			  &do_pvals,
+			  &do_asts,
+			  &mt_figs,
+			  &mt_fmt);    
+
     plain_print_model_table(prn);
 
     if (real_table_n_models() > 5) {
@@ -1272,6 +1279,13 @@ int special_print_model_table (PRN *prn)
 {
     set_alt_gettext_mode(prn);
 
+    get_model_table_prefs(&colheads,
+			  &use_tstats,
+			  &do_pvals,
+			  &do_asts,
+			  &mt_figs,
+			  &mt_fmt);    
+
     if (tex_format(prn)) {
 	return tex_print_model_table(prn);
     } else if (rtf_format(prn)) {
@@ -1375,19 +1389,16 @@ static int print_model_table_direct (const char *fname,
     return err;
 }
 
-int modeltab_parse_line (const char *line, gretlopt opt, PRN *prn)
+int modeltab_exec (const char *param, gretlopt opt, PRN *prn)
 {
-    char cmdword[16];
-    int gotcmd = 0;
     int err = 0;
 
-    if (sscanf(line, "%*s %15s", cmdword) == 1) {
-	gotcmd = 1;
-    }
-
-    if (gotcmd && (opt & OPT_O)) {
+    if ((param != NULL && (opt & OPT_O)) ||
+	(param == NULL && !(opt & OPT_O))) {
 	/* the --output option rules out the various
-	   command words */
+	   command params; otherwise a param value is
+	   needed
+	*/
 	return E_PARSE;    
     } else if (opt & OPT_O) {
 	/* --output="filename" */
@@ -1399,11 +1410,11 @@ int modeltab_parse_line (const char *line, gretlopt opt, PRN *prn)
 	} else {
 	    err = print_model_table_direct(outfile, opt, prn);
 	}
-    } else if (!strcmp(cmdword, "add")) {
+    } else if (!strcmp(param, "add")) {
 	err = cli_modeltab_add(prn);
-    } else if (!strcmp(cmdword, "show")) {
+    } else if (!strcmp(param, "show")) {
 	err = display_model_table(0);
-    } else if (!strcmp(cmdword, "free")) {
+    } else if (!strcmp(param, "free")) {
 	if (!model_table_is_empty()) {
 	    clear_model_table(0, prn);
 	}
@@ -1416,14 +1427,26 @@ int modeltab_parse_line (const char *line, gretlopt opt, PRN *prn)
 
 void format_model_table (windata_t *vwin)
 {
-    int colhead_opt = colheads;
-    int se_opt = use_tstats;
-    int pv_opt = do_pvals;
-    int ast_opt = do_asts;
-    int figs = mt_figs;
-    char fmt = mt_fmt;
+    int colhead_opt;
+    int se_opt, pv_opt;
+    int ast_opt, figs;
+    char fmt;
     int resp;
 
+    get_model_table_prefs(&colheads,
+			  &use_tstats,
+			  &do_pvals,
+			  &do_asts,
+			  &figs,
+			  &fmt);
+
+    colhead_opt = colheads;
+    se_opt = use_tstats;
+    pv_opt = do_pvals;
+    ast_opt = do_asts;
+    figs = mt_figs;
+    fmt = mt_fmt;    
+
     resp = model_table_dialog(&colhead_opt, &se_opt, &pv_opt, &ast_opt,
 			      &figs, &fmt, vwin->main);
 
@@ -1447,6 +1470,13 @@ void format_model_table (windata_t *vwin)
 	mt_figs = figs;
 	mt_fmt = fmt;
 
+	set_model_table_prefs(colheads,
+			      use_tstats,
+			      do_pvals,
+			      do_asts,
+			      mt_figs,
+			      mt_fmt);	
+
 	if (bufopen(&prn)) {
 	    return;
 	}
