@@ -593,10 +593,10 @@ static SESSION_TEXT *get_session_text_by_name (const char *name)
     return NULL;
 }
 
-int real_add_text_to_session (PRN *prn, const char *tname)
+int real_add_text_to_session (PRN *prn, int pos, const char *tname)
 {
     SESSION_TEXT *text = get_session_text_by_name(tname);
-    char *buf = gretl_print_get_chunk(prn);
+    char *buf = gretl_print_get_chunk_at(prn, pos);
     int replace = 0;
 
     if (buf == NULL) {
@@ -1009,6 +1009,37 @@ static void delete_icon_for_data (void *data)
     }
 }
 
+static int show_model_in_window (void *ptr, GretlObjType type)
+{ 
+    char *name = NULL;
+    PRN *prn;
+
+    if (type != GRETL_OBJ_EQN && type != GRETL_OBJ_VAR) {
+	return 1;
+    }
+
+    if (bufopen(&prn)) {
+	return 1;
+    }
+
+    gretl_object_compose_unique_name(ptr, type);
+    name = gretl_object_get_name(ptr, type);
+
+    if (type == GRETL_OBJ_EQN) {
+	MODEL *pmod = (MODEL *) ptr;
+
+	printmodel(pmod, dataset, OPT_NONE, prn);
+	view_model(prn, pmod, name);
+    } else if (type == GRETL_OBJ_VAR) {
+	GRETL_VAR *var = (GRETL_VAR *) ptr;
+
+	gretl_VAR_print(var, dataset, OPT_NONE, prn);
+	view_buffer(prn, 78, 450, name, var->ci, var);
+    }
+
+    return 0;
+}
+
 /* Callback (indirect) from libgretl for a model created via 
    script command. When this function is called, the model
    in question has already been stacked; it is just a matter
@@ -1021,8 +1052,13 @@ int add_model_to_session_callback (void *ptr, GretlObjType type)
     char *name = gretl_object_get_name(ptr, type);
     int ret = 0;
 
+    if (name == NULL || *name == '\0') {
+	/* we got the --window callback */
+	return show_model_in_window(ptr, type);
+    }
+
     /* are we replacing a session model's content? */
-    targ = get_session_model_by_name(name); 
+    targ = get_session_model_by_name(name);
 
     if (targ != NULL) {
 	windata_t *vwin = get_viewer_for_data(targ->ptr);
@@ -1286,7 +1322,6 @@ gboolean do_open_session (void)
     char gdtname[MAXLEN]; /* path to session data file */
     char fname[MAXLEN];   /* multi-purpose temp variable */
     gchar *zdirname = NULL;
-    GError *gerr = NULL;
     FILE *fp;
     int nodata = 0;
     int err = 0;
@@ -1313,20 +1348,10 @@ gboolean do_open_session (void)
     maybe_absolutize_tryfile();
     
     gretl_chdir(gretl_dotdir());
-    err = gretl_unzip_session_file(tryfile, &zdirname, &gerr);
-
-    if (gerr != NULL && err == 0) {
-	err = 1;
-    } 
+    err = gretl_unzip_session_file(tryfile, &zdirname);
 
     if (err) {
-	if (gerr != NULL) {
-	    errbox(gerr->message);
-	    g_error_free(gerr);
-	} else {
-	    /* FIXME more explicit error message */
-	    file_read_errbox(tryfile);
-	}
+	gui_errmsg(err);
 	g_free(zdirname);
 	goto bailout;
     } 
@@ -1407,8 +1432,7 @@ gboolean do_open_session (void)
 	if (sinfo.resample_n > 0) {
 	    err = dataset_resample(dataset, sinfo.resample_n, sinfo.seed); 
 	} else if (sinfo.mask != NULL) {
-	    err = restrict_sample_from_mask(sinfo.mask, dataset,
-					    OPT_NONE, NULL);
+	    err = restrict_sample_from_mask(sinfo.mask, dataset, OPT_NONE);
 	    if (!err) {
 		dataset->restriction = sinfo.restriction;
 		sinfo.restriction = NULL;
@@ -1465,10 +1489,10 @@ void verify_clear_data (void)
 	return;
     }
 
-    if (yes_no_dialog ("gretl",                      
-		       _("Clearing the data set will end\n"
-			 "your current session.  Continue?"), 
-		       0) != GRETL_YES) {
+    if (yes_no_dialog("gretl",                      
+		      _("Clearing the data set will end\n"
+			"your current session.  Continue?"), 
+		      NULL) != GRETL_YES) {
 	return;
     }
 
@@ -1744,7 +1768,7 @@ static int real_save_session_dataset (const char *dname)
 	/* reset the prior subsample */
 	if (!err) {
 	    err = restrict_sample_from_mask(mask, dataset, 
-					    OPT_NONE, NULL);
+					    OPT_NONE);
 	}
 	free(mask);
     }
@@ -1835,7 +1859,6 @@ int save_session (char *fname)
     char dirname[MAXLEN];
     int len, err = 0;
     int log_code = LOG_SAVE;
-    GError *gerr = NULL;
 
     if (fname == NULL) {
 	/* re-saving session 'as is' */
@@ -1926,18 +1949,11 @@ int save_session (char *fname)
     
     if (!err) {
 	/* make zipfile containing session files */
-	err = gretl_make_zipfile(fname, dirname, &gerr);
-	fprintf(stderr, " gretl_make_zipfile: err = %d\n", err);
-
-	if (gerr != NULL && !err) {
-	    err = 1;
-	}
+	err = gretl_make_zipfile(fname, dirname);
 
 	if (err) {
-	    if (gerr != NULL) {
-		errbox(gerr->message);
-		g_error_free(gerr);
-	    }
+	    fprintf(stderr, " gretl_make_zipfile: err = %d\n", err);
+	    gui_errmsg(err);
 	} else {
 	    mkfilelist(FILE_LIST_SESSION, fname);
 	    if (fname != sessionfile) {
@@ -2390,12 +2406,12 @@ static void maybe_delete_session_object (gui_obj *obj)
     }
 
     if (busy) {
-	warnbox(_("%s: please close this object's window first"),
-		obj->name);
+	warnbox_printf(_("%s: please close this object's window first"),
+		       obj->name);
     } else {	    
 	gchar *msg = g_strdup_printf(_("Really delete %s?"), obj->name);
 
-	if (yes_no_dialog(_("gretl: delete"), msg, 0) == GRETL_YES) {
+	if (yes_no_dialog(_("gretl: delete"), msg, iconview) == GRETL_YES) {
 	    delete_session_object(obj);
 	}
 	g_free(msg);
@@ -2542,7 +2558,8 @@ static int rename_session_object (gui_obj *obj, const char *newname)
     }
 
     if (err) {
-	errbox(_("'%s': there is already an object of this name"), newname);
+	errbox_printf(_("'%s': there is already an object of this name"), 
+		      newname);
     } else {
 	free(obj->name);
 	obj->name = g_strdup(newname);
@@ -2567,7 +2584,8 @@ static int copy_session_object (gui_obj *obj, const char *cpyname)
     } 
 
     if (oldp != NULL) {
-	errbox(_("'%s': there is already an object of this name"), cpyname);
+	errbox_printf(_("'%s': there is already an object of this name"), 
+		      cpyname);
     } else if (obj->sort == GRETL_OBJ_GRAPH || obj->sort == GRETL_OBJ_PLOT) {
 	SESSION_GRAPH *g0 = obj->data;
 
@@ -2898,7 +2916,8 @@ static int gui_user_var_callback (const char *name, GretlType type,
 	    if (u != NULL) {
 		session_add_icon(u, otype, ICON_ADD_SINGLE);
 	    }
-	} else if (autoicon_on()) {
+	} else if (type != GRETL_TYPE_MATRIX && autoicon_on()) {
+	    /* auto-open icon view unless we added a matrix */
 	    auto_view_session();
 	}
 	if (iconview != NULL && waiting_for_output()) {
