@@ -104,6 +104,7 @@ PangoFontDescription *fixed_font;
 static int usecwd;
 static int shellok;
 static int manpref;
+static int robust_z;
 static int autoicon = 1;
 static int session_prompt = 1;
 static int keep_folder = 1;
@@ -144,6 +145,14 @@ static char themepref[12] = "Clearlooks";
 char midiplayer[MAXSTR] = "timidity -ig";
 #endif
 
+/* model table display variables */
+static int modtab_colheads;
+static gboolean modtab_tstats;
+static gboolean modtab_pvalues;
+static gboolean modtab_asterisks = TRUE;
+static int modtab_digits = 4;
+static gboolean modtab_decimals;
+
 typedef enum {
     USERSET  = 1 << 0,  /* user-level variable */
     BOOLSET  = 1 << 1,  /* boolean value (user) */
@@ -212,8 +221,6 @@ RCVAR rc_vars[] = {
       BOOLSET, 0, TAB_MAIN, NULL },
     { "session_prompt", N_("Prompt to save session"), NULL, &session_prompt, 
       BOOLSET, 0, TAB_MAIN, NULL },
-    { "oxsupport", N_("Enable Ox support"), NULL, &ox_support, 
-      BOOLSET | RESTART, 0, TAB_MAIN, NULL },
     { "usecwd", N_("Set working directory from shell"), NULL, &usecwd, 
       INVISET | BOOLSET | RESTART, 0, TAB_NONE, NULL },
     { "keepfolder", N_("File selector remembers folder"), NULL, &keep_folder, 
@@ -314,6 +321,8 @@ RCVAR rc_vars[] = {
       INVISET | INTSET, 0, TAB_NONE, NULL },
     { "HC_by_default", N_("Use robust covariance matrix by default"), NULL,
       &hc_by_default, BOOLSET, 0, TAB_VCV, NULL },
+    { "robust_z", N_("Use the normal distribution for robust p-values"), NULL,
+      &robust_z, BOOLSET, 0, TAB_VCV, NULL },
     { "HC_xsect", N_("For cross-sectional data"), NULL, hc_xsect, 
       LISTSET, 5, TAB_VCV, NULL },
     { "HC_tseri", N_("For time-series data"), NULL, hc_tseri, 
@@ -324,6 +333,18 @@ RCVAR rc_vars[] = {
       LISTSET, 5, TAB_VCV, NULL },
     { "manpref", N_("PDF manual preference"), NULL, &manpref, 
       RADIOSET | INTSET, 0, TAB_MAN, NULL },
+    { "modtab_colheads", "Model table column heads", NULL, &modtab_colheads, 
+      INVISET | INTSET, 0, TAB_NONE, NULL },
+    { "modtab_tstats", "Model table t-ratios", NULL, &modtab_tstats, 
+      INVISET | BOOLSET, 0, TAB_NONE, NULL },
+    { "modtab_pvalues", "Model table p-values", NULL, &modtab_pvalues, 
+      INVISET | BOOLSET, 0, TAB_NONE, NULL },
+    { "modtab_asterisks", "Model table asterisks", NULL, &modtab_asterisks, 
+      INVISET | BOOLSET, 0, TAB_NONE, NULL },
+    { "modtab_digits", "Model table digits", NULL, &modtab_digits, 
+      INVISET | INTSET, 0, TAB_NONE, NULL },
+    { "modtab_decimals", "Model table decimal places", NULL, &modtab_decimals, 
+      INVISET | INTSET, 0, TAB_NONE, NULL },
     { NULL, NULL, NULL, NULL, 0, 0, TAB_NONE, NULL }
 };
 
@@ -401,9 +422,10 @@ void set_script_output_policy (int p, windata_t *vwin)
 {
     script_output_policy = p;
 
-    if (script_output_policy < 0 || 
-	script_output_policy > OUTPUT_APPEND) {
-	script_output_policy = OUTPUT_NEW_WINDOW;
+    if (script_output_policy < 0 ||
+	script_output_policy > OUTPUT_POLICY_NEW_WINDOW) {
+	/* invalid setting */
+	script_output_policy = OUTPUT_POLICY_REPLACE;
     }
 
     set_reuseable_output_window(p, vwin);
@@ -428,6 +450,36 @@ int get_use_wimp (void)
 
 #endif
 
+void get_model_table_prefs (int *colheads,
+			    int *use_tstats,
+			    int *do_pvals,
+			    int *do_asts,
+			    int *figs,
+			    char *fmt)
+{
+    *colheads   = modtab_colheads;
+    *use_tstats = modtab_tstats;
+    *do_pvals   = modtab_pvalues;
+    *do_asts    = modtab_asterisks;
+    *figs       = modtab_digits;
+    *fmt        = modtab_decimals ? 'f' : 'g';
+}    
+
+void set_model_table_prefs (int colheads,
+			    int use_tstats,
+			    int do_pvals,
+			    int do_asts,
+			    int figs,
+			    char fmt)
+{
+    modtab_colheads  = colheads;
+    modtab_tstats    = use_tstats;
+    modtab_pvalues   = do_pvals;
+    modtab_asterisks = do_asts;
+    modtab_digits    = figs;
+    modtab_decimals  = (fmt == 'f')? 1 : 0;
+}
+
 static gretlopt update_paths_opt = OPT_NONE;
 
 void force_english_help (void)
@@ -438,15 +490,18 @@ void force_english_help (void)
 
 void set_fixed_font (const char *fontname)
 {
-    if (fixed_font != NULL) {
-	pango_font_description_free(fixed_font);
-    }
-    
-    if (fontname != NULL && fontname != fixedfontname) {
+    if (fontname == NULL) {
+	/* initial set-up */
+	fixed_font = pango_font_description_from_string(fixedfontname);
+    } else if (strcmp(fontname, fixedfontname)) {
+	/* changed via the GUI */
+	if (fixed_font != NULL) {
+	    pango_font_description_free(fixed_font);
+	}
 	strcpy(fixedfontname, fontname);
+	fixed_font = pango_font_description_from_string(fixedfontname);
+	infobox(_("This change will apply to newly opened windows"));
     }
-
-    fixed_font = pango_font_description_from_string(fixedfontname);
 }
 
 #ifndef G_OS_WIN32
@@ -571,11 +626,11 @@ static int write_OK (gchar *dirname)
     int ok = 0;
 
     if (gretl_mkdir(dirname) == 0) {
-	gchar  *target = g_strdup_printf("%s%c%s", dirname, SLASH, "wtest");
+	gchar *test = g_strdup_printf("%s%c%s", dirname, SLASH, "wtest");
 
-	if (target != NULL) {
-	    ok = (gretl_test_fopen(target, "w") == 0);
-	    g_free(target);
+	if (test != NULL) {
+	    ok = (gretl_test_fopen(test, "w") == 0);
+	    g_free(test);
 	}
     }
 
@@ -628,7 +683,7 @@ static void get_pkg_save_dir (char *dirname, int action)
     }
 
 #ifdef G_OS_WIN32
-    if (windows_uses_virtual_store()) {
+    if (win32_uses_virtual_store()) {
 	/* don't write to virtualized location */
 	try_sysdir = 0;
     }
@@ -751,7 +806,7 @@ static void set_tramo_status (void)
     }
 
     if (tramo_ok && !ok && gui_up) {
-	warnbox("Invalid path for %s", "TRAMO");
+	warnbox_printf("Invalid path for %s", "TRAMO");
     }
 
     tramo_ok = ok;
@@ -794,7 +849,7 @@ static void set_x12a_status (void)
     }
 
     if (x12a_ok && !ok && gui_up) {
-	warnbox("Invalid path for %s", "X-12-ARIMA");
+	warnbox_printf("Invalid path for %s", "X-12-ARIMA");
     }
 
     x12a_ok = ok;
@@ -816,7 +871,7 @@ static void root_check (void)
 
 	resp = yes_no_dialog("gretl", _("You seem to be running gretl " 
 					"as root.  Do you really want to do this?"), 
-			     0);
+			     NULL);
 	if (resp == GRETL_NO) {
 	    exit(EXIT_FAILURE);
 	}
@@ -1274,11 +1329,6 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	    continue;
 	}
 
-	if (!ox_support && rc->var == paths.oxlpath) {
-	    /* don't show ox path entry if support is not enabled */
-	    continue;
-	}
-
 	if ((rc->flags & BOOLSET) && rc->link == NULL) { 
 	    /* simple boolean variable (check box) */
 	    int rcval = *(int *) (rc->var);
@@ -1305,7 +1355,10 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	    gtk_widget_show(rc->widget);
 	    b_col++;
 
-	    if (b_col == 2) {
+	    if (tab == TAB_VCV || b_col == 2) {
+		/* boolean strings under TAB_VCV are too long to
+		   appear column-wise 
+		*/
 		b_col = 0;
 		b_len++;
 		gtk_table_resize(GTK_TABLE(b_table), b_len + 1, 2);
@@ -1729,6 +1782,7 @@ static void apply_changes (GtkWidget *widget, gpointer data)
        the "libset" apparatus 
     */
     libset_set_bool(SHELL_OK, shellok);
+    libset_set_bool(ROBUST_Z, robust_z);
     set_xsect_hccme(hc_xsect);
     set_tseries_hccme(hc_tseri);
     set_panel_hccme(hc_panel);
@@ -1880,6 +1934,7 @@ static int common_read_rc_setup (void)
 
     libset_set_bool(SHELL_OK, shellok);
     libset_set_bool(USE_CWD, usecwd);
+    libset_set_bool(ROBUST_Z, robust_z);
     set_gp_colors();
     set_gp_scale();
 
@@ -2086,6 +2141,8 @@ int read_win32_config (int debug)
 
     if (chinese_locale()) {
 	strcpy(fixedfontname, "NSimSun 10");
+    } else if (japanese_locale()) {
+	strcpy(fixedfontname, "MS Gothic 10");
     }
 
     appdata = appdata_path();
@@ -2297,6 +2354,8 @@ static void font_selection_ok (GtkWidget *w, GtkFontChooser *fc)
 {
     gchar *fontname = gtk_font_chooser_get_font(fc);
 
+    gtk_widget_hide(GTK_WIDGET(fc));
+
     if (fontname != NULL && *fontname != '\0') {
 	int mono = widget_get_int(fc, "mono");
 
@@ -2386,6 +2445,7 @@ static void font_selection_ok (GtkWidget *w, GtkFontselHackDialog *fs)
     gchar *fontname;
 
     fontname = gtk_fontsel_hack_dialog_get_font_name(fs);
+    gtk_widget_hide(GTK_WIDGET(fs));
 
     if (fontname != NULL && *fontname != '\0') {
 	int filter = gtk_fontsel_hack_dialog_get_filter(fs); 
