@@ -31,7 +31,7 @@
 struct tabwin_t_ {
     int role;             /* what's tabwin doing? */ 
     GtkWidget *main;      /* top-level GTK window */
-    GtkWidget *mbox;      /* horizontal box to hold menu bar */
+    GtkWidget *hbox;      /* horizontal box to hold menu bar */
     GtkWidget *mbar;      /* menu bar */
     GtkWidget *tabs;      /* notebook for tabs */
     GtkWidget *dialog;    /* associated dialog */
@@ -65,25 +65,18 @@ static tabwin_t *vwin_get_tabwin (windata_t *vwin)
 }
 
 static gboolean maybe_block_tabedit_quit (tabwin_t *tabwin,
-					  GtkWindow *topwin)
+					  GtkWidget *parent)
 {
     GtkNotebook *notebook = GTK_NOTEBOOK(tabwin->tabs);
     int np = gtk_notebook_get_n_pages(notebook);
     gboolean ret = FALSE;
 
-    if (tabwin->role != EDIT_SCRIPT) {
-	return FALSE;
-    }
-
     if (np > 1) {
 	gchar *msg = g_strdup_printf(_("Editing %d scripts: really quit?"), np);
 	gint resp;
 
-	if (topwin != NULL) {
-	    gtk_window_present(topwin);
-	}
-
-	resp = yes_no_dialog(_("gretl: script editor"), msg, 0);
+	gtk_window_present(GTK_WINDOW(parent));
+	resp = yes_no_dialog(_("gretl: script editor"), msg, parent);
 	if (resp != GRETL_YES) {
 	    ret = TRUE;
 	}
@@ -95,9 +88,7 @@ static gboolean maybe_block_tabedit_quit (tabwin_t *tabwin,
 	    windata_t *vwin = g_object_get_data(G_OBJECT(page), "vwin");
     
 	    if (vwin_content_changed(vwin)) {
-		if (topwin != NULL) {
-		    gtk_window_present(topwin);
-		}
+		gtk_window_present(GTK_WINDOW(parent));
 		ret = query_save_text(NULL, NULL, vwin);
 	    }
 	}
@@ -106,32 +97,30 @@ static gboolean maybe_block_tabedit_quit (tabwin_t *tabwin,
     return ret;
 }
 
-/* called on program exit */
+/* called from winstack.c on program exit: @w will
+   be the top-level of a tabbed window */
 
 gboolean tabwin_exit_check (GtkWidget *w)
 {
     tabwin_t *tabwin = g_object_get_data(G_OBJECT(w), "tabwin");
 
-    return maybe_block_tabedit_quit(tabwin, GTK_WINDOW(w));
+    if (tabwin->role != EDIT_SCRIPT) {
+	return FALSE;
+    }    
+
+    return maybe_block_tabedit_quit(tabwin, w);
 }
 
-/* called on delete-event */
+/* called on delete-event: @w is the top-level */
 
 static gboolean tabedit_quit_check (GtkWidget *w, GdkEvent *event, 
 				    tabwin_t *tabwin)
 {
-    return maybe_block_tabedit_quit(tabwin, NULL);
-}
-
-/* called by top-level toolbar closer button */
-
-void maybe_destroy_tabwin (windata_t *vwin)
-{
-    tabwin_t *tabwin = vwin_get_tabwin(vwin);
-
-    if (!maybe_block_tabedit_quit(tabwin, NULL)) {
-	gtk_widget_destroy(vwin->topmain);
+    if (tabwin->role != EDIT_SCRIPT) {
+	return FALSE;
     }
+    
+    return maybe_block_tabedit_quit(tabwin, w);
 }
 
 /* activate or de-activate a tab's closer button */
@@ -155,9 +144,10 @@ static void viewer_tab_show_closer (GtkNotebook *notebook,
 static void tabwin_remove_toolbar (tabwin_t *tabwin)
 {
 #if TDEBUG
-    fprintf(stderr, "*** removing toolbar at %p\n", (void *) tabwin->mbar);
+    fprintf(stderr, "*** tabwin_remove_toolbar: removing toolbar at %p\n", 
+	    (void *) tabwin->mbar);
 #endif
-    gtk_container_remove(GTK_CONTAINER(tabwin->mbox), 
+    gtk_container_remove(GTK_CONTAINER(tabwin->hbox), 
 			 tabwin->mbar);
     tabwin->mbar = NULL;
 }
@@ -165,9 +155,10 @@ static void tabwin_remove_toolbar (tabwin_t *tabwin)
 static void tabwin_insert_toolbar (tabwin_t *tabwin, windata_t *vwin)
 {
 #if TDEBUG
-    fprintf(stderr, "*** inserting toolbar at %p\n", (void *) vwin->mbar);
+    fprintf(stderr, "*** tabwin_insert_toolbar: inserting toolbar at %p\n", 
+	    (void *) vwin->mbar);
 #endif
-    gtk_box_pack_start(GTK_BOX(tabwin->mbox), vwin->mbar, 
+    gtk_box_pack_start(GTK_BOX(tabwin->hbox), vwin->mbar, 
 		       TRUE, TRUE, 0);
     gtk_widget_show_all(vwin->mbar);
     tabwin->mbar = vwin->mbar;
@@ -245,6 +236,14 @@ static void tabwin_tab_close (GtkWidget *w, windata_t *vwin)
 	return;
     }
 
+    if (tabwin->role == EDIT_SCRIPT && vwin_content_changed(vwin)) {
+	gint cancel = query_save_text(NULL, NULL, vwin);
+
+	if (cancel) {
+	    return;
+	}
+    }
+
     /* note: vwin->mbar is packed under tabwin, so it will not
        get destroyed automatically when the page is removed
     */
@@ -421,7 +420,7 @@ static GtkWidget *make_viewer_tab (tabwin_t *tabwin,
 	if (strstr(info, "script_tmp") != NULL) {
 	    title = untitled_title(tabwin);
 	} else {
-	    title = title_from_filename(info, FALSE);
+	    title = title_from_filename(info, EDIT_SCRIPT, FALSE);
 	}
     } else if (info != NULL) {
 	title = g_strdup(info);
@@ -553,8 +552,8 @@ static tabwin_t *make_tabbed_viewer (int role)
     gtk_container_add(GTK_CONTAINER(tabwin->main), vbox);
 
     /* box to hold menu bar */
-    tabwin->mbox = gtk_hbox_new(FALSE, 0);
-    gtk_box_pack_start(GTK_BOX(vbox), tabwin->mbox, FALSE, FALSE, 0);
+    tabwin->hbox = gtk_hbox_new(FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), tabwin->hbox, FALSE, FALSE, 0);
     tabwin->mbar = NULL;
 
     /* notebook with its signal handlers */
@@ -675,7 +674,7 @@ void tabwin_register_toolbar (windata_t *vwin)
 		      GUINT_TO_POINTER(handler_id));
 
 #if TDEBUG
-    fprintf(stderr, "*** register_toolbar: vwin=%p has toolbar=%p\n",
+    fprintf(stderr, "*** tabwin_register_toolbar: vwin=%p has toolbar=%p\n",
 	    (void *) vwin, (void *) vwin->mbar);
 #endif
 
@@ -826,7 +825,7 @@ static gchar *title_from_vwin (windata_t *vwin)
 
 	return g_strdup_printf(_("gretl: model %d"), pmod->ID);
     } else {
-	return title_from_filename(vwin->fname, TRUE);
+	return title_from_filename(vwin->fname, vwin->role, TRUE);
     }
 }
 
@@ -917,7 +916,7 @@ void undock_tabbed_viewer (GtkWidget *w, windata_t *vwin)
 
     /* add box for toolbar, pack it, drop extra ref., then
        remove the "tabbed" flag (note that the tabbed flag
-       is wanted so that vwin_pack_toolbar() will put the
+       is wanted so that vwin_pack_toolbar will put the
        toolbar up top)
     */
     vwin_pack_toolbar(vwin);
@@ -1019,6 +1018,7 @@ static void dock_viewer (GtkWidget *w, windata_t *vwin)
     g_object_ref(vwin->mbar);
     box = gtk_widget_get_parent(vwin->mbar);
     gtk_container_remove(GTK_CONTAINER(box), vwin->mbar);
+    gtk_widget_destroy(box);
 
     /* create new vwin->main, etc. */
     vwin->main = gtk_hbox_new(FALSE, 0);
