@@ -56,7 +56,7 @@ enum {
 #define gui_help(r) (r == GUI_HELP || r == GUI_HELP_EN)
 #define foreign_script_role(r) (r == EDIT_GP || r == EDIT_R || \
 				r == EDIT_OX || r == EDIT_OCTAVE || \
-				r == EDIT_PYTHON)
+				r == EDIT_PYTHON || r == EDIT_STATA)
 
 /* globals accessed in settings.c */
 int tabwidth = 4;
@@ -68,6 +68,10 @@ static gboolean script_tab_handler (windata_t *vwin, GdkModifierType mods);
 static gboolean 
 script_popup_handler (GtkWidget *w, GdkEventButton *event, gpointer p);
 static gchar *textview_get_current_line_with_newline (GtkWidget *view);
+static void
+insert_text_with_markup (GtkTextBuffer *tbuf, GtkTextIter *iter,
+			 const char *s, int role);
+static void connect_link_signals (windata_t *vwin);
 
 void text_set_cursor (GtkWidget *w, GdkCursorType cspec)
 {
@@ -423,6 +427,8 @@ static void sourceview_apply_language (windata_t *vwin)
 	id = "octave";
     } else if (vwin->role == EDIT_PYTHON) {
 	id = "python";
+    } else if (vwin->role == EDIT_STATA) {
+	id = "stata";
     } else {
 	id = "gretl";
     }
@@ -687,7 +693,8 @@ static void set_sourceview_data_path (GtkSourceLanguageManager *lm)
 #define gretl_script_role(r) (r == EDIT_SCRIPT || \
 			      r == VIEW_SCRIPT || \
 			      r == EDIT_PKG_CODE || \
-			      r == EDIT_PKG_SAMPLE)
+			      r == EDIT_PKG_SAMPLE || \
+			      r == VIEW_PKG_SAMPLE)
 
 void create_source (windata_t *vwin, int hsize, int vsize, 
 		    gboolean editable)
@@ -832,6 +839,10 @@ static GtkTextTagTable *gretl_tags_new (void)
     g_object_set(tag, "foreground", "red", NULL);
     gtk_text_tag_table_add(table, tag);
 
+    tag = gtk_text_tag_new("greentext");
+    g_object_set(tag, "foreground", "green", NULL);
+    gtk_text_tag_table_add(table, tag);
+
     tag = gtk_text_tag_new("title");
     g_object_set(tag, "justification", GTK_JUSTIFY_CENTER,
 		 "pixels_above_lines", 15,
@@ -996,6 +1007,35 @@ void textview_append_text_colorized (GtkWidget *view, const char *buf, int trim)
     real_textview_add_colorized(view, buf, 1, trim);
 }
 
+void textview_set_text_report (GtkWidget *view, const char *buf)
+{
+    GtkTextBuffer *tbuf;
+    GtkTextIter iter;
+    const char *p;
+
+    /* plain text except for "<@ok>" represented as "OK" in
+       green and "<@fail>" as "failed" in red
+    */
+
+    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
+    gtk_text_buffer_get_iter_at_offset(tbuf, &iter, 0);
+    
+    while ((p = strstr(buf, "<@"))) {
+	gtk_text_buffer_insert(tbuf, &iter, buf, p - buf);
+	if (!strncmp(p, "<@ok>", 5)) {
+	    gtk_text_buffer_insert_with_tags_by_name(tbuf, &iter, _("OK"),
+						     -1, "greentext", NULL);
+	    buf = p + 5;
+	} else if (!strncmp(p, "<@fail>", 7)) {
+	    gtk_text_buffer_insert_with_tags_by_name(tbuf, &iter, _("failed"),
+						     -1, "redtext", NULL);
+	    buf = p + 7;
+	}
+    }
+
+    gtk_text_buffer_insert(tbuf, &iter, buf, -1);
+}
+
 void textview_delete_processing_message (GtkWidget *view)
 {
     GtkTextBuffer *tbuf;
@@ -1051,6 +1091,7 @@ void textview_insert_file (windata_t *vwin, const char *fname)
     GtkTextIter iter;    
     int thiscolor, nextcolor;
     char fline[MAXSTR], *chunk;
+    int links = 0;
     int i = 0;
 
     g_return_if_fail(GTK_IS_TEXT_VIEW(vwin->text));
@@ -1084,31 +1125,36 @@ void textview_insert_file (windata_t *vwin, const char *fname)
 	}
 
 	nextcolor = PLAIN_TEXT;
-	
-	if (vwin->role == SCRIPT_OUT && ends_with_backslash(chunk)) {
-	    nextcolor = BLUE_TEXT;
-	}
 
-	if (*chunk == '?') {
-	    thiscolor = (vwin->role == CONSOLE)? RED_TEXT : BLUE_TEXT;
-	} else if (*chunk == '#') {
-	    thiscolor = BLUE_TEXT;
-	} 
+	if (vwin->role == VIEW_DOC && strchr(chunk, '<')) {
+	    insert_text_with_markup(tbuf, &iter, chunk, vwin->role);
+	    links = 1;
+	} else {
+	    if (vwin->role == SCRIPT_OUT && ends_with_backslash(chunk)) {
+		nextcolor = BLUE_TEXT;
+	    }
 
-	switch (thiscolor) {
-	case PLAIN_TEXT:
-	    gtk_text_buffer_insert(tbuf, &iter, chunk, -1);
-	    break;
-	case BLUE_TEXT:
-	    gtk_text_buffer_insert_with_tags_by_name(tbuf, &iter,
-						     chunk, -1,
-						     "bluetext", NULL);
-	    break;
-	case RED_TEXT:
-	    gtk_text_buffer_insert_with_tags_by_name(tbuf, &iter,
-						     chunk, -1,
-						     "redtext", NULL);
-	    break;
+	    if (*chunk == '?') {
+		thiscolor = (vwin->role == CONSOLE)? RED_TEXT : BLUE_TEXT;
+	    } else if (*chunk == '#') {
+		thiscolor = BLUE_TEXT;
+	    } 
+
+	    switch (thiscolor) {
+	    case PLAIN_TEXT:
+		gtk_text_buffer_insert(tbuf, &iter, chunk, -1);
+		break;
+	    case BLUE_TEXT:
+		gtk_text_buffer_insert_with_tags_by_name(tbuf, &iter,
+							 chunk, -1,
+							 "bluetext", NULL);
+		break;
+	    case RED_TEXT:
+		gtk_text_buffer_insert_with_tags_by_name(tbuf, &iter,
+							 chunk, -1,
+							 "redtext", NULL);
+		break;
+	    }
 	}
 
 	if (chunk != fline) {
@@ -1120,6 +1166,10 @@ void textview_insert_file (windata_t *vwin, const char *fname)
     }
 
     fclose(fp);
+
+    if (links) {
+	connect_link_signals(vwin);
+    }
 }
 
 void textview_insert_from_tempfile (windata_t *vwin, PRN *prn)
@@ -1517,6 +1567,26 @@ cmdref_visibility_notify (GtkWidget *w,  GdkEventVisibility *e)
     return FALSE;
 }
 
+static void connect_link_signals (windata_t *vwin)
+{
+    if (hand_cursor == NULL) {
+	hand_cursor = gdk_cursor_new(GDK_HAND2);
+    }
+
+    if (regular_cursor == NULL) {
+	regular_cursor = gdk_cursor_new(GDK_XTERM);
+    }    
+
+    g_signal_connect(G_OBJECT(vwin->text), "key-press-event", 
+		     G_CALLBACK(cmdref_key_press), NULL);
+    g_signal_connect(G_OBJECT(vwin->text), "event-after", 
+		     G_CALLBACK(cmdref_event_after), NULL);
+    g_signal_connect(G_OBJECT(vwin->text), "motion-notify-event", 
+		     G_CALLBACK(cmdref_motion_notify), NULL);
+    g_signal_connect(G_OBJECT(vwin->text), "visibility-notify-event", 
+		     G_CALLBACK(cmdref_visibility_notify), NULL);
+}
+
 static void maybe_connect_help_signals (windata_t *hwin, int en)
 {
     int done = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(hwin->text), 
@@ -2222,6 +2292,26 @@ static void check_for_comment (const char *s, int *incomm)
     }
 }
 
+static int line_broken (const char *s)
+{
+    int ret = 0;
+
+    if (*s != '\0') {
+	int i, n = strlen(s);
+
+	for (i=n-1; i>0; i--) {
+	    if (s[i] == '\\' || s[i] == ',') {
+		ret = 1;
+		break;
+	    } else if (!isspace(s[i])) {
+		break;
+	    }
+	}
+    }
+
+    return ret;
+}
+
 static void normalize_indent (GtkTextBuffer *tbuf, 
 			      const gchar *buf,
 			      GtkTextIter *start,
@@ -2230,6 +2320,7 @@ static void normalize_indent (GtkTextBuffer *tbuf,
     int this_indent = 0;
     int next_indent = 0;
     char word[9], line[1024];
+    char lastline[1024];
     const char *ins;
     int incomment = 0;
     int inforeign = 0;
@@ -2241,9 +2332,12 @@ static void normalize_indent (GtkTextBuffer *tbuf,
 
     gtk_text_buffer_delete(tbuf, start, end);
 
+    lastline[0] = '\0';
     bufgets_init(buf);
 
     while (bufgets(line, sizeof line, buf)) {
+	int handled = 0;
+
 	if (string_is_blank(line)) {
 	    gtk_text_buffer_insert(tbuf, start, line, -1);
 	    continue;
@@ -2267,20 +2361,25 @@ static void normalize_indent (GtkTextBuffer *tbuf,
 		    inforeign = 0;
 		} else {
 		    gtk_text_buffer_insert(tbuf, start, line, -1);
-		    continue;
+		    handled = 1;
 		}
 	    } else {
 		adjust_indent(word, &this_indent, &next_indent);
 	    }
 	}
-	nsp = this_indent * tabwidth;
-	if (incomment == 2) {
-	    nsp += 3;
+	if (!handled) {
+	    nsp = this_indent * tabwidth;
+	    if (incomment == 2) {
+		nsp += 3;
+	    } else if (line_broken(lastline)) {
+		nsp += 2;
+	    }
+	    for (i=0; i<nsp; i++) {
+		gtk_text_buffer_insert(tbuf, start, " ", -1);
+	    }
+	    gtk_text_buffer_insert(tbuf, start, ins, -1);
 	}
-	for (i=0; i<nsp; i++) {
-	    gtk_text_buffer_insert(tbuf, start, " ", -1);
-	}
-	gtk_text_buffer_insert(tbuf, start, ins, -1);
+	strcpy(lastline, line);
     }
 
     bufgets_finalize(buf); 
@@ -3479,6 +3578,67 @@ void create_text (windata_t *vwin, int hsize, int vsize,
     gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(w), editable);
 }
 
+static GtkTextTagTable *gretl_console_tags_new (void)
+{
+    GtkTextTagTable *table;
+    GtkTextTag *tag;
+
+    table = gtk_text_tag_table_new(); 
+
+    tag = gtk_text_tag_new("bluetext");
+    g_object_set(tag, "foreground", "blue", NULL);
+    gtk_text_tag_table_add(table, tag);
+    
+    tag = gtk_text_tag_new("redtext");
+    g_object_set(tag, "foreground", "red", NULL);
+    gtk_text_tag_table_add(table, tag);
+
+    return table;
+}
+
+void create_console (windata_t *vwin, int hsize, int vsize)
+{
+    static GtkTextTagTable *console_tags = NULL;
+    GtkSourceBuffer *sbuf;
+    GtkTextView *view;
+    int cw;
+
+    if (console_tags == NULL) {
+	console_tags = gretl_console_tags_new();
+    }
+
+    sbuf = gtk_source_buffer_new(console_tags);
+    gtk_source_buffer_set_highlight_matching_brackets(sbuf, TRUE);
+
+    vwin->text = gtk_source_view_new_with_buffer(sbuf);
+    vwin->sbuf = sbuf;
+
+    view = GTK_TEXT_VIEW(vwin->text);
+
+    gtk_text_view_set_wrap_mode(view, GTK_WRAP_NONE);
+    gtk_text_view_set_left_margin(view, 4);
+    gtk_text_view_set_right_margin(view, 4);
+
+    gtk_widget_modify_font(GTK_WIDGET(vwin->text), fixed_font);
+
+    cw = get_char_width(vwin->text);
+    set_source_tabs(vwin->text, cw);
+
+    if (hsize > 0) {
+	hsize *= cw;
+	hsize += 48; /* ?? */
+    }
+
+    if (hsize > 0 && vsize > 0) {
+	GtkWidget *vmain = vwin_toplevel(vwin);
+
+	gtk_window_set_default_size(GTK_WINDOW(vmain), hsize, vsize);
+    }
+
+    gtk_text_view_set_editable(view, TRUE);
+    gtk_text_view_set_cursor_visible(view, TRUE);
+}
+
 void text_set_word_wrap (GtkWidget *w, gboolean wrap)
 {
     if (wrap) {
@@ -3506,20 +3666,25 @@ void text_table_setup (GtkWidget *vbox, GtkWidget *w)
     gtk_widget_show(sw);
 }
 
-static void set_pane_text_properties (GtkTextView *w)
+static void set_pane_text_properties (GtkTextView *w2,
+				      GtkTextView *w1)
 {
-    gtk_text_view_set_wrap_mode(w, 0);
-    gtk_text_view_set_left_margin(w, 4);
-    gtk_text_view_set_right_margin(w, 4);
+    gtk_text_view_set_wrap_mode(w2, 0);
+    gtk_text_view_set_left_margin(w2, 4);
+    gtk_text_view_set_right_margin(w2, 4);
 
-    gtk_widget_modify_font(GTK_WIDGET(w), fixed_font);
+    gtk_widget_modify_font(GTK_WIDGET(w2), fixed_font);
 
-    gtk_text_view_set_editable(w, FALSE);
-    gtk_text_view_set_cursor_visible(w, FALSE);
+    if (gtk_text_view_get_editable(w1)) {
+	gtk_text_view_set_editable(w2, TRUE);
+	gtk_text_view_set_cursor_visible(w2, TRUE);
+    } else {
+	gtk_text_view_set_editable(w2, FALSE);
+	gtk_text_view_set_cursor_visible(w2, FALSE);
+    }
 }
 
-/* limited to script output window at present: divide the
-   window into two panes */
+/* divide a text window into two panes */
 
 void viewer_split_pane (windata_t *vwin, int vertical)
 {
@@ -3551,8 +3716,15 @@ void viewer_split_pane (windata_t *vwin, int vertical)
     g_object_set_data(G_OBJECT(vwin->vbox), "sw", NULL);
 
     tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view1));
-    view2 = gtk_text_view_new_with_buffer(tbuf);
-    set_pane_text_properties(GTK_TEXT_VIEW(view2));
+
+    if (GTK_IS_SOURCE_VIEW(view1)) {
+	view2 = gtk_source_view_new_with_buffer(GTK_SOURCE_BUFFER(tbuf));
+    } else {
+	view2 = gtk_text_view_new_with_buffer(tbuf);
+    }
+
+    set_pane_text_properties(GTK_TEXT_VIEW(view2),
+			     GTK_TEXT_VIEW(view1));
 
     g_signal_connect(G_OBJECT(view2), "button-press-event", 
 		     G_CALLBACK(text_popup_handler), vwin);
@@ -3601,4 +3773,3 @@ void viewer_close_pane (windata_t *vwin)
     gtk_widget_show(sw);
     g_object_unref(sw);
 }
-
