@@ -54,8 +54,9 @@
 #include "../pixmaps/mini.en.xpm"
 #include "../pixmaps/mini.split_h.xpm"
 #include "../pixmaps/mini.split_v.xpm"
+#include "../pixmaps/mini.join_h.xpm"
+#include "../pixmaps/mini.join_v.xpm"
 #include "../pixmaps/mini.winlist.xpm"
-#include "../pixmaps/mini.spreadsheet.xpm"
 #include "../pixmaps/mini.bundle.xpm"
 
 /* for main-window toolbar */
@@ -107,7 +108,9 @@ enum {
     NEW_ITEM,
     CLOSE_ITEM,
     BUNDLE_ITEM,
-    FIND_ITEM
+    FIND_ITEM,
+    CLEAR_ITEM,
+    LOG_COPY_ITEM
 } viewbar_flags;
 
 struct stock_maker {
@@ -135,8 +138,9 @@ void gretl_stock_icons_init (void)
 	{ mini_en_xpm, GRETL_STOCK_EN },
 	{ mini_split_h_xpm, GRETL_STOCK_SPLIT_H },
 	{ mini_split_v_xpm, GRETL_STOCK_SPLIT_V },
+	{ mini_join_h_xpm, GRETL_STOCK_JOIN_H },
+	{ mini_join_v_xpm, GRETL_STOCK_JOIN_V },
 	{ mini_winlist_xpm, GRETL_STOCK_WINLIST },
-	{ mini_spreadsheet_xpm, GRETL_STOCK_SHEET },
 	{ mini_bundle_xpm, GRETL_STOCK_BUNDLE },
 	{ mini_db_xpm, GRETL_STOCK_DB},
 	{ mini_gretl_xpm, GRETL_STOCK_GRETL},
@@ -170,6 +174,33 @@ void gretl_stock_icons_init (void)
 
 /* callbacks for viewer window toolbar */
 
+static void copy_log_callback (GtkWidget *w, windata_t *vwin)
+{
+    gchar *buf = textview_get_text(vwin->text);
+    gchar *s = buf;
+    int n = 0;
+
+    while (s != NULL && n < 3) {
+	s = strchr(s, '\n');
+	if (s != NULL) {
+	    s++;
+	    n++;
+	}
+    }
+
+    if (s != NULL) {
+	gchar *modbuf;
+
+	modbuf = g_strdup_printf("# logged commands\n%s", s);
+	do_new_script(EDIT_SCRIPT, modbuf);
+	g_free(modbuf);
+    } else {
+	do_new_script(EDIT_SCRIPT, buf);
+    }
+    
+    g_free(buf);
+}
+
 static void save_as_callback (GtkWidget *w, windata_t *vwin)
 {
     GtkWidget *vmain = vwin_toplevel(vwin);
@@ -194,6 +225,7 @@ static void save_as_callback (GtkWidget *w, windata_t *vwin)
 	}
     } else if (vwin->role == EDIT_SCRIPT ||
 	       vwin->role == VIEW_SCRIPT ||
+	       vwin->role == VIEW_PKG_SAMPLE ||
 	       vwin->role == VIEW_LOG ||
 	       vwin->role == VIEW_PKG_CODE) {
 	u = SAVE_SCRIPT;
@@ -207,6 +239,8 @@ static void save_as_callback (GtkWidget *w, windata_t *vwin)
 	u = SAVE_OCTAVE_CMDS;
     } else if (vwin->role == EDIT_PYTHON) {
         u = SAVE_PYTHON_CMDS;
+    } else if (vwin->role == EDIT_STATA) {
+	u = SAVE_STATA_CMDS;
     } else if (vwin->role == VIEW_FILE) {
 	u = SAVE_TEXT;
     } else {
@@ -238,16 +272,21 @@ static void mail_script_callback (GtkWidget *w, windata_t *vwin)
 
 static void file_open_callback (GtkWidget *w, windata_t *vwin)
 {
-    /* don't proceed unconditionally if there's unsaved
-       text in the window 
-    */
-    if (query_save_text(NULL, NULL, vwin)) {
-	return;
+    int tabbed = window_is_tab(vwin);
+    
+    if (!tabbed) {
+	/* Don't proceed unconditionally if there's unsaved
+	   text in a single-script window; this doesn't
+	   apply if we're just opening another tab.
+	*/
+	if (query_save_text(NULL, NULL, vwin)) {
+	    return;
+	}
     }
 
     file_selector(OPEN_SCRIPT, FSEL_DATA_VWIN, vwin);
 
-    if (vwin->flags & VWIN_CONTENT_CHANGED) {
+    if (!tabbed && (vwin->flags & VWIN_CONTENT_CHANGED)) {
 	mark_vwin_content_saved(vwin);
 	vwin_set_filename(vwin, tryfile);
     }
@@ -255,7 +294,12 @@ static void file_open_callback (GtkWidget *w, windata_t *vwin)
 
 static void toolbar_new_callback (GtkWidget *w, windata_t *vwin)
 {
-    do_new_script(vwin->role);
+    do_new_script(vwin->role, NULL);
+}
+
+static void clear_text_callback (GtkWidget *w, windata_t *vwin)
+{
+    textview_set_text(vwin->text, "");
 }
 
 static void window_print_callback (GtkWidget *w, windata_t *vwin)
@@ -265,7 +309,9 @@ static void window_print_callback (GtkWidget *w, windata_t *vwin)
     window_print(NULL, vwin);
 #else
     if (textview_use_highlighting(vwin->role)) {
-	int resp = yes_no_dialog(NULL, _("Print with syntax highlighting?"), 1);
+	int resp = yes_no_cancel_dialog(NULL,
+					_("Print with syntax highlighting?"),
+					vwin_toplevel(vwin));
 
 	if (resp == GRETL_YES) {
 	    sourceview_print(vwin);
@@ -303,9 +349,10 @@ static void toolbar_refresh (GtkWidget *w, windata_t *vwin)
 static void set_matrix_name (GtkWidget *widget, dialog_t *dlg)
 {
     char *vname = (char *) edit_dialog_get_data(dlg);
+    GtkWidget *parent =  edit_dialog_get_window(dlg);
     const gchar *s = edit_dialog_get_text(dlg);
 
-    if (s == NULL || gui_validate_varname(s, GRETL_TYPE_MATRIX)) {
+    if (s == NULL || gui_validate_varname(s, GRETL_TYPE_MATRIX, parent)) {
 	edit_dialog_reset(dlg);
     } else {
 	strcpy(vname, s);
@@ -338,7 +385,7 @@ static void add_matrix_callback (GtkWidget *w, windata_t *vwin)
 		    gretl_matrix_free(m);
 		    gui_errmsg(err);
 		} else {
-		    infobox(_("Saved matrix as %s"), mname);
+		    infobox_printf(_("Saved matrix as %s"), mname);
 		}
 	    }
 	}
@@ -428,16 +475,42 @@ static void split_pane_callback (GtkWidget *w, windata_t *vwin)
 	hb = w;
     }
 
+    /* Note: by "vertical" here we mean that the split runs vertically,
+       dividing the pane into left- and right-hand sections; otherwise
+       the split runs horizontally.
+    */
+
     if (g_object_get_data(G_OBJECT(vwin->vbox), "sw") != NULL) {
-	/* currently in single-view mode */
+	/* currently in single-view mode: so split */
 	viewer_split_pane(vwin, vertical);
 	gtk_widget_set_sensitive(vb, vertical);
 	gtk_widget_set_sensitive(hb, !vertical);
-    } else if (g_object_get_data(G_OBJECT(vwin->vbox), "paned") != NULL) {
-	/* currently in split-view mode */
-	viewer_close_pane(vwin);
-	gtk_widget_set_sensitive(hb, TRUE);
-	gtk_widget_set_sensitive(vb, TRUE);
+	if (vertical) {
+	    gtk_tool_button_set_stock_id(GTK_TOOL_BUTTON(vb), 
+					 GRETL_STOCK_JOIN_V);
+	} else {
+	    gtk_tool_button_set_stock_id(GTK_TOOL_BUTTON(hb), 
+					 GRETL_STOCK_JOIN_H);
+	}
+    } else {
+	GtkWidget *paned;
+
+	paned = g_object_get_data(G_OBJECT(vwin->vbox), "paned");
+
+	if (paned != NULL) {
+	    /* currently in split-view mode: so rejoin */
+	    vertical = GTK_IS_HPANED(paned);
+	    viewer_close_pane(vwin);
+	    gtk_widget_set_sensitive(hb, TRUE);
+	    gtk_widget_set_sensitive(vb, TRUE);
+	    if (vertical) {
+		gtk_tool_button_set_stock_id(GTK_TOOL_BUTTON(vb), 
+					     GRETL_STOCK_SPLIT_V);
+	    } else {
+		gtk_tool_button_set_stock_id(GTK_TOOL_BUTTON(hb), 
+					     GRETL_STOCK_SPLIT_H);
+	    }
+	}
     }
 }
 
@@ -540,24 +613,9 @@ static void coeffint_set_alpha (GtkWidget *w, windata_t *vwin)
     gtk_widget_show_all(dialog);
 }
 
-static void set_output_stickiness (GtkWidget *w, windata_t *vwin)
+static void stickiness_callback (GtkWidget *w, windata_t *vwin)
 {
-    const char *opts[] = {
-	N_("Always goes to a new window"),
-	N_("Replaces output in this window"),
-	N_("Adds to output in this window")
-    };
-    const char *label = N_("New script output:");
-    int resp, deflt;
-
-    deflt = get_script_output_policy();
-
-    resp = radio_dialog(NULL, label, opts, 3, deflt, 0, 
-			vwin_toplevel(vwin));
-
-    if (resp != GRETL_CANCEL) {
-	set_script_output_policy(resp, vwin);
-    }
+    output_policy_dialog(vwin, vwin, 1);
 }
 
 static void toolbar_plot_callback (GtkWidget *w, windata_t *vwin)
@@ -591,24 +649,6 @@ static void activate_script_help (GtkWidget *widget, windata_t *vwin)
     set_window_help_active(vwin);
 }
 
-static void delete_file_viewer (GtkWidget *widget, windata_t *vwin) 
-{
-    gint resp = 0;
-
-    if (window_is_tab(vwin)) {
-	maybe_destroy_tabwin(vwin);
-	return;
-    }
-
-    if (vwin_is_editing(vwin) && vwin_content_changed(vwin)) {
-	resp = query_save_text(NULL, NULL, vwin);
-    }
-
-    if (!resp) {
-	gtk_widget_destroy(vwin->main); 
-    }
-}
-
 static int edit_script_popup_item (GretlToolItem *item)
 {
     return !strcmp(item->icon, GTK_STOCK_COPY) ||
@@ -641,6 +681,8 @@ static GretlToolItem viewbar_items[] = {
     { N_("Open..."), GTK_STOCK_OPEN, G_CALLBACK(file_open_callback), OPEN_ITEM },
     { N_("Save"), GTK_STOCK_SAVE, G_CALLBACK(vwin_save_callback), SAVE_ITEM },
     { N_("Save as..."), GTK_STOCK_SAVE_AS, G_CALLBACK(save_as_callback), SAVE_AS_ITEM },
+    { N_("Open in script editor"), GTK_STOCK_EDIT, G_CALLBACK(copy_log_callback), LOG_COPY_ITEM },
+    { N_("Clear"), GTK_STOCK_CLEAR, G_CALLBACK(clear_text_callback), CLEAR_ITEM },
     { N_("Save bundle content..."), GRETL_STOCK_BUNDLE, NULL, BUNDLE_ITEM },
     { N_("Print..."), GTK_STOCK_PRINT, G_CALLBACK(window_print_callback), 0 },
     { N_("Show/hide"), GRETL_STOCK_PIN, G_CALLBACK(session_notes_callback), NOTES_ITEM },
@@ -661,25 +703,24 @@ static GretlToolItem viewbar_items[] = {
     { N_("LaTeX"), GRETL_STOCK_TEX, G_CALLBACK(window_tex_callback), TEX_ITEM },
     { N_("Graph"), GRETL_STOCK_TS, G_CALLBACK(toolbar_plot_callback), PLOT_ITEM },
     { N_("Reformat..."), GTK_STOCK_CONVERT, G_CALLBACK(reformat_callback), FORMAT_ITEM },
-    { N_("Edit values..."), GRETL_STOCK_SHEET, G_CALLBACK(series_view_edit), EDITOR_ITEM },
+    { N_("Edit values..."), GTK_STOCK_EDIT, G_CALLBACK(series_view_edit), EDITOR_ITEM },
     { N_("Refresh"), GTK_STOCK_REFRESH, G_CALLBACK(toolbar_refresh), REFRESH_ITEM },
     { N_("Add to dataset..."), GTK_STOCK_ADD, G_CALLBACK(add_data_callback), ADD_DATA_ITEM },
     { N_("Add as matrix..."), GTK_STOCK_ADD, G_CALLBACK(add_matrix_callback), ADD_MATRIX_ITEM },
-    { N_("Stickiness..."), GRETL_STOCK_PIN, G_CALLBACK(set_output_stickiness), STICKIFY_ITEM },
+    { N_("Stickiness..."), GRETL_STOCK_PIN, G_CALLBACK(stickiness_callback), STICKIFY_ITEM },
     { N_("Toggle split pane"), GRETL_STOCK_SPLIT_H, G_CALLBACK(split_pane_callback), SPLIT_ITEM },
     { N_("Toggle split pane"), GRETL_STOCK_SPLIT_V, G_CALLBACK(split_pane_callback), SPLIT_ITEM },
     { N_("Help on command"), GTK_STOCK_HELP, G_CALLBACK(activate_script_help), CMD_HELP_ITEM },
     { N_("Help"), GTK_STOCK_HELP, G_CALLBACK(window_help), HELP_ITEM },
     { N_("Help"), GTK_STOCK_HELP, G_CALLBACK(display_gnuplot_help), GP_HELP_ITEM },
-    { N_("Help"), GTK_STOCK_HELP, G_CALLBACK(display_x12a_help), X12A_HELP_ITEM },
-    { N_("Windows"), GRETL_STOCK_WINLIST, GNULL, 0 },
-    { N_("Close"), GTK_STOCK_CLOSE, G_CALLBACK(delete_file_viewer), CLOSE_ITEM }
+    { N_("Help"), GTK_STOCK_HELP, G_CALLBACK(display_x12a_help), X12A_HELP_ITEM }
 };
 
 static int n_viewbar_items = G_N_ELEMENTS(viewbar_items);
 
 #define exec_ok(r) (vwin_editing_script(r) || \
 		    r == VIEW_SCRIPT || \
+		    r == VIEW_PKG_SAMPLE || \
 	            r == EDIT_PKG_SAMPLE)
 
 #define open_ok(r) (vwin_editing_script(r))
@@ -707,6 +748,7 @@ static int n_viewbar_items = G_N_ELEMENTS(viewbar_items);
 #define cmd_help_ok(r) (r == EDIT_SCRIPT || \
 	                r == EDIT_PKG_CODE || \
                         r == EDIT_PKG_SAMPLE || \
+			r == VIEW_PKG_SAMPLE || \
 			r == VIEW_SCRIPT || \
 			r == VIEW_LOG)
 
@@ -720,7 +762,10 @@ static int n_viewbar_items = G_N_ELEMENTS(viewbar_items);
                         r == MAHAL || r == FCAST || \
 			r == LOESS || r == NADARWAT)
 
-#define split_ok(r) (r == SCRIPT_OUT || FNCALL_OUT)
+#define split_ok(r) (r == SCRIPT_OUT || r == FNCALL_OUT || \
+                     r == VIEW_LOG || vwin_editing_script(r))
+
+#define clear_ok(r) (r == VIEW_LOG)
 
 /* Screen out unwanted menu items depending on the context; also
    adjust the callbacks associated with some items based on
@@ -735,12 +780,16 @@ static GCallback tool_item_get_callback (GretlToolItem *item, windata_t *vwin,
     int f = item->flag;
     int r = vwin->role;
 
-    if (!edit_ok(r) && f == EDIT_ITEM) {
+    if (r != VIEW_LOG && f == LOG_COPY_ITEM) {
+	return NULL;
+    } else if (!edit_ok(r) && f == EDIT_ITEM) {
 	return NULL;
     } else if (!open_ok(r) && f == OPEN_ITEM) {
 	return NULL;
     } else if (!new_ok(r) && f == NEW_ITEM) {
 	return NULL;
+    } else if (!clear_ok(r) && f == CLEAR_ITEM) {
+	return NULL;
     } else if (!exec_ok(r) && f == EXEC_ITEM) {
 	return NULL;
     } else if (!cmd_help_ok(r) && f == CMD_HELP_ITEM) {
@@ -829,6 +878,21 @@ static GtkWidget *tool_item_get_menu (GretlToolItem *item, windata_t *vwin)
     return menu;
 }
 
+static void gretl_toolbar_relief (GtkWidget *w)
+{
+    static int style_done;
+
+    gtk_widget_set_name(w, "gretl_toolbar");
+
+    if (!style_done) {
+	gtk_rc_parse_string("style \"gretl-tb-style\"\n{\n"
+			    "  GtkToolbar::shadow-type = GTK_SHADOW_NONE\n"
+			    "}\n"
+			    "widget \"*.gretl_toolbar\" style \"gretl-tb-style\"");
+	style_done = 1;
+    }
+}
+
 GtkWidget *gretl_toolbar_new (void)
 {
     GtkWidget *tb = gtk_toolbar_new();
@@ -836,6 +900,7 @@ GtkWidget *gretl_toolbar_new (void)
     gtk_toolbar_set_icon_size(GTK_TOOLBAR(tb), GTK_ICON_SIZE_MENU);
     gtk_toolbar_set_style(GTK_TOOLBAR(tb), GTK_TOOLBAR_ICONS);
     gtk_toolbar_set_show_arrow(GTK_TOOLBAR(tb), FALSE);
+    gretl_toolbar_relief(tb);
 
     return tb;
 }
@@ -862,18 +927,6 @@ static GtkToolItem *gretl_menu_button (const char *icon,
     return item;
 }
 
-void vwin_toolbar_insert_winlist (windata_t *vwin)
-{
-    GtkWidget *button;
-    GtkToolItem *item;
-
-    item = gretl_menu_button(GRETL_STOCK_WINLIST, N_("Windows"),
-			     &button);
-    g_signal_connect(G_OBJECT(button), "button-press-event", 
-		     G_CALLBACK(vwin_winlist_popup), vwin);
-    gtk_toolbar_insert(GTK_TOOLBAR(vwin->mbar), item, -1);
-}
-
 static void gretl_tool_item_set_tip (GtkWidget *item,
 				     GretlToolItem *tool)
 {
@@ -910,6 +963,7 @@ GtkWidget *gretl_toolbar_insert (GtkWidget *tbar,
     item = gtk_tool_button_new_from_stock(tool->icon);
     gretl_tool_item_set_tip(GTK_WIDGET(item), tool);
     g_signal_connect(G_OBJECT(item), "clicked", func, data);
+    gtk_widget_set_size_request(GTK_WIDGET(item), 30, -1);
     gtk_toolbar_insert(GTK_TOOLBAR(tbar), item, pos);
 
     return GTK_WIDGET(item);
@@ -966,6 +1020,11 @@ static GtkWidget *vwin_toolbar_insert (GretlToolItem *tool,
 	}
     }
 
+#if 0
+    /* separate the buttons a tad more? */
+    gtk_widget_set_size_request(GTK_WIDGET(item), 30, -1);
+#endif
+    
     gtk_toolbar_insert(GTK_TOOLBAR(vwin->mbar), item, pos);
 
     return GTK_WIDGET(item);
@@ -990,12 +1049,6 @@ static void viewbar_add_items (windata_t *vwin, ViewbarFlags flags)
 
 	item = &viewbar_items[i];
 
-	if (winlist_item(item)) {
-	    /* the window-list item is special */
-	    vwin_toolbar_insert_winlist(vwin);
-	    continue;
-	}
-
 	/* Is there anything to hook up, in context? We
 	   try first for a menu to attach to the toolbar
 	   button; failing that we test for a "direct"
@@ -1111,10 +1164,6 @@ GtkWidget *build_text_popup (windata_t *vwin)
 		}
 	    } else if (func == G_CALLBACK(text_undo) && !text_can_undo(vwin)) {
 		continue;
-	    } else if (func == G_CALLBACK(delete_file_viewer)) {
-		if (window_is_tab(vwin)) {
-		    continue;
-		}
 	    }
 	    w = gtk_menu_item_new_with_label(_(item->tip));
 	    g_signal_connect(G_OBJECT(w), "activate", func, vwin);
@@ -1150,11 +1199,6 @@ static void tbar_open_data (void)
     display_files(TEXTBOOK_DATA, NULL);
 }
 
-static void tbar_users_guide (void)
-{
-    display_pdf_help(NULL);
-}
-
 static void tbar_command_ref (void)
 {
     plain_text_cmdref(NULL);
@@ -1185,18 +1229,9 @@ static void tbar_model (void)
     }
 }
 
-static void tbar_iconview (void)
-{
-    if (data_status) {
-	view_session();
-    } else {
-	warnbox(_("Please open a data file first"));
-    }
-}
-
 static void tbar_new_script (void)
 {
-    do_new_script(EDIT_SCRIPT);
+    do_new_script(EDIT_SCRIPT, NULL);
 }
 
 static void tbar_show_funcs (GtkWidget *w, gpointer p)
@@ -1210,15 +1245,13 @@ static GretlToolItem mainbar_items[] = {
     { N_("launch calculator"),  GRETL_STOCK_CALC,    G_CALLBACK(tbar_calc), 0 },
     { N_("new script"),         GTK_STOCK_EDIT,      G_CALLBACK(tbar_new_script), 0 },
     { N_("open gretl console"), GRETL_STOCK_CONSOLE, G_CALLBACK(gretl_console), 0 },
-    { N_("session icon view"),  GRETL_STOCK_ICONS,   G_CALLBACK(tbar_iconview), 0 },
+    { N_("session icon view"),  GRETL_STOCK_ICONS,   G_CALLBACK(view_session), 0 },
     { N_("function packages"),  GRETL_STOCK_FUNC,    G_CALLBACK(tbar_show_funcs), 0 },
-    { N_("user's guide"),       GRETL_STOCK_PDF,     G_CALLBACK(tbar_users_guide), 0 },
     { N_("command reference"),  GTK_STOCK_HELP,      G_CALLBACK(tbar_command_ref), 0 },
     { N_("X-Y graph"),          GRETL_STOCK_SCATTER, G_CALLBACK(tbar_xy_graph), 0 },
     { N_("OLS model"),          GRETL_STOCK_MODEL,   G_CALLBACK(tbar_model), 0 },
     { N_("gretl database"),     GRETL_STOCK_DB,      G_CALLBACK(show_native_dbs), 0 },
     { N_("open dataset"),       GTK_STOCK_OPEN,      G_CALLBACK(tbar_open_data), 0 },
-    { N_("Windows"),            GRETL_STOCK_WINLIST, GNULL, 0 },
 };
 
 void add_mainwin_toolbar (GtkWidget *vbox)
@@ -1231,11 +1264,7 @@ void add_mainwin_toolbar (GtkWidget *vbox)
 
     for (i=0; i<n; i++) {
 	item = &mainbar_items[i];
-	if (winlist_item(item)) {
-	    vwin_toolbar_insert_winlist(mdata);
-	} else {
-	    gretl_toolbar_insert(mdata->mbar, item, item->func, mdata, -1);
-	}
+	gretl_toolbar_insert(mdata->mbar, item, item->func, mdata, -1);
     }
 
     hbox = gtk_hbox_new(FALSE, 0);
@@ -1243,8 +1272,12 @@ void add_mainwin_toolbar (GtkWidget *vbox)
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 }
 
-/* add a temporary top-bar for use in a script output
-   window, while we're waiting for the output */
+/* Add a temporary menubar for use in a script output
+   window, while we're waiting for the output. If the
+   output window is being reused this is a bit more
+   complicated; we have to "hide" the regular menubar
+   before inserting the temporary one.
+ */
 
 void vwin_add_tmpbar (windata_t *vwin)
 {
@@ -1256,15 +1289,32 @@ void vwin_add_tmpbar (windata_t *vwin)
     };
     GtkWidget *hbox, *tmp;
 
-    hbox = gtk_hbox_new(FALSE, 0);
-    g_object_set_data(G_OBJECT(vwin->main), "top-hbox", hbox);
+    hbox = g_object_get_data(G_OBJECT(vwin->main), "top-hbox");
+    
+    if (hbox != NULL) {
+	/* We're replacing a "real" menubar temporarily: ref. the
+	   widgets in @hbox before removing them so we can put
+	   them back later.
+	*/
+	GtkWidget *winlist = g_object_get_data(G_OBJECT(hbox), "winlist");
+
+	g_object_ref(G_OBJECT(vwin->mbar));
+	gtk_container_remove(GTK_CONTAINER(hbox), vwin->mbar);
+	if (winlist != NULL) {
+	    g_object_ref(G_OBJECT(winlist));
+	    gtk_container_remove(GTK_CONTAINER(hbox), winlist);
+	}
+    } else {
+	/* starting from scratch */
+	hbox = gtk_hbox_new(FALSE, 0);
+	g_object_set_data(G_OBJECT(vwin->main), "top-hbox", hbox);
+	gtk_box_pack_start(GTK_BOX(vwin->vbox), hbox, FALSE, FALSE, 0);
+    }
 
     tmp = gretl_toolbar_new();
     gretl_toolbar_insert(tmp, &item, item.func, NULL, 0);
     gtk_box_pack_start(GTK_BOX(hbox), tmp, FALSE, FALSE, 5);
 
-    gtk_box_pack_start(GTK_BOX(vwin->vbox), hbox, FALSE, FALSE, 0);
-    start_wait_for_output(hbox);
+    start_wait_for_output(vwin, hbox);
     gtk_widget_show_all(hbox);
 }
-
