@@ -23,6 +23,7 @@
 #include "treeutils.h"
 #include "datafiles.h"
 #include "dlgutils.h"
+#include "menustate.h"
 
 /* special comparator which always preserves "const" (variable 0) in
    the first position when sorting variables by name */
@@ -439,6 +440,14 @@ static void check_db_series_selection (GtkTreeSelection *sel,
     }  
 }
 
+static void mdata_selection_changed (GtkTreeSelection *sel, 
+				     windata_t *vwin)
+{
+    int nsel = gtk_tree_selection_count_selected_rows(sel);
+
+    single_var_menu_state(nsel);
+}
+
 static void id_col_clicked (GtkTreeViewColumn *column, GtkWidget *view)
 {
     GtkTreeModel *model;
@@ -459,7 +468,7 @@ static void id_col_clicked (GtkTreeViewColumn *column, GtkWidget *view)
                              v->role == REMOTE_SERIES)
 
 void vwin_add_list_box (windata_t *vwin, GtkBox *box, 
-			int ncols, gboolean hidden_col,
+			int ncols, int hidden_cols,
 			GType *types, const char **titles,
 			int tree) 
 {
@@ -470,11 +479,9 @@ void vwin_add_list_box (windata_t *vwin, GtkBox *box,
     GtkCellRenderer *bool_renderer = NULL;
     GtkTreeViewColumn *column;
     GtkTreeSelection *select;
-    int i, viscols = ncols;
+    int i, viscols;
 
-    if (hidden_col) {
-	viscols--;
-    }
+    viscols = ncols - hidden_cols;
 
     if (tree) {
 	tstore = gtk_tree_store_newv(ncols, types);
@@ -535,7 +542,7 @@ void vwin_add_list_box (windata_t *vwin, GtkBox *box,
 	}	
     }
 
-    if (hidden_col) {
+    for (i=viscols; i<ncols; i++) {
 	column = gtk_tree_view_column_new_with_attributes(NULL,
 							  renderer,
 							  "text", i, 
@@ -558,6 +565,9 @@ void vwin_add_list_box (windata_t *vwin, GtkBox *box,
 			      | GDK_POINTER_MOTION_HINT_MASK);
         g_signal_connect(G_OBJECT(view), "motion-notify-event",
 			 G_CALLBACK(listbox_drag), NULL);
+	g_signal_connect(G_OBJECT(select), "changed",
+			 G_CALLBACK(mdata_selection_changed),
+			 vwin);
     } else if (db_series_window(vwin)) {
 	gtk_tree_selection_set_mode(select, GTK_SELECTION_MULTIPLE);
 	g_signal_connect(G_OBJECT(select), "changed",
@@ -640,6 +650,20 @@ void tree_view_get_bool (GtkTreeView *view, int row, int col, gboolean *val)
     g_free(path);
 }
 
+void tree_view_get_int (GtkTreeView *view, int row, int col, gint *val)
+{
+    GtkTreeModel *model;
+    GtkTreeIter iter;
+    gchar *path;
+
+    model = gtk_tree_view_get_model(view);
+    path = g_strdup_printf("%d", row);
+    if (gtk_tree_model_get_iter_from_string(model, &iter, path)) {
+	gtk_tree_model_get(model, &iter, col, val, -1);
+    }
+    g_free(path);
+}
+
 void tree_view_get_string (GtkTreeView *view, int row, int col, gchar **val)
 {
     GtkTreeModel *model;
@@ -683,20 +707,6 @@ void tree_store_set_string (GtkTreeView *view, int row, int col, const gchar *va
     tree_view_set_string(view, row, col, val, 1);
 }
 
-void tree_view_get_int (GtkTreeView *view, int row, int col, int *val)
-{
-    GtkTreeModel *model;
-    GtkTreeIter iter;
-    gchar *path;
-
-    model = gtk_tree_view_get_model(view);
-    path = g_strdup_printf("%d", row);
-    if (gtk_tree_model_get_iter_from_string(model, &iter, path)) {
-	gtk_tree_model_get(model, &iter, col, val, -1);
-    }
-    g_free(path);
-}
-
 int tree_path_get_row_number (GtkTreePath *path)
 {
     return gtk_tree_path_get_indices(path)[0];
@@ -714,6 +724,21 @@ void tree_model_get_iter_last (GtkTreeModel *mod, GtkTreeIter *iter)
     }
 }
 
+int tree_model_count_rows (GtkTreeModel *mod)
+{
+    GtkTreeIter iter;
+    int nrows = 0;
+
+    if (gtk_tree_model_get_iter_first(mod, &iter)) {
+	nrows = 1;
+	while (gtk_tree_model_iter_next(mod, &iter)) {
+	    nrows++;
+	}
+    }
+
+    return nrows;
+}
+
 gboolean tree_model_iter_prev (GtkTreeModel *mod, GtkTreeIter *iter)
 {
     GtkTreePath *path;
@@ -771,3 +796,17 @@ int *main_window_selection_as_list (void)
 
     return list;
 }
+
+char *main_window_selection_as_string (void)
+{
+    int *list = main_window_selection_as_list();
+    char *liststr = NULL;
+
+    if (list != NULL) {
+	int err = 0;
+
+	liststr = gretl_list_to_string(list, dataset, &err);
+    }
+
+    return liststr;
+}
