@@ -212,8 +212,6 @@ static int try_regenerate_var (int v, const char *s)
     PRN *prn;
     int err;
 
-    /* FIXME use gretl_command_sprintf, etc? */
-
     if (bufopen(&prn)) {
 	return 0;
     }
@@ -224,6 +222,9 @@ static int try_regenerate_var (int v, const char *s)
 
     if (err) {
 	errbox(gretl_print_get_buffer(prn));
+    } else {
+	lib_command_strcpy(line);
+	record_command_verbatim();
     }
 
     gretl_print_destroy(prn);
@@ -250,8 +251,8 @@ static void really_set_variable_info (GtkWidget *w, gui_varinfo *vset)
 
     if (vset->changed[VSET_VARNAME]) {
 	newstr = entry_get_trimmed_text(vset->name_entry);
-	/* note: do_rename_var() logs the corresponding command */
-	err = do_rename_variable(v, newstr);
+	/* note: do_rename_variable() logs the corresponding command */
+	err = do_rename_variable(v, newstr, vset->dlg);
 	g_free(newstr);
     }
 
@@ -261,6 +262,7 @@ static void really_set_variable_info (GtkWidget *w, gui_varinfo *vset)
 	    err = try_regenerate_var(v, newstr);
 	} else {
 	    series_record_label(dataset, v, newstr);
+	    record_varlabel_change(v, 1, 0);
 	}
 	g_free(newstr);
     }
@@ -273,6 +275,7 @@ static void really_set_variable_info (GtkWidget *w, gui_varinfo *vset)
 	    err = E_DATA;
 	} else {
 	    series_record_display_name(dataset, v, newstr);
+	    record_varlabel_change(v, 0, 1);
 	}
 	g_free(newstr);
     }
@@ -299,9 +302,6 @@ static void really_set_variable_info (GtkWidget *w, gui_varinfo *vset)
     }  
 
     if (!err) {
-	if (vset->changed[VSET_LABEL] || vset->changed[VSET_DISPLAY]) {
-	    record_varlabel_change(v);
-	}
 	if (vset->changed[VSET_IDNUM]) {
 	    ival = spinner_get_int(vset->id_spin);
 	    dataset_renumber_variable(v, ival, dataset);
@@ -354,9 +354,9 @@ static void set_series_name_and_desc (GtkWidget *w, name_setter *nset)
 	}
 
 	if (allow_overwrite) {
-	    err = gui_validate_varname(s, GRETL_TYPE_SERIES);
+	    err = gui_validate_varname(s, GRETL_TYPE_SERIES, nset->dlg);
 	} else {
-	    err = gui_validate_varname_strict(s, GRETL_TYPE_SERIES);
+	    err = gui_validate_varname_strict(s, GRETL_TYPE_SERIES, nset->dlg);
 	}
 
 	if (err) {
@@ -665,6 +665,7 @@ static void varinfo_add_toolbar (gui_varinfo *vset, GtkWidget *hbox)
 
     item = gtk_tool_button_new_from_stock(GTK_STOCK_APPLY);
     g_signal_connect(G_OBJECT(item), "clicked", G_CALLBACK(varinfo_apply), vset);
+    gtk_widget_set_size_request(GTK_WIDGET(item), 30, -1);
     gtk_toolbar_insert(GTK_TOOLBAR(tbar), item, -1);
     vset->apply = GTK_WIDGET(item);
     add_stock_tooltip(vset->apply, GTK_STOCK_APPLY);
@@ -672,12 +673,14 @@ static void varinfo_add_toolbar (gui_varinfo *vset, GtkWidget *hbox)
 
     item = gtk_tool_button_new_from_stock(GTK_STOCK_GO_UP);
     g_signal_connect(G_OBJECT(item), "clicked", G_CALLBACK(varinfo_up_down), vset);
+    gtk_widget_set_size_request(GTK_WIDGET(item), 30, -1);
     gtk_toolbar_insert(GTK_TOOLBAR(tbar), item, -1);
     vset->up = GTK_WIDGET(item);
     gretl_tooltips_add(vset->up, _("Previous series"));
     
     item = gtk_tool_button_new_from_stock(GTK_STOCK_GO_DOWN);
     g_signal_connect(G_OBJECT(item), "clicked", G_CALLBACK(varinfo_up_down), vset);
+    gtk_widget_set_size_request(GTK_WIDGET(item), 30, -1);
     gtk_toolbar_insert(GTK_TOOLBAR(tbar), item, -1);
     vset->down = GTK_WIDGET(item);
     gretl_tooltips_add(vset->down, _("Next series"));
