@@ -23,6 +23,9 @@
 #include "guiprint.h"
 #include "session.h"
 #include "tabwin.h"
+#include "fnsave.h"
+#include "toolbar.h"
+#include "cmdstack.h"
 #include "winstack.h"
 
 #if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 20
@@ -128,11 +131,14 @@ static const gchar *window_list_icon (int role)
 	id = GRETL_STOCK_BOOK;
     } else if (role == STAT_TABLE) {
 	id = GRETL_STOCK_CALC;
-    } else if (role == VIEW_SCRIPT) {
+    } else if (role == VIEW_SCRIPT ||
+	       role == VIEW_PKG_SAMPLE) {
 	id = GTK_STOCK_EXECUTE;
     } else if (role == OPEN_SESSION) {
 	id = GRETL_STOCK_ICONS;
-    } else if (role == PRINT || role == SCRIPT_OUT) {
+    } else if (role == PRINT || 
+	       role == SCRIPT_OUT ||
+	       role == VIEW_LOG) {
 	id = GRETL_STOCK_PAGE;
     } else if (role == SSHEET) {
 	id = GRETL_STOCK_TABLE;
@@ -456,17 +462,70 @@ static gboolean winlist_popup_done (GtkMenuShell *mshell,
     windata_t *vwin = window_get_active_vwin(window);
 
     if (vwin != NULL) {
+	/* don't leave focus on the winlist button */
 	if (vwin->role == VIEW_MODEL || 
 	    vwin->role == VAR || 
 	    vwin->role == VECM) {
-	    /* don't leave focus on the winlist button */
 	    gtk_widget_grab_focus(vwin->text);
+	} else if (vwin == mdata) {
+	    gtk_widget_grab_focus(vwin->listbox);
 	}
     }
 
     return FALSE;
 }
 
+static void add_cascade_item (GtkWidget *menu,
+			      GtkWidget *item)
+{
+    GtkWidget *image;
+
+    item = gtk_image_menu_item_new_with_label(_("Arrange"));
+    image = gtk_image_new_from_stock(GRETL_STOCK_WINLIST, 
+				     GTK_ICON_SIZE_MENU);
+    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), 
+				  image);
+    g_signal_connect(G_OBJECT(item), "activate", 
+		     G_CALLBACK(cascade_session_windows), 
+		     NULL);
+    gtk_widget_show(item);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+}
+
+static void add_log_item (GtkWidget *menu, 
+			  GtkWidget *item)
+{
+    GtkWidget *image;
+
+    item = gtk_image_menu_item_new_with_label(_("command log"));
+    image = gtk_image_new_from_stock(GRETL_STOCK_PAGE, 
+				     GTK_ICON_SIZE_MENU);
+    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), 
+				  image);
+    g_signal_connect(G_OBJECT(item), "activate", 
+		     G_CALLBACK(view_command_log), 
+		     NULL);
+    gtk_widget_show(item);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+}
+
+static void add_iconview_item (GtkWidget *menu,
+			       GtkWidget *item)
+{
+    GtkWidget *image;
+
+    item = gtk_image_menu_item_new_with_label(_("icon view"));
+    image = gtk_image_new_from_stock(GRETL_STOCK_ICONS, 
+				     GTK_ICON_SIZE_MENU);
+    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), 
+				  image);
+    g_signal_connect(G_OBJECT(item), "activate", 
+		     G_CALLBACK(view_session), 
+		     NULL);
+    gtk_widget_show(item);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+}
+
 /* pop up a list of open windows from which the user can
    select one to raise and focus */
 
@@ -477,8 +536,11 @@ void window_list_popup (GtkWidget *src, GdkEvent *event,
     GdkEventType evtype;
     GList *wlist = gtk_action_group_list_actions(window_group);
     GList *list;
-    GtkWidget *item, *win = NULL;
+    GtkWidget *item, *lwin;
+    GtkWidget *thiswin = NULL;
     GtkAction *action;
+    int log_up = 0;
+    int icons_up = 0;
 
     if (menu != NULL) {
 	/* we need to make sure this is up to date */
@@ -488,18 +550,25 @@ void window_list_popup (GtkWidget *src, GdkEvent *event,
 
     if (n_listed_windows > 1) {
 	wlist = g_list_sort(wlist, sort_window_list);
-	if (data != NULL) {
-	    win = GTK_WIDGET(data);
-	}
     }
 
+    if (data != NULL) {
+	thiswin = GTK_WIDGET(data);
+    }    
+
     menu = gtk_menu_new();
     list = wlist;
 
     while (list) {
 	action = (GtkAction *) list->data;
-	if (win != NULL) {
-	    maybe_revise_action_label(action, win);
+	lwin = window_from_action(action);
+	if (is_command_log_viewer(lwin)) {
+	    log_up = 1;
+	} else if (widget_is_iconview(lwin)) {
+	    icons_up = 1;
+	}
+	if (n_listed_windows > 1 && thiswin != NULL) {
+	    maybe_revise_action_label(action, thiswin);
 	}
 	gtk_action_set_accel_path(action, NULL);
 	item = gtk_action_create_menu_item(action);
@@ -511,25 +580,25 @@ void window_list_popup (GtkWidget *src, GdkEvent *event,
     g_list_free(wlist);
 
     if (n_listed_windows > 1) {
-	/* add "cascade" menu item */
-	GtkWidget *image;
-
-	item = gtk_image_menu_item_new_with_label(_("Arrange"));
-	image = gtk_image_new_from_stock(GRETL_STOCK_WINLIST, 
-					 GTK_ICON_SIZE_MENU);
-	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), 
-				      image);
-	g_signal_connect(G_OBJECT(item), "activate", 
-			 G_CALLBACK(cascade_session_windows), 
-			 NULL);
-	gtk_widget_show(item);
-	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+	add_cascade_item(menu, item);
     }
 
-    if (win != NULL) {
+    if (!log_up || !icons_up) {
+	item = gtk_separator_menu_item_new();
+	gtk_widget_show(item);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+	if (!log_up) {
+	    add_log_item(menu, item);
+	}
+	if (!icons_up) {
+	    add_iconview_item(menu, item);
+	}
+    }
+
+    if (thiswin != NULL) {
 	g_signal_connect(G_OBJECT(menu), "deactivate", 
 			 G_CALLBACK(winlist_popup_done),
-			 win);
+			 thiswin);
     }
 
     evtype = event != NULL ? event->type : 0;
@@ -562,9 +631,20 @@ void vwin_winlist_popup (GtkWidget *src, GdkEvent *event,
     window_list_popup(src, event, vwin_toplevel(vwin));
 }
 
-/* on exiting, check for any editing windows with unsaved
-   changes, and if we find any give the user a chance to
-   save the changes, or to cancel the exit
+static int window_is_package_editor (GtkWidget *w)
+{
+    const gchar *wname = gtk_widget_get_name(w);
+
+    if (wname != NULL && strcmp(wname, "pkg-editor") == 0) {
+	return 1;
+    } else {
+	return 0;
+    }
+}
+
+/* On exiting, check for any editing windows with unsaved
+   changes: if we find any, give the user a chance to
+   save the changes or cancel the exit.
 */
 
 gboolean window_list_exit_check (void)
@@ -580,14 +660,18 @@ gboolean window_list_exit_check (void)
 	    w = window_from_action((GtkAction *) list->data);
 	    if (w != NULL) {
 		vwin = g_object_get_data(G_OBJECT(w), "vwin");
-		if (vwin != NULL && vwin_is_editing(vwin)) {
-		    if (vwin_content_changed(vwin)) {
+		if (vwin != NULL) {
+		    if (vwin_is_editing(vwin) && vwin_content_changed(vwin)) {
 			gtk_window_present(GTK_WINDOW(vwin->main));
 			ret = query_save_text(NULL, NULL, vwin);
 		    }
-		}
-		if (vwin == NULL && g_object_get_data(G_OBJECT(w), "tabwin")) {
-		    ret = tabwin_exit_check(w);
+		} else {
+		    /* vwin is NULL */
+		    if (g_object_get_data(G_OBJECT(w), "tabwin")) {
+			ret = tabwin_exit_check(w);
+		    } else if (window_is_package_editor(w)) {
+			ret = package_editor_exit_check(w);
+		    }
 		}
 	    }
 	    list = list->next;
@@ -609,6 +693,7 @@ gboolean window_list_exit_check (void)
 			     r == EDIT_PKG_SAMPLE ||	\
 			     r == VIEW_LOG ||		\
 			     r == VIEW_SCRIPT ||	\
+			     r == VIEW_PKG_SAMPLE ||	\
 	                     r == TEXTBOOK_DATA ||	\
 			     r == PS_FILES ||		\
 			     r == NATIVE_DB ||		\
@@ -620,15 +705,9 @@ gboolean window_list_exit_check (void)
 
 static int keep_window_open (GtkWidget *w, gretlopt opt)
 {
-    const gchar *wname = gtk_widget_get_name(w);
+    return window_is_package_editor(w);
 
     /* FIXME maybe keep plot windows open if opt & OPT_P? */
-
-    if (wname != NULL && strcmp(wname, "pkg-editor") == 0) {
-	return 1;
-    } else {
-	return 0;
-    }
 }
 
 /* called from session.c on switching the session: close all
@@ -875,6 +954,34 @@ int get_script_output_number (void)
     return ret;
 }
 
+windata_t *get_unique_output_viewer (void)
+{
+    windata_t *ret = NULL;
+    int vcount = 0;
+
+    if (n_listed_windows > 1) {
+	GList *list = gtk_action_group_list_actions(window_group);
+	windata_t *vwin;
+
+	while (list != NULL) {
+	    vwin = vwin_from_action((GtkAction *) list->data);
+	    if (vwin != NULL && vwin->role == SCRIPT_OUT) {
+		vcount++;
+		if (vcount == 1) {
+		    ret = vwin;
+		} else {
+		    ret = NULL;
+		    break;
+		}
+	    }
+	    list = list->next;
+	}
+	g_list_free(list);
+    }
+
+    return ret;
+}
+
 GtkWidget *get_window_for_data (const gpointer data)
 {
     GtkWidget *ret = NULL;
@@ -1065,42 +1172,64 @@ windata_t *gretl_viewer_new (int role, const gchar *title,
 
 GtkWidget *vwin_toplevel (windata_t *vwin)
 {
-    return vwin->topmain != NULL ? vwin->topmain : vwin->main;
+    if (vwin == NULL) {
+	return NULL;
+    } else {
+	return vwin->topmain != NULL ? vwin->topmain : vwin->main;
+    }
 }
 
-static void menu_bar_add_winlist (windata_t *vwin)
+static GtkWidget *real_add_winlist (windata_t *vwin,
+				    GtkWidget *window,
+				    GtkWidget *hbox)
+{
+    GtkWidget *button, *img, *tbar;
+    GtkToolItem *item;
+
+    button = gtk_button_new();
+    item = gtk_tool_item_new();
+    tbar = gretl_toolbar_new();
+    
+    gtk_widget_set_tooltip_text(GTK_WIDGET(item), _("Windows"));
+    gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
+    img = gtk_image_new_from_stock(GRETL_STOCK_WINLIST, GTK_ICON_SIZE_MENU);
+    gtk_container_add(GTK_CONTAINER(button), img);
+    gtk_container_add(GTK_CONTAINER(item), button);
+
+    if (vwin != NULL) {
+	g_signal_connect(G_OBJECT(button), "button-press-event", 
+			 G_CALLBACK(vwin_winlist_popup), vwin);
+    } else {
+	g_signal_connect(G_OBJECT(button), "button-press-event", 
+			 G_CALLBACK(window_list_popup), window);
+    }
+    gtk_toolbar_insert(GTK_TOOLBAR(tbar), item, -1);
+    gtk_widget_show_all(tbar);
+    gtk_box_pack_end(GTK_BOX(hbox), tbar, FALSE, FALSE, 0);
+
+    return tbar;
+}
+
+void vwin_add_winlist (windata_t *vwin)
 {
-    GtkWidget *img, *button = gtk_button_new();
     GtkWidget *hbox = gtk_widget_get_parent(vwin->mbar);
 
-#if GTK_MAJOR_VERSION == 3 || MAC_NATIVE
-    /* looks better with Adwaita */
-    gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-#elif GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 16
-    /* doesn't work with gtk 3.2.4 or 3.4.0 */
-    GtkStyle *style = gtk_widget_get_style(vwin->mbar);
-    GValue val = {0};
-
-    g_value_init(&val, G_TYPE_INT);
-    gtk_style_get_style_property(style,
-				 GTK_TYPE_MENU_BAR,
-				 "shadow-type", 
-				 &val);
-    if (g_value_get_int(&val) == GTK_SHADOW_NONE) {
-	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
+    if (g_object_get_data(G_OBJECT(hbox), "winlist") == NULL) {
+	GtkWidget *winlist;
+	
+	winlist = real_add_winlist(vwin, NULL, hbox);
+	g_object_set_data(G_OBJECT(hbox), "winlist", winlist);
     }
-    g_value_unset(&val);
-#endif
+}
 
-    img = gtk_image_new_from_stock(GRETL_STOCK_WINLIST, 
-				   GTK_ICON_SIZE_MENU);
-    gtk_container_add(GTK_CONTAINER(button), img);
-    gtk_widget_set_tooltip_text(GTK_WIDGET(button), _("Windows"));
-    g_signal_connect(G_OBJECT(button), "button-press-event",
-		     G_CALLBACK(window_list_popup), 
-		     vwin_toplevel(vwin));
-    gtk_widget_show_all(button);
-    gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+void window_add_winlist (GtkWidget *window, GtkWidget *hbox)
+{
+    if (g_object_get_data(G_OBJECT(hbox), "winlist") == NULL) {
+	GtkWidget *winlist;
+
+	winlist = real_add_winlist(NULL, window, hbox);
+	g_object_set_data(G_OBJECT(hbox), "winlist", winlist);
+    }    
 }
 
 static void destroy_hbox_child (GtkWidget *w, gpointer p)
@@ -1111,17 +1240,25 @@ static void destroy_hbox_child (GtkWidget *w, gpointer p)
     gtk_widget_destroy(w);
 }
 
+static int want_winlist (windata_t *vwin)
+{
+    GtkWidget *hbox = gtk_widget_get_parent(vwin->mbar);
+
+    return g_object_get_data(G_OBJECT(hbox), "winlist") == NULL;
+}
+
 void vwin_pack_toolbar (windata_t *vwin)
 {
     if (vwin->topmain != NULL) {
 	/* @vwin is embedded in a tabbed window */
 	tabwin_register_toolbar(vwin);
-	if (vwin->role == VIEW_MODEL && viewer_n_siblings(vwin) == 0) {
-	    menu_bar_add_winlist(vwin);
-	}
     } else {
 	GtkWidget *hbox;
 
+	/* check for presence of a temporary "top-hbox" -- as
+	   in a script output window that's waiting for full
+	   output
+	*/
 	hbox = g_object_get_data(G_OBJECT(vwin->main), "top-hbox");
 
 	if (hbox != NULL) {
@@ -1136,7 +1273,6 @@ void vwin_pack_toolbar (windata_t *vwin)
 		vwin->role == VECM) {
 		/* model viewer: the menubar extends full-length */
 		gtk_box_pack_start(GTK_BOX(hbox), vwin->mbar, TRUE, TRUE, 0);
-		menu_bar_add_winlist(vwin);
 	    } else {
 		gtk_box_pack_start(GTK_BOX(hbox), vwin->mbar, FALSE, FALSE, 0);
 	    }
@@ -1147,6 +1283,34 @@ void vwin_pack_toolbar (windata_t *vwin)
 	    gtk_widget_show_all(hbox);
 	}
     }
+
+    if (want_winlist(vwin)) {
+	vwin_add_winlist(vwin);
+    }
+}
+
+void vwin_reinstate_toolbar (windata_t *vwin)
+{
+    GtkWidget *hbox;
+
+    hbox = g_object_get_data(G_OBJECT(vwin->main), "top-hbox");
+
+    if (hbox != NULL) {
+	/* destroy the temporary stuff, put the "real" stuff back,
+	   and drop the extra references 
+	*/
+	GtkWidget *winlist;
+	
+	gtk_container_foreach(GTK_CONTAINER(hbox), destroy_hbox_child, NULL);
+	gtk_box_pack_start(GTK_BOX(hbox), vwin->mbar, FALSE, FALSE, 0);
+	g_object_unref(G_OBJECT(vwin->mbar));
+	winlist = g_object_get_data(G_OBJECT(hbox), "winlist");
+	if (winlist != NULL) {
+	    gtk_box_pack_end(GTK_BOX(hbox), winlist, FALSE, FALSE, 0);
+	    g_object_unref(G_OBJECT(winlist));
+	}
+	gtk_widget_show_all(hbox);
+    }
 }
 
 windata_t *gretl_browser_new (int role, const gchar *title)
