@@ -51,6 +51,8 @@ GtkWidget *get_window_for_plot (const char *plotfile);
 
 int get_script_output_number (void);
 
+windata_t *get_unique_output_viewer (void);
+
 void maybe_close_window_for_user_var (const gpointer data,
 				      GretlObjType otype);
 
@@ -58,6 +60,10 @@ void close_session_windows (gretlopt opt);
 
 void cascade_session_windows (void);
 
+void vwin_add_winlist (windata_t *vwin);
+
+void window_add_winlist (GtkWidget *window, GtkWidget *hbox);
+
 windata_t *vwin_new (int role, gpointer data);
 
 int highest_numbered_variable_in_winstack (void);
@@ -76,6 +82,8 @@ GtkWidget *vwin_toplevel (windata_t *vwin);
 
 void vwin_pack_toolbar (windata_t *vwin);
 
+void vwin_reinstate_toolbar (windata_t *vwin);
+
 void gretl_viewer_present (windata_t *vwin);
 
 void gretl_viewer_destroy (windata_t *vwin);
