@@ -60,6 +60,8 @@ typedef struct {
     const char *literal;
 } PLOTGROUP;
 
+#define BDEBUG 0
+
 #define show_mean(g)    (g->flags & BOX_SHOW_MEAN)
 #define do_intervals(g) (g->flags & BOX_INTERVALS)
 #define whiskerbars(g)  (g->flags & BOX_WHISKBARS)
@@ -74,20 +76,43 @@ typedef struct {
 #define unset_show_mean(g)    (g->flags &= ~BOX_SHOW_MEAN)
 #define unset_do_intervals(g) (g->flags &= ~BOX_INTERVALS)
 
-#define BPSTRLEN 128
-
-static char boxplots_string[BPSTRLEN];
-
-const char *get_last_boxplots_string (void)
-{
-    return boxplots_string;
-}
-
 static void quartiles_etc (double *x, int n, BOXPLOT *plt,
 			   double limit)
 {
     double p[3] = {0.25, 0.5, 0.75};
 
+    if (n <= 3) {
+	int i;
+
+	if (n == 1) {
+	    plt->wmin = plt->min = x[0];
+	    plt->wmax = plt->max = x[0];
+	    plt->mean = plt->median = NADBL;
+	    plt->lq = plt->uq = NADBL;
+	} else if (n == 2) {
+	    plt->wmin = plt->min = x[0];
+	    plt->wmax = plt->max = x[1];
+	    plt->mean = (x[0] + x[1]) / 2.0;
+	    plt->median = plt->lq = plt->uq = NADBL;
+	} else {
+	    plt->wmin = plt->min = x[0];
+	    plt->wmax = plt->max = x[2];
+	    plt->median = x[1];
+	    plt->lq = plt->uq = NADBL;
+	    plt->mean = (x[0] + x[1] + x[2]) / 3.0;
+	}
+
+	plt->outliers = gretl_vector_alloc(n);
+
+	if (plt->outliers != NULL) {
+	    for (i=0; i<n; i++) {
+		gretl_vector_set(plt->outliers, i, x[i]);
+	    }
+	}	
+	
+	return;
+    }
+
     plt->wmin = plt->min = x[0];
     plt->wmax = plt->max = x[n-1];
 
@@ -97,6 +122,15 @@ static void quartiles_etc (double *x, int n, BOXPLOT *plt,
     plt->median = p[1];
     plt->uq = p[2];
 
+    if (na(plt->lq) || na(plt->uq)) {
+	/* ensure that Q1 and Q3 are either both present
+	   or both missing
+	*/
+	plt->lq = plt->uq = NADBL;
+    }
+
+    plt->mean = gretl_mean(0, n - 1, x);
+
     if (limit > 0) {
 	double d = limit * (plt->uq - plt->lq);
 	double xlo = plt->lq - d;
@@ -133,17 +167,34 @@ static void quartiles_etc (double *x, int n, BOXPLOT *plt,
     }
 }
 
-static void six_numbers (BOXPLOT *plt, int do_mean, PRN *prn)
+/* backward compatibility, for when we were writing NAs into
+   gretl boxplot files using "%g"
+*/
+#define bpna(x) (na(x) || x > 1.0e308)
+
+static void six_numbers (BOXPLOT *p, int do_mean, PRN *prn)
 {
+    double vals[5] = {p->min, p->lq, p->median, p->uq, p->max};
+    int i;
+
     if (do_mean) {
-	pprintf(prn, "%#9.5g", plt->mean);
+	if (bpna(p->mean)) {
+	    pprintf(prn, "%9s", "NA");
+	} else {
+	    pprintf(prn, "%#9.5g", p->mean);
+	}
     }
 
-    pprintf(prn, "%#9.5g%#9.5g%#9.5g%#9.5g%#9.5g", plt->min, 
-	    plt->lq, plt->median, plt->uq, plt->max);
+    for (i=0; i<5; i++) {
+	if (bpna(vals[i])) {
+	    pprintf(prn, "%9s", "NA");
+	} else {
+	    pprintf(prn, "%#9.5g", vals[i]);
+	}
+    }
 
-    if (plt->n > 0) {
-	pprintf(prn, "  (n=%d)\n", plt->n);
+    if (p->n > 0) {
+	pprintf(prn, "  (n=%d)\n", p->n);
     } else {
 	pputc(prn, '\n');
     }
@@ -178,15 +229,22 @@ static void box_stats_leader (PLOTGROUP *grp, int i, int offset,
 
 static int has_mean (PLOTGROUP *grp)
 {
+    BOXPLOT *plt;
     int i;
 
     for (i=0; i<grp->nplots; i++) {
-	if (na(grp->plots[i].mean)) {
-	    return 0;
+	plt = &grp->plots[i];
+	if (na(plt->mean)) {
+	    if (plt->n == 1) {
+		plt->mean = plt->min;
+	    }
+	}
+	if (!na(plt->mean)) {
+	    return 1;
 	}
     }
 
-    return 1;
+    return 0;
 }
 
 static int get_format_offset (PLOTGROUP *grp)
@@ -314,20 +372,6 @@ median_interval (double *x, int n, double *low, double *high)
     return err;
 }
 
-static int special_varcount (const char *s)
-{
-    char test[36];
-    int n = 0;
-
-    while (sscanf(s, "%35s", test) == 1) {
-	if (*test != '(') n++;
-	s += strspn(s, " ");
-	s += strlen(test);
-    }
-
-    return n;
-}
-
 static void plotgroup_destroy (PLOTGROUP *grp)
 {
     int i;
@@ -376,6 +420,11 @@ static int plotgroup_factor_setup (PLOTGROUP *grp,
 	grp->nplots = gretl_vector_get_length(grp->dvals);
     }
 
+#if BDEBUG
+    fprintf(stderr, "plotgroup_factor_setup, nplots=%d\n", grp->nplots);
+    gretl_matrix_print(grp->dvals, "discrete vals");
+#endif
+
     return err;
 }
 
@@ -422,6 +471,7 @@ static PLOTGROUP *plotgroup_new (const int *list,
 
     if (!err) {
 	if (opt & OPT_Z) {
+	    /* --factorized */
 	    if (dset != NULL) {
 		strcpy(grp->ylabel, dset->varname[list[1]]);
 		strcpy(grp->xlabel, dset->varname[list[2]]);
@@ -501,13 +551,11 @@ static void get_box_y_range (PLOTGROUP *grp, double gyrange,
     }
 }
 
-static int write_gnuplot_boxplot (PLOTGROUP *grp, 
-				  gretlopt opt, 
-				  const char **labels)
+static int write_gnuplot_boxplot (PLOTGROUP *grp, gretlopt opt)
 {
     FILE *fp = NULL;
     BOXPLOT *bp;
-    int n = grp->nplots;
+    int np = grp->nplots;
     double lpos, h, gyrange;
     double ymin, ymax;
     int anybool, n_outliers;
@@ -536,7 +584,7 @@ static int write_gnuplot_boxplot (PLOTGROUP *grp,
     get_box_y_range(grp, gyrange, &ymin, &ymax);
 
     if (opt & (OPT_Z | OPT_P)) {
-	if (n > 1 && n <= 30) {
+	if (np > 1 && np <= 30) {
 	    set_use_xtics(grp);
 	}
     } 
@@ -546,10 +594,11 @@ static int write_gnuplot_boxplot (PLOTGROUP *grp,
     } 
 
     fputs("set nokey\n", fp);
+    fputs("set datafile missing \"?\"\n", fp);
 
     gretl_push_c_numeric_locale();
 
-    fprintf(fp, "set xrange [0:%d]\n", n + 1);
+    fprintf(fp, "set xrange [0:%d]\n", np + 1);
     fprintf(fp, "set yrange [%g:%g]\n", ymin, ymax);
 
     if (*grp->ylabel != '\0') {
@@ -560,7 +609,7 @@ static int write_gnuplot_boxplot (PLOTGROUP *grp,
 	fprintf(fp, "set xlabel \"%s\"\n", grp->xlabel);
     }    
 
-    if (n > 30) {
+    if (np > 30) {
 	lwidth = 1;
     } else {
 	fputs("set ytics nomirror\n", fp);
@@ -568,17 +617,14 @@ static int write_gnuplot_boxplot (PLOTGROUP *grp,
 
 	if (use_xtics(grp)) {
 	    fputs("set xtics (", fp);
-	    for (i=0; i<n; i++) {
+	    for (i=0; i<np; i++) {
+		bp = &grp->plots[i];
 		if (opt & OPT_Z) {
-		    if (labels==NULL) {
-			fprintf(fp, "\"%g\" %d", grp->dvals->val[i], i+1);
-		    } else {
-			fprintf(fp, "\"%s\" %d", labels[i], i+1);
-		    }
+		    fprintf(fp, "\"%s\" %d", bp->bool, i+1);
 		} else {
-		    fprintf(fp, "\"%s\" %d", grp->plots[i].varname, i+1);
+		    fprintf(fp, "\"%s\" %d", bp->varname, i+1);
 		}
-		if (i < n - 1) {
+		if (i < np - 1) {
 		    fputs(", ", fp);
 		} else {
 		    fputs(") scale 0.0\n", fp);
@@ -587,15 +633,16 @@ static int write_gnuplot_boxplot (PLOTGROUP *grp,
 	    fputs("set xtics nomirror\n", fp);
 	} else {
 	    fputs("set noxtics\n", fp);
-	    if (n > 1 || anybool) {
+	    if (np > 1 || anybool) {
 		fprintf(fp, "set bmargin %d\n", (anybool)? 4 : 3);
-		for (i=0; i<n; i++) {
+		for (i=0; i<np; i++) {
+		    bp = &grp->plots[i];
 		    fprintf(fp, "set label \"%s\" at %d,%g center\n", 
-			    grp->plots[i].varname, i+1, lpos);
+			    bp->varname, i+1, lpos);
 		    if (grp->plots[i].bool != NULL) {
 			/* FIXME positioning? */
 			fprintf(fp, "set label \"%s\" at %d,%g center\n", 
-				grp->plots[i].bool, i+1, lpos - .8 * h);
+				bp->bool, i+1, lpos - .8 * h);
 		    }
 		}
 	    }
@@ -639,38 +686,57 @@ static int write_gnuplot_boxplot (PLOTGROUP *grp,
 	fputc('\n', fp);
     }
 
-    /* data for quartiles and whiskers */
-    for (i=0; i<n; i++) {
+    /* data for quartiles and whiskers, plus plot->n */
+    for (i=0; i<np; i++) {
 	bp = &grp->plots[i];
 	if (na(bp->wmin)) bp->wmin = bp->min;
 	if (na(bp->wmax)) bp->wmax = bp->max;
-	fprintf(fp, "%d %g %g %g %g\n", i+1, bp->wmin, bp->lq, bp->uq, bp->wmax);
+	fprintf(fp, "%d %g", i+1, bp->wmin);
+	if (na(bp->lq)) {
+	    fputs(" ?", fp);
+	} else {
+	    fprintf(fp, " %g", bp->lq);
+	}
+	if (na(bp->uq)) {
+	    fputs(" ?", fp);
+	} else {
+	    fprintf(fp, " %g", bp->uq);
+	}
+	fprintf(fp, " %g %d\n", bp->wmax, bp->n);
     }
     fputs("e\n", fp);
 
     /* data for median */
-    for (i=0; i<n; i++) {
+    for (i=0; i<np; i++) {
 	bp = &grp->plots[i];
-	fprintf(fp, "%d %g\n", i+1, bp->median);
+	if (na(bp->median)) {
+	    fprintf(fp, "%d ?\n", i+1);
+	} else {
+	    fprintf(fp, "%d %g\n", i+1, bp->median);
+	}
     }
     fputs("e\n", fp);
 
     if (show_mean(grp)) {
 	/* data for mean */
-	for (i=0; i<n; i++) {
+	for (i=0; i<np; i++) {
 	    bp = &grp->plots[i];
-	    fprintf(fp, "%d %g\n", i+1, bp->mean);
+	    if (na(bp->mean)) {
+		fprintf(fp, "%d ?\n", i+1);
+	    } else {
+		fprintf(fp, "%d %g\n", i+1, bp->mean);
+	    }
 	}
 	fputs("e\n", fp);
     } else if (do_intervals(grp)) {
 	/* data for lower median bound */
-	for (i=0; i<n; i++) {
+	for (i=0; i<np; i++) {
 	    bp = &grp->plots[i];
 	    fprintf(fp, "%d %g\n", i+1, bp->conf[0]);
 	}
 	fputs("e\n", fp);
 	/* data for upper median bound */
-	for (i=0; i<n; i++) {
+	for (i=0; i<np; i++) {
 	    bp = &grp->plots[i];
 	    fprintf(fp, "%d %g\n", i+1, bp->conf[1]);
 	}
@@ -681,7 +747,7 @@ static int write_gnuplot_boxplot (PLOTGROUP *grp,
 	int j, nout;
 
 	fprintf(fp, "# auxdata %d 2\n", n_outliers);
-	for (i=0; i<n; i++) {
+	for (i=0; i<np; i++) {
 	    bp = &grp->plots[i];
 	    if (bp->outliers != NULL) {
 		nout = gretl_vector_get_length(bp->outliers);
@@ -716,15 +782,16 @@ static int transcribe_array_factorized (PLOTGROUP *grp, int i,
 }
 
 static int factorized_boxplot_check (const int *list, 
-				     char **bools, 
 				     int *haslabels,
 				     const DATASET *dset)
 {
     int err = 0;
 
-    if (list[0] != 2 || bools != NULL) {
+    if (list[0] != 2) {
+	/* exactly two variables needed */
 	err = E_DATA;
     } else {
+	/* the second of which must be discrete */
 	int v2 = list[2];
 	
 	if (!series_is_discrete(dset, v2) &&
@@ -732,9 +799,9 @@ static int factorized_boxplot_check (const int *list,
 	    gretl_errmsg_set(_("You must supply two variables, the second of "
 			       "which is discrete"));
 	    err = E_DATA;
+	} else {
+	    *haslabels = is_string_valued(dset, v2);
 	}
-
-	*haslabels = is_string_valued(dset, v2);
     }
 
     return err;
@@ -756,17 +823,17 @@ static int all_missing (int t1, int t2, const double *x)
 /* build the boxplot group data structure, then pass it to 
    gnuplot_do_boxplot to write the command file */
 
-static int real_boxplots (const int *list, char **bools, 
+static int real_boxplots (const int *list,
 			  const char *literal,
 			  const DATASET *dset,
 			  gretlopt opt)
 {
     PLOTGROUP *grp;
+    const char **labels = NULL;
     double lim = NADBL;
     int i, k, np;
+    int haslabels = 0;
     int err = 0;
-    int haslabels;
-    const char **labels = NULL;
 
     if (opt & OPT_L) {
 	/* we should have an explicit outlier limit */
@@ -788,12 +855,11 @@ static int real_boxplots (const int *list, char **bools,
     }
 
     if (opt & OPT_Z) {
-	err = factorized_boxplot_check(list, bools, &haslabels, dset);
+	/* --factorized */
+	err = factorized_boxplot_check(list, &haslabels, dset);
 	if (err) {
 	    return err;
-	} 
-
-	if (haslabels) {
+	} else if (haslabels) {
 	    int n_labels;
 
 	    labels = (const char **)
@@ -809,8 +875,8 @@ static int real_boxplots (const int *list, char **bools,
     if (*grp->ylabel != '\0' && *grp->xlabel != '\0') {
 	grp->title = gretl_strdup_printf(_("Distribution of %s by %s"),
 					 grp->ylabel, grp->xlabel);
-    } else if (list[0] == 1 && bools == NULL) {
-	/* doing a single, standard plot */
+    } else if (list[0] == 1) {
+	/* doing a single plot */
 	const char *s = series_get_label(dset, list[1]);
 	char yname[VNAMELEN];
 
@@ -845,19 +911,19 @@ static int real_boxplots (const int *list, char **bools,
 	}
 
 	if (n < 2) {
+	    /* less than two observations */
 	    if (grp->nplots == 1) {
 		/* we'll be out of data */
 		err = E_DATA;
 		break;
-	    } else {
-		/* discard this plot */
+	    } else if (n == 0) {
+		/* discard this particular plot */
 		grp->nplots -= 1;
 		continue;
 	    }
 	}
 
 	plot = &grp->plots[k]; /* note: k rather than i */
-	plot->mean = gretl_mean(0, n - 1, grp->x);
 	qsort(grp->x, n, sizeof *grp->x, gretl_compare_doubles);
 	quartiles_etc(grp->x, n, plot, grp->limit);
 	
@@ -881,12 +947,13 @@ static int real_boxplots (const int *list, char **bools,
 	strcpy(plot->varname, vname);
 
 	if (opt & OPT_Z) {
-	    plot->bool = gretl_strdup_printf("%g", grp->dvals->val[i]);
+	    if (labels != NULL) {
+		plot->bool = gretl_strdup(labels[i]);
+	    } else {
+		plot->bool = gretl_strdup_printf("%g", grp->dvals->val[i]);
+	    }
 	    grp->n_bools += 1;
-	} else if (bools != NULL && bools[i] != NULL) {
-	    plot->bool = bools[i];
-	    grp->n_bools += 1;
-	} 
+	}
 
 	k++; /* increment the actually used plot index */
     }
@@ -895,7 +962,7 @@ static int real_boxplots (const int *list, char **bools,
 	if (!do_intervals(grp)) {
 	    set_show_mean(grp);
 	}
-	err = write_gnuplot_boxplot(grp, opt, labels);
+	err = write_gnuplot_boxplot(grp, opt);
     }
 
     plotgroup_destroy(grp);   
@@ -942,7 +1009,7 @@ static int panel_group_boxplots (int vnum, const DATASET *dset,
 	}
     }
 
-    err = real_boxplots(list, NULL, NULL, gdset, opt);
+    err = real_boxplots(list, NULL, gdset, opt);
 
     destroy_dataset(gdset);
     free(list);
@@ -956,7 +1023,7 @@ static int panel_group_boxplots (int vnum, const DATASET *dset,
  * @dset: dataset struct.
  * @opt: may include OPT_P for a plot by panel group (in which
  * case @list should have just one member), or OPT_Z for a
- * "factorized" plot (@list should have exactly three members),
+ * "factorized" plot (@list should have exactly two members),
  * and/or OPT_L to set the limit for outliers.
  *
  * Constructs a boxplot command file for use with gnuplot.
@@ -977,189 +1044,12 @@ int boxplots (const int *list, const char *literal,
 	    err = panel_group_boxplots(list[1], dset, opt);
 	}
     } else {
-	err = real_boxplots(list, NULL, literal, dset, opt);
+	err = real_boxplots(list, literal, dset, opt);
     }
 
     return err;
 }
 
-/* remove extra spaces around operators in boxplots line */
-
-static char *boxplots_fix_parentheses (const char *line, int *err)
-{
-    char *s, *p;
-    int inparen = 0;
-    int i, flen, len = strlen(line);
-
-    /* make room to insert space before parens, if needed */
-    flen = len;
-    for (i=0; i<len; i++) {
-	if (i > 0 && line[i] == '(' && line[i-1] != ' ') {
-	    flen++;
-	}
-    }
-
-    s = malloc(flen + 1);
-    if (s == NULL) {
-	*err = E_ALLOC;
-	return NULL;
-    }
-
-    p = s;
-    for (i=0; i<len; i++) {
-	if (line[i] == '(') {
-	    if (i > 0 && line[i-1] != ' ') {
-		*p++ = ' ';
-	    }
-	    inparen = 1;
-	}
-	if (line[i] == ')') {
-	    if (inparen == 1) {
-		inparen = 0;
-	    } else {
-		*err = E_PARSE;
-		free(s);
-		return NULL;
-	    }
-	}
-	if (inparen && line[i] == ' ') ;
-	else *p++ = line[i];
-    }
-
-    *p = 0;
-
-    return s;
-}
-
-int boolean_boxplots (const char *line, const char *literal,
-		      DATASET *dset, gretlopt opt)
-{
-    int i, k, v, nvars, nbool;
-    int n = dset->n;
-    int origv = dset->v;
-    char *tok, *s = NULL;
-    char **bools = NULL;
-    int *list = NULL;
-    int err = 0;
-
-    if (!strncmp(line, "boxplots ", 9)) {
-	line += 9;
-    } else if (!strncmp(line, "boxplot ", 8)) {
-	line += 8;
-    }
-
-    s = boxplots_fix_parentheses(line, &err);
-
-    if (!err) {
-	nvars = special_varcount(s);
-	if (nvars == 0) {
-	    err = E_ARGS;
-	}
-    }
-
-    if (!err) {
-	list = gretl_list_new(nvars);
-	bools = strings_array_new(nvars);
-	if (list == NULL || bools == NULL) {
-	    err = E_ALLOC;
-	}
-    }
-
-    if (err) {
-	free(list);
-	free(bools);
-	free(s);
-	return err;
-    }
-
-    i = nbool = 0;
-
-    /* record the command string */
-    *boxplots_string = '\0';
-    strncat(boxplots_string, s, BPSTRLEN - 1);
-
-    while (!err && (tok = strtok((i)? NULL : s, " "))) {
-	if (tok[0] == '(') {
-	    if (i) {
-		bools[i-1] = gretl_strdup(tok);
-		nbool++;
-	    } else {
-		err = E_DATA;
-	    }
-	} else {
-	    if (isdigit(tok[0])) { 
-		v = atoi(tok);
-		if (v < origv) {
-		    list[++i] = v;
-		} else {
-		    gretl_errmsg_sprintf(_("got invalid variable number %d"), v);
-		    err = E_DATA;
-		}
-	    } else if (isalpha(tok[0])) {
-		v = current_series_index(dset, tok);
-		if (v >= 0) {
-		    list[++i] = v;
-		} else {
-		    gretl_errmsg_sprintf(_("got invalid varname '%s'"), tok);
-		    err = E_DATA;
-		}
-	    } else {
-		gretl_errmsg_sprintf(_("got invalid field '%s'"), tok);
-		err = E_DATA; 
-	    }
-	}
-    }
-
-    /* Now we add nbool new variables, with ID numbers origv,
-       origv + 1, and so on.  These are the original variables
-       that have boolean conditions attached, masked by those
-       conditions. 
-    */
-
-    k = origv;
-    nbool = 0;
-
-    for (i=0; i<nvars && !err; i++) {
-	char formula[80];
-	
-	if (bools[i] == NULL) {
-	    continue;
-	}
-
-	sprintf(formula, "bool_%d = %s", i, bools[i]);
-	err = generate(formula, dset, OPT_P, NULL);
-
-	if (!err) {
-	    int t, vi = list[i+1];
-
-	    for (t=0; t<n; t++) {
-		if (dset->Z[k][t] == 1.0) {
-		    dset->Z[k][t] = dset->Z[vi][t];
-		} else { 
-		    dset->Z[k][t] = NADBL;
-		}
-	    }
-	    strcpy(dset->varname[k], dset->varname[vi]);
-	    list[i+1] = k++;
-	    nbool++;
-	}
-    }
-
-    if (!err) {
-	err = real_boxplots(list, bools, literal, dset, opt);
-    } 
-    
-    free(list);
-    free(bools); /* the bool[i]s are now attached to the plots */
-    free(s);
-
-    if (nbool) {
-	dataset_drop_last_variables(dset, nbool);
-    }
-    
-    return err;
-}
-
 /* parse "factor" values out of the xtics line in a gretl
    boxplot file, e.g. ("0" 1, "1" 2, "4", 3) */
 
@@ -1230,6 +1120,54 @@ static int read_plot_outliers (PLOTGROUP *grp, char *line,
     return err;
 }
 
+static int scan_five (BOXPLOT *plot, int *j, const char *s)
+{
+    char lq[16], uq[16];
+    int nf, nobs = 0;
+
+    nf = sscanf(s, "%d %lf %15s %15s %lf %d", j, &plot->min, 
+		lq, uq, &plot->max, &nobs);
+
+    if (nf >= 5) {
+	if (nf == 6) {
+	    plot->n = nobs;
+	}
+	if (!strcmp(lq, "?")) {
+	    plot->lq = NADBL;
+	} else {
+	    nf--;
+	    nf += sscanf(lq, "%lf", &plot->lq);
+	}
+	if (!strcmp(uq, "?")) {
+	    plot->uq = NADBL;
+	} else {
+	    nf--;
+	    nf += sscanf(uq, "%lf", &plot->uq);	    
+	}
+    }
+
+    return nf;
+}
+
+static int scan_two (int *j, double *x, const char *s)
+{
+    char test[16];
+    int nf;
+
+    nf = sscanf(s, "%d %15s", j, test);
+
+    if (nf == 2) {
+	if (!strcmp(test, "?")) {
+	    *x = NADBL;
+	} else {
+	    nf--;
+	    nf += sscanf(test, "%lf", x);
+	}
+    }
+
+    return nf;
+}
+
 /* read data out of a boxplots file in order to reconstruct
    numerical summary */
 
@@ -1245,26 +1183,27 @@ static int read_plot_data_block (PLOTGROUP *grp, char *line,
 	plot = &grp->plots[i];
 	targ = 2; /* the number of values we should read */
 	nf = 0;   /* the number of values actually read */
+
 	if (*block == 0) {
 	    /* min, Q1, Q3, max */
 	    targ = 5;
-	    nf = sscanf(line, "%d %lf %lf %lf %lf", 
-			&j, &plot->min, &plot->lq, &plot->uq, &plot->max);
+	    nf = scan_five(plot, &j, line);
 	} else if (*block == 1) {
 	    /* median */
-	    nf = sscanf(line, "%d %lf", &j, &plot->median);
+	    nf = scan_two(&j, &plot->median, line);
 	} else if (*block == 2) {
 	    /* mean, or lower bound of c.i. */
 	    if (do_intervals(grp)) {
-		nf = sscanf(line, "%d %lf", &j, &plot->conf[0]);
+		nf = scan_two(&j, &plot->conf[0], line);
 	    } else {
-		nf = sscanf(line, "%d %lf", &j, &plot->mean);
+		nf = scan_two(&j, &plot->mean, line);
 	    }
 	} else if (*block == 3) {
 	    /* upper bound of c.i. */
-	    nf = sscanf(line, "%d %lf", &j, &plot->conf[1]);
+	    nf = scan_two(&j, &plot->conf[1], line);
 	}
-	if (nf != targ || j != i + 1) {
+
+	if (nf < targ || j != i + 1) {
 	    /* data read screwed up */
 	    err = E_DATA;
 	} else if (fgets(line, lsize, fp) == NULL) {
