@@ -396,11 +396,11 @@ static void print_compare (struct COMPARE *cmp,
     if (cmp->score >= 0) {
 	pputs(prn, "  ");
 	if (cmp->ci == ADD) {
-	    pprintf(prn, _("Adding variables improved %d of %d model "
-			   "selection statistics.\n"), cmp->score, C_MAX);
+	    pprintf(prn, _("Adding variables improved %d of %d information "
+			   "criteria.\n"), cmp->score, C_MAX);
 	} else {
-	    pprintf(prn, _("Omitting variables improved %d of %d model "
-			   "selection statistics.\n"), cmp->score, C_MAX);
+	    pprintf(prn, _("Omitting variables improved %d of %d information "
+			   "criteria.\n"), cmp->score, C_MAX);
 	}
     } 
 
@@ -581,51 +581,47 @@ static int get_extra_var (const MODEL *pmod)
     }
 }
 
-/* reconstitute full varlist for WLS, AR and count models */
+/* reconstitute full varlist for WLS, AR, count and
+   duation models */
 
 static int *
 full_model_list (const MODEL *pmod, const int *inlist)
 {
-    int i, len = 0, pos = 0;
     int *flist = NULL;
 
-    if (pmod->ci == WLS) { 
-	len = inlist[0] + 2;
-    } else if (COUNT_MODEL(pmod->ci)) {
-	len = inlist[0] + 3;
-    } else if (pmod->ci == AR) {
-	pos = pmod->arinfo->arlist[0] + 1;
-	len = pos + inlist[0] + 2;
-    }
-
-    flist = malloc(len * sizeof *flist);
-    if (flist == NULL) {
-	return NULL;
-    }
-
-    if (pmod->ci == WLS) { 
-	flist[0] = len - 1;
-	flist[1] = pmod->nwt;
-	for (i=1; i<=inlist[0]; i++) {
-	    flist[i+1] = inlist[i];
+    if (pmod->ci == AR) {
+	/* cobble together arlist and @inlist */
+	flist = gretl_lists_join_with_separator(pmod->arinfo->arlist,
+						inlist);
+    } else {
+	int i, len = inlist[0];
+    
+	if (pmod->ci == WLS) {
+	    /* prepend the weight variable */
+	    len += 1;
+	} else if (COUNT_MODEL(pmod->ci) || pmod->ci == DURATION) {
+	    /* append list separator and offset or censoring var */
+	    len += 2;
 	}
-    } else if (COUNT_MODEL(pmod->ci) || pmod->ci == DURATION) {
-	int extra = get_extra_var(pmod);
 
-	flist[0] = len - 1;
-	for (i=1; i<=inlist[0]; i++) {
-	    flist[i] = inlist[i];
+	flist = gretl_list_new(len);
+	if (flist == NULL) {
+	    return NULL;
 	}
-	flist[flist[0] - 1] = LISTSEP;
-	flist[flist[0]] = extra;
-    } else if (pmod->ci == AR) {
-	flist[0] = len - 2;
-	for (i=1; i<pos; i++) {
-	    flist[i] = pmod->arinfo->arlist[i];
-	}
-	flist[pos] = LISTSEP;
-	for (i=1; i<=inlist[0]; i++) {
-	    flist[pos+i] = inlist[i];
+
+	if (pmod->ci == WLS) { 
+	    flist[1] = pmod->nwt;
+	    for (i=1; i<=inlist[0]; i++) {
+		flist[i+1] = inlist[i];
+	    }
+	} else if (COUNT_MODEL(pmod->ci) || pmod->ci == DURATION) {
+	    int extra = get_extra_var(pmod);
+
+	    for (i=1; i<=inlist[0]; i++) {
+		flist[i] = inlist[i];
+	    }
+	    flist[flist[0]-1] = LISTSEP;
+	    flist[flist[0]] = extra;
 	}
     }
 
@@ -747,9 +743,9 @@ static MODEL replicate_estimator (const MODEL *orig, int *list,
 	} else if (orig->opt & OPT_U) {
 	    /* random effects */
 	    myopt |= OPT_U;
-	} else if (orig->opt & OPT_W) {
+	} else if (orig->opt & OPT_H) {
 	    /* unit weights */
-	    myopt |= OPT_W;
+	    myopt |= OPT_H;
 	    if (gretl_model_get_int(orig, "iters")) {
 		myopt |= OPT_I;
 	    }
@@ -2099,20 +2095,19 @@ static int lb_autocorr_test (MODEL *pmod, int order,
 			     gretlopt opt, PRN *prn)
 {
     double lb, pval;
-    int npq, df = order;
-    int err = 0;
+    int df, err = 0;
 
-    npq = arma_model_get_n_arma_coeffs(pmod);
-    if (npq < df) {
-	df -= npq; /* FIXME? */
+    df = order - arma_model_get_n_arma_coeffs(pmod);
+
+    if (df < 0) {
+	gretl_errmsg_set("Insufficient degrees of freedom for test");
+	return E_DATA;
     }
 
     lb = ljung_box(order, pmod->t1, pmod->t2, pmod->uhat, &err);
 
-    if (err) {
-	lb = pval = NADBL;
-    } else {
-	pval = chisq_cdf_comp(order, lb);
+    if (!err) {
+	pval = chisq_cdf_comp(df, lb);
 	if (na(pval)) {
 	    err = E_DATA;
 	} 
@@ -2348,18 +2343,49 @@ static int chow_active (int split, const double *x, int t)
     } 
 }
 
+static int *get_break_list (const MODEL *pmod, int ci, int *err)
+{
+    const char *lname = get_optval_string(ci, OPT_L);
+    int *list = NULL;
+
+    if (lname == NULL) {
+	*err = E_DATA;
+    } else {
+	list = get_list_by_name(lname);
+    }
+
+    if (list != NULL) {
+	if (list[0] == 0) {
+	    *err = E_DATA;
+	} else {
+	    int i;
+
+	    for (i=1; i<=list[0] && !*err; i++) {
+		if (!in_gretl_list(pmod->list, list[i])) {
+		    *err = E_DATA;
+		} else if (list[i] == 0) {
+		    *err = E_DATA;
+		}
+	    }
+	}
+    }
+
+    return list;
+}
+
 /* compose list of variables to be used for the Chow test and add
    them to the data set */
 
 static int *
 make_chow_list (const MODEL *pmod, DATASET *dset,
-		int split, int dumv, int *err)
+		int split, int dumv, int ci,
+		gretlopt opt, int *err)
 {
     int *chowlist = NULL;
+    int *brklist = NULL;
     int l0 = pmod->list[0];
-    int ninter = pmod->ncoeff - pmod->ifc; /* number of interaction terms */
+    int ninter = 0, newvars = 0;
     int havedum = (dumv > 0);
-    int newvars = ninter + 1 - havedum;
     int i, t, v = dset->v;
 
     if (havedum && in_gretl_list(pmod->list, dumv)) {
@@ -2369,6 +2395,21 @@ make_chow_list (const MODEL *pmod, DATASET *dset,
 	return NULL;
     }
 
+    if (opt & OPT_L) {
+	/* --limit-to */
+	brklist = get_break_list(pmod, ci, err);
+	if (*err) {
+	    return NULL;
+	} else {
+	    ninter = brklist[0];
+	}
+    } else {
+	/* number of interaction terms */
+	ninter = pmod->ncoeff - pmod->ifc;
+    }
+
+    newvars = ninter + 1 - havedum;
+
     if (dataset_add_series(dset, newvars)) {
 	*err = E_ALLOC;
     } else {
@@ -2401,8 +2442,13 @@ make_chow_list (const MODEL *pmod, DATASET *dset,
 
 	/* and the interaction terms */
 	for (i=0; i<ninter; i++) {
-	    int pv = pmod->list[i + 2 + pmod->ifc];
-	    int sv = v + i + 1 - havedum;
+	    int pv, sv = v + i + 1 - havedum;
+
+	    if (brklist != NULL) {
+		pv = brklist[i+1];
+	    } else {
+		pv = pmod->list[i + 2 + pmod->ifc];
+	    }
 
 	    for (t=0; t<dset->n; t++) {
 		if (model_missing(pmod, t)) {
@@ -2427,26 +2473,73 @@ make_chow_list (const MODEL *pmod, DATASET *dset,
     return chowlist;
 }
 
+static void write_plot_x_range (const double *x, int t1, int t2,
+				FILE *fp)
+{
+    double xmin0, xmin, xmax0, xmax;
+    double xrange;
+
+    gretl_minmax(t1, t2, x, &xmin0, &xmax0);
+    xrange = xmax0 - xmin0;
+    xmin = xmin0 - xrange * .025;
+    if (xmin0 >= 0.0 && xmin < 0.0) {
+	xmin = 0.0;
+    }
+    xmax = xmax0 + xrange * .025;
+    fprintf(fp, "set xrange [%.10g:%.10g]\n", xmin, xmax);
+}
+
 static int QLR_graph (const double *testvec, int t1, int t2, 
-		      const DATASET *dset, int robust)
+		      const DATASET *dset, int df, int robust)
 {
     const double *x = gretl_plotx(dset, OPT_NONE);
+    const char *titles[] = {
+	N_("Chow F-test for break"),
+	N_("Robust Wald test for break")
+    };
+    const char *title;
+    double (*qlr_critval) (int);
+    double critval = NADBL;
     FILE *fp;
     int t, err = 0;
 
+    set_effective_plot_ci(QLRTEST);
+
     fp = open_plot_input_file(PLOT_REGULAR, &err);
     if (err) {
 	return err;
     }
 
+    qlr_critval = get_plugin_function("qlr_critval_15_05");
+    if (qlr_critval != NULL) {
+	critval = qlr_critval(df);
+    }
+
     print_keypos_string(GP_KEY_LEFT_TOP, fp);
 
+    if (robust) {
+	title = titles[1];
+    } else {
+	title = titles[0];
+	if (!na(critval)) {
+	    critval /= df;
+	}
+    }
+
     gretl_push_c_numeric_locale();
 
-    fprintf(fp, "plot \\\n"
-	    "'-' using 1:2 title '%s' w lines\n",
-	    robust ? _("Robust Wald test for break") : 
-	    _("Chow F-test for break"));
+    write_plot_x_range(x, t1, t2, fp);
+
+    if (!na(critval)) {
+	fprintf(fp, "plot \\\n"
+		"'-' using 1:2 title \"%s\" w lines , \\\n"
+		"%g title \"%s\" w lines lt 0\n", 
+		_(title), critval, "5% QLR critical value");
+    } else {
+	fprintf(fp, "plot \\\n"
+		"'-' using 1:2 title \"%s\" w lines\n", _(title));
+    }
+    
     for (t=t1; t<=t2; t++) {
 	fprintf(fp, "%g %g\n", x[t], testvec[t-t1]);
     }
@@ -2454,7 +2547,11 @@ static int QLR_graph (const double *testvec, int t1, int t2,
 
     gretl_pop_c_numeric_locale();
 
-    return finalize_plot_input_file(fp);
+    err = finalize_plot_input_file(fp);
+
+    set_effective_plot_ci(GNUPLOT);
+
+    return err;
 }
 
 static void save_QLR_test (MODEL *pmod, const char *datestr,
@@ -2472,6 +2569,8 @@ static void save_QLR_test (MODEL *pmod, const char *datestr,
     }	  
 }
 
+/* for internal use by the "qlrtest" command */
+
 static double get_QLR_pval (double test, int df, int k1, int k2, 
 			    MODEL *pmod, PRN *prn)
 {
@@ -2492,20 +2591,47 @@ static double get_QLR_pval (double test, int df, int k1, int k2,
 
     pval = (*qlr_asy_pvalue) (test, df, lam0);
 
+#if 0
+    fprintf(stderr, "k1 = %d, k2 = %d, pi_1 = %g pi_2 = %g\n",
+	    k1, k2, pi_1, pi_2);
+    fprintf(stderr, "lambda0 = %g, pi0 = %g, test = %g, pval = %g\n",
+	    lam0, 1.0/(1 + sqrt(lam0)), test, pval);
+#endif
+
     if (!na(pval)) {
-	pprintf(prn, _("Asymptotic p-value = %.4g for "
+	pprintf(prn, _("Asymptotic p-value = %.6g for "
 		       "chi-square(%d) = %g"),
 		pval, df, test);
 	pputc(prn, '\n');
-#if 0
-	pprintf(prn, "(k1 = %d, k2 = %d, lambda0 = %g, pi0 = %g)\n", 
-		k1, k2, lam0, 1.0/(1 + sqrt(lam0)));
-#endif
     }
 
     return pval;
 }
 
+/* for use by the qlrpval() function */
+
+double QLR_pval (double X2, int df, double p1, double p2)
+{
+    double (*qlr_asy_pvalue) (double, int, double);
+    double lamda, pval;
+
+    if (X2 < 0 || df <= 0 || p1 <= 0 || p2 <= p1 || p2 >= 1) {
+	return NADBL;
+    }
+
+    qlr_asy_pvalue = get_plugin_function("qlr_asy_pvalue");
+
+    if (qlr_asy_pvalue == NULL) {
+	fputs(I_("Couldn't load plugin function\n"), stderr);
+	return NADBL;
+    }
+
+    lamda = (p2*(1.0 - p1)) / (p1*(1.0 - p2));
+    pval = (*qlr_asy_pvalue) (X2, df, lamda);
+
+    return pval;
+}
+
 static void QLR_print_result (MODEL *pmod,
 			      double test, int tmax, 
 			      int k1, int k2, 
@@ -2536,24 +2662,13 @@ static void QLR_print_result (MODEL *pmod,
 
     pval = get_QLR_pval(X2, dfn, k1, k2, pmod, prn);
 
-    record_test_result(X2, pval, "QLR");
+    record_QLR_test_result(X2, pval, tmax + 1);
 
     if (opt & OPT_S) {
 	save_QLR_test(pmod, datestr, X2, pval, dfn);
     }
 }
 
-static double robust_chow_test (MODEL *pmod, const int *testlist)
-{
-    double test = wald_omit_F(testlist, pmod);
-
-    if (!na(test)) {
-	test *= testlist[0]; /* chi-square form */
-    }
-
-    return test;
-}
-
 static void save_chow_test (MODEL *pmod, const char *chowstr,
 			    double test, double pval,
 			    int dfn, int dfd, 
@@ -2577,12 +2692,34 @@ static void save_chow_test (MODEL *pmod, const char *chowstr,
     }	  
 }
 
+static int QLR_plot_wanted (gretlopt opt)
+{
+    /* the default: show a plot only if in interactive mode */
+    int ret = !gretl_in_batch_mode();
+
+    if (opt & OPT_U) {
+	/* but the default can be overruled by @opt */
+	const char *plotparm = get_optval_string(QLRTEST, OPT_U);
+
+	if (plotparm != NULL) {
+	    if (!strcmp(plotparm, "none")) {
+		ret = 0;
+	    } else {
+		ret = 1;
+	    }
+	}
+    }
+
+    return ret;
+}
+
 /*
  * real_chow_test:
  * @chowparm: sample breakpoint; or ID number of dummy
  * variable if given OPT_D; or ignored if given OPT_T.
  * @pmod: pointer to model to be tested.
  * @dset: dataset information.
+ * @ci: either CHOW or QLRTEST.
  * @opt: if flags include OPT_S, save test results to model;
  * if OPT_D included, do the Chow test based on a given dummy
  * variable; if OPT_T included, do the QLR test.
@@ -2592,7 +2729,7 @@ static void save_chow_test (MODEL *pmod, const char *chowstr,
  */
 
 static int real_chow_test (int chowparm, MODEL *pmod, DATASET *dset, 
-			   gretlopt opt, PRN *prn)
+			   int ci, gretlopt opt, PRN *prn)
 {
     int save_t1 = dset->t1;
     int save_t2 = dset->t2;
@@ -2634,7 +2771,8 @@ static int real_chow_test (int chowparm, MODEL *pmod, DATASET *dset,
     }
 
     if (!err) {
-	chowlist = make_chow_list(pmod, dset, split, dumv, &err);
+	chowlist = make_chow_list(pmod, dset, split, dumv, ci,
+				  opt, &err);
     }
 
     if (err) {
@@ -2644,15 +2782,17 @@ static int real_chow_test (int chowparm, MODEL *pmod, DATASET *dset,
     if (QLR) {
 	/* Quandt likelihood ratio */
 	int robust = (pmod->opt & OPT_R);
+	int do_plot = QLR_plot_wanted(opt);
 	gretlopt lsqopt = OPT_A;
 	double test, testmax = 0.0;
 	double *testvec = NULL;
 	int *testlist = NULL;
 	int dfn = 0, dfd = 0;
+	int nextra = dset->v - origv;
 	int tmax = 0;
 	int i, t;
 
-	if (gretl_in_gui_mode()) {
+	if (do_plot) {
 	    testvec = malloc((smax - split + 1) * sizeof *testvec);
 	}
 
@@ -2673,7 +2813,7 @@ static int real_chow_test (int chowparm, MODEL *pmod, DATASET *dset,
 	    }
 	    dfn = chow_mod.ncoeff - pmod->ncoeff;
 	    if (robust) {
-		test = robust_chow_test(&chow_mod, testlist);
+		test = wald_omit_chisq(testlist, &chow_mod);
 #if 0
 		fprintf(stderr, "%d %g\n", t+1, test);
 #endif
@@ -2698,7 +2838,7 @@ static int real_chow_test (int chowparm, MODEL *pmod, DATASET *dset,
 		    pmod->ess, chow_mod.ess);
 #endif
 	    clear_model(&chow_mod);
-	    for (i=0; i<pmod->ncoeff; i++) {
+	    for (i=0; i<nextra; i++) {
 		dset->Z[origv+i][t] = 0.0;
 	    }
 	}
@@ -2707,7 +2847,7 @@ static int real_chow_test (int chowparm, MODEL *pmod, DATASET *dset,
 	    QLR_print_result(pmod, testmax, tmax, split, smax, 
 			     dfn, dfd, dset, opt, robust, prn);
 	    if (testvec != NULL) {
-		QLR_graph(testvec, split, smax, dset, robust);
+		QLR_graph(testvec, split, smax, dset, dfn, robust);
 	    }
 	}
 
@@ -2747,7 +2887,7 @@ static int real_chow_test (int chowparm, MODEL *pmod, DATASET *dset,
 	    }
 
 	    if (robust) {
-		test = robust_chow_test(&chow_mod, testlist);
+		test = wald_omit_chisq(testlist, &chow_mod);
 		if (!na(test)) {
 		    pval = chisq_cdf_comp(dfn, test);
 		}
@@ -2837,7 +2977,7 @@ int chow_test (int splitobs, MODEL *pmod, DATASET *dset,
 	gretl_errmsg_set(_("Invalid sample split for Chow test"));
 	err = E_DATA;
     } else {
-	err = real_chow_test(splitobs, pmod, dset, opt, prn);
+	err = real_chow_test(splitobs, pmod, dset, CHOW, opt, prn);
     }
 
     return err;
@@ -2868,7 +3008,8 @@ int chow_test_from_dummy (int splitvar, MODEL *pmod, DATASET *dset,
 	gretl_errmsg_set(_("Invalid sample split for Chow test"));
 	err = E_DATA;
     } else {
-	err = real_chow_test(splitvar, pmod, dset, opt | OPT_D, prn);
+	err = real_chow_test(splitvar, pmod, dset, CHOW,
+			     opt | OPT_D, prn);
     }
 
     return err;
@@ -2891,7 +3032,8 @@ int chow_test_from_dummy (int splitvar, MODEL *pmod, DATASET *dset,
 int QLR_test (MODEL *pmod, DATASET *dset,
 	      gretlopt opt, PRN *prn)
 {
-    return real_chow_test(0, pmod, dset, opt | OPT_T, prn);
+    return real_chow_test(0, pmod, dset, QLRTEST,
+			  opt | OPT_T, prn);
 }
 
 /* compute v'Mv, for symmetric M */
