@@ -329,6 +329,9 @@ void copy_varinfo (VARINFO *targ, const VARINFO *src)
     targ->lag = src->lag;
     targ->compact_method = src->compact_method;
     targ->stack_level = src->stack_level;
+    if (src->st != NULL) {
+	targ->st = series_table_copy(src->st);
+    }
 }
 
 /**
@@ -408,7 +411,8 @@ void datainfo_init (DATASET *dset)
     dset->varname = NULL;
     dset->varinfo = NULL;    
 
-    dset->markers = NO_MARKERS;  
+    dset->markers = NO_MARKERS;
+    dset->modflag = 0;
 
     dset->S = NULL;
     dset->descrip = NULL;
@@ -1235,15 +1239,17 @@ int dataset_add_series (DATASET *dset, int newvars)
 /**
  * dataset_add_NA_series:
  * @dset: pointer to dataset.
+ * @newvars: number of series to add.
  *
- * Adds space for one additional series in the dataset; the
- * values of the new series are initialized to NADBL.
+ * Adds space for the specified number of additional series
+ * in the dataset. Values are initialized to NA.
  *
  * Returns: 0 on success, E_ALLOC on error.
  */
 
-int dataset_add_NA_series (DATASET *dset)
+int dataset_add_NA_series (DATASET *dset, int newvars)
 {
+    int v0 = dset->v;
     int err;
 
     if (dset_zcols_borrowed(dset)) {
@@ -1251,13 +1257,16 @@ int dataset_add_NA_series (DATASET *dset)
 	return E_DATA;
     }
 
-    err = real_add_series(1, NULL, dset);
+    err = real_add_series(newvars, NULL, dset);
 
     if (!err) {
-	int t, v = dset->v - 1;
+	int i, v, t;
 
-	for (t=0; t<dset->n; t++) {
-	    dset->Z[v][t] = NADBL;
+	for (i=0; i<newvars; i++) {
+	    v = v0 + i;
+	    for (t=0; t<dset->n; t++) {
+		dset->Z[v][t] = NADBL;
+	    }
 	}
     }
 
@@ -1363,12 +1372,20 @@ int dataset_copy_series_as (DATASET *dset, int v, const char *name)
 	    dset->Z[vnew][t] = dset->Z[v][t];
 	}
 	strcpy(dset->varname[vnew], name);
-	dset->varinfo[vnew]->stack_level += 1;
-	/* FIXME other varinfo stuff? */
+	copy_varinfo(dset->varinfo[vnew], dset->varinfo[v]);
+	if (dset->varinfo[v]->flags & VAR_LISTARG) {
+	    dset->varinfo[vnew]->flags &= ~VAR_LISTARG;
+	}
+	if (dset->varinfo[v]->stack_level == gretl_function_depth()) {
+	    /* the source series has not already been shifted to
+	       greater stack level 
+	    */
+	    dset->varinfo[vnew]->stack_level += 1;
+	}
 #if 0
 	fprintf(stderr, "copied var %d ('%s', level %d) as var %d ('%s', level %d): ",
 		v, dset->varname[v], dset->varinfo[v]->stack_level,
-		vnew, newname, dset->varinfo[vnew]->stack_level);
+		vnew, name, dset->varinfo[vnew]->stack_level);
 	fprintf(stderr, "Z[%d][0] = %g\n", vnew, dset->Z[vnew][0]);
 #endif
     }
@@ -3645,7 +3662,6 @@ int series_get_flags (DATASET *dset, int i)
     }
 }
 
-
 /**
  * series_zero_flags:
  * @dset: pointer to dataset.
@@ -4015,6 +4031,43 @@ int series_set_string_val (DATASET *dset, int i, int t, const char *s)
     return err;
 }
 
+/**
+ * string_series_assign_value:
+ * @dset: pointer to dataset.
+ * @i: index number of string-valued series.
+ * @t: 0-based index of observation.
+ * @x: the numeric value to set.
+ *
+ * Attempts to set the value for observation @t of series @i
+ * to @x. This will fail if the @x falls outside of the range
+ * of values supported by the string table for the series.
+ *
+ * Returns: 0 on success, non-zero code on error.
+ */
+
+int string_series_assign_value (DATASET *dset, int i,
+				int t, double x)
+{
+    series_table *st = NULL;
+    int err = 0;
+
+    if (i <= 0 || i >= dset->v) {
+	err = E_DATA;
+    } else if (na(x)) {
+	dset->Z[i][t] = x;
+    } else if (x != floor(x)) {
+	err = E_TYPES;
+    } else if ((st = dset->varinfo[i]->st) == NULL) {
+	err = E_TYPES;
+    } else if (series_table_get_string(st, x) == NULL) {
+	err = E_DATA;
+    } else {
+	dset->Z[i][t] = x;
+    }
+
+    return err;
+}
+
 /**
  * series_decode_string:
  * @dset: pointer to dataset.
@@ -4126,7 +4179,7 @@ int series_set_string_vals (DATASET *dset, int i,
     gretl_matrix *vals = NULL;
     int err = 0;
 
-    if (S == NULL || dset == NULL || i < 1 || i >= dset->n) {
+    if (S == NULL || dset == NULL || i < 1 || i >= dset->v) {
 	return E_DATA;
     }
 
@@ -4135,32 +4188,33 @@ int series_set_string_vals (DATASET *dset, int i,
 
     if (!err) {
 	int i, nvals = gretl_vector_get_length(vals);
+	double x0 = gretl_vector_get(vals, 0);
+	double x1 = gretl_vector_get(vals, nvals - 1);
 
-	if (ns != nvals) {
-	    /* we want exactly one string per (non-missing) value */
-	    gretl_errmsg_sprintf("Wrong number of strings, %d: should be %d",
-				 ns, nvals);
+	if (x0 < 1.0) {
+	    gretl_errmsg_set("The minimum value of the target series "
+			     "must be >= 1");
 	    err = E_DATA;
-	} else {
-	    /* the values should be successive 1-based integers */
-	    double x0 = gretl_vector_get(vals, 0);
-	    double x1 = gretl_vector_get(vals, nvals - 1);
+	} else if (x1 > ns) {
+	    gretl_errmsg_sprintf("Too few strings: %d are needed", ns);
+	    err = E_DATA;
+	}
 
-	    if (x0 != 1.0 || x1 != nvals) {
-		gretl_errmsg_set("The series values must be successive integers");
-		err = E_DATA;
-	    }
-	    for (i=1; i<nvals && !err; i++) {
+	if (!err) {
+	    /* the values should all be integers */
+	    for (i=0; i<nvals && !err; i++) {
 		x1 = gretl_vector_get(vals, i);
-		if (x1 != x0 + 1.0) {
-		    gretl_errmsg_set("The series values must be successive integers");
+		if (x1 != floor(x1)) {
+		    gretl_errmsg_set("The series values must be integers");
 		    err = E_DATA;
 		}
-		x0 = x1;
 	    }
 	    /* and the strings should all be UTF8 */
 	    for (i=0; i<ns && !err; i++) {
-		if (!g_utf8_validate(S[i], -1, NULL)) {
+		if (S[i] == NULL) {
+		    gretl_errmsg_sprintf("String %d is NULL", i+1);
+		    err = E_DATA;
+		} else if (!g_utf8_validate(S[i], -1, NULL)) {
 		    gretl_errmsg_sprintf("String %d is not valid UTF-8", i+1);
 		    err = E_DATA;
 		}
@@ -4183,6 +4237,7 @@ int series_set_string_vals (DATASET *dset, int i,
 	}
 	if (!err) {
 	    if (dset->varinfo[i]->st != NULL) {
+		/* remove any pre-existing table */
 		series_table_destroy(dset->varinfo[i]->st);
 	    }
 	    series_set_discrete(dset, i, 1);
@@ -4274,6 +4329,19 @@ int is_panel_group_names_series (const DATASET *dset, int v)
     }
 }
 
+int is_dataset_series (const DATASET *dset, const double *x)
+{
+    int i;
+
+    for (i=dset->v-1; i>=0; i--) {
+	if (x == dset->Z[i]) {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
 static int effective_daily_skip (int delta, int wd, int pd)
 {
     int k, skip = delta - 1;
