@@ -352,7 +352,7 @@ int dataset_shrink_obs_range (DATASET *dset);
 
 int dataset_add_series (DATASET *dset, int newvars);
 
-int dataset_add_NA_series (DATASET *dset);
+int dataset_add_NA_series (DATASET *dset, int newvars);
 
 int dataset_add_allocated_series (DATASET *dset, double *x);
 
@@ -509,6 +509,9 @@ const char *series_get_string_for_value (const DATASET *dset, int i,
 
 int series_set_string_val (DATASET *dset, int i, int t, const char *s);
 
+int string_series_assign_value (DATASET *dset, int i,
+				int t, double x);
+
 int series_set_string_vals (DATASET *dset, int i, 
 			    char **S, int ns);
 
@@ -530,4 +533,6 @@ const char *panel_group_names_varname (const DATASET *dset);
 
 int is_panel_group_names_series (const DATASET *dset, int v);
 
+int is_dataset_series (const DATASET *dset, const double *x);
+
 #endif /* DATASET_H */
