@@ -124,8 +124,8 @@ float retrieve_float (netfloat nf)
 #endif
 
 static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo, 
-			    DATASET *dset, CompactMethod method, 
-			    int dbv, PRN *prn);
+			    DATASET *dset, CompactMethod cmethod,
+			    int interpolate, int dbv, PRN *prn);
 
 static FILE *open_binfile (const char *dbbase, int code, int offset, int *err)
 {
@@ -1649,6 +1649,8 @@ static double *interpolate_db_series (const double *src,
     return ret;
 }
 
+#define EXPAND_DEBUG 0
+
 /* Expand a single series from a database, for importation 
    into a working dataset of higher frequency.  At present
    this is permitted only for the cases:
@@ -1695,6 +1697,11 @@ double *expand_db_series (const double *src, SERIESINFO *sinfo,
 	}
     }
 
+#if EXPAND_DEBUG
+    fprintf(stderr, "expand_db_series 1: mult=%d, newn=%d, sinfo->stobs='%s'\n",
+	    mult, newn, sinfo->stobs);
+#endif
+
     if (err) {
 	return NULL;
     }
@@ -1718,6 +1725,11 @@ double *expand_db_series (const double *src, SERIESINFO *sinfo,
     sinfo->pd = target_pd;
     sinfo->nobs = newn;
 
+#if EXPAND_DEBUG
+    fprintf(stderr, "expand_db_series 2: sinfo->pd=%d, sinfo->stobs='%s'\n",
+	    sinfo->pd, sinfo->stobs);
+#endif
+
     return x;
 }
 
@@ -1919,16 +1931,10 @@ int set_odbc_dsn (const char *line, PRN *prn)
     return err;
 }
 
-int db_set_sample (const char *s, DATASET *dset)
+int db_set_sample (const char *start, const char *stop, DATASET *dset)
 {
-    char start[OBSLEN], stop[OBSLEN];
     int t1 = 0, t2 = 0;
 
-    if (sscanf(s, "%10s %10s", start, stop) != 2) {
-	gretl_errmsg_set(_("error reading smpl line"));
-	return 1;
-    }
-
     if (strcmp(start, ";")) {
 	t1 = dateton(start, dset);
 	if (t1 < 0) {
@@ -1962,8 +1968,8 @@ int db_set_sample (const char *s, DATASET *dset)
     return 0;
 }
 
-static char *
-get_word_and_advance (char *s, char *word, size_t maxlen)
+static const char *
+get_word_and_advance (const char *s, char *word, size_t maxlen)
 {
     size_t i = 0;
 
@@ -2499,12 +2505,13 @@ static int update_sinfo_masked (SERIESINFO *sinfo, int nobs)
 
 static int get_one_db_series (const char *series, 
 			      DATASET *dset,
-			      CompactMethod method,
+			      CompactMethod cmethod,
+			      int interpolate, 
 			      const gretl_matrix *mask,
 			      const char *idxname,
 			      PRN *prn)
 {
-    CompactMethod this_method = method;
+    CompactMethod this_method = cmethod;
     SERIESINFO sinfo;
     double **dbZ;
     int v, nobs;
@@ -2514,7 +2521,7 @@ static int get_one_db_series (const char *series,
 
     /* see if the series is already in the dataset */
     v = series_index(dset, series);
-    if (v < dset->v && method == COMPACT_NONE) {
+    if (v < dset->v && cmethod == COMPACT_NONE) {
 	this_method = series_get_compact_method(dset, v);
     }
 
@@ -2572,7 +2579,7 @@ static int get_one_db_series (const char *series,
     } 
 
 #if DB_DEBUG
-    fprintf(stderr, "db_get_series: get_XXX_db_data gave %d\n", err);
+    fprintf(stderr, "db_get_series: get_db_data gave %d\n", err);
 #endif
 
     if (err == DB_MISSING_DATA) {
@@ -2586,7 +2593,7 @@ static int get_one_db_series (const char *series,
 
     if (!err) {
 	err = cli_add_db_data(dbZ, &sinfo, dset, this_method, 
-			      v, prn);
+			      interpolate, v, prn);
     }
 
     free_dbZ(dbZ);
@@ -2609,9 +2616,10 @@ int db_get_series (const char *line, DATASET *dset,
     const gretl_matrix *rowmask = NULL;
     char **vnames = NULL;
     char *idxname = NULL;
-    CompactMethod method;
+    CompactMethod cmethod;
     int i, nnames = 0;
     int from_scratch = 0;
+    int interpolate = 0;
     int err = 0;
 
     if (!strncmp(line, "data ", 5)) {
@@ -2625,6 +2633,7 @@ int db_get_series (const char *line, DATASET *dset,
     }
 
     if (opt & OPT_M) {
+	/* FIXME this is undocumented: is it useful? */
 	err = db_get_row_mask(&rowmask);
 	if (err) {
 	    return err;
@@ -2646,9 +2655,9 @@ int db_get_series (const char *line, DATASET *dset,
 
     if (opt & OPT_C) {
 	/* new-style: compaction method supplied as option */
-	method = compact_method_from_option(&err);
+	cmethod = compact_method_from_option(&err);
     } else {
-	line = get_compact_method_and_advance(line, &method);
+	line = get_compact_method_and_advance(line, &cmethod);
     }
 
     if (!err) {
@@ -2674,7 +2683,12 @@ int db_get_series (const char *line, DATASET *dset,
 	}
     }
 
-    /* and process them individually */
+    if (!err && (opt & OPT_I)) {
+	interpolate = 1;
+    }
+
+    /* now process the db series individually */
+
     for (i=0; i<nnames && !err; i++) {
 	if (is_glob(vnames[i])) {
 	    if (saved_db_type == GRETL_NATIVE_DB ||
@@ -2685,16 +2699,18 @@ int db_get_series (const char *line, DATASET *dset,
 		tmp = native_db_match_series(vnames[i], &nmatch, 
 					     idxname, &err);
 		for (j=0; j<nmatch && !err; j++) {
-		    err = get_one_db_series(tmp[j], dset, method, 
-					    rowmask, idxname, prn);
+		    err = get_one_db_series(tmp[j], dset, cmethod, 
+					    interpolate, rowmask, 
+					    idxname, prn);
 		}
 		strings_array_free(tmp, nmatch);
 	    } else {
 		err = E_DATA;
 	    }
 	} else {
-	    err = get_one_db_series(vnames[i], dset, method, 
-				    rowmask, idxname, prn);
+	    err = get_one_db_series(vnames[i], dset, cmethod,
+				    interpolate, rowmask,
+				    idxname, prn);
 	}
     }
 
@@ -2741,7 +2757,7 @@ static void maybe_fclose (FILE *fp)
 
 #define DBUFLEN 1024
 
-static int db_delete_series (char *line, const int *list,
+static int db_delete_series (const char *line, const int *list,
 			     const char *fname, PRN *prn)
 {
     dbnumber buf[DBUFLEN];
@@ -2898,7 +2914,7 @@ static int db_delete_series (char *line, const int *list,
     return err;
 }
 
-int db_delete_series_by_name (char *line, PRN *prn)
+int db_delete_series_by_name (const char *line, PRN *prn)
 {
     return db_delete_series(line, NULL, NULL, prn);
 }
@@ -2911,12 +2927,16 @@ int db_delete_series_by_number (const int *list, const char *fname)
 void get_db_padding (SERIESINFO *sinfo, DATASET *dset, 
 		     int *pad1, int *pad2)
 {
-    *pad1 = dateton(sinfo->stobs, dset); 
+    *pad1 = dateton(sinfo->stobs, dset);
+#if DB_DEBUG    
     fprintf(stderr, "pad1 = dateton(%s) = %d\n", sinfo->stobs, *pad1);
+#endif    
 
     *pad2 = dset->n - sinfo->nobs - *pad1;
+#if DB_DEBUG
     fprintf(stderr, "pad2 = dset->n - sinfo->nobs - pad1 = %d - %d - %d = %d\n", 
 	    dset->n, sinfo->nobs, *pad1, *pad2);
+#endif    
 } 
 
 int db_range_check (SERIESINFO *sinfo, DATASET *dset)
@@ -2982,8 +3002,8 @@ init_datainfo_from_sinfo (DATASET *dset, SERIESINFO *sinfo)
 }
 
 static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo, 
-			    DATASET *dset, CompactMethod method, 
-			    int dbv, PRN *prn)
+			    DATASET *dset, CompactMethod cmethod, 
+			    int interpolate, int dbv, PRN *prn)
 {
     double *xvec = NULL;
     int pad1 = 0, pad2 = 0;
@@ -3022,8 +3042,11 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
 	    dset->n, dset->v, dbv);
 #endif
 
-    /* is the frequency of the new var higher? */
-    if (sinfo->pd > dset->pd) {
+    if (sinfo->pd < dset->pd) {
+	/* the series needs to be expanded */
+	xvec = expand_db_series(dbZ[1], sinfo, dset->pd, interpolate);
+    } else if (sinfo->pd > dset->pd) {
+	/* the series needs to be compacted */
 	if (dset->pd != 1 && dset->pd != 4 &&
 	    sinfo->pd != 12) {
 	    gretl_errmsg_set(_("Sorry, can't handle this conversion yet!"));
@@ -3032,13 +3055,13 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
 	    }
 	    return 1;
 	}
-	if (method == COMPACT_NONE) {
-	    method = COMPACT_AVG;
+	if (cmethod == COMPACT_NONE) {
+	    cmethod = COMPACT_AVG;
 	    pprintf(prn, _("%s: using default compaction method: averaging"), 
 		    sinfo->varname);
 	    pputc(prn, '\n');
 	}
-	xvec = compact_db_series(dbZ[1], sinfo, dset->pd, method);
+	xvec = compact_db_series(dbZ[1], sinfo, dset->pd, cmethod);
 	if (xvec == NULL) {
 	    gretl_errmsg_set(_("Out of memory!"));
 	    if (new) {
@@ -3055,7 +3078,7 @@ static int cli_add_db_data (double **dbZ, SERIESINFO *sinfo,
     /* common stuff for adding a var */
     strcpy(dset->varname[dbv], sinfo->varname);
     series_set_label(dset, dbv, sinfo->descrip);
-    series_set_compact_method(dset, dbv, method);
+    series_set_compact_method(dset, dbv, cmethod);
     get_db_padding(sinfo, dset, &pad1, &pad2);
 
     if (pad1 > 0) {
@@ -4028,7 +4051,9 @@ int maybe_expand_daily_data (DATASET *dset)
  * @dset: dataset struct.
  * @newpd: target data frequency.
  * @default_method: code for the default compaction method.
- * @monstart: FIXME add explanation.
+ * @monstart: if non-zero, take Monday rather than Sunday as
+ * the "start of the week" (only relevant for 7-day daily
+ * data).
  * @repday: "representative day" for conversion from daily
  * to weekly data (with method %COMPACT_WDAY only).
  * 
