@@ -177,6 +177,7 @@ static op_container *op_container_new (int ci, int ndum,
     fprintf(stderr, "nobs = %d\n", OC->nobs);
     fprintf(stderr, "t1-t2 = %d-%d\n", OC->t1, OC->t2);
     fprintf(stderr, "k = %d\n", OC->k);
+    fprintf(stderr, "ndum = %d\n", ndum);
     fprintf(stderr, "nx = %d\n", OC->nx);
     fprintf(stderr, "Max(y) = M = %d\n", OC->ymax);
     printlist(OC->list, "list, in op_container_new");
@@ -303,11 +304,11 @@ static int op_compute_probs (const double *theta, op_container *OC)
 
 /* Below: method for getting around the "non-increasing cut point"
    issue in ordered models by construction: the 2nd and higher cut
-   points are represented to BFGS in the form of the log-difference
-   from the previous cut point.
+   points are represented to the optimizer in the form of the 
+   log-difference from the previous cut point.
 */
 
-static void op_make_BFGS_theta (op_container *OC, double *theta)
+static void op_transform_theta (op_container *OC, double *theta)
 {
     int i;
 
@@ -322,7 +323,7 @@ static void op_make_BFGS_theta (op_container *OC, double *theta)
 }
 
 /* Inverse operation for the transformation done by
-   op_make_BFGS_theta() */
+   op_transform_theta() */
 
 static void op_get_real_theta (op_container *OC, const double *theta)
 {
@@ -406,7 +407,7 @@ static int op_score (double *theta, double *s, int npar, BFGS_CRIT_FUNC ll,
 	}
     }
 
-    return 1;
+    return 0;
 }
 
 static int ordered_hessian (op_container *OC, gretl_matrix *H) 
@@ -905,7 +906,7 @@ static int do_ordered (int ci, int ndum,
 
     npar = OC->k;
 
-    /* transformed theta to pass to BFGS */
+    /* transformed theta to pass to optimizer */
     theta = malloc(npar * sizeof *theta);
     if (theta == NULL) {
 	op_container_destroy(OC);
@@ -921,11 +922,11 @@ static int do_ordered (int ci, int ndum,
     cut_points_init(OC, pmod, (const double **) dset->Z);
 
     /* transform theta to log-diff form */
-    op_make_BFGS_theta(OC, theta);
+    op_transform_theta(OC, theta);
 
 #if LPDEBUG
     for (i=0; i<npar; i++) {
-	fprintf(stderr, "theta[%d]: 'real' = %g, BFGS = %g\n", i, 
+	fprintf(stderr, "theta[%d]: 'real' = %g, transformed = %g\n", i, 
 		OC->theta[i], theta[i]);
     }
     fprintf(stderr, "\ninitial loglikelihood = %.12g\n", 
@@ -946,6 +947,7 @@ static int do_ordered (int ci, int ndum,
 				 op_score, NULL, OC, 
 				 (prn != NULL)? OPT_V : OPT_NONE,
 				 prn);
+	fprintf(stderr, "use_newton: err = %d\n", err);
     } else {
 	BFGS_defaults(&maxit, &toler, PROBIT);
 	err = BFGS_max(theta, npar, maxit, toler, 
@@ -954,19 +956,12 @@ static int do_ordered (int ci, int ndum,
 		       (prn != NULL)? OPT_V : OPT_NONE, prn);
     }
 
-    if (err) {
-	goto bailout;
-    } 
-
-    /* transform back to 'real' theta */
-    op_get_real_theta(OC, theta);
-
     if (!err) {
+	/* transform back to 'real' theta */
+	op_get_real_theta(OC, theta);
 	fill_op_model(pmod, list, dset, OC, fncount, grcount);
     }
 
- bailout:
-
     free(theta);
     op_container_destroy(OC);
 
@@ -982,96 +977,97 @@ static int do_ordered (int ci, int ndum,
 static int maybe_fix_op_depvar (MODEL *pmod, DATASET *dset,
 				double **orig_y, int *ndum)
 {
-    struct sorter *s = NULL;
-    double *sy = NULL;
-    double nexty, bady;
+    gretl_matrix *v = NULL;
+    double *yvals = NULL;
     int dv = pmod->list[1];
     int i, t, n = 0;
     int fixit = 0;
+    int nv = 0;
     int err = 0;
 
-    /* make a copy of the observations on the dep. var. that
-       were actually used in the initial OLS, recording
-       the observation numbers, then sort
-    */
-
     for (t=pmod->t1; t<=pmod->t2; t++) {
 	if (!na(pmod->uhat[t])) {
 	    n++;
 	}
     }
 
-    s = malloc(n * sizeof *s);
-    if (s == NULL) {
+    /* Transcribe the y values that were used in 
+       the initial OLS
+    */    
+
+    yvals = malloc(n * sizeof *yvals);
+    if (yvals == NULL) {
 	return E_ALLOC;
-    }
+    }    
 
     i = 0;
     for (t=pmod->t1; t<=pmod->t2; t++) {
 	if (!na(pmod->uhat[t])) {
-	    s[i].x = dset->Z[dv][t];
-	    s[i].t = t;
-	    i++;
+	    yvals[i++] = dset->Z[dv][t];
 	}
     }
 
-    qsort(s, n, sizeof *s, gretl_compare_doubles);
-
-    /* normalize to minimum zero */
-    if (s[0].x != 0) {
-	double ymin = s[0].x;
-
-	for (i=0; i<n && s[i].x == ymin; i++) {
-	    s[i].x = 0.0;
-	}
-	fixit = 1;
-    }
-
-    /* ensure that the sorted values increase by steps of one */
-    for (i=1; i<n; i++) {
-	if (s[i].x != s[i-1].x) {
-	    nexty = s[i-1].x + 1;
-	    if (s[i].x != nexty) {
-		bady = s[i].x;
-		while (i < n && s[i].x == bady) {
-		    s[i++].x = nexty;
-		}
-		i--; /* compensate for outer i++ */
-		fixit = 1;
-	    } 
-	} 
-    }
-
-    /* the number of dummies actually used will equal the
-       max of normalized y
+    /* Make a sorted vector containing the distinct
+       values of y 
     */
-    *ndum = (int) s[n-1].x;
+    v = gretl_matrix_values(yvals, n, OPT_S, &err);
+
+#if LPDEBUG   
+    gretl_matrix_print(v, "distinct y values");
+#endif    
+
+    if (!err) {
+	nv = gretl_vector_get_length(v);
+	*ndum = nv - 1;
+	if (v->val[0] != 0.0) {
+	    /* the y min. is not zero */
+	    fixit = 1;
+	} else {
+	    for (i=1; i<nv; i++) {
+		if (v->val[i] != v->val[i-1] + 1) {
+		    /* y values are not consecutive integers */
+		    fixit = 1;
+		    break;
+		}
+	    }
+	}
+    }
 
     if (fixit) {
-	/* the dependent var needs transforming */
-	sy = copyvec(dset->Z[dv], dset->n);
-	if (sy == NULL) {
+	double *normy = malloc(dset->n * sizeof *normy);
+
+	if (normy == NULL) {
 	    err = E_ALLOC;
 	} else {
-	    for (i=0; i<n; i++) {
-		sy[s[i].t] = s[i].x;
+	    for (t=0; t<dset->n; t++) {
+		normy[t] = NADBL;
+		if (!na(pmod->uhat[t])) {
+		    for (i=0; i<nv; i++) {
+			if (dset->Z[dv][t] == v->val[i]) {
+			    normy[t] = i;
+			    break;
+			}
+		    }
+		}
 	    }
-	    /* back up the original dep. var and replace it for
-	       the duration of the ordered analysis */
+	    /* Back up the original y and replace it for
+	       the duration of the ordered analysis.
+	    */
 	    *orig_y = dset->Z[dv];
-	    dset->Z[dv] = sy;
-	}
+	    dset->Z[dv] = normy;
+	}	
     }
 
-#if 0
-    if (fixit) {
-	fputs("oprobit: using normalized y\n", stderr);
+#if LPDEBUG
+    if (!err && fixit) {
+	fputs("ordered model: using normalized y\n", stderr);
     } else {
-	fputs("oprobit: using original y\n", stderr);
+	fputs("ordered model: using original y\n", stderr);
     }
 #endif
 
-    free(s);
+    free(yvals);
+    gretl_matrix_free(v);
 
     return err;
 }
@@ -1178,21 +1174,6 @@ static void list_purge_const (int *list, DATASET *dset)
     list[1] = depvar;
 }
 
-static int check_for_missing_dummies (MODEL *pmod, int *dlist)
-{
-    int i, dv;
-
-    for (i=1; i<=dlist[0]; i++) {
-	dv = dlist[i];
-	if (!in_gretl_list(pmod->list, dv)) {
-	    fprintf(stderr, "check for missing dummies: var %d is gone!\n", dv);
-	    return E_SINGULAR;
-	}
-    }
-
-    return 0;
-}
-
 static int ordered_depvar_check (int v, const DATASET *dset)
 {
     if (!series_is_discrete(dset, v) && 
@@ -1247,22 +1228,16 @@ static MODEL ordered_estimate (int *list, DATASET *dset, int ci,
     }
 
 #if LPDEBUG
-    pprintf(prn, "ordered_estimate: initial OLS\n");
-    printmodel(&model, dset, OPT_S, prn);
+    PRN *vprn = gretl_print_new(GRETL_PRINT_STDERR, NULL);
+    
+    pputs(vprn, "ordered_estimate: initial OLS\n");
+    printmodel(&model, dset, OPT_S, vprn);
+    gretl_print_destroy(vprn);
 #endif
 
-    if (model.list[0] < biglist[0]) {
-	/* One or more regressors were dropped in OLS, most likely due
-	   to collinearity.  We can accept the dropping of
-	   user-selected regressors, but all the added dummies must be
-	   retained, so we'd better investigate.
-	*/
-	model.errcode = check_for_missing_dummies(&model, dumlist);
-    }
-	
     if (!model.errcode) {
-	/* after accounting for any missing observations, check that
-	   the dependent variable is acceptable 
+	/* after accounting for any missing observations, normalize
+	   the dependent variable if necessary 
 	*/
 	model.errcode = maybe_fix_op_depvar(&model, dset,
 					    &orig_y, &ndum);
@@ -1328,14 +1303,39 @@ static double logit_pdf (double x)
     return l;
 }
 
+/* Here we're checking for a dummy variable that acts as a
+   "one-way perfect predictor" of the binary dependent variable;
+   that is, an x such that Prob(y = A | x = B) = 1 for some
+   assignment of values 0 or 1 to A and B. The MLE does not
+   exist in the presence of such a regressor, so we'll remove
+   it from the model (after alerting the user). 
+
+   The approach taken by Stata is not only to drop such a
+   regressor but also to drop the observations that are thus
+   perfectly predicted. As the outcome of disussions in
+   November-December 2014 we decided not to follow Stata in
+   this policy: we just drop the regressor. However, in case
+   we want to revisit this point I'm leaving in place the
+   apparatus required to implement the Stata policy.
+
+   AC, 2014-12-04
+*/
+
+#define LIKE_STATA 0
+
 static char *classifier_check (int *list, const DATASET *dset,
-			       PRN *prn, int *err)
+			       PRN *prn, int *ndropped, int *err)
 {
     char *mask = NULL;
-    const double *y = dset->Z[list[1]];
+    int yno = list[1];
+    const double *y = dset->Z[yno];
     int i, v, ni, t;
 
-    for (i=2; i<=list[0]; i++) {
+    *ndropped = 0;
+
+    for (i=list[0]; i>=2; i--) {
+	int getout = 0;
+	
 	v = list[i];
 	if (v == 0) {
 	    continue;
@@ -1344,40 +1344,52 @@ static char *classifier_check (int *list, const DATASET *dset,
 	ni = gretl_isdummy(dset->t1, dset->t2, dset->Z[v]);
 
 	if (ni > 0) {
-	    int same[2] = {0};
-	    int diff[2] = {0};
-	    int maskval, pc = 1;
+	    const double *x = dset->Z[v];
+	    int xytab[4] = {0};
+	    int pp0 = 1, pp1 = 1;
+	    int maskval = -1;
 
-	    for (t=dset->t1; t<=dset->t2 && pc; t++) {
-		if (dset->Z[v][t] > 0) {
-		    if (y[t] > 0) {
-			same[0] += 1;
-		    } else {
-			diff[0] += 1;
-		    }
-		} else {
-		    if (y[t] == 0) {
-			same[1] += 1;
-		    } else {
-			diff[1] += 1;
-		    }
+	    for (t=dset->t1; t<=dset->t2; t++) {
+		xytab[0] += (x[t] == 0 && y[t] == 0);
+		xytab[1] += (x[t] == 0 && y[t] == 1);
+		xytab[2] += (x[t] == 1 && y[t] == 0);
+		xytab[3] += (x[t] == 1 && y[t] == 1);
+		if (xytab[1] && xytab[3]) {
+		    /* x does not perfectly predict y == 0 */
+		    pp0 = 0;
 		}
-		if (same[0] && diff[0] && same[1] && diff[1]) {
-		    pc = 0;
+		if (xytab[0] && xytab[2]) {
+		    /* x does not perfectly predict y == 1 */
+		    pp1 = 0;
+		}
+		if (!pp0 && !pp1) {
+		    break;
 		}
 	    }
 
-	    if (pc) {
-		if (!(same[0] && diff[0])) {
-		    maskval = 1;
-		    pprintf(prn, "Note: %s != 0 predicts %s perfectly\n",
-			    dset->varname[v], (same[0])? "success" : "failure");
-		} else {
-		    maskval = 0;
-		    pprintf(prn, "Note: %s == 0 predicts %s perfectly\n",
-			    dset->varname[v], (diff[1])? "success" : "failure");
-		}
-
+#if LIKE_STATA	    
+	    if (pp0 && pp1) {
+		pputc(prn, '\n');
+		pprintf(prn, "Note: %s = %s%s at all observations\n",
+			dset->varname[yno], xytab[0] ? "" : "not-",
+			dset->varname[v]);
+		*err = E_NOCONV;
+		getout = 1;
+	    } else if (pp0) {
+		maskval = xytab[1] ? 1 : 0;
+		pputc(prn, '\n');
+		pprintf(prn, "Note: Prob(%s = %d | %s = %d) = 1\n",
+			dset->varname[yno], 0, dset->varname[v],
+			maskval);
+	    } else if (pp1) {
+		pputc(prn, '\n');
+		maskval = xytab[0] ? 1 : 0;
+		pprintf(prn, "Note: Prob(%s = %d | %s = %d) = 1\n",
+			dset->varname[yno], 1, dset->varname[v],
+			maskval);
+	    }
+		
+	    if (maskval >= 0) {
 		if (mask == NULL) {
 		    mask = malloc(dset->n + 1);
 		    if (mask == NULL) {
@@ -1387,17 +1399,53 @@ static char *classifier_check (int *list, const DATASET *dset,
 			mask[dset->n] = 0;
 		    }
 		}
-
 		if (mask != NULL) {
 		    for (t=dset->t1; t<=dset->t2; t++) {
 			if (dset->Z[v][t] == maskval) {
 			    mask[t-dset->t1] = '1';
+			    *ndropped += 1;
 			}
 		    }
-		    pprintf(prn, "%d observations not used\n", ni);
+		    pprintf(prn, "%s dropped and %d observations not used\n",
+			    dset->varname[v], *ndropped);
 		}
-		gretl_list_delete_at_pos(list, i--);
+		
+		gretl_list_delete_at_pos(list, i);
+		/* It'll get too confusing if we try doing
+		   this for more than one regressor?
+		*/
+		getout = 1;
 	    }
+#else /* not like Stata */
+	    if (pp0 && pp1) {
+		pputc(prn, '\n');
+		pprintf(prn, _("Note: %s = %s%s at all observations\n"),
+			dset->varname[yno], xytab[0] ? "" : "!",
+			dset->varname[v]);
+		*err = E_NOCONV;
+		getout = 1;
+	    } else if (pp0) {
+		maskval = xytab[1] ? 1 : 0;
+		pputc(prn, '\n');
+		pprintf(prn, _("Note: Prob(%s = %d | %s = %d) = 1\n"),
+			dset->varname[yno], 0, dset->varname[v],
+			maskval);
+	    } else if (pp1) {
+		maskval = xytab[0] ? 1 : 0;
+		pputc(prn, '\n');
+		pprintf(prn, _("Note: Prob(%s = %d | %s = %d) = 1\n"),
+			dset->varname[yno], 1, dset->varname[v],
+			maskval);
+	    }
+		
+	    if (maskval >= 0) {
+		pprintf(prn, _("Dropping %s\n"), dset->varname[v]);
+		gretl_list_delete_at_pos(list, i);
+	    }
+#endif	    
+	}
+	if (getout) {
+	    break;
 	}
     }
 
@@ -2852,6 +2900,7 @@ static MODEL binary_model (int ci, const int *inlist,
     double gradtol = 1.0e-7;
     int maxit = 100;
     int fncount = 0;
+    int ndropped = 0;
     MODEL mod;
     bin_info *bin = NULL;
     PRN *vprn = NULL;
@@ -2881,8 +2930,11 @@ static MODEL binary_model (int ci, const int *inlist,
 
     list_adjust_sample(list, &dset->t1, &dset->t2, dset, NULL);
 
-    mask = classifier_check(list, dset, prn, &mod.errcode);
+    mask = classifier_check(list, dset, prn, &ndropped, &mod.errcode);
     if (mod.errcode) {
+	if (mod.errcode == E_NOCONV) {
+	    gretl_errmsg_set(_("Perfect prediction obtained: no MLE exists"));
+	}
 	goto bailout;
     }
 
@@ -2957,6 +3009,9 @@ static MODEL binary_model (int ci, const int *inlist,
 
     if (!mod.errcode) {
 	binary_model_finish(bin, &mod, dset, opt);
+	if (!mod.errcode && ndropped > 0) {
+	    gretl_model_set_int(&mod, "binary_obs_dropped", ndropped);
+	}
     }  
 
  bailout:  
