@@ -66,7 +66,7 @@ struct str_table dummies[] = {
     { 0,        NULL }
 };
 
-/* dvars: dataset-related accessors */
+/* dvars: dataset- and test-related accessors */
 
 struct str_table dvars[] = {
     { R_NOBS,      "$nobs" },
@@ -75,6 +75,7 @@ struct str_table dvars[] = {
     { R_DATATYPE,  "$datatype" },
     { R_TEST_STAT, "$test" },
     { R_TEST_PVAL, "$pvalue" },
+    { R_TEST_BRK,  "$qlrbreak" },
     { R_TEST_LNL,  "$rlnl" },
     { R_KLNL,      "$kalman_lnl" },
     { R_KS2,       "$kalman_s2" },
@@ -155,6 +156,7 @@ struct str_table mvars[] = {
     { M_YLIST,   "$ylist" },
     { M_COMMAND, "$command" },
     { M_DEPVAR,  "$depvar" },
+    { M_MODEL,   "$model" },
     { 0,         NULL }
 };
 
@@ -317,6 +319,7 @@ struct str_table funcs[] = {
     { F_MRANDGEN, "mrandgen" },
     { F_RANDGEN1, "randgen1" },
     { F_URCPVAL,  "urcpval" },
+    { F_QLRPVAL,  "qlrpval" },
     { F_VALUES,   "values" },
     { F_UNIQ,     "uniq" },
     { F_MSHAPE,   "mshape" },
@@ -432,20 +435,26 @@ struct str_table funcs[] = {
     { F_ARRAY,     "array" },
     { F_STRVALS,   "strvals" },
     { F_STRINGIFY, "stringify" },
-    { F_PRINTARRAY, "printarray" },
+    { F_PUTARRAY,  "putarray" },
+    { F_DOTWRITE,  "dotwrite" },
+    { F_BOOTCI,    "bootci" },
+    { F_BOOTPVAL,  "bootpval" },
     { 0,           NULL }
 };
 
 struct str_table func_alias[] = {
     { F_GAMMA,     "gammafunc" },
     { F_GAMMA,     "gamma" },
-    { F_PVAL,      "pval" },
     { F_LOG,       "logs" },
-    { F_OBSLABEL,  "date" },
     { F_BACKTICK,  "$" },
     { 0,           NULL }
 };
 
+struct str_table hidden_funcs[] = {
+    { HF_CLOGFI,   "_clogitfi" },
+    { 0,           NULL }
+};
+
 int const_lookup (const char *s)
 {
     int i;
@@ -457,9 +466,7 @@ int const_lookup (const char *s)
     }
 
     /* backward comatibility: to be removed eventually */
-    if (!strcmp(s, "pi")) {
-	return CONST_PI;
-    } else if (!strcmp(s, "macheps")) {
+    if (!strcmp(s, "macheps")) {
 	return CONST_EPS;
     }
 
@@ -491,6 +498,11 @@ static GHashTable *gretl_function_hash_init (void)
 			    GINT_TO_POINTER(funcs[i].id));
     }
 
+    for (i=0; hidden_funcs[i].str != NULL; i++) {
+	g_hash_table_insert(ht, (gpointer) hidden_funcs[i].str, 
+			    GINT_TO_POINTER(hidden_funcs[i].id));
+    }    
+
     return ht;
 }
 
@@ -762,10 +774,13 @@ void context_error (int c, parser *p)
     } else {
 	const char *s = getsymb(p->sym, p);
 
-	if (s != NULL && *s != '\0') {
+	if (s != NULL && *s != '\0' && strcmp(s, "unknown")) {
 	    pprintf(p->prn, _("The symbol '%s' is not valid in this context\n"), 
 		    getsymb(p->sym, p));
-	}
+	} else {
+	    pprintf(p->prn, "The symbol %d is not valid in this context\n", 
+		    p->sym);
+	}	    
     }
 
     if (!p->err) {
@@ -998,12 +1013,18 @@ static int maybe_get_R_function (const char *s)
 */ 
 
 static int parsing_query;
+static int doing_genseries;
 
 void set_parsing_query (int s)
 {
     parsing_query = s;
 }
 
+void set_doing_genseries (int s)
+{
+    doing_genseries = s;
+}
+
 static void look_up_word (const char *s, parser *p)
 {
     int fsym, err = 0;
@@ -1063,6 +1084,10 @@ static void look_up_word (const char *s, parser *p)
 		    /* note: all "native" types take precedence over this */
 		    p->sym = RFUN;
 		    p->idstr = gretl_strdup(s + 2);
+		} else if (!strcmp(s, "pi")) {
+		    /* backward compat (get rid of this) */
+		    p->idnum = CONST_PI;
+		    p->sym = CON;
 		} else if (parsing_query) {
 		    p->sym = UNDEF;
 		    p->idstr = gretl_strdup(s);
@@ -1200,7 +1225,7 @@ static int is_word_char (parser *p)
 {
     if (strchr(wordchars, p->ch) != NULL) {
 	return 1;
-    } else if (p->targ == LIST && p->ch == '*') {
+    } else if (p->targ == LIST && !doing_genseries && p->ch == '*') {
 	return 1;
     } 
 
@@ -1401,7 +1426,7 @@ void lex (parser *p)
 	case ' ':
 	case '\t':
 	case '\r':
-        case '\n': 
+        case '\n':
 	    parser_getc(p);
 	    break;
         case '+': 
@@ -1412,10 +1437,16 @@ void lex (parser *p)
 	    p->sym = B_SUB;
 	    parser_getc(p);
 	    return;
-        case '*': 
-	    if (p->targ == LIST) {
-		/* treat '*' as wildcard */
-		getword(p);
+        case '*':
+	    if (p->targ == LIST && !doing_genseries) {
+		/* allow for '*' as wildcard */
+		if (*(p->point - 2) == ' ' &&
+		    (bare_data_type(p->sym) || closing_sym(p->sym) ||
+		     (p->sym == LAG))) {
+		    p->sym = B_LCAT;
+		} else {
+		    getword(p);
+		}
 		return;
 	    }
 	    parser_getc(p);
@@ -1613,11 +1644,13 @@ void lex (parser *p)
 		parser_getc(p);
 		return;
 	    }
-	    if (p->targ == LIST && *(p->point - 2) == ' ' && 
+
+	    if (p->targ == LIST && !doing_genseries && *(p->point - 2) == ' ' && 
 		(bare_data_type(p->sym) || closing_sym(p->sym) ||
 		 (p->sym == LAG))) {
 		/* may be forming a list, but only if there are 
-		   spaces between the terms */
+		   spaces between the terms 
+		*/
 		p->sym = B_LCAT;
 		return;
 	    }
@@ -1658,6 +1691,8 @@ const char *getsymb (int t, const parser *p)
 	return "OBS";
     } else if (t == MSL) {
 	return "MSL";
+    } else if (t == OSL) {
+	return "OSL";
     } else if (t == DMSL) {
 	return "DMSL";
     } else if (t == DMSTR) {
@@ -1696,6 +1731,12 @@ const char *getsymb (int t, const parser *p)
 	return "UNDEF";
     } else if (t == NUM) {
 	return "NUM";
+    } else if (t == IVEC) {
+	return "IVEC";
+    } else if (t == UNUM_P) {
+	return "UNUM_P";
+    } else if (t == UNUM_M) {
+	return "UNUM_M";
     }
 
     if (p != NULL) {
@@ -1787,6 +1828,8 @@ const char *getsymb (int t, const parser *p)
 	return "<=";
     case B_AND: 
 	return "&&";
+    case B_JOIN:
+	return "JOIN";
     case B_RANGE:
 	return " to ";
     case U_ADDR:
