@@ -71,6 +71,8 @@ static void gen_write_message (const parser *p, int oldv, PRN *prn)
 		    p->lh.name, p->lh.v);
 	}
     } else if (p->targ == MAT) {
+	gretl_matrix *m = get_matrix_by_name(p->lh.name);
+	
 	if (p->lh.t == MAT && p->lh.substr != NULL && 
 	    *p->lh.substr != '\0') {
 	    pprintf(prn, _("Modified matrix %s"), p->lh.name);
@@ -79,6 +81,9 @@ static void gen_write_message (const parser *p, int oldv, PRN *prn)
 	} else {
 	    pprintf(prn, _("Generated matrix %s"), p->lh.name);
 	}
+	if (m != NULL && m->rows == 1 && m->cols == 1) {
+	    pprintf(prn, " = {%g}", m->val[0]);
+	}
     } else if (p->targ == LIST) {
 	if (p->lh.t == LIST) {
 	    pprintf(prn, _("Replaced list %s"), p->lh.name);
@@ -223,7 +228,6 @@ int function_from_string (const char *s)
 static const char *reswords[] = {
     /* constants */
     "const",
-    "pi",
     "NA",
     "null",
     "obs", /* not exactly a constant, but hey */
@@ -234,6 +238,7 @@ static const char *reswords[] = {
     "string",
     "list",
     "bundle",
+    "array",
     "kalman",
     "void",
     /* debugging instructions, etc. */
@@ -377,7 +382,7 @@ int series_index (const DATASET *dset, const char *varname)
 	       not just be the result of its being a member of a list
 	       that was passed as an argument.
 	    */
-	    for (i=1; i<dset->v; i++) { 
+	    for (i=1; i<dset->v; i++) {
 		if (fd == series_get_stack_level(dset, i) &&
 		    !series_is_listarg(dset, i) && 
 		    strcmp(dset->varname[i], s) == 0) {
@@ -873,12 +878,27 @@ char *generate_string (const char *s, DATASET *dset, int *err)
     return ret;
 }
 
+static int x_to_list_member (double x, DATASET *dset, int *err)
+{
+    int v = -1;
+
+    if (x < 0 || x >= dset->v) {
+	*err = E_INVARG;
+    } else if (fabs(x - nearbyint(x)) > 1.0e-8) {
+	*err = E_INVARG;
+    } else {
+	v = nearbyint(x);
+    }
+
+    return v;
+}
+
 /* retrieve a list result directly */
 
 int *generate_list (const char *s, DATASET *dset, int *err)
 {
-    parser p;
     int *ret = NULL;
+    parser p;
 
     if (dset == NULL) {
 	*err = E_NODATA;
@@ -903,10 +923,16 @@ int *generate_list (const char *s, DATASET *dset, int *err)
 	    if (ret != NULL) {
 		ret[1] = n->vnum;
 	    }
-	} else if (n->t == NUM && n->v.xval >= 0 && n->v.xval < dset->v) {
-	    ret = gretl_list_new(1);
-	    if (ret != NULL) {
-		ret[1] = n->v.xval;
+	} else if (n->t == NUM) {
+	    int v = x_to_list_member(n->v.xval, dset, err);
+
+	    if (!*err) {
+		ret = gretl_list_new(1);
+		if (ret == NULL) {
+		    *err = E_ALLOC;
+		} else {
+		    ret[1] = v;
+		}
 	    }	    
 	} else {
 	    *err = E_TYPES;
