@@ -108,7 +108,7 @@ enum {
               OVAR,	  /* object variable: variable "under" an object */
               LIST,	  /* list, evaluated */
 	      LISTVAR,    /* variable in list, dot syntax */
-              ELEMENT,    /* element of list, array [...] syntax) */
+              ELEMENT,    /* element of list or array, [...] syntax) */
   /* 80 */    STR,	  /* string */
 	      BUNDLE,     /* gretl bundle (hash table) */
 	      BMEMB,      /* member of bundle */
@@ -122,6 +122,7 @@ enum {
   /* 90 */    RFUN,       /* GNU R function */
 	      USTR,       /* string variable */
 	      IVEC,       /* array of ints, not a varlist */
+	      OSL,        /* "slice" of object other than matrix */
               INC,        /* increment */
               DEC,        /* decrement */
               QUERY,      /* ternary "?" expression */
@@ -280,6 +281,7 @@ enum {
     F_NLINES,
     F_ARRAY,
     F_STRVALS,
+    F_DOTWRITE,
     F1_MAX,	  /* SEPARATOR: end of single-arg functions */
     F_COR,
     F_COV,
@@ -356,7 +358,7 @@ enum {
     F_JSONGET,
     F_KPSSCRIT,
     F_STRINGIFY,
-    F_PRINTARRAY,
+    F_PUTARRAY,
     F2_MAX,	  /* SEPARATOR: end of two-arg functions */
     F_LLAG,
     F_PRINCOMP,
@@ -410,6 +412,10 @@ enum {
     F_GHK,
     F_QUADTAB,
     F_ISOCONV,
+    F_QLRPVAL,
+    F_BOOTCI,
+    F_BOOTPVAL,
+    HF_CLOGFI,
     FN_MAX,	  /* SEPARATOR: end of n-arg functions */
 };
 
@@ -458,16 +464,14 @@ enum {
                              s == F_NRMAX || s == F_DESEAS || \
 			     s == F_AGGRBY || s == F_INBUNDLE || \
 			     s == F_SSCANF || s == F_PRINTF || \
-			     s == F_SPRINTF || s == F_ALLREDUCE || \
-			     s == F_GENSERIES)
+			     s == F_SPRINTF || s == F_ALLREDUCE)
 
 /* functions taking string arg in middle position */
 #define string_mid_func(s) (s == F_REDUCE || s == F_SCATTER)
 
 /* functions taking one or more "fncall" (string) arguments */
 #define fncall_func(s) (s == F_BFGSMAX || s == F_NRMAX || \
-			s == F_FDJAC || s == F_SIMANN || \
-			s == F_GENSERIES)
+			s == F_FDJAC || s == F_SIMANN)
 
 #define unary_op(s)  (s >= 1 && s < U_MAX)
 #define binary_op(s) (s > U_MAX && s < OP_MAX)
@@ -477,7 +481,7 @@ enum {
 
 #define evalb2(s) (binary_op(s) || func2_symb(s) || s == MSL || \
                    s == MSLRAW || s == SUBSL || s == LAG || \
-                   s == OBS || s == BMEMB || s == ELEMENT)
+                   s == OBS || s == BMEMB || s == ELEMENT || s == OSL)
 
 #define b1sym(s) (unary_op(s) || func1_symb(s) || funcn_symb(s) || \
                   s == G_LPR || s == EROOT)
@@ -585,7 +589,7 @@ enum {
     P_SAVEAUX = 1 << 17, /* indicates try aux node compilation */
     P_AUXDONE = 1 << 18, /* indicates aux nodes compiled */
     P_DELTAN  = 1 << 19, /* flag for change in series length */
-    P_LHBKVAR = 1 << 20  /* LHS bundle key is string variable */  
+    P_LHBKVAR = 1 << 20  /* LHS bundle key is string variable */
 };
 
 struct lhinfo {
@@ -653,6 +657,7 @@ void context_error (int c, parser *p);
 void undefined_symbol_error (const char *s, parser *p);
 const char *getsymb (int t, const parser *p);
 void set_parsing_query (int s);
+void set_doing_genseries (int s);
 
 int realgen (const char *s, parser *p, DATASET *dset, 
 	     PRN *prn, int flags, int targtype);
