@@ -319,6 +319,10 @@ static NODE *base (parser *p, NODE *up)
 	lex(p);
 	t = expr(p);
 	break;
+    case B_JOIN:
+	/* list joiner with empty LHS */
+	t = newempty();
+	break;
     case G_LPR: /* left paren '(' */
 	lex(p);
 	t = expr(p);
@@ -714,22 +718,6 @@ static NODE *get_bundle_member_name (parser *p)
     if (p->ch == '.') {
 	/* using bundle dot notation */
 	n = gretl_namechar_spn(s);
-	if (*(s + n) == '[') {
-	    /* support "[...]" spec following member name */
-	    const char *q = s + (++n);
-	    int br = 1;
-
-	    while (*q && br > 0) {
-		if (*q == '[') br++;
-		else if (*q == ']') br--;
-		n++;
-		q++;
-	    }
-
-	    if (br != 0) {
-		unmatched_symbol_error('[', p);
-	    }
-	}
     } else if (p->ch == '[') {
 	/* using bracketed key notation */
 	if (*s == '"') {
@@ -799,11 +787,17 @@ static void get_matrix_def (NODE *t, parser *p, int *sub)
     char cexp = 0;
 
 #if SDEBUG
-    fprintf(stderr, "get_matrix_def, p->sym = %d\n", p->sym);
+    fprintf(stderr, "get_matrix_def, p->sym = %d ('%s')\n",
+	    p->sym, getsymb(p->sym, NULL));
 #endif    
 
     if (p->sym == G_LCB) {
 	lex(p);
+	if (!p->err && p->sym == G_RCB) {
+	    /* empty matrix def, OK */
+	    lex(p);
+	    return;
+	}
 	while (p->ch != 0 && !p->err) {
 	    n = expr(p);
 	    if (p->err) {
@@ -836,6 +830,11 @@ static void get_matrix_def (NODE *t, parser *p, int *sub)
 	cexp = '{';
     }
 
+#if SDEBUG
+    fprintf(stderr, " after processing, sym = %d, err = %d\n",
+	    p->sym, p->err);
+#endif     
+
     if (!p->err && cexp == 0) {
 	if (p->sym == G_RCB) {
 	    lex(p);
@@ -929,6 +928,8 @@ static void get_slice_parts (NODE *t, parser *p)
     set_slice_off(p);
 }
 
+
+
 static void attach_child (NODE *parent, NODE *child, int k, int i,
 			  parser *p)
 {
@@ -976,7 +977,7 @@ static void pad_parent (NODE *parent, int k, int i, parser *p)
 #define next_arg_is_string(i,k,o) ((i>0 && i<k-1 && (o & (MID_STR|MID_FNCALL))) || \
                                    (i==k-1 && (o & RIGHT_STR)))
 
-/* Get up to k comma-separated arguments (possibly optional).
+/* Get up to @k comma-separated arguments (possibly optional).
    However, if k < 0 this is a signal to get as many arguments as we
    can find (the number unknown in advance).
 */
@@ -1121,11 +1122,20 @@ static NODE *powterm (parser *p)
             t->v.b1.b = powterm(p);
         }
     } else if (func2_symb(sym)) {
+	int unset = 0;
+	
+	if (sym == F_GENSERIES) {
+	    set_doing_genseries(1);
+	    unset = 1;
+	}
 	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
 	    lex(p);
 	    get_args(t, p, sym, 2, opt, &next);
 	}
+	if (unset) {
+	    set_doing_genseries(0);
+	}
     } else if (func3_symb(sym)) {
 	t = newb3(sym, NULL);
 	if (t != NULL) {
@@ -1215,7 +1225,7 @@ static NODE *powterm (parser *p)
 	if (t != NULL) {
 	    t->v.b2.l = newref(p, BUNDLE);
 	    t->v.b2.r = get_bundle_member_name(p);
-	}	
+	}
     } else if (sym == OVAR) {
 	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
@@ -1287,7 +1297,28 @@ static NODE *powterm (parser *p)
 	t = base(p, NULL);
     }
 
-    if (next == '[') {
+    if (t != NULL && (sym == ELEMENT || sym == BMEMB)) {
+	if (p->sym == G_LBR) {
+	    /* followed by subspec */
+	    t = newb2(OSL, t, NULL);
+	    if (t != NULL) {
+		t->v.b2.r = newb2(MSLRAW, NULL, NULL);
+		if (t->v.b2.r != NULL) {
+		    get_slice_parts(t->v.b2.r, p);
+		}
+	    }
+	    if (sym == BMEMB && p->sym == G_LBR) {
+		/* followed by another subspec */
+		t = newb2(OSL, t, NULL);
+		if (t != NULL) {
+		    t->v.b2.r = newb2(MSLRAW, NULL, NULL);
+		    if (t->v.b2.r != NULL) {
+			get_slice_parts(t->v.b2.r, p);
+		    }
+		}		
+	    }
+	}
+    } else if (next == '[') {
 	/* support func(args)[slice] */
 	t = newb2(MSL, t, NULL);
 	if (t != NULL) {
