@@ -52,7 +52,8 @@ typedef enum {
     GPT_POLAR          = 1 << 24, /* plot is in polar mode */
     GPT_XL             = 1 << 25, /* large */
     GPT_XXL            = 1 << 26, /* extra-large */
-    GPT_TIMEFMT        = 1 << 27  /* using gnuplot "timefmt" */
+    GPT_TIMEFMT        = 1 << 27, /* using gnuplot "timefmt" */
+    GPT_ICON           = 1 << 28  /* saving plot "as icon" */
 } GptFlags; 
 
 typedef struct gretlRGB_ gretlRGB;
@@ -125,6 +126,7 @@ typedef enum {
     PLOT_FIT_INVERSE,
     PLOT_FIT_LOESS,
     PLOT_FIT_LOGLIN,
+    PLOT_FIT_LINLOG,
     PLOT_FIT_NA       /* fit option not applicable */
 } FitType;
 
@@ -175,7 +177,7 @@ const char *get_gretl_png_term_line (PlotType ptype, GptFlags flags);
 
 const char *get_png_line_for_plotspec (const GPT_SPEC *spec);
 
-const char *get_gretl_emf_term_line (PlotType ptype, int color);
+const char *get_gretl_emf_term_line (PlotType ptype, GptFlags flags);
 
 const char *get_gretl_pdf_term_line (PlotType ptype, GptFlags flags);
 
@@ -205,6 +207,8 @@ void write_plot_line_styles (int ptype, FILE *fp);
 
 void write_plot_bounding_box_request (FILE *fp);
 
+void set_effective_plot_ci (int ci);
+
 PlotType plot_type_from_string (const char *str);
 
 void plot_get_scaled_dimensions (int *width, int *height, double scale);
@@ -255,6 +259,9 @@ int theil_forecast_plot (const int *plotlist, const DATASET *dset,
 
 int gretl_panel_ts_plot (int vnum, DATASET *dset, gretlopt opt);
 
+int panel_means_XY_scatter (const int *list, const DATASET *dset,
+			    gretlopt opt);
+
 int plot_fcast_errs (const FITRESID *fr, const double *maxerr,
 		     const DATASET *dset, gretlopt opt);
 
