@@ -333,10 +333,13 @@ int gretl_array_set_string (gretl_array *A, int i,
 {
     int err = 0;
 
-    if (A == NULL || i < 0 || i >= A->n) {
+    if (A == NULL) {
 	err = E_DATA;
     } else if (A->type != GRETL_TYPE_STRINGS) {
 	err = E_TYPES;
+    } else if (i < 0 || i >= A->n) {
+	gretl_errmsg_sprintf(_("Index value %d is out of bounds"), i+1);
+	err = E_DATA;
     } else {
 	free(A->data[i]);
 	err = set_string(A, i, s, copy);
@@ -375,10 +378,13 @@ int gretl_array_set_matrix (gretl_array *A, int i,
 {
     int err = 0;
 
-    if (A == NULL || i < 0 || i >= A->n) {
+    if (A == NULL) {
 	err = E_DATA;
     } else if (A->type != GRETL_TYPE_MATRICES) {
 	err = E_TYPES;
+    } else if (i < 0 || i >= A->n) {
+	gretl_errmsg_sprintf(_("Index value %d is out of bounds"), i+1);
+	err = E_DATA;
     } else {
 	gretl_matrix_free(A->data[i]);
 	err = set_matrix(A, i, m, copy);
@@ -417,10 +423,13 @@ int gretl_array_set_bundle (gretl_array *A, int i,
 {
     int err = 0;
 
-    if (A == NULL || i < 0 || i >= A->n) {
+    if (A == NULL) {
 	err = E_DATA;
     } else if (A->type != GRETL_TYPE_BUNDLES) {
 	err = E_TYPES;
+    } else if (i < 0 || i >= A->n) {
+	gretl_errmsg_sprintf(_("Index value %d is out of bounds"), i+1);
+	err = E_DATA;
     } else {
 	gretl_bundle_destroy(A->data[i]);
 	err = set_bundle(A, i, b, copy);
@@ -458,10 +467,13 @@ int gretl_array_set_list (gretl_array *A, int i,
 {
     int err = 0;
 
-    if (A == NULL || i < 0 || i >= A->n) {
+    if (A == NULL) {
 	err = E_DATA;
     } else if (A->type != GRETL_TYPE_LISTS) {
 	err = E_TYPES;
+    } else if (i < 0 || i >= A->n) {
+	gretl_errmsg_sprintf(_("Index value %d is out of bounds"), i+1);
+	err = E_DATA;
     } else {
 	free(A->data[i]);
 	err = set_list(A, i, L, copy);
@@ -481,6 +493,7 @@ int gretl_array_append_list (gretl_array *A,
 	err = E_DATA;
     } else if (A->type != GRETL_TYPE_LISTS) {
 	err = E_TYPES;
+	
     } else {
 	err = array_extend_content(A, 1);
 	if (!err) {
@@ -703,40 +716,90 @@ int gretl_array_print (gretl_array *A, PRN *prn)
     return 0;
 }
 
+static void real_array_print_full (gretl_array *A, 
+				   const DATASET *dset,
+				   PRN *prn)
+{
+    int i;
+
+    /* FIXME what exactly this does may be debatable */
+
+    for (i=0; i<A->n; i++) {
+	void *data = A->data[i];
+
+	if (data == NULL) {
+	    pprintf(prn, "%d: null\n", i + 1);
+	} else if (A->type == GRETL_TYPE_STRINGS) {
+	    pprintf(prn, "%s\n", (char *) data);
+	} else if (A->type == GRETL_TYPE_MATRICES) {
+	    char numstr[16];
+
+	    sprintf(numstr, "%d", i + 1);
+	    gretl_matrix_print_to_prn(data, numstr, prn);
+	} else if (A->type == GRETL_TYPE_BUNDLES) {
+	    gretl_bundle_print(data, prn);
+	} else if (A->type == GRETL_TYPE_LISTS) {
+	    gretl_list_print(data, dset, prn);
+	}
+    }
+}
+
 int gretl_array_print_full (gretl_array *A, const DATASET *dset,
 			    PRN *prn)
 {
     if (A != NULL) {
-	const char *s = gretl_type_get_name(A->type);
-	int i;
-
-	/* FIXME this may need some more work */
-
-	pprintf(prn, _("Array of %s, length %d\n"), s, A->n);
-
-	for (i=0; i<A->n; i++) {
-	    void *data = A->data[i];
-
-	    if (data == NULL) {
-		pprintf(prn, "%d: null\n", i + 1);
-	    } else if (A->type == GRETL_TYPE_STRINGS) {
-		pprintf(prn, "%d: %s\n", i + 1, (char *) data);
-	    } else if (A->type == GRETL_TYPE_MATRICES) {
-		char numstr[8];
-
-		sprintf(numstr, "%d", i + 1);
-		gretl_matrix_print_to_prn(data, numstr, prn);
-	    } else if (A->type == GRETL_TYPE_BUNDLES) {
-		gretl_bundle_print(data, prn);
-	    } else if (A->type == GRETL_TYPE_LISTS) {
-		gretl_list_print(data, dset, prn);
-	    }
-	}
+	real_array_print_full(A, dset, prn);
     }
 
     return 0;
 }
 
+static PRN *get_dotdir_printer (char **pfname, int *err)
+{
+    const char *dotdir = gretl_dotdir();
+    char *fname = NULL;
+    FILE *fp = NULL;
+    PRN *prn = NULL;
+
+    fname = malloc(strlen(dotdir) + 16);
+
+    if (fname == NULL) {
+	*err = E_ALLOC;
+    } else {
+	sprintf(fname, "%sprntmp.XXXXXX", dotdir);
+	fp = gretl_mktemp(fname, "w");
+	if (fp == NULL) {
+	    *err = E_FOPEN;
+	    free(fname);
+	    fname = NULL;
+	}
+    }
+
+    if (fp != NULL) {
+	prn = gretl_print_new_with_stream(fp);
+	*pfname = fname;
+    }
+
+    return prn;
+}
+
+char *gretl_array_print_to_dotdir (gretl_array *A, 
+				   const DATASET *dset,
+				   int *err)
+{
+    char *fname = NULL;
+    PRN *prn;
+
+    prn = get_dotdir_printer(&fname, err);
+
+    if (prn != NULL) {
+	real_array_print_full(A, dset, prn);
+	gretl_print_destroy(prn);
+    }
+
+    return fname;
+}
+
 /* Called from gretl_bundle.c when serializing a bundle
    which contains one or more arrays.
 */
