@@ -1226,10 +1226,10 @@ static void reverse_gradient (double *g, int n)
     }
 }
 
-int LBFGS_max (double *b, int n, int maxit, double reltol,
-	       int *fncount, int *grcount, BFGS_CRIT_FUNC cfunc, 
-	       int crittype, BFGS_GRAD_FUNC gradfunc, void *data, 
-	       gretlopt opt, PRN *prn)
+static int LBFGS_max (double *b, int n, int maxit, double reltol,
+		      int *fncount, int *grcount, BFGS_CRIT_FUNC cfunc, 
+		      int crittype, BFGS_GRAD_FUNC gradfunc, void *data, 
+		      gretlopt opt, PRN *prn)
 {
     double *wspace = NULL;
     double *g, *l, *u, *wa;
@@ -1391,11 +1391,14 @@ int LBFGS_max (double *b, int n, int maxit, double reltol,
  * includes %OPT_V.
  *
  * Obtains the set of values for @b which jointly maximize the
- * criterion value as calculated by @cfunc.  Uses the BFGS
- * variable-metric method.  Based on Pascal code in J. C. Nash,
+ * criterion value as calculated by @cfunc. By default uses the BFGS
+ * variable-metric method (based on Pascal code in J. C. Nash,
  * "Compact Numerical Methods for Computers," 2nd edition, converted
- * by p2c then re-crafted by B. D. Ripley for gnu R.  Revised for 
- * gretl by Allin Cottrell.
+ * by p2c then re-crafted by B. D. Ripley for gnu R; revised for 
+ * gretl by Allin Cottrell and Jack Lucchetti). Alternatively,
+ * if OPT_L is given, uses the L-BFGS-B method (limited memory
+ * BFGS), based on Lbfgsb.2.1 by Ciyou Zhu, Richard Byrd, Jorge 
+ * Nocedal and Jose Luis Morales. 
  * 
  * Returns: 0 on successful completion, non-zero error code
  * on error.
@@ -1408,6 +1411,8 @@ int BFGS_max (double *b, int n, int maxit, double reltol,
 {
     int ret, wnum;
 
+    gretl_iteration_push();
+
     if ((opt & OPT_L) || libset_get_bool(USE_LBFGS)) {
 	ret = LBFGS_max(b, n, maxit, reltol,
 			fncount, grcount, cfunc, 
@@ -1420,6 +1425,8 @@ int BFGS_max (double *b, int n, int maxit, double reltol,
 			A0, opt, prn);
     }
 
+    gretl_iteration_pop();
+
     wnum = check_gretl_warning();
     if (wnum != W_GRADIENT) {
 	/* suppress expected numerical warnings */
@@ -1720,9 +1727,10 @@ double user_BFGS (gretl_matrix *b,
 
     if (verbose) {
 	opt = OPT_V;
-	u->prn = prn;
     }
 
+    u->prn = prn; /* 2015-03-10: this was conditional on OPT_V */
+
     *err = BFGS_max(b->val, u->ncoeff, 
 		    maxit, tol, &fcount, &gcount,
 		    user_get_criterion, C_OTHER, 
@@ -1782,9 +1790,11 @@ double user_NR (gretl_matrix *b,
 
     if (libset_get_bool(MAX_VERBOSE)) {
 	opt = OPT_V;
-	u->prn = prn;
     }
 
+    u->prn = prn; /* 2015-03-10: this was conditional on OPT_V */
+
+
     *err = newton_raphson_max(b->val, u->ncoeff, maxit, 
 			      crittol, gradtol, 
 			      &iters, C_OTHER, 
@@ -2288,6 +2298,10 @@ int newton_raphson_max (double *b, int n, int maxit,
 	} else {
 	    err = numeric_gradient(b, g->val, n, cfunc, data);
 	}
+	if (err) {
+	    fprintf(stderr, "newton_raphson_max: err = %d from %s\n", err,
+		    gradfunc != NULL ? "supplied gradfunc" : "numeric_gradient");
+	}
     }
 
     if (!err) {
@@ -2302,6 +2316,8 @@ int newton_raphson_max (double *b, int n, int maxit,
 	}
     }
 
+    gretl_iteration_push();
+
     while (status == 0 && !err) {
 	iter++;
 	steplen = 1.0;
@@ -2382,6 +2398,8 @@ int newton_raphson_max (double *b, int n, int maxit,
 	}
     }
 
+    gretl_iteration_pop();
+
     if (verbose) {
 	print_iter_info(-1, f1, crittype, n, b1, g->val, 
 			steplen, prn);
@@ -2469,6 +2487,8 @@ int gretl_simann (double *theta, int n, int maxit,
 		f0);
     }
 
+    gretl_iteration_push();
+
     /* Question: should the initial radius be a function of the scale
        of the initial parameter vector?
     */
@@ -2508,6 +2528,8 @@ int gretl_simann (double *theta, int n, int maxit,
 	radius *= 0.9999;
     }
 
+    gretl_iteration_pop();
+
     if (improved) {
 	/* use the best point */
 	gretl_matrix_copy_values(&b, bstar);
