@@ -27,6 +27,7 @@
 #include "gretl_xml.h"
 #include "gretl_foreign.h"
 #include "gretl_typemap.h"
+#include "genparse.h"
 #include "gretl_bundle.h"
 
 #define BDEBUG 0
@@ -632,53 +633,6 @@ double *gretl_bundle_get_series (gretl_bundle *bundle,
     return x;
 }
 
-/**
- * gretl_bundle_get_payload_matrix:
- * @bundle: bundle to access.
- * @err: location to receive error code.
- *
- * Returns: a copy of the matrix associated with the key 
- * "payload" in the specified @bundle, if any; otherwise NULL.
- */
-
-gretl_matrix *gretl_bundle_get_payload_matrix (gretl_bundle *bundle,
-					       int *err)
-{
-    gretl_matrix *m = NULL;
-    GretlType type;
-    void *ptr;
-
-    ptr = gretl_bundle_get_data(bundle, "payload", &type, 
-				NULL, err);
-    if (!*err && type != GRETL_TYPE_MATRIX) {
-	*err = E_TYPES;
-    }
-
-    if (!*err) {
-	m = gretl_matrix_copy((gretl_matrix *) ptr);
-	if (m == NULL) {
-	    *err = E_ALLOC;
-	}
-    }
-
-    return m;
-}
-
-/**
- * gretl_bundle_set_payload_matrix:
- * @bundle: bundle to access.
- * @m: matrix to set as payload.
- *
- * Returns: 0 on success, non-zero code on error.
- */
-
-int gretl_bundle_set_payload_matrix (gretl_bundle *bundle,
-				     const gretl_matrix *m)
-{
-    return real_bundle_set_data(bundle, "payload", (void *) m,
-				GRETL_TYPE_MATRIX, 0, 1, NULL);
-}
-
 /**
  * gretl_bundle_get_scalar:
  * @bundle: bundle to access.
@@ -1750,6 +1704,141 @@ gretl_bundle *get_sysinfo_bundle (int *err)
     return sysinfo_bundle;
 }
 
+static gretl_matrix *matrix_from_list (const int *list)
+{
+    gretl_matrix *m = NULL;
+    int i;
+    
+    if (list != NULL && list[0] > 0) {
+	m = gretl_matrix_alloc(1, list[0]);
+	if (m != NULL) {
+	    for (i=0; i<list[0]; i++) {
+		m->val[i] = list[i+1];
+	    }
+	}
+    }
+
+    return m;
+}
+
+/* For a single-equation model, create a bundle containing
+   all the data available via $-accessors.
+*/
+
+gretl_bundle *bundle_from_model (MODEL *pmod,
+				 DATASET *dset,
+				 int *err)
+{
+    gretl_bundle *b = NULL;
+    gretl_matrix *m;
+    double *x;
+    double val;
+    int *list;
+    const char *s;
+    const char *key;
+    int i, t, berr;
+
+    if (pmod == NULL) {
+	/* get the "last model" */
+	GretlObjType type = 0;
+	void *p = get_last_model(&type);
+
+	if (p == NULL || type != GRETL_OBJ_EQN) {
+	    *err = E_DATA;
+	    return NULL;
+	} else {
+	    pmod = p;
+	}
+    }
+
+    x = malloc(dset->n * sizeof *x);
+    if (x == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    b = gretl_bundle_new();
+    if (b == NULL) {
+	free(x);
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    for (i=M_ESS; i<M_SCALAR_MAX && !*err; i++) {
+	berr = 0;
+	val = gretl_model_get_scalar(pmod, i, dset, &berr);
+	if (!berr) {
+	    key = mvarname(i) + 1;
+	    *err = gretl_bundle_set_scalar(b, key, val);	    
+	}
+    }
+
+    for (i=M_SCALAR_MAX+1; i<M_SERIES_MAX && !*err; i++) {
+	for (t=0; t<dset->n; t++) {
+	    x[t] = NADBL;
+	}
+	berr = gretl_model_get_series(x, pmod, dset, i);
+	if (!berr) {
+	    key = mvarname(i) + 1;
+	    *err = gretl_bundle_set_series(b, key, x, dset->n);
+	}	
+    }    
+
+    for (i=M_SERIES_MAX+1; i<M_MATRIX_MAX && !*err; i++) {
+	berr = 0;
+	m = gretl_model_get_matrix(pmod, i, &berr);
+	if (!berr) {
+	    key = mvarname(i) + 1;
+	    *err = gretl_bundle_donate_data(b, key, m,
+					    GRETL_TYPE_MATRIX,
+					    0);
+	}
+    }
+
+    for (i=M_MBUILD_MAX+1; i<M_LIST_MAX && !*err; i++) {
+	list = NULL;
+	if (i == M_XLIST) {
+	    list = gretl_model_get_x_list(pmod);
+	} else if (i == M_YLIST) {
+	    list = gretl_model_get_y_list(pmod);
+	}
+	if (list != NULL) {
+	    /* convert list to matrix for bundling */
+	    m = matrix_from_list(list);
+	    if (m != NULL) {
+		key = mvarname(i) + 1;
+		*err = gretl_bundle_donate_data(b, key, m,
+						GRETL_TYPE_MATRIX,
+						0);
+	    }
+	    free(list);
+	}
+    }
+
+    for (i=M_LIST_MAX+1; i<M_STR_MAX && !*err; i++) {
+	s = NULL;
+	if (i == M_DEPVAR) {
+	    s = gretl_model_get_depvar_name(pmod, dset);
+	} else if (i == M_COMMAND) {
+	    s = gretl_command_word(pmod->ci);
+	}
+	if (s != NULL && *s != '\0') {
+	    key = mvarname(i) + 1;
+	    *err = gretl_bundle_set_string(b, key, s);	    
+	}	    
+    }    
+
+    free(x);
+
+    /* don't return a broken bundle */
+    if (*err && b != NULL) {
+	gretl_bundle_destroy(b);
+	b = NULL;
+    }
+
+    return b;
+}
+
 void gretl_bundle_cleanup (void)
 {
     if (sysinfo_bundle != NULL) {
