@@ -66,12 +66,6 @@ const char *gretl_bundle_get_string (gretl_bundle *bundle,
 				     const char *key,
 				     int *err);
 
-gretl_matrix *gretl_bundle_get_payload_matrix (gretl_bundle *bundle,
-					       int *err);
-
-int gretl_bundle_set_payload_matrix (gretl_bundle *bundle,
-				     const gretl_matrix *m);
-
 const char *gretl_bundle_get_note (gretl_bundle *bundle, const char *key);
 
 const char *gretl_bundle_get_creator (gretl_bundle *bundle);
@@ -153,6 +147,10 @@ int bundle_contains_data (gretl_bundle *b, void *data);
 
 gretl_bundle *get_sysinfo_bundle (int *err);
 
+gretl_bundle *bundle_from_model (MODEL *pmod,
+				 DATASET *dset,
+				 int *err);
+
 void gretl_bundle_cleanup (void);
 
 #endif /* GRETL_BUNDLE_H_ */
