@@ -380,7 +380,7 @@ void gretl_errmsg_sprintf (const char *fmt, ...)
 	    va_end(ap);
 
 	    if (gretl_errmsg[len0 - 1] != '\n') {
-		strncat(gretl_errmsg, "\n", 1);
+		strcat(gretl_errmsg, "\n");
 	    }
 	    strncat(gretl_errmsg, tmp, strlen(tmp));
 	} 
@@ -443,7 +443,7 @@ void gretl_errmsg_set_from_errno (const char *s)
  * Blank out any previously recorded error message.
  */
 
-void gretl_error_clear (void)
+int gretl_error_clear (void)
 {
 #if EDEBUG
     fprintf(stderr, "gretl_error_clear\n");
@@ -455,6 +455,8 @@ void gretl_error_clear (void)
     errno = 0;
     gretl_warnnum = 0;
     *gretl_warnmsg = '\0';
+
+    return 0;
 }
 
 /**
@@ -529,8 +531,6 @@ int gretl_error_is_fatal (void)
 	return 1;
     } else if (gretl_compiling_loop()) {
 	return 1;
-    } else if (libset_get_bool(HALT_ON_ERR) == 0) {
-	return 0;
     } else {
 	return gretl_in_batch_mode();
     }
