@@ -26,14 +26,12 @@
 # include "gretl_mpi.h"
 #endif
 
-#include <glib.h>
-
 #ifdef USE_RLIB
 # include <Rinternals.h> /* for SEXP and friends */
 #endif
 
 #ifdef G_OS_WIN32
-# include <windows.h>
+# include "gretl_win32.h"
 #else
 # include <signal.h>
 #endif
@@ -64,10 +62,12 @@ static gchar *gretl_python_prog;
 static gchar *gretl_mpi_prog;
 #endif
 
-static void destroy_foreign (void)
+void foreign_destroy (void)
 {
-    strings_array_free(foreign_lines, foreign_n_lines);
-    foreign_lines = NULL;
+    if (foreign_lines != NULL) {
+	strings_array_free(foreign_lines, foreign_n_lines);
+	foreign_lines = NULL;
+    }
     foreign_started = 0;
     foreign_n_lines = 0;
     foreign_opt = OPT_NONE;
@@ -328,7 +328,7 @@ static int lib_run_other_sync (gretlopt opt, PRN *prn)
 	cmd = g_strdup_printf("\"%s\" --silent \"%s\"", path, fname);
     } else if (foreign_lang == LANG_STATA) {
 	path = gretl_stata_path();
-	cmd = g_strdup_printf("%s /q /e gretltmp.do", path);
+	cmd = g_strdup_printf("\"%s\" /q /e gretltmp.do", path);
     } else if (foreign_lang == LANG_PYTHON) {
 	path = gretl_python_path();
 	fname = gretl_python_filename();
@@ -1132,19 +1132,47 @@ static int write_gretl_mpi_file (gretlopt opt)
 
 #endif /* HAVE_MPI */
 
+static int *get_send_data_list (const DATASET *dset, int *err)
+{
+    const char *lname = get_optval_string(FOREIGN, OPT_D);
+    int *list = NULL;
+
+    if (lname != NULL) {
+	list = get_list_by_name(lname);
+	if (list == NULL) {
+	    *err = E_DATA;
+	} else {
+	    int i;
+
+	    for (i=1; i<=list[0] && !*err; i++) {
+		if (list[i] < 0 || list[i] >= dset->v) {
+		    *err = E_DATA;
+		}
+	    }
+	}
+    }
+
+    return list;
+}
+
 static int write_data_for_stata (const DATASET *dset,
 				 FILE *fp)
 {
+    int *list = NULL;
+    gchar *sdata = NULL;
     char save_na[8];
-    gchar *sdata;
-    int err;
+    int err = 0;
 
-    *save_na = '\0';
-    strncat(save_na, get_csv_na_write_string(), 7);
-    set_csv_na_write_string(".");
-    sdata = g_strdup_printf("%sstata.csv", gretl_dotdir());
-    err = write_data(sdata, NULL, dset, OPT_C, NULL);
-    set_csv_na_write_string(save_na);
+    list = get_send_data_list(dset, &err);
+    
+    if (!err) {
+	*save_na = '\0';
+	strncat(save_na, get_csv_na_write_string(), 7);
+	set_csv_na_write_string(".");
+	sdata = g_strdup_printf("%sstata.csv", gretl_dotdir());
+	err = write_data(sdata, list, dset, OPT_C, NULL);
+	set_csv_na_write_string(save_na);
+    }
  
     if (err) {
 	gretl_errmsg_sprintf("write_data_for_stata: failed with err = %d\n", err);
@@ -1205,11 +1233,16 @@ int write_gretl_stata_file (const char *buf, gretlopt opt,
 static int write_data_for_octave (const DATASET *dset,
 				  FILE *fp)
 {
-    gchar *mdata;
-    int err;
+    int *list = NULL;
+    gchar *mdata = NULL;
+    int err = 0;
 
-    mdata = g_strdup_printf("%smdata.tmp", gretl_dotdir());
-    err = write_data(mdata, NULL, dset, OPT_M, NULL);
+    list = get_send_data_list(dset, &err);
+
+    if (!err) {
+	mdata = g_strdup_printf("%smdata.tmp", gretl_dotdir());
+	err = write_data(mdata, list, dset, OPT_M, NULL);
+    }
  
     if (err) {
 	gretl_errmsg_sprintf("write_data_for_octave: failed with err = %d\n", err);
@@ -1272,12 +1305,18 @@ static int write_data_for_R (const DATASET *dset,
 			     FILE *fp)
 {
     int ts = dataset_is_time_series(dset);
+    int *list = NULL;
     gchar *Rdata;
-    int err;
+    int err = 0;
 
     Rdata = g_strdup_printf("%sRdata.tmp", gretl_dot_dir);
 
-    err = write_data(Rdata, NULL, dset, OPT_R, NULL);
+    list = get_send_data_list(dset, &err);
+
+    if (!err) {
+	err = write_data(Rdata, list, dset, OPT_R, NULL);
+    }
+
     if (err) {
 	gretl_errmsg_sprintf("write_data_for_R: failed with err = %d\n", err);
 	g_free(Rdata);
@@ -2191,36 +2230,50 @@ static int run_R_lib (const DATASET *dset,
 
 #endif /* USE_RLIB */
 
-static int foreign_block_init (const char *line, gretlopt opt, PRN *prn)
+/**
+ * foreign_start:
+ * @ci: either FOREIGN or MPI.
+ * @param: string specifying language, for FOREIGN.
+ * @opt: may include %OPT_V for verbose operation.
+ * @prn: struct for printing output.
+ *
+ * Starts a new "foreign" block if no such block is
+ * currently defined.
+ * 
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int foreign_start (int ci, const char *param, gretlopt opt, 
+		   PRN *prn)
 {
     int err = 0;
 
+    if (foreign_started) {
+	gretl_errmsg_sprintf("%s: a block is already started",
+			     gretl_command_word(ci));
+	return E_DATA;
+    }
+
     foreign_opt = OPT_NONE;
 
-    if (!strncmp(line, "foreign ", 8)) {
-	char lang[16];
+    if (ci == FOREIGN) {
+	if (param == NULL || *param == '\0') {
+	    err = E_ARGS;
+	} else {
+	    char lang[16];
 
-	line += 8;
-	line += strspn(line, " ");
-	if (!strncmp(line, "language", 8)) {
-	    line += 8;
-	    line += strspn(line, " =");
-	    if (sscanf(line, "%15s", lang) == 1) {
+	    if (sscanf(param, "language=%15s", lang) == 1) {
 		err = set_foreign_lang(lang, prn);
 	    } else {
 		err = E_PARSE;
 	    }
-	} else {
-	    err = E_PARSE;
 	}
-    } else if (!strncmp(line, "mpi", 3)) {
+    } else if (ci == MPI) {
 	err = set_foreign_lang("mpi", prn);
-    } else {
-	/* we'll default to R for now */
-	foreign_lang = LANG_R;
     }
-
+	    
     if (!err) {
+	foreign_started = 1;
 	foreign_opt = opt;
     }
 
@@ -2228,43 +2281,34 @@ static int foreign_block_init (const char *line, gretlopt opt, PRN *prn)
 }
 
 /**
- * foreign_append_line:
- * @line: command line.
- * @opt: may include %OPT_V for verbose operation
- * @prn: struct for printing output.
+ * foreign_append:
+ * @line: line to append.
+ * @context: either FOREIGN or MPI.
  *
  * Appends @line to an internally stored block of "foreign"
- * commands, or starts a new block if no such block is
- * currently defined.
+ * or MPI commands, if such a block is currently defined.
  * 
  * Returns: 0 on success, non-zero on error.
  */
 
-int foreign_append_line (const char *line, gretlopt opt, PRN *prn)
+int foreign_append (const char *line, int context)
 {
     int err = 0;
 
-    if (string_is_blank(line)) {
-	return 0;
-    }
-
 #if 0
-    fprintf(stderr, "foreign_append_line: '%s'\n", line);
+    fprintf(stderr, "foreign_append: '%s'\n", line);
 #endif
 
     if (!foreign_started) {
-	/* starting from scratch */
-	err = foreign_block_init(line, opt, prn);
-	if (!err) {
-	    foreign_started = 1;
-	}
-    } else {
-	/* appending */
+	gretl_errmsg_sprintf("%s: no block is in progress",
+			     gretl_command_word(context));
+	err = E_DATA;
+    } else if (!string_is_blank(line)) {
 	err = strings_array_add(&foreign_lines, &foreign_n_lines, line);
 	if (err) {
-	    destroy_foreign();
+	    foreign_destroy();
 	}
-    }	
+    }
 
     return err;
 }
@@ -2324,7 +2368,7 @@ int foreign_execute (const DATASET *dset,
 	} else {
 	    err = lib_run_mpi_sync(foreign_opt, prn);
 	}
-	destroy_foreign();
+	foreign_destroy();
 	return err;
     }
 #endif
@@ -2375,7 +2419,7 @@ int foreign_execute (const DATASET *dset,
 	err = E_DATA;
     }
     
-    destroy_foreign();
+    foreign_destroy();
 
     return err;
 }
