@@ -28,10 +28,14 @@ typedef enum {
     LANG_MPI
 } ForeignLangs;
 
-int foreign_append_line(const char *line, gretlopt opt, PRN *prn);
+int foreign_start (int ci, const char *param, gretlopt opt, PRN *prn);
+
+int foreign_append (const char *line, int context);
 
 int foreign_execute (const DATASET *dset, gretlopt opt, PRN *prn);
 
+void foreign_destroy (void);
+
 int write_gretl_R_files (const char *buf,
 			 const DATASET *dset,
 			 gretlopt opt);
@@ -46,6 +50,10 @@ int write_gretl_octave_file (const char *buf, gretlopt opt,
 			     const DATASET *dset,
 			     const char **pfname);
 
+int write_gretl_stata_file (const char *buf, gretlopt opt, 
+			    const DATASET *dset,
+			    const char **pfname);
+
 int gretl_max_mpi_processes (void);
 
 #ifdef HAVE_MPI
