@@ -61,7 +61,6 @@ typedef enum {
 
 typedef struct ufunc_ ufunc;
 typedef struct fnpkg_ fnpkg;
-typedef struct fnargs_ fnargs;
 
 int n_user_functions (void);
 
@@ -100,6 +99,8 @@ int user_func_get_return_type (const ufunc *fun);
 
 int user_func_is_noprint (const ufunc *fun);
 
+int user_func_is_menu_only (const ufunc *fun);
+
 const char *user_function_name_by_index (int i);
 
 int user_function_index_by_name (const char *name, 
@@ -123,9 +124,12 @@ int gretl_function_append_line (const char *line);
 
 int gretl_is_public_user_function (const char *name);
 
-int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
-			 DATASET *dset, void *ret, 
-			 char **descrip, PRN *prn);
+int gretl_function_exec (ufunc *u, int rtype, DATASET *dset,
+			 void *ret, char **descrip, PRN *prn);
+
+int attach_loop_to_function (void *ptr);
+
+int current_function_size (void);
 
 char *gretl_func_get_arg_name (const char *argvar, int *err);
 
@@ -137,7 +141,10 @@ void sample_range_get_extrema (const DATASET *dset, int *t1, int *t2);
 
 int function_return_type_from_string (const char *s);
 
-int gretl_function_print_code (ufunc *u, PRN *prn);
+int gretl_function_print_code (ufunc *u, int tabwidth, PRN *prn);
+
+int print_function_package_sample (const char *fname, int tabwidth,
+				   PRN *prn);
 
 void set_current_function_package (fnpkg *pkg);
 
@@ -154,6 +161,13 @@ int function_package_set_properties (fnpkg *pkg, ...);
 
 int function_package_get_properties (fnpkg *pkg, ...);
 
+const char *function_package_get_string (fnpkg *pkg,
+					 const char *id);
+
+int function_package_set_data_files (fnpkg *pkg, char **S, int n);
+
+char **function_package_get_data_files (fnpkg *pkg, int *n);
+
 const char *function_package_get_name (fnpkg *pkg);
 
 int function_package_write_file (fnpkg *pkg);
@@ -191,11 +205,13 @@ void function_package_unload_full_by_filename (const char *fname);
 
 int print_function_package_info (const char *fname, PRN *prn);
 
-int print_function_package_code (const char *fname, PRN *prn);
+int print_function_package_code (const char *fname, int tabwidth,
+				 PRN *prn);
 
 ufunc *get_function_from_package (const char *funname, fnpkg *pkg);
 
-int get_function_file_header (const char *fname, char **pdesc, char **pver);
+int get_function_file_header (const char *fname, char **pdesc,
+			      char **pver, int *pdfdoc);
 
 double function_package_get_version (const char *fname);
 
@@ -205,12 +221,10 @@ int function_package_has_PDF_doc (fnpkg *pkg, char **pdfname);
 
 void gretl_functions_cleanup (void);
 
-fnargs *fn_args_new (int argc);
+void function_clear_args (ufunc *fun);
 
-void fn_args_free (fnargs *args);
-
-int push_fn_arg (fnargs *args, const char *name, GretlType type,
-		 void *value);
+int push_function_arg (ufunc *fun, const char *name,
+		       GretlType type, void *value);
 
 void adjust_indent (const char *line, int *this_indent,
 		    int *next_indent);
