@@ -80,14 +80,18 @@ typedef enum {
     RQ_NID    /* sandwich */
 } RQVCVType;
 
+typedef enum {
+    HAC_PREWHITEN = 1
+} VCVFlags;
+
 typedef struct VCVInfo_ VCVInfo;
 
 struct VCVInfo_ {
-    int vmaj;
-    int vmin;
-    int order;
-    int flags;
-    double bw;
+    int vmaj;       /* general type of VCV */
+    int vmin;       /* HC variant, etc. */
+    int order;      /* for use with HAC */
+    VCVFlags flags; /* includes prewhitening */
+    double bw;      /* for use with QS HAC kernel */ 
 };
 
 /* single-equation model commands */
@@ -278,6 +282,8 @@ int gretl_model_set_vcv_info (MODEL *pmod, int vmaj, int vmin);
 
 int gretl_model_get_vcv_type (const MODEL *pmod);
 
+int gretl_model_get_hc_version (const MODEL *pmod);
+
 int gretl_model_get_cluster_var (const MODEL *pmod);
 
 void *gretl_model_get_data (const MODEL *pmod, const char *key);
@@ -436,6 +442,8 @@ int mle_criteria (MODEL *pmod, int addk);
 
 double coeff_pval (int ci, double x, int df);
 
+double model_coeff_pval (const MODEL *pmod, double x);
+
 int exact_fit_check (const MODEL *pmod, PRN *prn);
 
 void maybe_suppress_time_dummies (MODEL *pmod, int ndum);
