@@ -33,7 +33,7 @@
 #include <unistd.h>
 
 #ifdef WIN32
-# include <windows.h>
+# include "gretl_win32.h"
 #else
 # include <sys/stat.h>
 # include <sys/types.h>
@@ -1496,13 +1496,15 @@ static char *gretl_addon_get_path (const char *name)
 
     dirs = get_plausible_search_dirs(FUNCS_SEARCH, &n_dirs);
 
+    gretl_push_c_numeric_locale();
+
     for (i=0; i<n_dirs; i++) {
 	const char *fndir = dirs[i];
 	struct dirent *dirent;
 	const char *dname;
 	DIR *dir;
 	int found = 0;
-	
+
 	if ((dir = gretl_opendir(fndir)) == NULL) {
 	    continue;
 	}
@@ -1528,6 +1530,8 @@ static char *gretl_addon_get_path (const char *name)
 	closedir(dir);
     }
 
+    gretl_pop_c_numeric_locale();
+
     strings_array_free(dirs, n_dirs);
 
     return ret;
@@ -1537,6 +1541,7 @@ static int is_addon (const char *name)
 {
     if (!strcmp(name, "gig") ||
 	!strcmp(name, "SVAR") ||
+	!strcmp(name, "HIP") ||
 	!strcmp(name, "ivpanel")) {
 	return 1;
     } else {
@@ -1640,6 +1645,51 @@ char *gretl_function_package_get_path (const char *name,
     return ret;
 }
 
+/**
+ * get_package_data_path:
+ * @fname: the basename of the file whose full path is wanted.
+ * @fullname: location to which the full path should be written
+ * (should be at least FILENAME_MAX bytes).
+ * 
+ * Looks for @fname in association with the name of a function
+ * package, which must have been set previously using the
+ * --frompkg option with the "open" command. 
+ *
+ * Returns: 0 on success, non-zero code on error.
+ */
+
+int get_package_data_path (const char *fname, char *fullname)
+{
+    const char *pkgname;
+    int err = 0;
+
+    *fullname = '\0';
+    pkgname = get_optval_string(OPEN, OPT_K);
+
+    if (pkgname == NULL) {
+	err = E_DATA;
+    } else {
+	char *gfnpath = gretl_addon_get_path(pkgname);
+
+	if (gfnpath == NULL) {
+	    err = E_DATA;
+	} else {
+	    char *p = strrchr(gfnpath, SLASH);
+
+	    if (p != NULL) {
+		*(p + 1) = '\0';
+		strcat(fullname, gfnpath);
+		strcat(fullname, fname);
+	    } else {
+		strcpy(fullname, fname);
+	    }
+	    free(gfnpath);
+	}
+    }
+
+    return err;
+}
+
 /**
  * gretl_addpath:
  * @fname: the initially given file name.
@@ -1647,7 +1697,7 @@ char *gretl_function_package_get_path (const char *name,
  * is a hansl script.
  * 
  * Elementary path-searching: try adding various paths to the given
- * @fname and see if it can be opened. Usually called by getopenfile().
+ * @fname and see if it can be opened. Usually called by get_full_filename().
  * If @fname does not have a dot-extension we may also try adding
  * an appropriate gretl extension in case no file is found.
  *
@@ -1833,27 +1883,6 @@ char *gretl_addpath (char *fname, int script)
     return NULL;
 }
 
-static int get_quoted_filename (const char *s, char *fname)
-{
-    int ret = 0;
-
-    if (*s == '"' || *s == '\'') {
-	const char *p = strchr(s + 1, *s);
-
-	if (p != NULL) {
-	    size_t len = p - s;
-
-	    if (len > 0) {
-		*fname = 0;
-		strncat(fname, s+1, len-1);
-		ret = 1;
-	    } 
-	}
-    }
-
-    return ret;
-}
-
 static int substitute_homedir (char *fname)
 {
     char *homedir = getenv("HOME");
@@ -1901,67 +1930,64 @@ static int get_gfn_special (char *fname)
 }
 
 /**
- * getopenfile:
- * @line: command line (e.g. "open foo").
- * @fname: filename to be filled out.
- * @opt: if includes OPT_W, treat as web filename and don't
- * try to add path; if OPT_S, treat as a script; if OPT_I
- * we're responding to the "include" command.
+ * get_full_filename:
+ * @fname: input filename.
+ * @fullname: filename to be filled out: must be at least #MAXLEN bytes.
+ * @opt: if OPT_S, treat as a script; if OPT_I we're responding
+ * to the "include" command; if OPT_W to pass @fname through as is.
  * 
- * Elementary path-searching: try adding various paths to the given
- * @fname and see if it can be opened.
+ * Includes elementary path-searching: try adding various paths to the
+ * given @fname, if appropriate, and see if it can be opened. For
+ * internal gretl use.
  *
- * Returns: 0 on successful parsing of @line, 1 on error.
+ * Returns: 0 on success, non-zero on error.
  */
 
-int getopenfile (const char *line, char *fname, gretlopt opt)
+int get_full_filename (const char *fname, char *fullname, gretlopt opt)
 {
     int script = (opt & (OPT_S | OPT_I))? 1 : 0;
-    int got_quoted;
-    char *test;
+    char *test = NULL;
 
-    /* skip past command word */
-    line += strcspn(line, " ");
-    line += strspn(line, " ");
+    *fullname = '\0';
 
-    got_quoted = get_quoted_filename(line, fname);
-
-    if (!got_quoted && sscanf(line, "%s", fname) != 1) {
-	return E_PARSE;
+    if (fname == NULL || *fname == '\0') {
+	return E_DATA;
     }
 
+    strncat(fullname, fname, MAXLEN - 1);
+
     if (opt & OPT_W) {
 	return 0;
     }
 
     /* handle tilde == HOME */
-    if (fname[0] == '~' && fname[1] == '/') {
-	substitute_homedir(fname);
+    if (fullname[0] == '~' && fullname[1] == '/') {
+	substitute_homedir(fullname);
     }
 
-    if (g_path_is_absolute(fname)) {
+    if (g_path_is_absolute(fullname)) {
 	return 0;
     }
 
-    if (has_suffix(fname, ".gfn") && get_gfn_special(fname)) {
+    if (has_suffix(fullname, ".gfn") && get_gfn_special(fullname)) {
 	return 0;
     }
 
     /* try a basic path search on this filename */
 #if SEARCH_DEBUG
-    fprintf(stderr, "getopenfile: calling addpath on '%s'\n", fname);
+    fprintf(stderr, "get_full_filename: calling addpath on '%s'\n", fullname);
 #endif
-    test = gretl_addpath(fname, script);
+    test = gretl_addpath(fullname, script);
 #if SEARCH_DEBUG
-    fprintf(stderr, "getopenfile: after: '%s'\n", fname);
+    fprintf(stderr, "get_full_filename: after: '%s'\n", fullname);
 #endif
 
     if (test != NULL && (opt & OPT_S)) {
-	int spos = gretl_slashpos(fname);
+	int spos = gretl_slashpos(fullname);
 
 	if (spos) {
 	    *current_dir = '\0';
-	    strncat(current_dir, fname, spos + 1);
+	    strncat(current_dir, fullname, spos + 1);
 	} else {
 	    current_dir[0] = '.';
 	    current_dir[1] = SLASH;
@@ -2981,7 +3007,13 @@ static void load_default_path (char *targ)
     } else if (targ == paths.mpi_hosts) {
 	*targ = '\0';
     } else if (targ == paths.pngfont) {
-	strcpy(targ, "verdana 8");
+	if (chinese_locale()) {
+	    strcpy(targ, "SimSun 8");
+	} else if (japanese_locale()) {
+	    strcpy(targ, "Meiryo 8");
+	} else {
+	    strcpy(targ, "verdana 8");
+	}
     }
 
     free(progfiles);
@@ -3512,6 +3544,8 @@ void get_gretl_config_from_file (FILE *fp, ConfigPaths *cpaths,
 	    libset_set_bool(SHELL_OK, rc_bool(val));
 	} else if (!strcmp(key, "usecwd")) {
 	    handle_use_cwd(rc_bool(val), cpaths);
+	} else if (!strcmp(key, "lcnumeric")) {
+	    libset_set_bool(FORCE_DECP, !rc_bool(val));
 	} else if (!strcmp(key, "dbhost")) {
 	    strncat(cpaths->dbhost, val, 32 - 1);
 	} else if (!strcmp(key, "dbproxy")) {
@@ -3667,3 +3701,91 @@ const char *gretl_app_support_dir (void)
 }
 
 #endif
+
+static int dir_is_writable (const char *dirname)
+{
+    int ok = 0;
+
+    if (gretl_mkdir(dirname) == 0) {
+	gchar *test = g_strdup_printf("%s%c%s", dirname, SLASH, "wtest");
+
+	if (test != NULL) {
+	    ok = (gretl_test_fopen(test, "w") == 0);
+	    g_free(test);
+	}
+    }
+
+    return ok;
+}
+
+static int get_user_install_path (char *path, const char *subdir)
+{
+#ifdef OS_OSX
+    const char *dirname = gretl_app_support_dir();
+#else
+    const char *dirname = gretl_dotdir();
+#endif
+    int err = 0;
+
+    if (dirname == NULL || *dirname == '\0') {
+	err = E_FOPEN;
+    } else {
+	sprintf(path, "%s%s", dirname, subdir);
+	err = (dir_is_writable(path) == 0);
+    }
+
+    return err;
+}
+
+static int get_system_install_path (char *path, const char *subdir)
+{
+    sprintf(path, "%s%s", gretl_home(), subdir);
+
+    if (dir_is_writable(path)) {
+	return 0;
+    } else {
+	return E_FOPEN;
+    }
+}
+
+/* get a path that's suitable for writing a function
+   package on installation 
+*/
+
+const char *gretl_function_package_path (void)
+{
+    static char path[FILENAME_MAX];
+
+    if (*path == '\0') {
+	int sys_first = 1;
+	int err = 0;
+	
+#if defined(OS_OSX)
+	/* we prefer writing to ~/Library/Application Support */
+	sys_first = 0;
+#elif defined(WIN32)
+	if (win32_uses_virtual_store()) {
+	    /* don't write to virtualized location */
+	    sys_first = 0;
+	}    
+#endif
+
+	if (sys_first) {
+	    err = get_system_install_path(path, "functions");
+	    if (err) {
+		err = get_user_install_path(path, "functions");
+	    }
+	} else {
+	    err = get_user_install_path(path, "functions");
+	}
+
+	if (err) {
+	    *path = '\0';
+	} else {
+	    slash_terminate(path);
+	}
+    }
+
+    return path;
+}
+
