@@ -128,7 +128,8 @@ int gretl_isdir (const char *path);
 
 char *gretl_addpath (char *fname, int script);
 
-int getopenfile (const char *line, char *fname, gretlopt opt);
+int get_full_filename (const char *fname, char *fullname, 
+		       gretlopt opt);
 
 int fname_has_path (const char *fname);
 
@@ -145,6 +146,8 @@ char **get_plausible_search_dirs (SearchType stype, int *n_dirs);
 char *gretl_function_package_get_path (const char *name,
 				       PkgType type);
 
+int get_package_data_path (const char *fname, char *fullname);
+
 void set_gretl_plugin_path (const char *path);
 
 const char *helpfile_path (int id);
@@ -209,6 +212,8 @@ const char *gretl_mpiexec (void);
 
 const char *gretl_current_dir (void);
 
+const char *gretl_function_package_path (void);
+
 void gretl_set_current_dir (const char *s);
 
 void set_gretl_png_font (const char *s);
