@@ -286,6 +286,34 @@ series_table *series_table_new (char **strs, int n_strs)
     return st;
 }
 
+series_table *series_table_copy (series_table *st)
+{
+    series_table *ret = NULL;
+
+    if (st != NULL) {
+	ret = series_table_alloc();
+    }
+
+    if (ret != NULL) {
+	char **S = strings_array_dup(st->strs, st->n_strs);
+	int i;
+
+	if (S == NULL) {
+	    series_table_destroy(ret);
+	    ret = NULL;
+	} else {
+	    ret->n_strs = st->n_strs;
+	    ret->strs = S;
+	    for (i=0; i<ret->n_strs; i++) {
+		g_hash_table_insert(ret->ht, (gpointer) ret->strs[i], 
+				    GINT_TO_POINTER(i+1));
+	    }
+	}
+    }
+
+    return ret;
+}
+
 static series_table *
 gretl_string_table_add_column (gretl_string_table *gst, int colnum)
 {
@@ -530,6 +558,53 @@ int gretl_string_table_print (gretl_string_table *gst, DATASET *dset,
     return err;
 }
 
+/**
+ * gretl_string_table_validate:
+ * @gst: gretl string table.
+ *
+ * Checks that the "string values" in @gst are not in fact
+ * undigested quasi-numerical values. We run this on 
+ * imported "CSV" data to ensure we don't produce
+ * misleading results.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int gretl_string_table_validate (gretl_string_table *gst)
+{
+    const char *test = "0123456789.,";
+    int i, ncols = 0;
+    int err = 0;
+
+    if (gst != NULL && gst->cols_list != NULL) {
+	ncols = gst->cols_list[0];
+    }
+
+    for (i=0; i<ncols; i++) {
+	series_table *st = gst->cols[i];
+	const char *s;
+	int j, myerr = E_DATA;
+
+	for (j=0; j<st->n_strs; j++) {
+	    s = st->strs[j];
+	    if (*s == '-' || *s == '+') s++;
+	    if (strspn(s, test) < strlen(s)) {
+		/* not quasi-numeric */
+		myerr = 0;
+		break;
+
+	    }
+	}
+
+	if (myerr) {
+	    err = myerr;
+	    break;
+	}
+    }
+
+    return err;
+}
+
 /**
  * gretl_string_table_save:
  * @gst: gretl string table.
@@ -852,6 +927,11 @@ char *retrieve_file_content (const char *fname, const char *codeset,
 	}
     }
 
+    if (*err && content != NULL) {
+	free(content);
+	content = NULL;
+    }
+
     return content;
 }
 
@@ -926,8 +1006,6 @@ static void too_long (void)
 			   "(%d bytes) exceeded\n"), MAXLINE);
 }
 
-#define PRINTF_SPECIAL 0
-
 int substitute_named_strings (char *line, int *subst)
 {
     char sname[VNAMELEN];
@@ -958,9 +1036,6 @@ int substitute_named_strings (char *line, int *subst)
 	    if (*s == '"' && (bs % 2 == 0)) {
 		/* reached end of (s)printf format string */
 		in_format = 0;
-#if PRINTF_SPECIAL
-		break;
-#endif
 	    }
 	    if (*s == '\\') {
 		bs++;
