@@ -33,6 +33,10 @@
 # include "gretl_www.h"
 #endif
 
+#ifdef WIN32
+# include "gretl_win32.h"
+#endif
+
 #ifdef _OPENMP
 # include <omp.h>
 #endif
@@ -49,6 +53,7 @@
 #include <gmp.h>
 
 static void gretl_tests_cleanup (void);
+static int have_optimized_blas (void);
 
 /**
  * date_as_double:
@@ -521,6 +526,8 @@ int true_const (int v, const DATASET *dset)
     return gretl_isunits(dset->t1, dset->t2, dset->Z[v]);
 }
 
+#define NAN_HIGH 1 /* like Matlab/Octave */
+
 /**
  * gretl_compare_doubles:
  * @a: pointer to first element to compare.
@@ -536,8 +543,22 @@ int gretl_compare_doubles (const void *a, const void *b)
 {
     const double *da = (const double *) a;
     const double *db = (const double *) b;
-     
+
+#if NAN_HIGH
+    if (isnan(*da) || isnan(*db)) {
+	if (!isnan(*da)) {
+	    return -1;
+	} else if (!isnan(*db)) {
+	    return 1;
+	} else {
+	    return 0;
+	}
+    } else {
+	return (*da > *db) - (*da < *db);
+    }
+#else
     return (*da > *db) - (*da < *db);
+#endif    
 }
 
 /**
@@ -556,7 +577,21 @@ int gretl_inverse_compare_doubles (const void *a, const void *b)
     const double *da = (const double *) a;
     const double *db = (const double *) b;
 
+#if NAN_HIGH    
+    if (isnan(*da) || isnan(*db)) {
+	if (!isnan(*da)) {
+	    return 1;
+	} else if (!isnan(*db)) {
+	    return -1;
+	} else {
+	    return 0;
+	}
+    } else {
+	return (*da < *db) - (*da > *db);
+    }      
+#else
     return (*da < *db) - (*da > *db);
+#endif    
 }
 
 /**
@@ -916,9 +951,11 @@ static void maybe_fix_daily_start (long *ed, int pd)
 
 #define recognized_ts_frequency(f) (f == 4 || f == 12 || f == 24)
 
-static int process_starting_obs (char *stobs, int pd, int *pstructure,
-				 double *psd0, int *pdated)
+static int process_starting_obs (const char *stobs_in, int pd, 
+				 int *pstructure, double *psd0,
+				 int *pdated)
 {
+    char stobs[OBSLEN];
     int structure = *pstructure;
     double sd0 = 0.0;
     int maybe_tseries = 1;
@@ -931,6 +968,9 @@ static int process_starting_obs (char *stobs, int pd, int *pstructure,
 	maybe_tseries = 0;
     }
 
+    *stobs = '\0';
+    strncat(stobs, stobs_in, OBSLEN - 1);
+
     /* truncate stobs if not a calendar date */
 
     if (likely_calendar_obs_string(stobs)) {
@@ -1014,7 +1054,8 @@ static int process_starting_obs (char *stobs, int pd, int *pstructure,
 
 /**
  * set_obs:
- * @line: command line.
+ * @parm1: first parameter.
+ * @parm2: second parameter.
  * @dset: dataset struct.
  * @opt: %OPT_S for stacked time-series, %OPT_C for stacked cross-section,
  * %OPT_T for time series, %OPT_X for cross section, %OPT_P to set
@@ -1028,10 +1069,10 @@ static int process_starting_obs (char *stobs, int pd, int *pstructure,
  * Returns: 0 on successful completion, non-zero on error.
  */
 
-int set_obs (const char *line, DATASET *dset, gretlopt opt)
+int set_obs (const char *parm1, const char *parm2,
+	     DATASET *dset, gretlopt opt)
 {
-    char pdstr[VNAMELEN];
-    char stobs[OBSLEN];
+    const char *stobs = NULL;
     int structure = STRUCTURE_UNKNOWN;
     double sd0 = dset->sd0;
     int pd, dated = 0;
@@ -1057,34 +1098,32 @@ int set_obs (const char *line, DATASET *dset, gretlopt opt)
 	return switch_panel_orientation(dset);
     }    
 
-    if (!strcmp(line, "setobs")) {
+    if (opt == OPT_NONE && parm1 == NULL && parm2 == NULL) {
 	/* we'll just print current obs info */
 	return 0;
     }
 
+    if (parm1 == NULL || parm2 == NULL) {
+	return E_ARGS;
+    }
+
     if (opt & OPT_P) {
-	return set_panel_structure_from_line(line, dset);
+	return set_panel_structure_from_varnames(parm1, parm2, dset);
     } else if (opt & OPT_G) {
 	/* --panel-groups */
-	return set_panel_group_strings(line, dset);
+	return set_panel_group_strings(parm1, parm2, dset);
     }
 
     /* now we get down to business */
 
-    if (sscanf(line, "%*s %15s %10s", pdstr, stobs) != 2) {
-	gretl_errmsg_set(_("Failed to parse line as frequency, startobs"));
-	return E_PARSE;
-    }
-
-    pd = gretl_int_from_string(pdstr, &err);
+    pd = gretl_int_from_string(parm1, &err);
     if (!err && pd < 1) {
 	gretl_errmsg_sprintf(_("frequency (%d) does not make seem to make sense"), pd);
-	err = E_DATA;
-    }
-    if (err) {
-	return err;
+	return E_DATA;
     }
 
+    stobs = parm2;
+
     /* if an explicit structure option was passed in, respect it */
     if (opt == OPT_X) {
 	structure = CROSS_SECTION;
@@ -1159,6 +1198,78 @@ int set_obs (const char *line, DATASET *dset, gretlopt opt)
     return err;
 }
 
+/**
+ * simple_set_obs:
+ * @dset: pointer to data information struct.
+ * @pd: data frequency.
+ * @stobs: string representation of starting observation.
+ * @opt: options flags.
+ * 
+ * See the "setobs" command.
+ *
+ * Returns: 0 on success, non-zero code on error.
+ */
+
+int simple_set_obs (DATASET *dset, int pd, const char *stobs, 
+		    gretlopt opt)
+{
+    int structure = STRUCTURE_UNKNOWN;
+    double sd0 = dset->sd0;
+    int dated = 0;
+    int panel = 0;
+    int err = 0;
+
+    if (opt == OPT_X) {
+	structure = CROSS_SECTION;
+    } else if (opt == OPT_T) {
+	structure = TIME_SERIES;
+    } else if (opt == OPT_S) {
+	structure = STACKED_TIME_SERIES;
+	panel = 1;
+    } else if (opt == OPT_C) {
+	structure = STACKED_CROSS_SECTION;
+	panel = 1;
+    } else if (opt == OPT_N) {
+	structure = SPECIAL_TIME_SERIES;
+    } else if (opt == OPT_I) {
+	/* --panel-time */
+	structure = TIME_SERIES;
+    }
+
+    err = process_starting_obs(stobs, pd, &structure, &sd0, &dated);
+
+    if (err) {
+	return err;
+    }
+
+    if (panel && dset->n % pd != 0) {
+	int sts = structure == STACKED_TIME_SERIES;
+
+	gretl_errmsg_sprintf(_("Panel datasets must be balanced.\n"
+			       "The number of observations (%d) is not a multiple\n"
+			       "of the number of %s (%d)."), 
+			     dset->n, sts ? _("periods") : _("units"), pd);
+	return E_DATA;
+    }
+
+    if (dated) {
+	/* replace any existing markers with date strings */
+	dataset_destroy_obs_markers(dset);
+    } else if (structure == TIME_SERIES && (pd == 1 || pd == 4 || pd == 12)) {
+	/* force use of regular time-series obs labels */
+	dataset_destroy_obs_markers(dset);
+    }
+
+    dset->pd = pd;
+    dset->structure = structure;
+    dset->sd0 = sd0;
+
+    ntodate(dset->stobs, 0, dset); 
+    ntodate(dset->endobs, dset->n - 1, dset);
+
+    return err;
+}
+
 /**
  * gretl_double_from_string:
  * @s: string to examine.
@@ -1177,7 +1288,7 @@ double gretl_double_from_string (const char *s, int *err)
     char *test;
     double x;
 
-    if (s == NULL || *s == 0) {
+    if (s == NULL || *s == '\0') {
 	*err = E_DATA;
 	return NADBL;
     }
@@ -1196,14 +1307,7 @@ double gretl_double_from_string (const char *s, int *err)
 	return x;
     }
 
-    if (gretl_is_scalar(s)) {
-	x = gretl_scalar_get_value(s, NULL);
-    } else {
-	*err = E_DATA;
-	x = NADBL;
-    } 
-
-    return x;    
+    return get_scalar_value_by_name(s, err);
 }
 
 /**
@@ -1242,18 +1346,19 @@ int gretl_int_from_string (const char *s, int *err)
 
     if (*test == '\0') {
 	return n;
-    } else if (gretl_is_scalar(s)) {
-	x = gretl_scalar_get_value(s, NULL);
+    }
+
+    x = get_scalar_value_by_name(s, err);
+
+    if (!*err) {
 	if (na(x)) {
 	    *err = E_MISSDATA;
 	} else if (fabs(x) > INT_MAX) {
 	    *err = E_DATA;
 	} else {
 	    n = (int) x;
-	}	
-    } else {
-	*err = E_DATA;
-    } 
+	}
+    }
 
     return n;    
 }
@@ -1861,6 +1966,10 @@ void libgretl_init (void)
     gretl_xml_init();
     gretl_stopwatch_init();
     mpf_set_default_prec(get_mp_bits());
+
+    if (have_optimized_blas()) {
+	set_blas_mnk_min(90000);
+    } 
 }
 
 #ifdef HAVE_MPI
@@ -1893,7 +2002,7 @@ int libgretl_mpi_init (int self, int np, int dcmt)
 
     if (dcmt && np > 1) {
 	/* use DCMT for multiple RNGs */
-	gretl_dcmt_init(np, self, 4172);
+	gretl_dcmt_init(np, self, 0);
     } else {
 	/* just use one RNG */
 	gretl_rand_init();
@@ -1929,6 +2038,8 @@ void libgretl_session_cleanup (int mode)
     gretl_lists_cleanup();
     gretl_tests_cleanup();
     gretl_plotx(NULL, OPT_NONE);
+
+    /* scrub options set via "setopt" */
     setopt_cleanup();
 
     if (mode != SESSION_CLEAR_DATASET) {
@@ -1958,7 +2069,7 @@ void libgretl_cleanup (void)
     gretl_function_hash_cleanup();
     lapack_mem_free();
     forecast_matrix_cleanup();
-    option_params_cleanup();
+    stored_options_cleanup();
     option_printing_cleanup();
     kalman_cleanup();
     gnuplot_cleanup();
@@ -1984,22 +2095,27 @@ enum {
     GET_TEST_STAT,
     GET_TEST_PVAL,
     GET_TEST_LNL,
+    GET_TEST_BRK,
     TESTS_CLEANUP
 };
 
 #define getcode(c) (c == GET_TEST_STAT || c == GET_TEST_PVAL || \
-                    c == GET_TEST_LNL)
+                    c == GET_TEST_LNL || c == GET_TEST_BRK)
 
 static GretlType last_test_type;
 
-static double
-record_or_get_test_result (double teststat, double pval, double lnl,
-			   char *instr, int code)
+static double record_or_get_test_result (double teststat,
+					 double pval,
+					 double lnl,
+					 double inbrk,
+					 char *instr,
+					 int code)
 {
     static char savestr[MAXLABEL] = {0};
     static double val = NADBL;
     static double pv = NADBL;
     static double ll = NADBL;
+    static double brk = NADBL;
     double ret = NADBL;
 
     if (code == SET_TEST_STAT) {
@@ -2007,6 +2123,7 @@ record_or_get_test_result (double teststat, double pval, double lnl,
 	val = teststat;
 	pv = pval;
 	ll = lnl;
+	brk = inbrk;
 	*savestr = '\0';
 	if (instr != NULL) {
 	    strncat(savestr, instr, MAXLABEL - 1);
@@ -2019,6 +2136,8 @@ record_or_get_test_result (double teststat, double pval, double lnl,
 		sprintf(instr, _("p-value for %s test"), savestr);
 	    } else if (code == GET_TEST_LNL) {
 		sprintf(instr, _("log-likelihood for %s test"), savestr);
+	    } else if (code == GET_TEST_BRK) {
+		sprintf(instr, _("break-point for %s test"), savestr);
 	    }
 	}
 	if (code == GET_TEST_STAT) {
@@ -2027,6 +2146,8 @@ record_or_get_test_result (double teststat, double pval, double lnl,
 	    ret = pv;
 	} else if (code == GET_TEST_LNL) {
 	    ret = ll;
+	} else if (code == GET_TEST_BRK) {
+	    ret = brk;
 	}
     } 
 	
@@ -2088,13 +2209,21 @@ int get_last_test_type (void)
 
 void record_test_result (double teststat, double pval, char *blurb)
 {
-    record_or_get_test_result(teststat, pval, NADBL, blurb, SET_TEST_STAT);
+    record_or_get_test_result(teststat, pval, NADBL, NADBL, blurb,
+			      SET_TEST_STAT);
 }
 
 void record_LR_test_result (double teststat, double pval, double lnl,
 			    char *blurb)
 {
-    record_or_get_test_result(teststat, pval, lnl, blurb, SET_TEST_STAT);
+    record_or_get_test_result(teststat, pval, lnl, NADBL, blurb,
+			      SET_TEST_STAT);
+}
+
+void record_QLR_test_result (double teststat, double pval, double brk)
+{
+    record_or_get_test_result(teststat, pval, NADBL, brk, "QLR",
+			      SET_TEST_STAT);
 }
 
 /* Note: the hypothesis-test recorder "takes ownership" of the
@@ -2110,17 +2239,22 @@ void record_matrix_test_result (gretl_matrix *tests,
 
 double get_last_test_statistic (char *blurb)
 {
-    return record_or_get_test_result(0, 0, 0, blurb, GET_TEST_STAT);
+    return record_or_get_test_result(0, 0, 0, 0, blurb, GET_TEST_STAT);
 }
 
 double get_last_pvalue (char *blurb)
 {
-    return record_or_get_test_result(0, 0, 0, blurb, GET_TEST_PVAL);
+    return record_or_get_test_result(0, 0, 0, 0, blurb, GET_TEST_PVAL);
 }
 
 double get_last_lnl (char *blurb)
 {
-    return record_or_get_test_result(0, 0, 0, blurb, GET_TEST_LNL);
+    return record_or_get_test_result(0, 0, 0, 0, blurb, GET_TEST_LNL);
+}
+
+double get_last_break (char *blurb)
+{
+    return record_or_get_test_result(0, 0, 0, 0, blurb, GET_TEST_BRK);
 }
 
 gretl_matrix *get_last_test_matrix (int *err)
@@ -2361,6 +2495,7 @@ static int real_detect_blas (const char *s)
 
     while (*s) {
 	if (*s == '\n') {
+	    /* got to the end of a line */
 	    line[i] = '\0';
 	    if (strstr(line, "libmkl")) {
 		found[3] = 'y';
@@ -2380,24 +2515,26 @@ static int real_detect_blas (const char *s)
 	s++;
     }
 
-    fputs("detect blas: ", stderr);
+    /* allow for the possibility that we have a symlink
+       from libblas to one or other of the optimized 
+       BLAS variants 
+    */
+    if (found[0] == 'y' && strcmp(found + 1, "nnn")) {
+	found[0] = 'n';
+    }
 
     if (strcmp(found, "nnny") == 0) {
-	fputs("MKL\n", stderr);
 	ret = BLAS_MKL;
     } else if (strcmp(found, "nnyn") == 0) {
-	fputs("OpenBLAS\n", stderr);
 	ret = BLAS_OPENBLAS;
     } else if (strcmp(found, "nynn") == 0) {
-	fputs("ATLAS\n", stderr);
 	ret = BLAS_ATLAS;
     } else if (strcmp(found, "ynnn") == 0) {
-	fputs("Netlib\n", stderr);
 	ret = BLAS_NETLIB;
     } else if (strcmp(found, "nnnn") == 0) {
-	fputs("found no relevant libs!\n", stderr);
+	fputs("detect blas: found no relevant libs!\n", stderr);
     } else {
-	fputs("confused, found too many blas libs!\n", stderr);
+	fputs("detect blas: confused, found too many blas libs!\n", stderr);
     }
 
     return ret;
@@ -2454,7 +2591,7 @@ static int blas_variant_code (void)
 #elif defined(OS_OSX)
     return BLAS_VECLIB;
 #else
-    /* not a packaged build: we have to check */
+    /* not an own-packaged build: we have to check */
     static int bver = -1;
 
     if (bver < 0) {
@@ -2465,6 +2602,16 @@ static int blas_variant_code (void)
 #endif
 }
 
+static int have_optimized_blas (void)
+{
+    int bvc = blas_variant_code();
+
+    return bvc == BLAS_OPENBLAS ||
+	bvc == BLAS_VECLIB ||
+	bvc == BLAS_MKL ||
+	bvc == BLAS_ATLAS;
+}
+
 const char *blas_variant_string (void)
 {
     int bver = blas_variant_code();
