@@ -78,7 +78,11 @@ int true_const (int v, const DATASET *dset);
 
 char *format_obs (char *obs, int maj, int min, int pd);
 
-int set_obs (const char *line, DATASET *dset, gretlopt opt);
+int set_obs (const char *parm1, const char *parm2,
+	     DATASET *dset, gretlopt opt);
+
+int simple_set_obs (DATASET *dset, int pd, const char *stobs, 
+		    gretlopt opt);
 
 /* sorting and comparison */
 
@@ -154,12 +158,16 @@ void record_matrix_test_result (gretl_matrix *tests,
 void record_LR_test_result (double teststat, double pval, double lnl,
 			    char *blurb);
 
+void record_QLR_test_result (double teststat, double pval, double brk);
+
 double get_last_test_statistic (char *blurb);
 
 double get_last_pvalue (char *blurb);
 
 double get_last_lnl (char *blurb);
 
+double get_last_break (char *blurb);
+
 gretl_matrix *get_last_test_matrix (int *err);
 
 gretl_matrix *get_last_pvals_matrix (int *err);
