@@ -23,9 +23,7 @@
 #include "libset.h"
 #include "gretl_www.h"
 
-#include <glib.h>
-
-#include <windows.h>
+#include "gretl_win32.h"
 #include <shlobj.h>
 #include <aclapi.h>
 
@@ -329,28 +327,13 @@ int gretl_spawn (char *cmdline)
 static char *win_special_path (int folder)
 {
     TCHAR dpath[MAX_PATH];
-    LPITEMIDLIST id_list;
-    DWORD result;
-    LPMALLOC allocator;
-    char *ret = NULL;
 
-    if (SHGetSpecialFolderLocation(NULL, folder | CSIDL_FLAG_CREATE, 
-				   &id_list) != S_OK) {
+    if (SHGetFolderPath(NULL, folder | CSIDL_FLAG_CREATE,
+			NULL, 0, dpath) != S_OK) {
 	return NULL;
     }
 
-    result = SHGetPathFromIDList(id_list, dpath);
-
-    if (result) {
-	ret = gretl_strdup(dpath);
-    }
-
-    if (SHGetMalloc(&allocator) == S_OK) {
-	allocator->lpVtbl->Free(allocator, id_list);
-	allocator->lpVtbl->Release(allocator);
-    }
-
-    return ret;
+    return gretl_strdup(dpath);
 }
 
 char *desktop_path (void)
@@ -570,10 +553,9 @@ int gretl_shell_grab (const char *arg, char **sout)
     return run_cmd_with_pipes(arg, sout, NULL, SHELL_RUN);
 }
 
-int gretl_shell (const char *arg, PRN *prn)
+int gretl_shell (const char *arg, gretlopt opt, PRN *prn)
 {
     UINT winret;
-    int async = 0;
     int err = 0;
 
     if (arg == NULL || *arg == '\0') {
@@ -585,16 +567,9 @@ int gretl_shell (const char *arg, PRN *prn)
 	return 1;
     }
 
-    if (!strncmp(arg, "launch ", 7)) {
-	async = 1;
-	arg += 7;
-    } else if (*arg == '!') {
-	arg++;
-    }
-
     arg += strspn(arg, " \t");
 
-    if (async) {
+    if (opt & OPT_A) {
 	winret = WinExec(arg, SW_SHOWNORMAL);
 	if (winret <= 31) {
 	    err = 1;
@@ -638,7 +613,7 @@ int win32_write_access (char *path)
     domain = LocalAlloc(0, dlen * sizeof *domain);
     if (sid == NULL || domain == NULL) {
 	err = 1;
-    } 
+    }
 
     if (!err) {
 	/* call the function for real */
@@ -650,10 +625,17 @@ int win32_write_access (char *path)
     if (!err) {
 	/* build a trustee and get the file's DACL */
 	BuildTrusteeWithSid(&t, sid);
+#ifdef _WIN64
 	ret = GetNamedSecurityInfo(path, SE_FILE_OBJECT, 
 				   DACL_SECURITY_INFORMATION, 
 				   NULL, NULL, &dacl, NULL, 
-				   (void **) &sd);
+				   (PSECURITY_DESCRIPTOR) &sd);
+#else
+	ret = GetNamedSecurityInfo(path, SE_FILE_OBJECT, 
+				   DACL_SECURITY_INFORMATION, 
+				   NULL, NULL, &dacl, NULL, 
+				   &sd);
+#endif
 	err = (ret != ERROR_SUCCESS);
     }
 
@@ -670,9 +652,6 @@ int win32_write_access (char *path)
 	}
     }
 
-    if (dacl != NULL) {
-	LocalFree(dacl);
-    }
     if (sid != NULL) {
 	LocalFree(sid);
     }    
@@ -719,6 +698,58 @@ int win32_delete_dir (const char *path)
     return err;
 }
 
+static BOOL running_as_admin (void)
+{
+    SID_IDENTIFIER_AUTHORITY auth = {SECURITY_NT_AUTHORITY};
+    PSID admin_group = NULL;
+    BOOL ok, ret = FALSE;
+
+    ok = AllocateAndInitializeSid(&auth, 2,
+				  SECURITY_BUILTIN_DOMAIN_RID, 
+				  DOMAIN_ALIAS_RID_ADMINS, 
+				  0, 0, 0, 0, 0, 0, 
+				  &admin_group);
+
+    if (ok) {
+	ok = CheckTokenMembership(NULL, admin_group, &ret);
+    }
+
+    if (!ok) {
+	fprintf(stderr, "running_as_admin: the check failed\n");
+    }
+ 
+    if (admin_group != NULL) {
+	FreeSid(admin_group);
+    }
+ 
+    return ret;
+}
+
+/* Note: at some point MS will scrap the virtual store
+   apparatus (supposedly), in which case we'll want to
+   add a further condition in this function -- that is,
+   dwMajorVersion < something.
+*/
+
+int win32_uses_virtual_store (void)
+{
+    OSVERSIONINFO osvi;
+    int ret;
+
+    ZeroMemory(&osvi, sizeof osvi);
+    osvi.dwOSVersionInfoSize = sizeof osvi;
+
+    GetVersionEx(&osvi);
+    /* VirtualStore came in with Vista */
+    ret = osvi.dwMajorVersion > 5;
+    
+    if (ret && running_as_admin()) {
+	ret = 0;
+    }
+
+    return ret;
+}
+
 char *slash_convert (char *str, int which)
 {
     char *p;
