@@ -51,7 +51,7 @@ int win_run_sync (char *cmdline, const char *currdir);
 
 int gretl_spawn (char *cmdline);
 
-int gretl_shell (const char *arg, PRN *prn);
+int gretl_shell (const char *arg, gretlopt opt, PRN *prn);
 
 int gretl_win32_grab_output (const char *cmdline, char **sout);
 
@@ -69,6 +69,8 @@ int win32_write_access (char *path);
 
 int win32_delete_dir (const char *path);
 
+int win32_uses_virtual_store (void);
+
 int R_path_from_registry (char *s, int which);
 
 int maybe_print_R_path_addition (FILE *fp);
