@@ -19,10 +19,6 @@
 
 /* gretl_www.c for gretl -- uses libcurl API */
 
-#ifdef WIN64
-# include <winsock2.h>
-#endif
-
 #include "libgretl.h"
 #include "libset.h"
 #include "build.h"
@@ -57,10 +53,12 @@ static char proxyhost[128];
 static char sffiles[DBHLEN] = "downloads.sourceforge.net";
 static char sfweb[DBHLEN]   = "gretl.sourceforge.net";
 
+#define URLLEN 1024
+
 typedef struct urlinfo_ urlinfo;
 
 struct urlinfo_ {
-    char url[1024];          /* the URL */
+    char url[URLLEN];        /* the URL */
     int err;                 /* error code */
     int verbose;             /* verbosity level */
     int saveopt;             /* if saving data: to buffer or file? */
@@ -80,6 +78,7 @@ static void urlinfo_init (urlinfo *u,
 			  const char *localfile)
 {
     u->url[0] = '\0';
+
     if (hostname != NULL) {
 	sprintf(u->url, "http://%s", hostname);
     }
@@ -112,6 +111,12 @@ static void urlinfo_init (urlinfo *u,
 #endif
 }
 
+static void urlinfo_set_url (urlinfo *u, const char *url)
+{
+    u->url[0] = '\0';
+    strncat(u->url, url, URLLEN - 1);
+}
+
 static void urlinfo_finalize (urlinfo *u, char **getbuf, int *err)
 {
     if (u->fp != NULL) {
@@ -133,7 +138,7 @@ static void urlinfo_finalize (urlinfo *u, char **getbuf, int *err)
 
 static void urlinfo_set_show_progress (urlinfo *u)
 {
-    int (*show_progress) (gint64, gint64, int) = NULL;
+    int (*show_progress) (double, double, int) = NULL;
 
     show_progress = get_plugin_function("show_progress");
     if (show_progress != NULL) {
@@ -147,13 +152,11 @@ static int progress_func (void *clientp, double dltotal, double dlnow,
     urlinfo *u = (urlinfo *) clientp;
     int ret = 0;
 
-    if (u->progfunc != NULL && dltotal > 0) {
-	if (!u->pstarted) {
-	    u->progfunc((long) dlnow, (long) dltotal, SP_LOAD_INIT);
-	    u->pstarted = 1;
-	} else {
-	    ret = u->progfunc((long) dlnow, (long) dltotal, SP_TOTAL);
-	}
+    if (u->pstarted) {
+	ret = u->progfunc(dlnow, dltotal, SP_TOTAL);
+    } else if (u->progfunc != NULL && dltotal > 1024) {
+	u->progfunc(dlnow, dltotal, SP_LOAD_INIT);
+	u->pstarted = 1;
     }
 
     return (ret == SP_RETURN_CANCELED) ? 1 : 0;
@@ -161,8 +164,9 @@ static int progress_func (void *clientp, double dltotal, double dlnow,
 
 static void stop_progress_bar (urlinfo *u)
 {
-    if (u->progfunc != NULL) {
+    if (u->progfunc != NULL && u->pstarted) {
 	u->progfunc(0, 1024, SP_FINISH);
+	u->pstarted = 0;
     }
 }
 
@@ -180,18 +184,22 @@ static int grow_read_buffer (urlinfo *u, size_t bgot)
     if (newbuf == NULL) {
 	return E_ALLOC;
     } else {
-	size_t addlen = newlen - u->buflen;
+	size_t zerolen = newlen - u->datalen;
 
 	/* zero the additional memory chunk */
-	memset(newbuf + u->datalen, 0, addlen);
+	memset(newbuf + u->datalen, 0, zerolen);
 	u->getbuf = newbuf;
 	u->buflen = newlen;
+#if WDEBUG
+	fprintf(stderr, "u->getbuf realloc'd at %p (len %d)\n", 
+		(void *) u->getbuf, (int) u->buflen);
+#endif
 	return 0;
     }
 }
 
-static size_t write_func (void *buf, size_t size, size_t nmemb, 
-			  void *data)
+static size_t gretl_write_func (void *buf, size_t size, size_t nmemb, 
+				void *data)
 {
     urlinfo *u = (urlinfo *) data;
     size_t bgot = size * nmemb;
@@ -218,6 +226,9 @@ static size_t write_func (void *buf, size_t size, size_t nmemb,
     } else if (u->saveopt == SAVE_TO_BUFFER) {
 	if (u->getbuf == NULL) {
 	    u->getbuf = calloc(WBUFSIZE, 1);
+#if WDEBUG
+	    fprintf(stderr, "u->getbuf started at %p\n", (void *) u->getbuf);
+#endif
 	    if (u->getbuf == NULL) {
 		u->err = E_ALLOC;
 		return 0;
@@ -253,6 +264,7 @@ static int progress_bar_wanted (int opt)
 		opt == GRAB_PKG ||
 		opt == GRAB_FOREIGN);
     }
+    
     return 0;
 }
 
@@ -279,6 +291,10 @@ static const char *print_option (int opt)
 	return "CHECK_DB";
     case LIST_PKGS:
 	return "LIST_PKGS";
+    case GRAB_FUNC_INFO:
+	return "GRAB_FUNC_INFO";
+    case FUNC_FULLNAME:
+	return "FUNC_FULLNAME";
     default:
 	break;
     }
@@ -294,7 +310,8 @@ static void urlinfo_set_params (urlinfo *u, CGIOpt opt,
     strcat(u->url, print_option(opt));
 
     if (fname != NULL) {
-	if (opt == GRAB_FILE || opt == GRAB_FUNC) {
+	if (opt == GRAB_FILE || opt == GRAB_FUNC ||
+	    opt == GRAB_FUNC_INFO || opt == FUNC_FULLNAME) {
 	    strcat(u->url, "&fname=");
 	} else {
 	    strcat(u->url, "&dbase=");
@@ -362,7 +379,7 @@ static int curl_get (urlinfo *u)
 	    fprintf(stderr, "curl_get: %s\n", u->url);
 	}
 	curl_easy_setopt(curl, CURLOPT_URL, u->url);
-	curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_func);
+	curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, gretl_write_func);
 	curl_easy_setopt(curl, CURLOPT_WRITEDATA, u);
 	curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);
 	curl_easy_setopt(curl, CURLOPT_USERAGENT, u->agent);
@@ -536,7 +553,7 @@ int get_update_info (char **saver, int verbose)
 
 int upload_function_package (const char *login, const char *pass, 
 			     const char *fname, const char *buf,
-			     char **retbuf)
+			     size_t buflen, char **retbuf)
 {
     CURL *curl;
     CURLcode res;
@@ -567,19 +584,22 @@ int upload_function_package (const char *login, const char *pass,
     } else {
 	struct curl_httppost *post = NULL;
 	struct curl_httppost *last = NULL;
+	int zipfile = has_suffix(fname, ".zip");
+	char sizestr[32];
 	
 	curl_easy_setopt(curl, CURLOPT_URL, u.url);
 	curl_easy_setopt(curl, CURLOPT_USERAGENT, u.agent);
 	curl_easy_setopt(curl, CURLOPT_VERBOSE, u.verbose);
+
 	if (saveopt == SAVE_TO_BUFFER) {
-	    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_func);
+	    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, gretl_write_func);
 	    curl_easy_setopt(curl, CURLOPT_FILE, &u);
 	}
 
 	if (wproxy && *proxyhost != '\0') {
 	    curl_easy_setopt(curl, CURLOPT_PROXY, proxyhost);
 	}
-	
+
 	curl_formadd(&post, &last, 
 		     CURLFORM_COPYNAME, "login", 
 		     CURLFORM_PTRCONTENTS, login, 
@@ -588,13 +608,28 @@ int upload_function_package (const char *login, const char *pass,
 		     CURLFORM_COPYNAME, "pass", 
 		     CURLFORM_PTRCONTENTS, pass, 
 		     CURLFORM_END);
-	curl_formadd(&post, &last, 
-		     CURLFORM_COPYNAME, "pkg", 
-		     CURLFORM_BUFFER, fname,
-		     CURLFORM_CONTENTTYPE, "text/plain; charset=utf-8",
-		     CURLFORM_BUFFERPTR, buf,
-		     CURLFORM_BUFFERLENGTH, strlen(buf),
-		     CURLFORM_END);
+	if (zipfile) {
+	    sprintf(sizestr, "%d", (int) buflen);
+	    curl_formadd(&post, &last, 
+			 CURLFORM_COPYNAME, "datasize",
+			 CURLFORM_PTRCONTENTS, sizestr, 
+			 CURLFORM_END);		    
+	    curl_formadd(&post, &last, 
+			 CURLFORM_COPYNAME, "pkg", 
+			 CURLFORM_BUFFER, fname,
+			 CURLFORM_CONTENTTYPE, "application/x-zip-compressed",
+			 CURLFORM_BUFFERPTR, buf,
+			 CURLFORM_BUFFERLENGTH, buflen,
+			 CURLFORM_END);	    
+	} else {
+	    curl_formadd(&post, &last, 
+			 CURLFORM_COPYNAME, "pkg", 
+			 CURLFORM_BUFFER, fname,
+			 CURLFORM_CONTENTTYPE, "text/plain; charset=utf-8",
+			 CURLFORM_BUFFERPTR, buf,
+			 CURLFORM_BUFFERLENGTH, strlen(buf),
+			 CURLFORM_END);
+	}
 
 	curl_easy_setopt(curl, CURLOPT_HTTPPOST, post);
 	res = curl_easy_perform(curl);
@@ -693,6 +728,63 @@ int retrieve_remote_function_package (const char *pkgname,
 			localname, NULL);
 }
 
+/**
+ * retrieve_remote_gfn_content:
+ * @zipname: name of function package, e.g. "foo.zip".
+ * @localname: full path to which the gfn file should be
+ * written on the local machine.
+ *
+ * Retrieves the gfn file from within a function package on
+ * the gretl server that takes the form of a zip file.
+ *
+ * Returns: 0 on success, non-zero on failure.
+ */
+
+int retrieve_remote_gfn_content (const char *zipname, 
+				 const char *localname)
+{
+    return retrieve_url(gretlhost, GRAB_FUNC_INFO, zipname, NULL, 
+			localname, NULL);
+}
+
+/**
+ * retrieve_remote_pkg_filename:
+ * @pkgname: name of function package, without extension.
+ * @err: location to receive error code.
+ *
+ * Returns: the completed package filename, with .gfn or
+ * .zip extension, or NULL on failure.
+ */
+
+char *retrieve_remote_pkg_filename (const char *pkgname, 
+				    int *err)
+{
+    char *fname = NULL;
+    char *buf = NULL;
+
+    *err = retrieve_url(gretlhost, FUNC_FULLNAME, pkgname, NULL, 
+			NULL, &buf);
+    
+    if (!*err) {
+	if (buf == NULL) {
+	    *err = E_DATA;
+	} else {
+	    if (strstr(buf, "not found")) {
+		gretl_errmsg_set(buf);
+		*err = E_DATA;
+	    } else {
+		char tmp[64];
+
+		sscanf(buf, "%63s", tmp);
+		fname = gretl_strdup(tmp);
+	    }
+	    free(buf);
+	}
+    }
+
+    return fname;
+}
+
 /**
  * retrieve_remote_datafiles_package:
  * @pkgname: name of data files package to retrieve, e.g. 
@@ -808,7 +900,7 @@ int retrieve_public_file (const char *uri, char *localname)
 	urlinfo u;
 
 	urlinfo_init(&u, NULL, SAVE_TO_FILE, localname);
-	strcpy(u.url, uri);
+	urlinfo_set_url(&u, uri);
 	if (gretl_in_gui_mode()) {
 	    urlinfo_set_show_progress(&u);
 	}
@@ -850,7 +942,7 @@ char *retrieve_public_file_as_buffer (const char *uri, size_t *len,
 	urlinfo u;
 
 	urlinfo_init(&u, NULL, SAVE_TO_BUFFER, NULL);
-	strcpy(u.url, uri);
+	urlinfo_set_url(&u, uri);
 	*err = curl_get(&u);
 	urlinfo_finalize(&u, &buf, err);
 	*len = (*err)? 0 : u.datalen;
@@ -991,4 +1083,3 @@ int gretl_curl (const char *url, const char *header,
 
     return err;
 }
-
