@@ -36,7 +36,9 @@ typedef enum {
     LIST_PKGS,
     GRAB_PKG,
     GRAB_FOREIGN,
-    QUERY_SF
+    QUERY_SF,
+    GRAB_FUNC_INFO,
+    FUNC_FULLNAME
 } CGIOpt;
 
 int gretl_www_init (const char *host, const char *proxy, int use_proxy);
@@ -60,6 +62,12 @@ int check_remote_db (const char *dbname);
 int retrieve_remote_function_package (const char *pkgname, 
 				      const char *localname);
 
+int retrieve_remote_gfn_content (const char *zipname, 
+				 const char *localname);
+
+char *retrieve_remote_pkg_filename (const char *pkgname, 
+				    int *err);
+
 int retrieve_remote_datafiles_package (const char *pkgname, 
 				       const char *localname);
 
@@ -74,7 +82,7 @@ int get_update_info (char **saver, int verbose);
 
 int upload_function_package (const char *login, const char *pass, 
 			     const char *fname, const char *buf,
-			     char **retbuf);
+			     size_t buflen, char **retbuf);
 
 int retrieve_public_file (const char *uri, char *localname);
 
