@@ -29,6 +29,10 @@
 #include "swap_bytes.h"
 #include "gretl_zip.h"
 
+#ifdef WIN32
+# include "gretl_win32.h"
+#endif
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -310,6 +314,17 @@ void gretl_xml_put_strings_array (const char *tag, const char **strs,
 {
     int i;
 
+    if (n < 0) {
+	/* assume NULL termination */
+	const char **S = strs;
+
+	n = 0;
+	while (*S != NULL) {
+	    n++;
+	    S++;
+	}
+    }
+
     fprintf(fp, "<%s count=\"%d\">\n", tag, n);
     for (i=0; i<n; i++) {
 	gretl_xml_put_string(strs[i], fp);
@@ -364,6 +379,34 @@ void gretl_xml_put_tagged_string (const char *tag, const char *str,
     fprintf(fp, "</%s>\n", tag);
 }
 
+/**
+ * gretl_xml_put_tagged_string_plus:
+ * @tag: name to give string.
+ * @str: string to put.
+ * @attrib: name of attribute.
+ * @attval: value of attribute.
+ * @fp: file to which to write.
+ * 
+ * Similar to gretl_xml_put_tagged_string(), but allows
+ * (in fact, requires) an attribute name and value along
+ * with the element name and value.
+ *
+ * Returns: 0 on success, non-zero error code on failure.
+ */
+
+void gretl_xml_put_tagged_string_plus (const char *tag,
+				       const char *str,
+				       const char *attrib,
+				       const char *attval,
+				       FILE *fp)
+{
+    fprintf(fp, "<%s %s=\"", tag, attrib);
+    gretl_xml_put_string(attval, fp);
+    fputs("\">", fp);
+    gretl_xml_put_string(str, fp);
+    fprintf(fp, "</%s>\n", tag);
+}
+
 /**
  * gretl_list_serialize:
  * @list: list of integers to be written.
@@ -1607,8 +1650,8 @@ int gretl_matrix_write_as_gdt (const char *fname,
     gzFile fz = Z_NULL;
     char datname[MAXLEN];
     char *xmlbuf = NULL;
-    int (*show_progress) (gint64, gint64, int) = NULL;
-    gint64 sz = 0;
+    int (*show_progress) (double, double, int) = NULL;
+    double msize = 0;
     int T = X->rows;
     int k = X->cols;
     int in_c_locale = 0;
@@ -1621,22 +1664,23 @@ int gretl_matrix_write_as_gdt (const char *fname,
 	return 1;
     }
 
-    sz = (T * k * sizeof(double));
-    if (sz > 100000) {
-	fprintf(stderr, I_("Writing %ld Kbytes of data\n"), sz / 1024);
+    msize = T * k * sizeof(double);
+    
+    if (msize > 100000) {
+	fprintf(stderr, I_("Writing %.0f Kbytes of data\n"), msize / 1024);
     } else {
-	sz = 0;
+	msize = 0;
     }
 
-    if (sz) {
+    if (msize > 0) {
 	show_progress = get_plugin_function("show_progress");
 	if (show_progress == NULL) {
-	    sz = 0;
+	    msize = 0;
+	} else {
+	    (*show_progress)(0, msize, SP_SAVE_INIT);
 	}
     }
 
-    if (sz) (*show_progress)(0, sz, SP_SAVE_INIT); 
-
     simple_fname(datname, fname);
     xmlbuf = gretl_xml_encode(datname);
     if (xmlbuf == NULL) {
@@ -1675,7 +1719,7 @@ int gretl_matrix_write_as_gdt (const char *fname,
 	    gzprintf(fz, "%.12g ", gretl_matrix_get(X, t, i));
 	}
 	gzputs(fz, "</obs>\n");
-	if (sz && t && (t % 50 == 0)) { 
+	if (msize > 0 && t && (t % 50 == 0)) { 
 	    (*show_progress) (50, T, SP_NONE);
 	}
     }
@@ -1688,7 +1732,7 @@ int gretl_matrix_write_as_gdt (const char *fname,
 	gretl_pop_c_numeric_locale();
     }
 
-    if (sz) {
+    if (msize > 0) {
 	(*show_progress)(0, T, SP_FINISH);
     } 
 
@@ -1937,14 +1981,15 @@ static int read_binary_header (FILE *fp, int order)
     return err;
 }
 
-static int read_binary_data (const char *fname, DATASET *dset,
-			     int order)
+static int read_binary_data (const char *fname, 
+			     DATASET *dset,
+			     int order,
+			     int fullv,
+			     const int *vlist)
 {
     char *bname;
     FILE *fp;
-    size_t got;
-    int T = dset->n;
-    int i, err = 0;
+    int err = 0;
 
     bname = switch_ext_new(fname, "bin");
     fp = gretl_fopen(bname, "rb");
@@ -1952,11 +1997,21 @@ static int read_binary_data (const char *fname, DATASET *dset,
     if (fp == NULL) {
 	err = E_FOPEN;
     } else {
+	int T = dset->n;
+	long offset = T * sizeof(double);
+	size_t got;
+	int i, k = 1;
+
 	err = read_binary_header(fp, order);
-	for (i=1; i<dset->v && !err; i++) {
-	    got = fread(dset->Z[i], sizeof(double), T, fp);
-	    if (got != T) {
-		err = E_DATA;
+
+	for (i=1; i<fullv && !err; i++) {
+	    if (vlist == NULL || in_gretl_list(vlist, i)) {
+		got = fread(dset->Z[k++], sizeof(double), T, fp);
+		if (got != T) {
+		    err = E_DATA;
+		}
+	    } else {
+		fseek(fp, offset, SEEK_CUR);
 	    }
 	}
 	fclose(fp);
@@ -2028,8 +2083,8 @@ static int real_write_gdt (const char *fname, const int *list,
     char startdate[OBSLEN], enddate[OBSLEN];
     char datname[MAXLEN], freqstr[32];
     char numstr[128], xmlbuf[256];
-    int (*show_progress) (gint64, gint64, int) = NULL;
-    gint64 dsize = 0;
+    int (*show_progress) (double, double, int) = NULL;
+    double dsize = 0;
     int i, t, v, nvars, ntabs;
     int gz, have_markers, in_c_locale = 0;
     int binary = 0, skip_padding = 0;
@@ -2068,7 +2123,7 @@ static int real_write_gdt (const char *fname, const int *list,
     dsize = tsamp * nvars * sizeof(double);
 
     if (dsize > 100000) {
-	fprintf(stderr, I_("Writing %ld Kbytes of data\n"), (long) (dsize / 1024));
+	fprintf(stderr, I_("Writing %.0f Kbytes of data\n"), dsize / 1024);
     } else if (progress) {
 	/* suppress progress bar for smaller data */
 	progress = 0;
@@ -2437,7 +2492,6 @@ int gretl_write_gdt (const char *fname, const int *list,
 
 	if (!err) {
 	    char xmlfile[FILENAME_MAX];
-	    GError *gerr = NULL;
 
 	    build_path(xmlfile, zdir, "data.xml", NULL);
 	    err = real_write_gdt(xmlfile, list, dset, opt | OPT_B, 0);
@@ -2445,17 +2499,9 @@ int gretl_write_gdt (const char *fname, const int *list,
 	    if (!err) {
 		int level = get_compression_option(STORE);
 
-		err = gretl_zip_datafile(fname, zdir, level, &gerr);
-		if (gerr != NULL && err == 0) {
-		    err = 1;
-		} 
+		err = gretl_zip_datafile(fname, zdir, level);
 		if (err) {
-		    if (gerr != NULL) {
-			gretl_errmsg_set(gerr->message);
-			g_error_free(gerr);
-		    } else {
-			gretl_errmsg_set("Problem writing data file");
-		    }
+		    gretl_errmsg_ensure("Problem writing data file");
 		}
 	    }
 	    gretl_deltree(zdir);
@@ -2592,35 +2638,233 @@ static int process_varlist (xmlNodePtr node, DATASET *dset)
     return err;
 }
 
-static int process_values (DATASET *dset, int t, char *s)
+static int series_wanted (const char *name, const char **vnames, int nv)
+{
+    int i;
+
+    for (i=0; i<nv; i++) {
+	if (!strcmp(name, vnames[i])) {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
+#define GDT_DEBUG 0
+
+static int process_varlist_subset (xmlNodePtr node, DATASET *dset,
+				   const char **vnames, int nv,
+				   int *fullv, int **pvlist)
+{
+    xmlNodePtr vars_node, cur;
+    xmlChar *tmp = xmlGetProp(node, (XUC) "count");
+    int *vlist = NULL;
+    int nv_found = 0;
+    int i, k, err = 0;
+
+    if (tmp != NULL) {
+	if (sscanf((char *) tmp, "%d", fullv) != 1) {
+	    err = E_DATA;
+	}
+	free(tmp);
+    } else {
+	err = E_DATA;
+    }
+
+    if (err) {
+	return err;
+    }
+
+    *fullv += 1; /* allow for const */
+
+#if GDT_DEBUG
+    fprintf(stderr, "process_varlist_subset: fullv = %d\n", *fullv);
+#endif
+
+    cur = node->xmlChildrenNode;
+    while (cur && xmlIsBlankNode(cur)) {
+	cur = cur->next;
+    }
+
+    if (cur == NULL) {
+	gretl_errmsg_set(_("Got no variables"));
+	return E_DATA;
+    }
+
+    vars_node = cur;
+    nv_found = 0;
+
+    /* first pass: check for matches */
+    while (cur != NULL) {
+        if (!xmlStrcmp(cur->name, (XUC) "variable")) {
+	    tmp = xmlGetProp(cur, (XUC) "name");
+	    if (tmp != NULL) {
+		if (series_wanted((const char *) tmp, vnames, nv)) {
+		    nv_found++;
+		}
+		free(tmp);
+	    }
+	}	    
+	cur = cur->next;
+    }
+
+#if GDT_DEBUG
+    fprintf(stderr, " nv_wanted = %d, nv_found = %d\n", nv, nv_found);
+#endif
+
+    if (nv_found < nv) {
+	return E_DATA;
+    }
+
+    /* allocate the dataset content */
+
+    dset->v = nv + 1;
+    if (!err && dataset_allocate_varnames(dset)) {
+	err = E_ALLOC;
+    }
+    if (!err) {
+	dset->Z = malloc(dset->v * sizeof *dset->Z);
+	if (dset->Z == NULL) {
+	    err = E_ALLOC;
+	}
+    }
+
+    if (!err) {
+	*pvlist = vlist = gretl_list_new(nv);
+	if (vlist == NULL) {
+	    err = E_ALLOC;
+	}
+    }
+
+    if (err) {
+	return err;
+    }
+
+    /* second pass: actually read the info */
+
+    cur = vars_node;
+    k = i = 1;
+    
+    while (cur != NULL) {
+        if (!xmlStrcmp(cur->name, (XUC) "variable")) {
+	    int wanted = 0;
+	    
+	    tmp = xmlGetProp(cur, (XUC) "name");
+	    if (tmp != NULL) {
+		if (series_wanted((const char *) tmp, vnames, nv)) {
+		    wanted = 1;
+		    transcribe_string(dset->varname[k], (char *) tmp, VNAMELEN);
+		    vlist[k] = i;
+		}
+		free(tmp);
+	    } else {
+		return E_DATA;
+	    }
+
+	    if (!wanted) {
+		i++;
+		cur = cur->next;
+		continue;
+	    }
+
+	    tmp = xmlGetProp(cur, (XUC) "label");
+	    if (tmp != NULL) {
+		series_set_label(dset, k, (char *) tmp);
+		free(tmp);
+	    }
+
+	    tmp = xmlGetProp(cur, (XUC) "displayname");
+	    if (tmp != NULL) {
+		series_set_display_name(dset, k, (char *) tmp);
+		free(tmp);
+	    }
+
+	    tmp = xmlGetProp(cur, (XUC) "parent");
+	    if (tmp != NULL) {
+		series_set_parent(dset, k, (char *) tmp); 
+		free(tmp);
+	    }
+
+	    tmp = xmlGetProp(cur, (XUC) "transform");
+	    if (tmp != NULL) {
+		int ci = gretl_command_number((char *) tmp);
+
+		series_set_transform(dset, k, ci);
+		free(tmp);
+	    }
+	    tmp = xmlGetProp(cur, (XUC) "lag");
+	    if (tmp != NULL) {
+		series_set_lag(dset, k, atoi((char *) tmp));
+		free(tmp);
+	    }
+
+	    tmp = xmlGetProp(cur, (XUC) "compact-method");
+	    if (tmp != NULL) {
+		series_set_compact_method(dset, k, compact_string_to_int((char *) tmp));
+		free(tmp);
+	    }
+
+	    tmp = xmlGetProp(cur, (XUC) "discrete");
+	    if (tmp != NULL) {
+		if (!strcmp((char *) tmp, "true")) {
+		    series_set_flag(dset, k, VAR_DISCRETE);
+		}
+		free(tmp);
+	    }
+
+	    tmp = xmlGetProp(cur, (XUC) "role");
+	    if (tmp != NULL) {
+		free(tmp);
+	    }
+
+	    i++;
+	    k++;
+	}
+	
+	cur = cur->next;
+    }
+
+    return err;
+}
+
+static int process_values (DATASET *dset, 
+			   int t, char *s,
+			   int fullv, 
+			   const int *vlist)
 {
     char *test;
     double x;
-    int i, err = 0;
+    int i, k = 1;
+    int err = 0;
 
     gretl_error_clear();
 
-    for (i=1; i<dset->v && !err; i++) {
+    for (i=1; i<fullv && !err; i++) {
 	while (isspace(*s)) s++;
-	x = strtod(s, &test);
-	if (errno) {
-	    if (SMALLVAL(x)) {
-		errno = 0; /* underflow, OK */
-	    } else {
-		fprintf(stderr, "%s: %d: bad data\n", __FILE__, __LINE__);
-		perror(NULL);
-		err = 1;
-	    }
-	} else if (!strncmp(test, "NA", 2)) {
-	    x = NADBL;
-	    s = test + 2;
-	} else if (*test != '\0' && !isspace(*test)) {
-	    err = 1;
+	if (vlist != NULL && !in_gretl_list(vlist, i)) {
+	    s += strcspn(s, " \t\r\n");
 	} else {
-	    s = test;
-	}
-	if (t < dset->n) {
-	    dset->Z[i][t] = x;
+	    x = strtod(s, &test);
+	    if (errno) {
+		if (SMALLVAL(x)) {
+		    errno = 0; /* underflow, OK */
+		} else {
+		    fprintf(stderr, "%s: %d: bad data\n", __FILE__, __LINE__);
+		    perror(NULL);
+		    err = E_DATA;
+		}
+	    } else if (!strncmp(test, "NA", 2)) {
+		x = NADBL;
+		s = test + 2;
+	    } else if (*test != '\0' && !isspace(*test)) {
+		err = 1;
+	    } else {
+		s = test;
+	    }
+	    if (t < dset->n) {
+		dset->Z[k++][t] = x;
+	    }
 	}
     }	
 
@@ -2631,16 +2875,15 @@ static int process_values (DATASET *dset, int t, char *s)
     return err;
 }
 
-#define GDT_DEBUG 0
-
 static int read_observations (xmlDocPtr doc, xmlNodePtr node, 
-			      DATASET *dset, long progress,
+			      DATASET *dset, double dsize,
 			      int binary, const char *fname)
 {
     xmlNodePtr cur;
     xmlChar *tmp;
     int n, i, t;
-    int (*show_progress) (gint64, gint64, int) = NULL;
+    int (*show_progress) (double, double, int) = NULL;
+    int progbar = 0;
     int err = 0;
 
     tmp = xmlGetProp(node, (XUC) "count");
@@ -2657,14 +2900,10 @@ static int read_observations (xmlDocPtr doc, xmlNodePtr node,
 	return E_DATA;
     }
 
-    if (binary) {
-	progress = 0;
-    }
-
-    if (progress > 0) {
+    if (dsize > 100000 && !binary) {
 	show_progress = get_plugin_function("show_progress");
-	if (show_progress == NULL) {
-	    progress = 0;
+	if (show_progress != NULL) {
+	    progbar = 1;
 	}
     }
 
@@ -2698,7 +2937,7 @@ static int read_observations (xmlDocPtr doc, xmlNodePtr node,
     }
 
     if (binary) {
-	err = read_binary_data(fname, dset, binary);
+	err = read_binary_data(fname, dset, binary, dset->v, NULL);
 	if (!dset->markers) {
 	    goto bailout;
 	}
@@ -2715,8 +2954,8 @@ static int read_observations (xmlDocPtr doc, xmlNodePtr node,
 	return E_DATA;
     }
 
-    if (progress) {
-	(*show_progress)(0L, progress, SP_LOAD_INIT);
+    if (progbar) {
+	(*show_progress)(0, dsize, SP_LOAD_INIT);
 #if GDT_DEBUG
 	fprintf(stderr, "read_observations: inited progess bar (n=%d)\n",
 		dset->n);
@@ -2726,7 +2965,6 @@ static int read_observations (xmlDocPtr doc, xmlNodePtr node,
     t = 0;
     while (cur != NULL) {
         if (!xmlStrcmp(cur->name, (XUC) "obs")) {
-
 	    if (dset->markers) {
 		tmp = xmlGetProp(cur, (XUC) "label");
 		if (tmp) {
@@ -2737,28 +2975,159 @@ static int read_observations (xmlDocPtr doc, xmlNodePtr node,
 		    return E_DATA;
 		}
 	    }
-
-	    if (binary) {
-		t++;
-		continue;
-	    }
-
-	    tmp = xmlNodeListGetRawString(doc, cur->xmlChildrenNode, 1);
-
-	    if (tmp) {
-		if (process_values(dset, t, (char *) tmp)) {
-		    return 1;
+	    if (!binary) {
+		tmp = xmlNodeListGetRawString(doc, cur->xmlChildrenNode, 1);
+		if (tmp) {
+		    err = process_values(dset, t, (char *) tmp, dset->v, NULL);
+		    free(tmp);
+		} else if (dset->v > 1) {
+		    gretl_errmsg_sprintf(_("Values missing at observation %d"), t+1);
+		    err = E_DATA;
 		}
-		free(tmp);
-		t++;
-	    } else if (dset->v > 1) {
-		gretl_errmsg_sprintf(_("Values missing at observation %d"), t+1);
-		err = E_DATA;
-		goto bailout;
-	    } else {
-		t++;
 	    }
-	}	   
+	    t++;
+	}
+
+	if (err) {
+	    break;
+	}
+ 
+	cur = cur->next;
+
+	if (cur != NULL && t == dset->n) {
+	    /* got too many observations */
+	    t = dset->n + 1;
+	    break;
+	}
+
+	if (progbar && t > 0 && t % 50 == 0) {
+	    (*show_progress) (50, dset->n, SP_NONE);
+	}
+    }
+
+ bailout:
+
+    if (progbar) {
+#if GDT_DEBUG
+	fprintf(stderr, "finalizing progress bar (n = %d)\n", dset->n);
+#endif
+	(*show_progress)(0, dset->n, SP_FINISH);
+    }
+
+    if (!err && t != dset->n) {
+	gretl_errmsg_set(_("Number of observations does not match declaration"));
+	err = E_DATA;
+    }
+
+    return err;
+}
+
+static int read_observations_subset (xmlDocPtr doc,
+				     xmlNodePtr node, 
+				     DATASET *dset,
+				     int binary,
+				     const char *fname,
+				     int fullv,
+				     const int *vlist,
+				     gretlopt opt)
+{
+    xmlNodePtr cur;
+    xmlChar *tmp;
+    int n, i, t;
+    int err = 0;
+
+    tmp = xmlGetProp(node, (XUC) "count");
+    if (tmp == NULL) {
+	return E_DATA;
+    } 
+
+    if (sscanf((char *) tmp, "%d", &n) == 1) {
+	dset->n = n;
+	free(tmp);
+    } else {
+	gretl_errmsg_set(_("Failed to parse number of observations"));
+	free(tmp);
+	return E_DATA;
+    }
+
+    if (opt & OPT_M) {
+	tmp = xmlGetProp(node, (XUC) "labels");
+	if (tmp) {
+	    if (!strcmp((char *) tmp, "true")) {
+		if (dataset_allocate_obs_markers(dset)) {
+		    return E_ALLOC;
+		}
+	    }
+	    free(tmp);
+	} else {
+	    return E_DATA;
+	}
+    }
+
+    if (dset->endobs[0] == '\0') {
+	sprintf(dset->endobs, "%d", dset->n);
+    }
+
+    dset->t2 = dset->n - 1;
+
+    for (i=0; i<dset->v; i++) {
+	dset->Z[i] = malloc(dset->n * sizeof **dset->Z);
+	if (dset->Z[i] == NULL) {
+	    return E_ALLOC;
+	}
+    }
+
+    for (t=0; t<dset->n; t++) {
+	dset->Z[0][t] = 1.0;
+    }
+
+    if (binary) {
+	err = read_binary_data(fname, dset, binary, fullv, vlist);
+	if (!dset->markers) {
+	    goto bailout;
+	}
+    }
+
+    /* now get individual obs info: labels and values */
+    cur = node->xmlChildrenNode;
+    while (cur && xmlIsBlankNode(cur)) {
+	cur = cur->next;
+    }
+
+    if (cur == NULL) {
+	gretl_errmsg_set(_("Got no observations\n"));
+	return E_DATA;
+    }
+
+    t = 0;
+    while (cur != NULL) {
+        if (!xmlStrcmp(cur->name, (XUC) "obs")) {
+	    if (dset->markers) {
+		tmp = xmlGetProp(cur, (XUC) "label");
+		if (tmp) {
+		    transcribe_string(dset->S[t], (char *) tmp, OBSLEN);
+		    free(tmp);
+		} else {
+		    gretl_errmsg_sprintf(_("Case marker missing at obs %d"), t+1);
+		    return E_DATA;
+		}
+	    }
+	    if (!binary) {
+		tmp = xmlNodeListGetRawString(doc, cur->xmlChildrenNode, 1);
+		if (tmp) {
+		    err = process_values(dset, t, (char *) tmp, fullv, vlist);
+		    free(tmp);
+		} else if (dset->v > 1) {
+		    gretl_errmsg_sprintf(_("Values missing at observation %d"), t+1);
+		    err = E_DATA;
+		}
+	    }
+	    t++;
+	}
+
+	if (err) {
+	    break;
+	}
  
 	cur = cur->next;
 
@@ -2767,21 +3136,10 @@ static int read_observations (xmlDocPtr doc, xmlNodePtr node,
 	    t = dset->n + 1;
 	    goto bailout;
 	}
-
-	if (progress && t > 0 && t % 50 == 0) {
-	    (*show_progress) (50, dset->n, SP_NONE);
-	}
     }
 
  bailout:
 
-    if (progress) {
-#if GDT_DEBUG
-	fprintf(stderr, "finalizing progress bar (n = %d)\n", dset->n);
-#endif
-	(*show_progress)(0, dset->n, SP_FINISH);
-    }
-
     if (!err && t != dset->n) {
 	gretl_errmsg_set(_("Number of observations does not match declaration"));
 	err = E_DATA;
@@ -2803,25 +3161,27 @@ static int owner_id (const DATASET *dset, const char *s)
     return -1;
 }
 
-static int process_string_tables (xmlDocPtr doc, xmlNodePtr node, 
-				  DATASET *dset)
+static int process_string_tables (xmlDocPtr doc,
+				  xmlNodePtr node, 
+				  DATASET *dset,
+				  int subset)
 {
-    xmlNodePtr cur;
+    xmlNodePtr cur = NULL;
     xmlChar *tmp;
     int ntabs = 0;
     int err = 0;
 
     tmp = xmlGetProp(node, (XUC) "count");
+
     if (tmp == NULL) {
-	return E_DATA;
-    } 
-
-    if (sscanf((char *) tmp, "%d", &ntabs) != 1) {
 	err = E_DATA;
+    } else {
+	if (sscanf((char *) tmp, "%d", &ntabs) != 1) {
+	    err = E_DATA;
+	}
+	free(tmp);
     }
 
-    free(tmp);
-
     if (!err) {
 	cur = node->xmlChildrenNode;
 	while (cur && xmlIsBlankNode(cur)) {
@@ -2847,21 +3207,21 @@ static int process_string_tables (xmlDocPtr doc, xmlNodePtr node,
 		err = E_DATA;
 	    } else {
 		v = owner_id(dset, (const char *) owner);
-		if (v <= 0) {
+		if (v <= 0 && !subset) {
 		    err = E_DATA;
 		}
 	    }
-	    if (!err) {
+	    if (v > 0) {
 		strs = gretl_xml_get_strings_array(cur, doc, &n_strs, 
 						   0, &err);
-	    }
-	    if (!err) {
-		st = series_table_new(strs, n_strs);
-		if (st == NULL) {
-		    strings_array_free(strs, n_strs);
-		    err = E_ALLOC;
-		} else {
-		    series_attach_string_table(dset, v, st);
+		if (!err) {
+		    st = series_table_new(strs, n_strs);
+		    if (st == NULL) {
+			strings_array_free(strs, n_strs);
+			err = E_ALLOC;
+		    } else {
+			series_attach_string_table(dset, v, st);
+		    }
 		}
 	    }
 	    free(owner);
@@ -3221,24 +3581,20 @@ static int real_read_gdt (const char *fname, const char *srcname,
     int gotvars = 0, gotobs = 0, err = 0;
     int caldata = 0, repad = 0;
     double gdtversion = 1.0;
-    long fsz, progress = 0L;
     int in_c_locale = 0;
     int gz, binary = 0;
+    long fsz;
 
     gretl_error_clear();
-
     fsz = get_filesize(fname);
     gz = is_gzipped(fname);
 
     if (fsz < 0) {
 	return E_FOPEN;
     } else if (fsz > 100000) {
-	fprintf(stderr, "%s %ld bytes %s...\n", 
+	fprintf(stderr, "%s %.0f Kbytes %s...\n", 
 		gz ? I_("Uncompressing") : I_("Reading"),
-		fsz, I_("of data"));
-	if (opt & OPT_B) {
-	    progress = fsz;
-	}
+		(double) fsz / 1024.0, I_("of data"));
     }
 
     set_alt_gettext_mode(prn);
@@ -3310,17 +3666,20 @@ static int real_read_gdt (const char *fname, const char *srcname,
 	    if (!gotvars) {
 		gretl_errmsg_set(_("Variables information is missing"));
 		err = 1;
-	    } else if (read_observations(doc, cur, tmpset, progress,
-					 binary, fname)) {
-		err = 1;
 	    } else {
-		gotobs = 1;
+		double dsize = (opt & OPT_B)? (double) fsz : 0;
+		
+		err = read_observations(doc, cur, tmpset, dsize,
+					binary, fname);
+		if (!err) {
+		    gotobs = 1;
+		}
 	    }
 	} else if (!xmlStrcmp(cur->name, (XUC) "string-tables")) {
 	    if (!gotvars) {
 		gretl_errmsg_set(_("Variables information is missing"));
 		err = 1;
-	    } else if (process_string_tables(doc, cur, tmpset)) {
+	    } else if (process_string_tables(doc, cur, tmpset, 0)) {
 		err = 1;
 	    }	    
 	} else if (!xmlStrcmp(cur->name, (XUC) "panel-info")) {
@@ -3410,6 +3769,151 @@ static int real_read_gdt (const char *fname, const char *srcname,
     return err;
 }
 
+static int real_read_gdt_subset (const char *fname,
+				 DATASET *dset,
+				 const char **vnames,
+				 int nv,
+				 gretlopt opt) 
+{
+    DATASET *tmpset;
+    xmlDocPtr doc = NULL;
+    xmlNodePtr cur;
+    int *vlist = NULL;
+    int gotvars = 0, gotobs = 0;
+    int caldata = 0;
+    int in_c_locale = 0;
+    int binary = 0;
+    int fullv = 0;
+    int err = 0;
+
+    gretl_error_clear();
+
+    tmpset = datainfo_new();
+    if (tmpset == NULL) {
+	err = E_ALLOC;
+	goto bailout;
+    }
+
+    err = gretl_xml_open_doc_root(fname, "gretldata", &doc, &cur);
+    if (err) {
+	goto bailout;
+    }
+
+    /* set some datainfo parameters */
+
+    err = xml_get_data_structure(cur, &tmpset->structure);
+    if (err) {
+	goto bailout;
+    } 
+
+    err = xml_get_data_frequency(cur, &tmpset->pd, &tmpset->structure);
+    if (err) {
+	goto bailout;
+    }   
+
+    gretl_push_c_numeric_locale();
+    in_c_locale = 1;
+
+    strcpy(tmpset->stobs, "1");
+    caldata = dataset_is_daily(tmpset) || dataset_is_weekly(tmpset);
+
+    err = xml_get_startobs(cur, &tmpset->sd0, tmpset->stobs, caldata);
+    if (err) {
+	goto bailout;
+    }     
+
+    *tmpset->endobs = '\0';
+    caldata = calendar_data(tmpset);
+
+    err = xml_get_endobs(cur, tmpset->endobs, caldata);
+    if (err) {
+	goto bailout;
+    }
+
+    binary = gdt_binary_order(cur);
+
+#if GDT_DEBUG
+    fprintf(stderr, "starting to walk XML tree...\n");
+#endif
+
+    /* Now walk the tree */
+    cur = cur->xmlChildrenNode;
+    while (cur != NULL && !err) {
+        if (!xmlStrcmp(cur->name, (XUC) "variables")) {
+	    if (process_varlist_subset(cur, tmpset, vnames, nv,
+				       &fullv, &vlist)) {
+		err = 1;
+	    } else {
+		gotvars = 1;
+	    }
+	} else if (!xmlStrcmp(cur->name, (XUC) "observations")) {
+	    if (!gotvars) {
+		gretl_errmsg_set(_("Variables information is missing"));
+		err = 1;
+	    } else if (read_observations_subset(doc, cur, tmpset, 
+						binary, fname,
+						fullv, vlist, opt)) {
+		err = 1;
+	    } else {
+		gotobs = 1;
+	    }
+	} else if (!xmlStrcmp(cur->name, (XUC) "string-tables")) {
+	    if (!gotvars) {
+		gretl_errmsg_set(_("Variables information is missing"));
+		err = 1;
+	    } else if (process_string_tables(doc, cur, tmpset, 1)) {
+		err = 1;
+	    }	    
+	}
+	if (!err) {
+	    cur = cur->next;
+	}
+    }
+
+#if GDT_DEBUG
+    fprintf(stderr, "done walking XML tree...\n");
+#endif
+
+    if (!err && !gotvars) {
+	gretl_errmsg_set(_("Variables information is missing"));
+	err = 1;
+    }
+
+    if (!err && !gotobs) {
+	gretl_errmsg_set(_("No observations were found"));
+	err = 1;
+    }
+
+    if (!err && caldata && tmpset->S != NULL) {
+	check_for_daily_date_strings(tmpset);
+    }
+
+ bailout:
+
+    if (in_c_locale) {
+	gretl_pop_c_numeric_locale();
+    }
+
+    if (doc != NULL) {
+	xmlFreeDoc(doc);
+    }
+
+    free(vlist);
+
+    if (!err) {
+	*dset = *tmpset;
+	free(tmpset);
+    } else {
+	destroy_dataset(tmpset);
+    }
+
+#if GDT_DEBUG
+    fprintf(stderr, "gretl_read_gdt_subset: returning %d\n", err);
+#endif
+
+    return err;
+}
+
 /**
  * gretl_read_gdt:
  * @fname: name of file to open for reading.
@@ -3429,7 +3933,6 @@ int gretl_read_gdt (const char *fname, DATASET *dset,
 {
     if (has_suffix(fname, ".gdtb")) {
 	/* zipfile with gdt + binary */
-	GError *gerr = NULL;
 	gchar *zdir;
 	int err;
 
@@ -3437,17 +3940,9 @@ int gretl_read_gdt (const char *fname, DATASET *dset,
 	err = gretl_mkdir(zdir);
 
 	if (!err) {
-	    err = gretl_unzip_datafile(fname, zdir, &gerr);
-	    if (gerr != NULL && err == 0) {
-		err = 1;
-	    } 
+	    err = gretl_unzip_into(fname, zdir);
 	    if (err) {
-		if (gerr != NULL) {
-		    gretl_errmsg_set(gerr->message);
-		    g_error_free(gerr);
-		} else {
-		    gretl_errmsg_ensure("Problem opening data file");
-		}
+		gretl_errmsg_ensure("Problem opening data file");
 	    } else {
 		char xmlfile[FILENAME_MAX];
 
@@ -3465,6 +3960,55 @@ int gretl_read_gdt (const char *fname, DATASET *dset,
     }
 }
 
+/**
+ * gretl_read_gdt_subset:
+ * @fname: name of file to open for reading.
+ * @dset: dataset struct.
+ * @vnames: array of names of series to extract.
+ * @nv: the number of elements in @vnames.
+ * @opt: may include OPT_M to retrieve the observation
+ * markers associated with the data, if any.
+ * 
+ * Read specified series from native file into @dset,
+ * which should be "empty" on input.
+ * 
+ * Returns: 0 on successful completion, non-zero otherwise.
+ */
+
+int gretl_read_gdt_subset (const char *fname, DATASET *dset, 
+			   const char **vnames, int nv,
+			   gretlopt opt)
+{
+    if (has_suffix(fname, ".gdtb")) {
+	/* zipfile with gdt + binary */
+	gchar *zdir;
+	int err;
+
+	zdir = g_strdup_printf("%stmp-unzip", gretl_dotdir());
+	err = gretl_mkdir(zdir);
+
+	if (!err) {
+	    err = gretl_unzip_into(fname, zdir);
+	    if (err) {
+		gretl_errmsg_ensure("Problem opening data file");
+	    } else {
+		char xmlfile[FILENAME_MAX];
+
+		build_path(xmlfile, zdir, "data.xml", NULL);
+		err = real_read_gdt_subset(xmlfile, dset, vnames,
+					   nv, opt);
+	    }
+	    gretl_deltree(zdir);
+	}
+
+	g_free(zdir);
+	return err;
+    } else {
+	/* plain XML file */
+	return real_read_gdt_subset(fname, dset, vnames, nv, opt);
+    }
+}
+
 /**
  * gretl_get_gdt_description:
  * @fname: name of file to try.
