@@ -111,6 +111,12 @@ void gretl_xml_put_tagged_string (const char *tag,
 				  const char *str, 
 				  FILE *fp);
 
+void gretl_xml_put_tagged_string_plus (const char *tag,
+				       const char *str,
+				       const char *attrib,
+				       const char *attval,
+				       FILE *fp);
+
 void gretl_xml_put_tagged_list (const char *tag, 
 				const int *list, 
 				FILE *fp);
@@ -135,6 +141,10 @@ int gretl_write_gdt (const char *fname, const int *list,
 int gretl_read_gdt (const char *fname, DATASET *dset, 
 		    gretlopt opt, PRN *prn);
 
+int gretl_read_gdt_subset (const char *fname, DATASET *dset, 
+			   const char **vnames, int nv,
+			   gretlopt opt);
+
 char *gretl_get_gdt_description (const char *fname);
 
 int load_user_XML_file (const char *fname, PRN *prn);
