@@ -17,11 +17,6 @@
  * 
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <glib.h>
-
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
@@ -30,28 +25,38 @@
 # include "winconfig.h"
 #endif
 
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <glib.h>
+
 #include "gretl_zip.h"
 #include "strutils.h"
 
-#ifdef USE_GSF
+static int handle_zip_error (const char *fname,
+			     GError *gerr, int err,
+			     const char *action)
+{
+    if (gerr != NULL) {
+	fprintf(stderr, "handle_zip_error: '%s'\n", gerr->message);
+	gretl_errmsg_sprintf("%s: %s", fname, gerr->message);
+	g_error_free(gerr);
+    } else if (err) {
+	gretl_errmsg_sprintf("%s: error %s", fname, action);
+    }
 
-#include <gsf/gsf-utils.h>
+    return err;
+}
 
-#include <gsf/gsf-input-stdio.h>
-#include <gsf/gsf-infile.h>
-#include <gsf/gsf-infile-stdio.h>
-#include <gsf/gsf-infile-zip.h>
+#ifdef USE_GSF /* libgsf-1 >= 1.14.31 */
 
-#include <gsf/gsf-output-stdio.h>
-#include <gsf/gsf-outfile-stdio.h>
-#include <gsf/gsf-outfile.h>
-#include <gsf/gsf-outfile-zip.h>
-
-#define ZDEBUG 1
+#include <gsf/gsf.h>
 
+#define ZDEBUG 0
 #define CHUNK 32768
 
-#define gsf_is_dir(i) (GSF_IS_INFILE(i) && gsf_infile_num_children(GSF_INFILE(i)) >= 0)
+#define gsf_is_dir(i) (GSF_IS_INFILE(i) && \
+		       gsf_infile_num_children(GSF_INFILE(i)) >= 0)
 
 static void ensure_gsf_init (void)
 {
@@ -133,12 +138,13 @@ static int clone_gsf_tree (GsfInput *input, GsfOutput *output)
     return err;
 }
 
-int gretl_make_zipfile (const char *fname, const char *path,
-			GError **gerr)
+static int gretl_gsf_make_zipfile (const char *fname,
+				   const char *path)
 {
     GsfInfile *infile;
     GsfOutput *output, *ziproot;
     GsfOutfile *outfile = NULL;
+    GError *gerr = NULL;
     int err = 0;
 
     ensure_gsf_init();
@@ -148,19 +154,19 @@ int gretl_make_zipfile (const char *fname, const char *path,
 	    fname, path);
 #endif
 
-    infile = gsf_infile_stdio_new(path, gerr);
+    infile = gsf_infile_stdio_new(path, &gerr);
 
     if (infile == NULL) {
 	err = 1;
     } else {
-	output = gsf_output_stdio_new(fname, gerr);
+	output = gsf_output_stdio_new(fname, &gerr);
 	if (output == NULL) {
 	    err = 1;
 	}
     }  
     
     if (!err) {
-	outfile = gsf_outfile_zip_new(output, gerr);
+	outfile = gsf_outfile_zip_new(output, &gerr);
 	g_object_unref(G_OBJECT(output));
 	if (outfile == NULL) {
 	    err = 1;
@@ -186,18 +192,16 @@ int gretl_make_zipfile (const char *fname, const char *path,
     fprintf(stderr, "*** gretl_make_zipfile: returning %d\n", err);
 #endif
 
-    return err;
+    return handle_zip_error(fname, gerr, err, "zipping");
 }
 
-#if 0 /* libgsf not ready yet: doesn't support setting of
-	 zlib compression level as of version 1.14.30
-      */
-
-int gretl_zip_datafile (const char *fname, const char *path,
-			int level, GError **gerr)
+static int gretl_gsf_zip_datafile (const char *fname,
+				   const char *path,
+				   int level)
 {
     GsfOutput *output;
     GsfOutfile *outfile = NULL;
+    GError *gerr = NULL;
     int err = 0;
 
     ensure_gsf_init();
@@ -206,13 +210,13 @@ int gretl_zip_datafile (const char *fname, const char *path,
     fprintf(stderr, "gretl_zip_datafile (gsf):\n fname='%s'\n", fname);
 #endif
 
-    output = gsf_output_stdio_new(fname, gerr);
+    output = gsf_output_stdio_new(fname, &gerr);
     if (output == NULL) {
 	err = 1;
     }
      
     if (!err) {
-	outfile = gsf_outfile_zip_new(output, gerr);
+	outfile = gsf_outfile_zip_new(output, &gerr);
 	g_object_unref(G_OBJECT(output));
 	if (outfile == NULL) {
 	    err = 1;
@@ -228,11 +232,11 @@ int gretl_zip_datafile (const char *fname, const char *path,
 
 	for (i=0; i<2 && !err; i++) {
 	    build_path(fullname, path, names[i], NULL);
-	    zinp = gsf_input_stdio_new(fullname, gerr);
+	    zinp = gsf_input_stdio_new(fullname, &gerr);
 	    if (zinp == NULL) {
 		err = 1;
 	    } else {
-		/* property not present in libgsf <= 1.14.30 */
+		/* note: property not present in libgsf <= 1.14.30 */
 		zout = gsf_outfile_new_child_full(outfile, names[i], 0,
 						  "deflate-level", level,
 						  NULL);
@@ -250,50 +254,40 @@ int gretl_zip_datafile (const char *fname, const char *path,
     }
 
 #if ZDEBUG
-    fprintf(stderr, "*** gretl_zip_datafile: returning %d\n", err);
+    fprintf(stderr, "*** gretl_gsf_zip_datafile: returning %d\n", err);
 #endif
 
-    return err;
+    return handle_zip_error(fname, gerr, err, "zipping");
 }
 
-#else
+/* @fname: the name of the file to be unzipped.
+   @path: if non-NULL, the location into which the unzipping
+   should be done (if NULL, use pwd).
+   @zdirname: if non-NULL, set its content to the name of
+   the top-level directory within the zipfile (via g_strdup).
+*/
 
-/* use native gretlzip plugin code for now */
-
-int gretl_zip_datafile (const char *fname, const char *path,
-			int level, GError **gerr)
-{
-    int (*zfunc) (const char *, const char *, int, GError **);
-
-    zfunc = get_plugin_function("gretl_native_zip_datafile");
-    if (zfunc == NULL) {
-        return 1;
-    }
-
-    return (*zfunc)(fname, path, level, gerr);
-}
-
-#endif /* work-around for missing functionality in libgsf */
-
-static int real_unzip_file (const char *fname, const char *path,
-			    gchar **zdirname, GError **gerr)
+static int gretl_gsf_unzip (const char *fname,
+			    const char *path,
+			    gchar **zdirname)
 {
     GsfInput *input;
     GsfInfile *infile;
     GsfOutfile *outfile;
+    GError *gerr = NULL;
     int err = 0;
 
 #if ZDEBUG
-    fprintf(stderr, "*** real_unzip_file (gsf):\n fname='%s'\n", fname);
+    fprintf(stderr, "*** gretl_gsf_unzip_file:\n fname='%s'\n", fname);
 #endif
 
     ensure_gsf_init();
-    input = gsf_input_stdio_new(fname, gerr);
+    input = gsf_input_stdio_new(fname, &gerr);
 
     if (input == NULL) {
 	err = 1;
     } else {
-	infile = gsf_infile_zip_new(input, gerr);
+	infile = gsf_infile_zip_new(input, &gerr);
 	g_object_unref(G_OBJECT(input));
 	if (infile == NULL) {
 	    err = 1;
@@ -302,9 +296,9 @@ static int real_unzip_file (const char *fname, const char *path,
 		*zdirname = g_strdup(gsf_infile_name_by_index(infile, 0));
 	    }
 	    if (path != NULL) {
-		outfile = gsf_outfile_stdio_new(path, gerr);
+		outfile = gsf_outfile_stdio_new(path, &gerr);
 	    } else {
-		outfile = gsf_outfile_stdio_new(".", gerr);
+		outfile = gsf_outfile_stdio_new(".", &gerr);
 	    }
 	    if (outfile == NULL) {
 		err = 1;
@@ -315,42 +309,33 @@ static int real_unzip_file (const char *fname, const char *path,
     }
 
 #if ZDEBUG
-    fprintf(stderr, "*** real_unzip_file: returning %d\n", err);
+    fprintf(stderr, "*** gretl_gsf_unzip_file: returning %d\n", err);
 #endif
 
-    return err;
-}
-
-int gretl_unzip_file (const char *fname, GError **gerr)
-{
-    return real_unzip_file(fname, NULL, NULL, gerr);
-}
-
-int gretl_unzip_session_file (const char *fname, gchar **zdirname, GError **gerr)
-{
-    return real_unzip_file(fname, NULL, zdirname, gerr);
-}
-
-int gretl_unzip_datafile (const char *fname, const char *path, GError **gerr)
-{
-    return real_unzip_file(fname, path, NULL, gerr);
+    return handle_zip_error(fname, gerr, err, "unzipping");
 }
 
 #else /* native, using gretlzip plugin, not libgsf */
 
 #include "plugins.h"
 
-int gretl_unzip_file (const char *fname, GError **gerr)
+static int gretl_plugin_unzip (const char *fname,
+			       const char *path,
+			       gchar **zdirname)
 {
-    int (*zfunc) (const char *, GError **);
+    int (*zfunc) (const char *, const char *, gchar **, GError **);
+    GError *gerr = NULL;
+    int err = 0;
 
-    zfunc = get_plugin_function("gretl_native_unzip_file");
+    zfunc = get_plugin_function("gretl_native_unzip");
     if (zfunc == NULL) {
 	/* error message handled by get_plugin_function() */
         return 1;
     }
 
-    return (*zfunc)(fname, gerr);
+    err = (*zfunc)(fname, path, zdirname, &gerr);
+
+    return handle_zip_error(fname, gerr, err, "unzipping");
 }
 
 /*
@@ -359,57 +344,141 @@ int gretl_unzip_file (const char *fname, GError **gerr)
  * and zipped.
  */
 
-int gretl_make_zipfile (const char *fname, const char *path,
-			GError **gerr)
+static int gretl_plugin_make_zipfile (const char *fname,
+				      const char *path)
 {
     int (*zfunc) (const char *, const char *, GError **);
+    GError *gerr = NULL;
+    int err = 0;
 
     zfunc = get_plugin_function("gretl_native_make_zipfile");
     if (zfunc == NULL) {
         return 1;
     }
 
-    return (*zfunc)(fname, path, gerr);
+    err = (*zfunc)(fname, path, &gerr);
+
+    return handle_zip_error(fname, gerr, err, "zipping");
 }
 
-int gretl_unzip_session_file (const char *fname, gchar **zdirname, GError **gerr)
-{
-    int (*zfunc) (const char *, gchar **, GError **);
-
-    zfunc = get_plugin_function("gretl_native_unzip_session_file");
-    if (zfunc == NULL) {
-        return 1;
-    }
-
-    return (*zfunc)(fname, zdirname, gerr);
-}
-
-int gretl_zip_datafile (const char *fname, const char *path,
-			int level, GError **gerr)
+static int gretl_plugin_zip_datafile (const char *fname,
+				      const char *path,
+				      int level)
 {
     int (*zfunc) (const char *, const char *, int, GError **);
+    GError *gerr = NULL;
+    int err = 0;
 
     zfunc = get_plugin_function("gretl_native_zip_datafile");
     if (zfunc == NULL) {
         return 1;
     }
 
-    return (*zfunc)(fname, path, level, gerr);
-}
+    err = (*zfunc)(fname, path, level, &gerr);
 
-int gretl_unzip_datafile (const char *fname, const char *path,
-			  GError **gerr)
-{
-    int (*zfunc) (const char *, const char *, GError **);
-
-    zfunc = get_plugin_function("gretl_native_unzip_datafile");
-    if (zfunc == NULL) {
-        return 1;
-    }
-
-    return (*zfunc)(fname, path, gerr);
+    return handle_zip_error(fname, gerr, err, "zipping");
 }
 
 #endif /* zip/unzip variants */
 
+/**
+ * gretl_unzip:
+ * @fname: name of the file to unzip.
+ *
+ * Unzips @fname in the current directory, preserving any
+ * internal directory structure.
+ *
+ * Returns: 0 on success, non-zero code on error.
+ */
 
+int gretl_unzip (const char *fname)
+{
+#if USE_GSF
+    return gretl_gsf_unzip(fname, NULL, NULL);
+#else
+    return gretl_plugin_unzip(fname, NULL, NULL);
+#endif    
+}
+
+/**
+ * gretl_unzip_into:
+ * @fname: name of the file to unzip.
+ * @dirname: the name of the directory in which unzipping
+ * should take place.
+ *
+ * Unzips @fname in the specified directory, preserving any
+ * internal directory structure.
+ *
+ * Returns: 0 on success, non-zero code on error.
+ */
+
+int gretl_unzip_into (const char *fname, const char *dirname)
+{
+#if USE_GSF
+    return gretl_gsf_unzip(fname, dirname, NULL);
+#else
+    return gretl_plugin_unzip(fname, dirname, NULL);
+#endif 
+}
+
+/**
+ * gretl_make zipfile:
+ * @fname: name of the zip file to create.
+ * @path: the path to the content which should be zipped.
+ *
+ * Creates a zip file of the specified name, whose content
+ * is determined (recursively) by @path.
+ *
+ * Returns: 0 on success, non-zero code on error.
+ */
+
+int gretl_make_zipfile (const char *fname, const char *path)
+{
+#if USE_GSF
+    return gretl_gsf_make_zipfile(fname, path);    
+#else
+    return gretl_plugin_make_zipfile(fname, path);       
+#endif 
+}
+
+/**
+ * gretl_unzip_session_file:
+ * @fname: name of the file to unzip.
+ * @zdirname: location to receive the name of the top-level
+ * directory within the zip archive.
+ *
+ * Specialized (slightly) unzipper for gretl session files.
+ *
+ * Returns: 0 on success, non-zero code on error.
+ */
+
+int gretl_unzip_session_file (const char *fname, gchar **zdirname)
+{
+#if USE_GSF
+    return gretl_gsf_unzip(fname, NULL, zdirname);
+#else
+    return gretl_plugin_unzip(fname, NULL, zdirname);    
+#endif 
+}
+
+/**
+ * gretl_zip_datafile:
+ * @fname: name of the zip file to create.
+ * @path: the path to the content which should be zipped.
+ * @level: the zlib compression level to apply.
+ *
+ * Creates a zip file of the specified name, with content
+ * given by @path, using the specified compression level.
+ *
+ * Returns: 0 on success, non-zero code on error.
+ */
+
+int gretl_zip_datafile (const char *fname, const char *path,
+			int level)
+{
+#if USE_GSF
+    return gretl_gsf_zip_datafile(fname, path, level);
+#else
+    return gretl_plugin_zip_datafile(fname, path, level);
+#endif 
+}
