@@ -24,6 +24,10 @@
 # include "config.h"
 #endif
 
+#ifdef WIN32
+# include "winconfig.h"
+#endif
+
 #ifdef HAVE_VASPRINTF
 # define _GNU_SOURCE
 # include <stdio.h>
@@ -49,10 +53,6 @@
 extern "C" {
 #endif
 
-#ifdef WIN32
-# include "winconfig.h"
-#endif
-
 #ifdef WIN32
 # ifndef isnan
 #  define isnan(x) ((x) != (x))
@@ -134,7 +134,8 @@ typedef enum {
     GRETL_TYPE_MATRICES_REF,
     GRETL_TYPE_BUNDLES_REF,
     GRETL_TYPE_LISTS_REF,
-    GRETL_TYPE_VOID
+    GRETL_TYPE_VOID,
+    GRETL_TYPE_ANY
 } GretlType;
 
 #define gretl_scalar_type(t) (t == GRETL_TYPE_BOOL ||	\
@@ -280,6 +281,7 @@ typedef struct DATASET_ {
     char **varname;     /* array of names of variables */
     VARINFO **varinfo;  /* array of specific info on vars */
     char markers;       /* NO_MARKERS (0), REGULAR MARKERS or DAILY_DATE_STRINGS */
+    char modflag;       /* binary flag for dataset modified or not */
     char **S;           /* to hold observation markers */
     char *descrip;      /* to hold info on data sources etc. */
     char *submask;      /* subsampling mask */
@@ -297,6 +299,8 @@ typedef struct VMatrix_ {
     int t1, t2, n;
     char **names;
     double *vec;
+    double *xbar;
+    double *ssx;
     int *list;
     int missing;
 } VMatrix;
@@ -401,9 +405,6 @@ typedef struct MODEL_ {
 #include "nls.h"
 #include "missing.h"
 #include "transforms.h"
-#ifdef WIN32
-# include "gretl_win32.h"
-#endif
 
 #ifdef  __cplusplus
 }
