@@ -58,7 +58,6 @@ enum {
     STATE_USE_SVD         = 1 << 5,  /* SVD decomposition is matrix OLS default */
     STATE_PREWHITEN       = 1 << 6,  /* HAC pre-whitening? */
     STATE_FORCE_HC        = 1 << 7,  /* don't use HAC for time series */
-    STATE_HALT_ON_ERR     = 1 << 8,  /* errors fatal in batch mode */
     STATE_USE_LBFGS       = 1 << 9,  /* prefer LBFGS to BFGS? */
     STATE_SHELL_OK        = 1 << 10, /* "shell" facility is approved? */
     STATE_MAX_VERBOSE     = 1 << 11, /* verbose output from maximizer? */
@@ -71,7 +70,9 @@ enum {
     STATE_BFGS_RSTEP      = 1 << 18, /* use Richardson method in BFGS numerical
 					gradient */
     STATE_DPDSTYLE_ON     = 1 << 19, /* emulate dpd in dynamic panel data models */
-    STATE_OPENMP_ON       = 1 << 20  /* using openmp */
+    STATE_OPENMP_ON       = 1 << 20, /* using openmp */
+    STATE_ROBUST_Z        = 1 << 21, /* use z- not t-score with HCCM/HAC */
+    STATE_ECHO_SPACE      = 1 << 22  /* preserve vertical space in output */
 };    
 
 /* for values that really want a non-negative integer */
@@ -119,9 +120,9 @@ struct set_vars_ {
     char csv_read_na[8];        /* representation of NA in CSV input */
     double nadarwat_trim;       /* multiple of h to use in nadarwat() for trimming */
     int fdjac_qual;             /* quality of "fdjac" function */
+    int wildboot_dist;          /* distribution for wild bootstrap */
 };
 
-#define ECHO "echo"
 #define MESSAGES "messages"
 #define WARNINGS "warnings"
 #define GRETL_DEBUG "debug"
@@ -135,12 +136,10 @@ struct set_vars_ {
 #define OMP_MNK_MIN "omp_mnk_min"
 #define OMP_N_THREADS "omp_num_threads"
 
-#define libset_boolvar(s) (!strcmp(s, ECHO) || \
-                           !strcmp(s, MESSAGES) || \
+#define libset_boolvar(s) (!strcmp(s, MESSAGES) || \
                            !strcmp(s, WARNINGS) || \
                            !strcmp(s, FORCE_DECP) || \
 			   !strcmp(s, FORCE_HC) || \
-			   !strcmp(s, HALT_ON_ERR) || \
                            !strcmp(s, MAX_VERBOSE) || \
 			   !strcmp(s, USE_LBFGS) || \
 			   !strcmp(s, PCSE) || \
@@ -155,6 +154,7 @@ struct set_vars_ {
 			   !strcmp(s, BFGS_RSTEP) || \
 			   !strcmp(s, DPDSTYLE) || \
 			   !strcmp(s, USE_DCMT) || \
+			   !strcmp(s, ROBUST_Z) || \
 			   !strcmp(s, USE_OPENMP))
 
 #define libset_double(s) (!strcmp(s, CONV_HUGE) || \
@@ -186,10 +186,12 @@ struct set_vars_ {
 		       !strcmp(s, OMP_N_THREADS) || \
 		       !strcmp(s, SIMD_K_MAX) || \
 		       !strcmp(s, SIMD_MN_MIN) || \
-		       !strcmp(s, FDJAC_QUAL))
+		       !strcmp(s, FDJAC_QUAL) || \
+		       !strcmp(s, WILDBOOT_DIST))
 
 /* global state */
 set_vars *state;
+static int seed_is_set;
 static int gretl_debug;
 static int user_mp_bits;
 static int R_functions;
@@ -290,6 +292,12 @@ static const char *normal_rand_strs[] = {
     NULL
 };
 
+static const char *wildboot_strs[] = {
+    "rademacher",
+    "mammen",
+    NULL
+};
+
 static const char **libset_option_strings (const char *s)
 {
     if (!strcmp(s, GARCH_VCV)) {
@@ -310,6 +318,8 @@ static const char **libset_option_strings (const char *s)
 	return csv_delim_args;
     } else if (!strcmp(s, OPTIM_STEPLEN)) {
 	return steplen_strs;
+    } else if (!strcmp(s, WILDBOOT_DIST)) {
+	return wildboot_strs;
     } else {
 	return NULL;
     }
@@ -360,6 +370,8 @@ static const char *libset_option_string (const char *s)
 	return normal_rand_strs[gretl_rand_get_box_muller()];
     } else if (!strcmp(s, OPTIM_STEPLEN)) {
 	return steplen_strs[state->optim_steplen];
+    } else if (!strcmp(s, WILDBOOT_DIST)) {
+	return wildboot_strs[state->wildboot_dist];
     } else {
 	return "?";
     }
@@ -435,6 +447,7 @@ static void state_vars_copy (set_vars *sv)
     sv->garch_robust_vcv = state->garch_robust_vcv;
     sv->nadarwat_trim = state->nadarwat_trim;
     sv->fdjac_qual = state->fdjac_qual;
+    sv->wildboot_dist = state->wildboot_dist;
 
     sv->initvals = gretl_matrix_copy(state->initvals);
     sv->matmask = gretl_matrix_copy(state->matmask);
@@ -571,7 +584,7 @@ static void state_vars_init (set_vars *sv)
     fprintf(stderr, "state_vars_init called\n");
 #endif
     sv->flags = STATE_ECHO_ON | STATE_MSGS_ON | STATE_WARN_ON | 
-	STATE_HALT_ON_ERR | STATE_SKIP_MISSING;
+	STATE_SKIP_MISSING;
 #if defined(_OPENMP)
     if (openmp_by_default()) {
 	sv->flags |= STATE_OPENMP_ON;
@@ -603,6 +616,7 @@ static void state_vars_init (set_vars *sv)
     sv->garch_robust_vcv = ML_UNSET;
     sv->nadarwat_trim = 4.0;
     sv->fdjac_qual = 0;
+    sv->wildboot_dist = 0;
 
     *sv->shelldir = '\0';
 
@@ -651,6 +665,12 @@ int gretl_echo_on (void)
     return flag_to_bool(state, STATE_ECHO_ON);
 }
 
+int gretl_echo_space (void)
+{
+    if (check_for_state()) return 0;
+    return flag_to_bool(state, STATE_ECHO_SPACE);
+}
+
 void set_gretl_messages (int e)
 {
     if (check_for_state()) return;
@@ -922,12 +942,10 @@ static int libset_get_scalar (const char *var, const char *arg,
 	return err;
     }
 
-    if (gretl_is_scalar(arg)) {
-	x = gretl_scalar_get_value(arg, NULL);
-    } else {
-	gretl_errmsg_sprintf("'%s': not a scalar", arg);
-	return E_UNKVAR;
-    }
+    x = get_scalar_value_by_name(arg, &err);
+    if (err) {
+	return err;
+    }	
 
     if (negval_invalid(var) && x < 0.0) {
 	return E_DATA;
@@ -1028,6 +1046,14 @@ static int parse_libset_int_code (const char *key,
 		break;
 	    }
 	}
+    } else if (!g_ascii_strcasecmp(key, WILDBOOT_DIST)) {
+	for (i=0; wildboot_strs[i] != NULL; i++) {
+	    if (!g_ascii_strcasecmp(val, wildboot_strs[i])) {
+		state->wildboot_dist = i;
+		err = 0;
+		break;
+	    }
+	}
     } else if (!g_ascii_strcasecmp(key, OPTIM_STEPLEN)) {
 	for (i=0; i<STEPLEN_MAX; i++) {
 	    if (!g_ascii_strcasecmp(val, steplen_strs[i])) {
@@ -1137,6 +1163,47 @@ static int set_initvals (const char *mname, PRN *prn)
     return err;
 }
 
+static int set_echo_status (const char *arg)
+{
+    int err = 0;
+
+    if (check_for_state()) {
+	return 1;
+    }
+
+    if (!strcmp(arg, "on")) {
+	state->flags |= STATE_ECHO_ON;
+    } else if (!strcmp(arg, "off")) {
+	state->flags &= ~STATE_ECHO_ON;
+	state->flags &= ~STATE_ECHO_SPACE;
+    } else if (!strcmp(arg, "space")) {
+	state->flags |= STATE_ECHO_SPACE;
+    } else if (!strcmp(arg, "full")) {
+	state->flags |= STATE_ECHO_ON;
+	state->flags |= STATE_ECHO_SPACE;
+    } else {
+	err = E_INVARG;
+    }
+	
+    return err;
+}
+
+static const char *get_echo_status (void)
+{
+    if (check_for_state()) {
+	return "on";
+    } else if ((state->flags & STATE_ECHO_ON) && 
+	       (state->flags & STATE_ECHO_SPACE)) {
+	return "full";
+    } else if (state->flags & STATE_ECHO_ON) {
+	return "on";
+    } else if (state->flags & STATE_ECHO_SPACE) {
+	return "space";
+    } else {
+	return "off";
+    }
+}
+
 static int set_matmask (const char *vname, const DATASET *dset,
 			PRN *prn)
 {
@@ -1200,10 +1267,6 @@ static int set_shelldir (const char *s)
 {
     int len = 0, err = 0;
 
-    /* skip past "set shelldir" and space */
-    s += 12;
-    s += strspn(s, " ");
-
     if (*s == '\0') {
 	*state->shelldir = '\0';
     } else if (*s == '"') {
@@ -1271,10 +1334,6 @@ static int set_workdir (const char *s)
 	return 1;
     }
 
-    /* skip past "set workdir" and space */
-    s += 11;
-    s += strspn(s, " ");
-
     if (*s == '\0') {
 	err = E_DATA;
     } else {
@@ -1343,7 +1402,8 @@ static void libset_print_bool (const char *s, PRN *prn,
 			 !strcmp(s, VECM_NORM) || \
 			 !strcmp(s, GRETL_OPTIM) || \
 			 !strcmp(s, NORMAL_RAND) || \
-			 !strcmp(s, OPTIM_STEPLEN))
+			 !strcmp(s, OPTIM_STEPLEN) || \
+			 !strcmp(s, WILDBOOT_DIST))
 
 const char *intvar_code_string (const char *s)
 {
@@ -1433,10 +1493,15 @@ static int print_settings (PRN *prn, gretlopt opt)
 	pprintf(prn, "set csv_read_na %s\n", get_csv_na_read_string());
     }
 
-    libset_print_int(CSV_DIGITS, prn, opt);	    
-    libset_print_bool(ECHO, prn, opt);
+    libset_print_int(CSV_DIGITS, prn, opt);
+
+    if (opt & OPT_D) {
+	pprintf(prn, " echo = %s\n", get_echo_status());
+    } else {
+	pprintf(prn, "set echo %s\n", get_echo_status());
+    }
+	    
     libset_print_bool(FORCE_DECP, prn, opt);
-    libset_print_bool(HALT_ON_ERR, prn, opt);
     libset_print_int(LOOP_MAXITER, prn, opt);
     libset_print_bool(MAX_VERBOSE, prn, opt);
     libset_print_int(BFGS_VERBSKIP, prn, opt);
@@ -1451,13 +1516,23 @@ static int print_settings (PRN *prn, gretlopt opt)
     libset_print_int(SIMD_MN_MIN, prn, opt);
 
     if (opt & OPT_D) {
+	/* display only */
 	libset_print_bool(SHELL_OK, prn, opt);
 	if (*state->shelldir) {
 	    pprintf(prn, " shelldir = '%s'\n", state->shelldir);
 	} else {
 	    pputs(prn, " shelldir = unset\n");
 	}
-    } 
+    } else {
+	/* saving to file */
+	if (*state->shelldir) {
+	    if (strchr(state->shelldir, ' ')) {
+		pprintf(prn, "set shelldir \"%s\"\n", state->shelldir);
+	    } else {
+		pprintf(prn, "set shelldir %s\n", state->shelldir);
+	    }
+	}
+    }	
 
     libset_print_bool(USE_CWD, prn, opt);
     libset_print_bool(SKIP_MISSING, prn, opt);
@@ -1493,7 +1568,9 @@ static int print_settings (PRN *prn, gretlopt opt)
 	pprintf(prn, " seed = %u\n", gretl_rand_get_seed());
 	pprintf(prn, " normal_rand = %s\n", libset_option_string(NORMAL_RAND));
     } else {
-	pprintf(prn, "set seed %u\n", gretl_rand_get_seed());
+	if (seed_is_set) {
+	    pprintf(prn, "set seed %u\n", gretl_rand_get_seed());
+	}
 	pprintf(prn, "set normal_rand %s\n", libset_option_string(NORMAL_RAND));
     }
     if (gretl_mpi_initialized()) {
@@ -1527,7 +1604,9 @@ static int libset_query_settings (const char *s, PRN *prn)
 {
     int err = 0;
 
-    if (libset_boolvar(s)) {
+    if (!strcmp(s, "echo")) {
+	pprintf(prn, "%s: code, currently '%s'\n", s, get_echo_status());
+    } else if (libset_boolvar(s)) {
 	pprintf(prn, "%s: boolean (on/off), currently %s\n", 
 		s, libset_get_bool(s)? "on" : "off");
     } else if (coded_intvar(s)) {
@@ -1566,8 +1645,9 @@ static int libset_query_settings (const char *s, PRN *prn)
 	    pprintf(prn, "%s: matrix, currently null\n", s);
 	}
     } else if (!strcmp(s, "seed")) {
-	pprintf(prn, "%s: unsigned int, currently %u\n",
-		s, state->seed ? state->seed : gretl_rand_get_seed());
+	pprintf(prn, "%s: unsigned int, currently %u (%s)\n",
+		s, state->seed ? state->seed : gretl_rand_get_seed(),
+		seed_is_set ? "set by user" : "automatic");
     } else if (!strcmp(s, "csv_delim")) {
 	pprintf(prn, "%s: named character, currently \"%s\"\n", s,
 		arg_from_delim(data_delim));
@@ -1642,12 +1722,10 @@ static int check_set_bool (const char *setobj, const char *setarg)
     }
 }
 
-int execute_set_line (const char *line, DATASET *dset, 
-		      gretlopt opt, PRN *prn)
+int execute_set (const char *setobj, const char *setarg,
+		 DATASET *dset, gretlopt opt, PRN *prn)
 {
-    char setobj[32], setarg[32], junk[8];
-    int k, argc, err = E_PARSE;
-    double x;
+    int k, argc, err;
 
     check_for_state();
 
@@ -1655,40 +1733,28 @@ int execute_set_line (const char *line, DATASET *dset,
 	return write_or_read_settings(opt, prn);
     }
 
-    *setobj = *setarg = *junk = '\0';
+    argc = (setobj != NULL) + (setarg != NULL);
 
-    argc = sscanf(line, "%*s %31s %31s %7s", setobj, setarg, junk);
-
-    if (argc <= 0) {
+    if (argc == 0) {
 	return print_settings(prn, OPT_D);
     }
 
-    if (argc > 1) {
-	/* specials which need the whole line (FIXME) */
-	if (!strcmp(setobj, "shelldir")) {
-	    return set_shelldir(line);
-	} else if (!strcmp(setobj, "workdir")) {
-	    return set_workdir(line);
-	}
-    }
-
-    if (argc == 3) {
-	/* got some extraneous stuff */
-	return err;
-    }
+    /* set error default */
+    err = E_PARSE;
 
     if (argc == 1) {
-	if (!strcmp(setobj, ECHO)) {
-	    state->flags |= STATE_ECHO_ON;
-	    err = 0;
-	} else if (!strcmp(setobj, "stopwatch")) {
+	if (!strcmp(setobj, "stopwatch")) {
 	    gretl_stopwatch();
-	    err = 0;
+	    return 0;
 	} else {
 	    return libset_query_settings(setobj, prn);
 	}
     } else if (argc == 2) {
-	if (!strcmp(setobj, "csv_write_na") || !strcmp(setobj, "csv_na")) {
+	if (!strcmp(setobj, "shelldir")) {
+	    return set_shelldir(setarg);
+	} else if (!strcmp(setobj, "workdir")) {
+	    return set_workdir(setarg);
+	} else if (!strcmp(setobj, "csv_write_na") || !strcmp(setobj, "csv_na")) {
 	    return set_csv_na_write_string(setarg);
 	} else if (!strcmp(setobj, "csv_read_na")) {
 	    return set_csv_na_read_string(setarg);
@@ -1698,12 +1764,14 @@ int execute_set_line (const char *line, DATASET *dset,
 	    return set_initvals(setarg, prn);
 	} else if (!strcmp(setobj, "matrix_mask")) {
 	    return set_matmask(setarg, dset, prn);
+	} else if (!strcmp(setobj, "echo")) {
+	    return set_echo_status(setarg);
 	}
 
 	if (libset_boolvar(setobj)) {
 	    if (!strcmp(setobj, SHELL_OK)) {
-		pprintf(prn, "You can only set this variable "
-			"via the gretl GUI\n");
+		pprintf(prn, "'%s': this must be set via the gretl GUI\n", setobj);
+		err = E_DATA;
 	    } else if (!strcmp(setobj, USE_OPENMP)) {
 #if defined(_OPENMP)
 		err = check_set_bool(setobj, setarg);
@@ -1712,18 +1780,14 @@ int execute_set_line (const char *line, DATASET *dset,
 #endif
 	    } else {
 		err = check_set_bool(setobj, setarg);
-		if (!err && !strcmp(setobj, HALT_ON_ERR) &&
-		    boolean_off(setarg)) {
-		    pputs(prn, "Warning: \"set halt_on_error off\" is "
-			  "deprecated and will be removed.\nPlease use "
-			  "\"catch\" to trap errors instead.\n");
-		}
 	    }
 	} else if (libset_double(setobj)) {
 	    if (default_ok(setobj) && default_str(setarg)) {
 		libset_set_double(setobj, NADBL);
 		err = 0;
 	    } else {
+		double x;
+
 		err = libset_get_scalar(NULL, setarg, NULL, &x);
 		if (!err) {
 		    err = libset_set_double(setobj, x);
@@ -1745,6 +1809,7 @@ int execute_set_line (const char *line, DATASET *dset,
 			    _("Pseudo-random number generator seeded with %d\n"), k);
 		}
 		state->seed = k;
+		seed_is_set = 1;
 	    }
 	} else if (!strcmp(setobj, HORIZON)) {
 	    /* horizon for VAR impulse responses */
@@ -1934,6 +1999,8 @@ int libset_get_int (const char *key)
 	return csv_digits;
     } else if (!strcmp(key, FDJAC_QUAL)) {
 	return state->fdjac_qual;
+    } else if (!strcmp(key, WILDBOOT_DIST)) {
+	return state->wildboot_dist;
     } else {
 	fprintf(stderr, "libset_get_int: unrecognized "
 		"variable '%s'\n", key);	
@@ -2001,6 +2068,10 @@ static int intvar_min_max (const char *s, int *min, int *max,
 	*min = 0;
 	*max = 3;
 	*var = &state->fdjac_qual;
+    } else if (!strcmp(s, WILDBOOT_DIST)) {
+	*min = 0;
+	*max = 1;
+	*var = &state->wildboot_dist;
     } else {
 	fprintf(stderr, "libset_set_int: unrecognized "
 		"variable '%s'\n", s);	
@@ -2051,9 +2122,7 @@ int libset_set_int (const char *key, int val)
 
 static int boolvar_get_flag (const char *s)
 {
-    if (!strcmp(s, ECHO)) {
-	return STATE_ECHO_ON;
-    } else if (!strcmp(s, MESSAGES)) {
+    if (!strcmp(s, MESSAGES)) {
 	return STATE_MSGS_ON;
     } else if (!strcmp(s, WARNINGS)) {
 	return STATE_WARN_ON;
@@ -2067,8 +2136,6 @@ static int boolvar_get_flag (const char *s)
 	return STATE_USE_CWD;
     } else if (!strcmp(s, USE_FCP)) {
 	return STATE_USE_FCP;
-    } else if (!strcmp(s, HALT_ON_ERR)) {
-	return STATE_HALT_ON_ERR;
     } else if (!strcmp(s, MAX_VERBOSE)) {
 	return STATE_MAX_VERBOSE;
     } else if (!strcmp(s, SHELL_OK)) {
@@ -2087,6 +2154,8 @@ static int boolvar_get_flag (const char *s)
 	return STATE_DPDSTYLE_ON;
     } else if (!strcmp(s, USE_OPENMP)) {
 	return STATE_OPENMP_ON;
+    } else if (!strcmp(s, ROBUST_Z)) {
+	return STATE_ROBUST_Z;
     } else {
 	fprintf(stderr, "libset_get_bool: unrecognized "
 		"variable '%s'\n", s);	
@@ -2120,8 +2189,6 @@ static void maybe_check_env (const char *s)
 	set_flag_from_env(STATE_USE_SVD, "GRETL_USE_SVD", 0);
     } else if (!strcmp(s, USE_LBFGS)) {
 	set_flag_from_env(STATE_USE_LBFGS, "GRETL_USE_LBFGS", 0);
-    } else if (!strcmp(s, HALT_ON_ERR)) {
-	set_flag_from_env(STATE_HALT_ON_ERR, "GRETL_KEEP_GOING", 1);
     }
 }
 
@@ -2470,6 +2537,25 @@ int gretl_looping_quietly (void)
     return (state->flags & STATE_LOOP_QUIET)? 1 : 0;
 }
 
+static int iter_depth;
+
+void gretl_iteration_push (void)
+{
+    iter_depth++;
+}
+
+void gretl_iteration_pop (void)
+{
+    if (iter_depth > 0) {
+	iter_depth--;
+    }
+}
+
+int gretl_iteration_depth (void)
+{
+    return iter_depth;
+}
+
 int gretl_looping_progressive (void)
 {
     return (state->flags & STATE_LOOP_PROG)? 1 : 0;
@@ -2700,6 +2786,36 @@ int libset_write_script (const char *fname)
     return err;
 }
 
+static char *get_quoted_arg (const char *s, int *err)
+{
+    const char *p;
+    int n = 0, matched = 0;
+    char *ret = NULL;
+
+    p = s = strchr(s, '"') + 1;
+
+    while (*p) {
+	if (*p == '"') {
+	    matched = 1;
+	    break;
+	} else {
+	    n++;
+	}
+	p++;
+    }
+
+    if (!matched) {
+	*err = E_PARSE;
+    } else {
+	ret = gretl_strndup(s, n);
+	if (ret == NULL) {
+	    *err = E_ALLOC;
+	}
+    }
+
+    return ret;
+}
+
 /* If @prn is non-NULL we're called via the "set" command.
    Otherwise we're called by the gretl session apparatus.
 */
@@ -2724,14 +2840,29 @@ static int real_libset_read_script (const char *fname,
     }
 
     if (!err) {
-	char line[1024];
+	char setobj[32], setarg[32], line[1024];
+	int nf;
 
 	while (fgets(line, sizeof line, fp)) {
 	    if (*line == '#' || string_is_blank(line)) {
 		continue;
 	    }
 	    tailstrip(line);
-	    err = execute_set_line(line, NULL, OPT_NONE, prn);
+	    nf = sscanf(line, "%*s %31s %31s", setobj, setarg);
+	    if (nf == 1) {
+		err = execute_set(setobj, NULL, NULL, OPT_NONE, prn);
+	    } else if (nf == 2) {
+		if (*setarg == '"') {
+		    char *q = get_quoted_arg(line, &err);
+
+		    if (!err) {
+			err = execute_set(setobj, q, NULL, OPT_NONE, prn);
+			free(q);
+		    }
+		} else {
+		    err = execute_set(setobj, setarg, NULL, OPT_NONE, prn);
+		}
+	    }
 	    if (err && prn != NULL) {
 		break;
 	    }
