@@ -59,7 +59,6 @@ typedef enum {
 #define GARCH_ROBUST_VCV "garch_robust_vcv"
 #define HAC_KERNEL       "hac_kernel"
 #define HAC_LAG          "hac_lag"
-#define HALT_ON_ERR      "halt_on_error"
 #define HC_VERSION       "hc_version"
 #define HORIZON          "horizon"
 #define USE_LBFGS        "lbfgs"
@@ -88,6 +87,8 @@ typedef enum {
 #define CONV_HUGE        "huge"
 #define USE_OPENMP       "openmp"
 #define FDJAC_QUAL       "fdjac_quality"
+#define ROBUST_Z         "robust_z"
+#define WILDBOOT_DIST    "wildboot"
 
 typedef int (*ITER_PRINT_FUNC) (int, PRN *);
 typedef void (*SHOW_ACTIVITY_FUNC) (void);
@@ -153,6 +154,10 @@ int gretl_looping_currently (void);
 int gretl_looping_quietly (void);
 int gretl_looping_progressive (void);
 
+void gretl_iteration_push (void);
+void gretl_iteration_pop (void);
+int gretl_iteration_depth (void);
+
 void gretl_set_batch_mode (int b);
 int gretl_in_batch_mode (void);
 
@@ -161,6 +166,7 @@ int gretl_in_gui_mode (void);
 
 void set_gretl_echo (int e);
 int gretl_echo_on (void);
+int gretl_echo_space (void);
 
 void set_gretl_messages (int e);
 int gretl_messages_on (void);
@@ -183,8 +189,8 @@ int set_csv_na_write_string (const char *s);
 const char *get_csv_na_read_string (void);
 int set_csv_na_read_string (const char *s);
 
-int execute_set_line (const char *line, DATASET *dset, 
-		      gretlopt opt, PRN *prn);
+int execute_set (const char *setobj, const char *setarg,
+		 DATASET *dset, gretlopt opt, PRN *prn);
 
 void set_iter_print_func (ITER_PRINT_FUNC func);
 int iter_print_callback (int i, PRN *prn);
