@@ -21,6 +21,11 @@
 #include "usermat.h"
 #include "matrix_extra.h"
 #include "libset.h"
+#include "swap_bytes.h"
+
+#ifdef WIN32
+# include "gretl_win32.h"
+#endif
 
 #include <errno.h>
 
@@ -494,10 +499,10 @@ real_gretl_matrix_data_subset (const int *list,
     int skip;
     int j, vj, s, t;
 
-    if (k <= 0 || Tmax <= 0) {
+    if (k <= 0 || Tmax <= 0 || t2 >= dset->n) {
 	*err = E_DATA;
 	return NULL;
-    }	
+    }
 
     *err = 0;
     T = Tmax;
@@ -892,7 +897,10 @@ int gretl_plotfit_matrices (const double *yvar, const double *xvar,
     if (fit == PLOT_FIT_LOGLIN && !gretl_ispositive(t1, t2, yvar, 1)) {
 	gretl_errmsg_set(_("Non-positive values encountered"));
 	return E_DATA;
-    }
+    } else if (fit == PLOT_FIT_LINLOG && !gretl_ispositive(t1, t2, xvar, 1)) {
+	gretl_errmsg_set(_("Non-positive values encountered"));
+	return E_DATA;
+    }	
 
     mask = calloc(T, 1);
     if (mask == NULL) {
@@ -946,6 +954,8 @@ int gretl_plotfit_matrices (const double *yvar, const double *xvar,
 	    xt = (xvar != NULL)? xvar[t] : s;
 	    if (fit == PLOT_FIT_INVERSE) {
 		gretl_matrix_set(X, i, j++, 1.0 / xt);
+	    } else if (fit == PLOT_FIT_LINLOG) {
+		gretl_matrix_set(X, i, j++, log(xt));
 	    } else {
 		gretl_matrix_set(X, i, j++, xt);
 	    }
@@ -1063,36 +1073,104 @@ static char *decompress_matrix_buffer (gzFile fz, const char *fname,
     return buf;
 }
 
+#if G_BYTE_ORDER == G_BIG_ENDIAN
+
+static void matrix_swap_endianness (gretl_matrix *A)
+{
+    int i, n = A->rows * A->cols;
+
+    for (i=0; i<n; i++) {
+	reverse_double(A->val[i]);
+    }
+}
+
+#endif
+
+static gretl_matrix *read_binary_matrix_file (FILE *fp, int *err)
+{
+    gretl_matrix *A = NULL;
+    char header[20] = {0};
+    gint32 dim[2];
+
+    if (fread(header, 1, 19, fp) < 19) {
+	*err = E_DATA;
+    } else if (strcmp(header, "gretl_binary_matrix")) {
+	*err = E_DATA;
+    } else if (fread(dim, sizeof *dim, 2, fp) < 2) {
+	*err = E_DATA;
+    } else if (dim[0] <= 0 || dim[1] <= 0) {
+	*err = E_DATA;
+    }
+
+    if (!*err) {
+#if G_BYTE_ORDER == G_BIG_ENDIAN
+	reverse_int(dim[0]);
+	reverse_int(dim[1]);
+#endif    
+	A = gretl_matrix_alloc(dim[0], dim[1]);
+	if (A == NULL) {
+	    *err = E_ALLOC;
+	}
+    }
+
+    if (!*err) {
+	size_t n = dim[0] * dim[1];
+    
+	if (fread(A->val, sizeof *A->val, n, fp) < n) {
+	    *err = E_DATA;
+	    gretl_matrix_free(A);
+	    A = NULL;
+	}
+    }
+
+#if G_BYTE_ORDER == G_BIG_ENDIAN
+    if (A != NULL) {
+	matrix_swap_endianness(A);
+    }
+#endif    
+
+    fclose(fp);
+
+    return A;
+}
+
 /**
- * gretl_matrix_read_from_text:
- * @fname: name of text file.
+ * gretl_matrix_read_from_file:
+ * @fname: name of file.
  * @import: non-zero means we're importing via dotdir.
  * @err: location to receive error code.
  *
- * Reads a matrix from a text file by the name @fname; the column
- * separator must be space or tab. It is assumed that the dimensions of
- * the matrix (number of rows and columns) are found on the first line
- * of the csv file, so no heuristics are necessary. In case of error,
- * @err is filled appropriately.
+ * Reads a matrix from a file by the name @fname; if it's
+ * a text file the column separator must be space or tab. It is 
+ * assumed that the dimensions of the matrix (number of rows and 
+ * columns) are found on the first line of the file, so no 
+ * heuristics are necessary. In case of error,  @err is filled 
+ * appropriately.
  *
- * Returns: The matrix read from file, or NULL.
+ * Returns: The matrix as read from file, or NULL.
  */
 
-gretl_matrix *gretl_matrix_read_from_text (const char *fname, 
+gretl_matrix *gretl_matrix_read_from_file (const char *fname, 
 					   int import, int *err)
 {
     char fullname[FILENAME_MAX];
-    int r, c, n, gz;
+    int r, c, n, gz, bin = 0;
     gretl_matrix *A = NULL;
     gzFile fz = Z_NULL;
     FILE *fp = NULL;
 
     gz = has_suffix(fname, ".gz");
 
+    if (!gz) {
+	bin = has_suffix(fname, ".bin");
+    }
+
     if (import) {
 	build_path(fullname, gretl_dotdir(), fname, NULL);
 	if (gz) {
 	    fz = gretl_gzopen(fullname, "r");
+	} else if (bin) {
+	    fp = gretl_fopen(fullname, "rb");
 	} else {
 	    fp = gretl_fopen(fullname, "r");
 	}
@@ -1100,6 +1178,8 @@ gretl_matrix *gretl_matrix_read_from_text (const char *fname,
 	strcpy(fullname, fname);
 	if (gz) {
 	    fz = gretl_gzopen(fullname, "r");
+	} else if (bin) {
+	    fp = gretl_fopen(fullname, "rb");
 	} else {
 	    fp = gretl_fopen(fullname, "r");
 	}
@@ -1108,6 +1188,8 @@ gretl_matrix *gretl_matrix_read_from_text (const char *fname,
 	    if (strcmp(fullname, fname)) {
 		if (gz) {
 		    fz = gretl_gzopen(fullname, "r");
+		} else if (bin) {
+		    fp = gretl_fopen(fullname, "rb");
 		} else {
 		    fp = gretl_fopen(fullname, "r");
 		}
@@ -1120,6 +1202,10 @@ gretl_matrix *gretl_matrix_read_from_text (const char *fname,
 	return NULL;
     }
 
+    if (bin) {
+	return read_binary_matrix_file(fp, err);
+    }
+
     /* skip any leading comment lines starting with '#' */
     while (!*err && skip_matrix_comment(fp, fz, err)) {
 	;
@@ -1245,20 +1331,19 @@ static void win32_xna_out (double x, gzFile fz, FILE *fp,
 #endif
 
 /**
- * gretl_matrix_write_as_text:
- * @A: matrix.
- * @fname: name of file to write.
+ * gretl_matrix_write_to_file:
+ * @A: matrix to write.
+ * @fname: name of file.
  * @export: non-zero means we're exporting via dotdir.
  *
- * Writes the matrix @A to a plain text file by the name @fname; the
- * column separator is the tab. The number of rows and columns are
- * written on the first line of the file (which comes in handy for
- * reading the matrix).
+ * Writes the matrix @A to a file by the name @fname; if it's a text
+ * file, the column separator is tab. The number of rows and columns
+ * are written on the first line of the file.
  *
  * Returns: 0 on successful completion, non-zero code on error.
  */
 
-int gretl_matrix_write_as_text (gretl_matrix *A, const char *fname,
+int gretl_matrix_write_to_file (gretl_matrix *A, const char *fname,
 				int export)
 {
     int r = A->rows;
@@ -1268,10 +1353,14 @@ int gretl_matrix_write_as_text (gretl_matrix *A, const char *fname,
     FILE *fp = NULL;
     double x;
     char pad, d = '\t';
-    int gz;
+    int gz, bin = 0;
 
     gz = has_suffix(fname, ".gz");
 
+    if (!gz) {
+	bin = has_suffix(fname, ".bin");
+    }
+
     if (export) {
 	char targ[FILENAME_MAX];
 
@@ -1290,6 +1379,35 @@ int gretl_matrix_write_as_text (gretl_matrix *A, const char *fname,
 	}
     }
 
+    if (bin && fp != NULL) {
+	const char *header = "gretl_binary_matrix";
+	gint32 dim[2] = {r, c};
+	size_t n = r * c;
+	
+#if G_BYTE_ORDER == G_BIG_ENDIAN
+	double x;
+	size_t i;
+	int k;
+	
+	fwrite(header, 1, strlen(header), fp);
+	for (i=0; i<2; i++) {
+	    k = dim[i];
+	    reverse_int(k);
+	    fwrite(&k, sizeof k, 1, fp);
+	}
+	for (i=0; i<n; i++) {
+	    x = A->val[i];
+	    reverse_double(x);
+	    fwrite(&x, sizeof x, 1, fp);
+	}
+#else
+	fwrite(header, 1, strlen(header), fp);
+	fwrite(dim, sizeof *dim, 2, fp);
+	fwrite(A->val, sizeof *A->val, n, fp);
+#endif	
+	goto finish;
+    }
+
     if (fz) {
 	gzprintf(fz, "%d%c%d\n", r, d, c);
     } else if (fp) {
@@ -1322,6 +1440,8 @@ int gretl_matrix_write_as_text (gretl_matrix *A, const char *fname,
 
     gretl_pop_c_numeric_locale();
 
+ finish:
+
     if (fz) {
 	gzclose(fz);
     } else {
@@ -1390,7 +1510,7 @@ static int max_label_length (const char **names, int n)
 
     for (i=0; i<n; i++) {
 	if (names[i] != NULL) {
-	    len = strlen(names[i]);
+	    len = g_utf8_strlen(names[i], -1);
 	    if (len > maxlen) {
 		maxlen = len;
 	    }
@@ -2407,6 +2527,10 @@ static int legendre_scale (int n, double *x, double *w,
     double shft, slp;
     int i;
 
+    if (xna(a) || xna(b)) {
+	return E_INVARG;
+    }
+
     if (fabs(b - a) <= DBL_EPSILON) {
 	fprintf(stderr, "legendre: |b - a| too small\n");
 	return E_DATA;
@@ -2423,34 +2547,53 @@ static int legendre_scale (int n, double *x, double *w,
     return 0;
 }
 
-static void hermite_scale (int n, double *x, double *w,
-			   double mu, double sigma)
+static int hermite_scale (int n, double *x, double *w,
+			  double mu, double sigma)
 {
     double rtpi = sqrt(M_PI);
     int i;
 
+    if (xna(mu) || xna(sigma) || sigma <= 0.0) {
+	return E_INVARG;
+    }
+
     for (i=0; i<n; i++) {
 	x[i] = mu + M_SQRT2 * sigma * x[i];
 	w[i] /= rtpi;
     }
+
+    return 0;
 }
 
 /**
  * gretl_quadrule_matrix_new:
  * @n: the order (i.e. the number of abscissae and weights).
  * @method: should be one of the values in #QuadMethod.
- * @a: for method = QUAD_LEGENDRE, the lower limit of integration.
- * @b: for method = QUAD_LEGENDRE, the upper limit of integration.
+ * @a: optional parameter (see below).
+ * @b: optional parameter (see below).
  * @err: location to receive error code.
  *
  * Calculates a quadrature "rule" (i.e. a set of abscissae or
- * nodes and assoicated weights) for use in numerical integration.
+ * nodes and associated weights) for use in numerical integration.
  * The three supported methods are Gauss-Hermite, Gauss-Legendre
- * and Gauss-Laguerre. The arguments @a and @b are ignored for
- * methods other than Legendre: in the Gauss-Hermite case the
- * integral that is approximated runs from minus infinity to
- * plus infinity, and in the Laguerre case it runs from zero to
- * plus infinity.
+ * and Gauss-Laguerre.
+ *
+ * If the optional parameters are not to be used, both should
+ * be given the value NADBL. 
+ *
+ * If the method is QUAD_GHERMITE, the default weights are W(x)
+ * = exp(-x^2), but @a and @b can be used to apply a normal
+ * scaling with mean @a and standard deviation @b. The limits
+ * of integration are in principle minus infinity and plus
+ * infinity.
+ *
+ * If the method is QUAD_LEGENDRE the weights are W(x) = 1 and
+ * the default limits of integration are -1 and 1, but @a and @b
+ * can be used to adjust the lower and upper limit respectively.
+ *
+ * In the QUAD_LAGUERRE case W(x) = exp(-x) and the limits
+ * of integration are 0 and plus infinity; @a and @b are
+ * ignored.
  * 
  * Returns: an @n x 2 matrix containing the abscissae in the first
  * column and the weights in the second, or NULL on failure.
@@ -2494,13 +2637,11 @@ gretl_matrix *gretl_quadrule_matrix_new (int n, int method,
 		    *err = legendre_scale(n, x, w, a, b);
 		}
 	    } else if (method == QUAD_GHERMITE) {
-#if 1 /* FIXME: change this */
-		hermite_scale(n, x, w, 0.0, 1.0);
-#else
-		if (!na(a) && !na(b)) {
-		    hermite_scale(n, x, w, a, b);
+		if (!na(a) || !na(b)) {
+		    *err = hermite_scale(n, x, w, a, b);
 		}
-#endif
+	    } else if (method == QUAD_LAGUERRE) {
+		; /* a and b are ignored */
 	    }
 	}
     }
