@@ -39,6 +39,8 @@
 #define LOOP_DEBUG 0
 #define SUBST_DEBUG 0
 
+#define LOOPSAVE 0
+
 #include <gmp.h>
 
 typedef mpf_t bigval;
@@ -98,12 +100,13 @@ typedef struct {
     DATASET *dset;  /* temporary data storage */
 } LOOP_STORE;
 
-enum loop_flags {
+typedef enum {
     LOOP_PROGRESSIVE = 1 << 0,
     LOOP_VERBOSE     = 1 << 1,
     LOOP_QUIET       = 1 << 2,
-    LOOP_DELVAR      = 1 << 3
-};
+    LOOP_DELVAR      = 1 << 3,
+    LOOP_ATTACHED    = 1 << 4
+} LoopFlags;
 
 struct controller_ {
     double val;            /* evaluated value */
@@ -114,32 +117,31 @@ struct controller_ {
 
 typedef struct controller_ controller;
 
-enum loop_command_codes {
+typedef enum {
     LOOP_CMD_GENR    = 1 << 0, /* compiled "genr" */
     LOOP_CMD_LIT     = 1 << 1, /* literal printing */
     LOOP_CMD_NODOL   = 1 << 2, /* no $-substitution this line */
     LOOP_CMD_NOSUB   = 1 << 3, /* no @-substitution this line */
-    LOOP_CMD_NOOPT   = 1 << 4, /* no option flags in this line */
-    LOOP_CMD_CATCH   = 1 << 5, /* "catch" flag present */
-    LOOP_CMD_COND    = 1 << 6, /* compiled conditional */
-    LOOP_CMD_DONE    = 1 << 7  /* progressive loop command parsed */
-};
+    LOOP_CMD_CATCH   = 1 << 4, /* "catch" flag present */
+    LOOP_CMD_COND    = 1 << 5, /* compiled conditional */
+    LOOP_CMD_DONE    = 1 << 6, /* progressive loop command parsed */
+    LOOP_CMD_OK      = 1 << 7  /* command has been run without error */
+} LoopCmdFlags;
 
 struct loop_command_ {
     char *line;
     int ci;
-    char flags;
+    gretlopt opt;
+    LoopCmdFlags flags;
     GENERATOR *genr;
 };
 
 typedef struct loop_command_ loop_command;
 
-typedef struct LOOPSET_ LOOPSET;
-
 struct LOOPSET_ {
     /* basic characteristics */
     char type;
-    char flags;
+    LoopFlags flags;
     int level;
     int err;
 
@@ -188,6 +190,8 @@ struct LOOPSET_ {
 #define loop_set_verbose(l)     (l->flags |= LOOP_VERBOSE)
 #define loop_is_quiet(l)        (l->flags & LOOP_QUIET)
 #define loop_set_quiet(l)       (l->flags |= LOOP_QUIET)
+#define loop_is_attached(l)     (l->flags & LOOP_ATTACHED)
+#define loop_set_attached(l)    (l->flags |= LOOP_ATTACHED)
 
 #define model_print_deferred(o) (o & OPT_F)
 
@@ -345,20 +349,6 @@ loop_delta (LOOPSET *loop, DATASET *dset, int *err)
     }
 }
 
-static gretlopt get_loop_opts (ExecState *s, int *err)
-{
-    gretlopt opt = OPT_NONE;
-
-    if (compile_level > 0) {
-	/* options will not have been parsed out already */
-	opt = get_gretl_options(s->line, err);
-    } else {
-	opt = s->cmd->opt;
-    }
-
-    return opt;
-}
-
 static void set_loop_opts (LOOPSET *loop, gretlopt opt)
 {
     if (opt & OPT_P) {
@@ -455,7 +445,7 @@ static void gretl_loop_init (LOOPSET *loop)
     loop->itermax = 0;
     loop->iter = 0;
     loop->err = 0;
-    *loop->idxname = 0;
+    *loop->idxname = '\0';
     loop->idxval = 0;
     loop->brk = 0;
     *loop->listname = '\0';
@@ -509,7 +499,7 @@ static LOOPSET *gretl_loop_new (LOOPSET *parent)
     return loop;
 }
 
-static void gretl_loop_destroy (LOOPSET *loop)
+void gretl_loop_destroy (LOOPSET *loop)
 {
     int i;
 
@@ -517,6 +507,10 @@ static void gretl_loop_destroy (LOOPSET *loop)
 	return;
     }
 
+#if LOOP_DEBUG || defined(SHOW_STRUCTURE)
+    fprintf(stderr, "destroying LOOPSET at %p\n", (void *) loop);
+#endif
+
     for (i=0; i<loop->n_children; i++) {
 	gretl_loop_destroy(loop->children[i]);
 	loop->children[i] = NULL;
@@ -1248,18 +1242,11 @@ static LOOPSET *start_new_loop (char *s, LOOPSET *inloop,
 
     gretl_error_clear();
 
-#if LOOP_DEBUG
+#if LOOP_DEBUG || defined(SHOW_STRUCTURE)
     fprintf(stderr, "start_new_loop: inloop=%p, line='%s'\n", 
 	    (void *) inloop, s);
 #endif
 
-#if 0
-    if (inloop != NULL && (opt & OPT_P)) {
-	/* don't allow nesting of "progressive" loops? */
-	fprintf(stderr, "inner loop with OPT_P\n");
-    }
-#endif
-
     if (inloop == NULL || compile_level <= inloop->level) {
 	loop = gretl_loop_new(NULL);
     } else {
@@ -1273,7 +1260,7 @@ static LOOPSET *start_new_loop (char *s, LOOPSET *inloop,
 	return NULL;
     }
 
-#if LOOP_DEBUG
+#if LOOP_DEBUG || defined(SHOW_STRUCTURE)
     fprintf(stderr, " added loop at %p (%s)\n", (void *) loop,
 	    (*nested)? "nested" : "independent");
 #endif
@@ -1390,6 +1377,7 @@ static void loop_cmds_init (LOOPSET *loop, int i1, int i2)
     for (i=i1; i<i2; i++) {
 	loop->cmds[i].line = NULL;
 	loop->cmds[i].ci = 0;
+	loop->cmds[i].opt = 0;
 	loop->cmds[i].flags = 0;
 	loop->cmds[i].genr = NULL;
     }
@@ -1578,6 +1566,11 @@ static int loop_print_start (LOOP_PRINT *lprn, const char *namestr)
 {
     int i, nv;
 
+    if (namestr == NULL || *namestr == '\0') {
+	gretl_errmsg_set("'print' list is empty");
+	return E_DATA;
+    }
+
     lprn->names = gretl_string_split(namestr, &lprn->nvars, NULL);
     if (lprn->names == NULL) {
 	return E_ALLOC;
@@ -2047,9 +2040,8 @@ static int add_more_loop_commands (LOOPSET *loop)
 
 static int real_append_line (ExecState *s, LOOPSET *loop)
 {
-    const char *flagstr = NULL;
     int n = loop->n_cmds;
-    int len, err = 0;
+    int err = 0;
 
 #if LOOP_DEBUG
     fprintf(stderr, "real_append_line: s->line = '%s'\n", s->line);
@@ -2061,34 +2053,13 @@ static int real_append_line (ExecState *s, LOOPSET *loop)
 	}
     }
 
-    len = strlen(s->line);
+    loop->cmds[n].line = gretl_strdup(s->line);
 
-    if (s->cmd->opt) {
-	flagstr = print_flags(s->cmd->opt, s->cmd->ci);
-	len += strlen(flagstr);
-	if (len >= MAXLINE) {
-	    gretl_errmsg_set("loop: line is too long");
-	    err = 1;
-	}
-    }
-
-    if (!err) {
-	if (flagstr != NULL) {
-	    loop->cmds[n].line = malloc(len + 1);
-	} else {
-	    loop->cmds[n].line = gretl_strdup(s->line);
-	}
-	if (loop->cmds[n].line == NULL) {
-	    err = E_ALLOC;
-	} else {
-	    if (flagstr != NULL) {
-		sprintf(loop->cmds[n].line, "%s%s", s->line, flagstr);
-	    }
-	    compress_spaces(loop->cmds[n].line);
-	}
-    }
-
-    if (!err) {
+    if (loop->cmds[n].line == NULL) {
+	err = E_ALLOC;
+    } else {
+	/* FIXME: is this (still) wanted */
+	compress_spaces(loop->cmds[n].line);
 	if (s->cmd->ci == PRINT && (!loop_is_progressive(loop) || strchr(s->line, '"'))) {
 	    /* printing a literal string, not a variable's value */
 	    loop->cmds[n].flags |= LOOP_CMD_LIT;
@@ -2126,12 +2097,7 @@ static void destroy_loop_stack (void)
  * @s: program execution state.
  * @dset: dataset struct.
  *
- * Add command line to accumulated loop buffer.  This may be
- * called "starting from cold", in which case the "line"
- * member of @s will have been parsed and any options
- * extracted to s->cmd->opt.  But it may also be called in
- * the process of ongoing loop compilation, and in that
- * case option flags will not have been processed already.
+ * Add the command line @s->line to accumulated loop buffer.
  *
  * Returns: 0 on success, non-zero code on error.
  */
@@ -2159,12 +2125,18 @@ int gretl_loop_append_line (ExecState *s, DATASET *dset)
 
     if (s->cmd->ci == LOOP) {
 	/* starting from scratch */
-	gretlopt opt = get_loop_opts(s, &err);
+	char *spec = s->cmd->param;
+	gretlopt opt = s->cmd->opt;
 	int nested = 0;
 
+	if (spec == NULL) {
+	    fprintf(stderr, "GRETL_ERROR: loop line is unparsed\n");
+	    err = E_DATA;
+	}
+
 	if (!err) {
-	    newloop = start_new_loop(s->line, loop, dset, 
-				     opt, &nested, &err);
+	    newloop = start_new_loop(spec, loop, dset, opt, 
+				     &nested, &err);
 #if LOOP_DEBUG
 	    fprintf(stderr, "got LOOP: newloop at %p (err = %d)\n", 
 		    (void *) newloop, err);
@@ -2413,7 +2385,7 @@ static void print_loop_results (LOOPSET *loop, const DATASET *dset,
     }
 
     for (i=0; i<loop->n_cmds; i++) {
-	gretlopt opt = OPT_NONE;
+	gretlopt opt = loop->cmds[i].opt;
 	int ci = loop->cmds[i].ci;
 
 #if LOOP_DEBUG
@@ -2421,13 +2393,6 @@ static void print_loop_results (LOOPSET *loop, const DATASET *dset,
 		i, loop->cmds[i].line);
 #endif
 
-	if (plain_model_ci(ci)) {
-	    char linecpy[MAXLINE];
-
-	    strcpy(linecpy, loop->cmds[i].line);
-	    opt = get_gretl_options(linecpy, NULL);
-	}	    
-
 	if (!loop_is_progressive(loop) && ci == OLS) {
 	    if (model_print_deferred(opt)) {
 		MODEL *pmod = loop->models[j++];
@@ -2592,6 +2557,26 @@ static void progressive_loop_zero (LOOPSET *loop)
 {
     int i;
 
+    /* What we're doing here is debatable: could we get
+       away with just "zeroing" the relevant structures
+       in an appropriate way, rather than destroying
+       them? Maybe, but so long as we're destroying them
+       we have to remove the "preparsed" flags from
+       associated "print" and "store" commands, or else
+       things will go awry on the second execution of
+       a nested progressive loop.
+    */
+
+    if (loop->cmds != NULL) {
+	for (i=0; i<loop->n_cmds; i++) {
+	    if (loop->cmds[i].ci == PRINT ||
+		loop->cmds[i].ci == STORE) {
+		/* reset */
+		loop->cmds[i].flags &= ~LOOP_CMD_DONE;
+	    }
+	}
+    }    
+
     for (i=0; i<loop->n_loop_models; i++) {
 	loop_model_free(&loop->lmodels[i]);
     }
@@ -2599,7 +2584,7 @@ static void progressive_loop_zero (LOOPSET *loop)
     loop->lmodels = NULL;
     loop->n_loop_models = 0;
 
-    for (i=0; i<loop->n_prints; i++) { 
+    for (i=0; i<loop->n_prints; i++) {
 	loop_print_free(&loop->prns[i]);
     }
 
@@ -2792,23 +2777,22 @@ static int loop_next_command (char *targ, LOOPSET *loop, int *pj)
 #define genr_compiled(l,j) (l->cmds[j].flags & LOOP_CMD_GENR)
 #define loop_cmd_nodol(l,j) (l->cmds[j].flags & LOOP_CMD_NODOL)
 #define loop_cmd_nosub(l,j) (l->cmds[j].flags & LOOP_CMD_NOSUB)
-#define loop_cmd_noopt(l,j) (l->cmds[j].flags & LOOP_CMD_NOOPT)
 #define loop_cmd_catch(l,j) (l->cmds[j].flags & LOOP_CMD_CATCH)
 #define conditional_compiled(l,j) (l->cmds[j].flags & LOOP_CMD_COND)
 #define cmd_preparsed(l,j) (l->cmds[j].flags & LOOP_CMD_DONE)
+#define cmd_checked(l,j)   (l->cmds[j].flags & LOOP_CMD_OK)
 
 static int loop_process_error (LOOPSET *loop, int j, int err, PRN *prn)
 {
 #if LOOP_DEBUG
-    fprintf(stderr, "loop_process_error: j=%d, catch=%d\n",
-	    j, loop_cmd_catch(loop, j));
+    fprintf(stderr, "loop_process_error: j=%d, err=%d, catch=%d\n",
+	    j, err, loop_cmd_catch(loop, j));
+    fprintf(stderr, " line: '%s'\n", loop->cmds[j].line);
+    fprintf(stderr, " errmsg: '%s'\n", gretl_errmsg_get());
 #endif
     if (loop_cmd_catch(loop, j)) {
 	set_gretl_errno(err);
 	err = 0;
-    } else if (!libset_get_bool(HALT_ON_ERR)) {
-	errmsg(err, prn);
-	err = 0;
     }
 
     return err;
@@ -2817,12 +2801,17 @@ static int loop_process_error (LOOPSET *loop, int j, int err, PRN *prn)
 /* Based on the stored flags in the loop-line record, set
    or unset some flags for the command parser: this can 
    reduce the amount of work the parser has to do on each
-   iteration of a loop.
+   iteration of a loop (maybe some of this obsolete?).
 */
 
 static inline void loop_info_to_cmd (LOOPSET *loop, int j,
 				     CMD *cmd)
 {
+#if LOOP_DEBUG
+    fprintf(stderr, "loop_info_to_cmd: i=%d, j=%d: '%s'\n",
+	    loop->iter, j, loop->cmds[j].line);
+#endif
+    
     if (loop_is_progressive(loop)) {
 	cmd->flags |= CMD_PROG;
     } else {
@@ -2830,27 +2819,26 @@ static inline void loop_info_to_cmd (LOOPSET *loop, int j,
     }
 
     if (loop_cmd_nosub(loop, j)) {
-	/* tell parser not to try for @-substitution */
+	/* tell parser not to bother trying for @-substitution */
 	cmd->flags |= CMD_NOSUB;
     } else {
 	cmd->flags &= ~CMD_NOSUB;
     }
-    
-    if (loop_cmd_noopt(loop, j)) {
-	/* tell parser not to try for option flags */
-	cmd->flags |= CMD_NOOPT;
-	/* since the cmd->opt is (or can be) persistent, we
-	   should zero it here */
-	cmd->opt = OPT_NONE;
-    } else {
-	cmd->flags &= ~CMD_NOOPT;
-    }
 
+    /* readjust "catch" for commands that are not being
+       sent through the parser again */
     if (loop_cmd_catch(loop, j)) {
 	cmd->flags |= CMD_CATCH;
     } else if (!cmd->context) {
 	cmd->flags &= ~CMD_CATCH;
     }
+
+#if LOOP_DEBUG    
+    fprintf(stderr, " flagged: prog %d, nosub %d, catch %d\n",
+	    (cmd->flags & CMD_PROG)? 1 : 0,
+	    (cmd->flags & CMD_NOSUB)? 1 : 0,
+	    (cmd->flags & CMD_CATCH)? 1 : 0);
+#endif	    
 }
 
 /* Based on the parsed info in @cmd, maybe modify some flags in
@@ -2860,6 +2848,13 @@ static inline void loop_info_to_cmd (LOOPSET *loop, int j,
 static inline void cmd_info_to_loop (LOOPSET *loop, int j,
 				     CMD *cmd, int *subst)
 {
+    loop_command *lcmd = &loop->cmds[j];
+    
+#if LOOP_DEBUG
+    fprintf(stderr, "cmd_info_to_loop: j=%d: '%s'\n",
+	    j, lcmd->line);
+#endif
+
     if (!loop_cmd_nosub(loop, j)) {
 	/* this loop line has not already been marked as
 	   free of @-substitution
@@ -2868,20 +2863,27 @@ static inline void cmd_info_to_loop (LOOPSET *loop, int j,
 	    *subst = 1;
 	} else {
 	    /* record: no @-substitution in this line */
-	    loop->cmds[j].flags |= LOOP_CMD_NOSUB;
+	    lcmd->flags |= LOOP_CMD_NOSUB;
 	}
     }
 
-    if (loop_cmd_nosub(loop, j)) {
-	if (cmd->opt == OPT_NONE) {
-	    /* record: no options are present on this line */
-	    loop->cmds[j].flags |= LOOP_CMD_NOOPT;
-	}
+    if (lcmd->ci == ELSE || lcmd->ci == ENDIF ||
+	lcmd->ci == BREAK || lcmd->ci == LOOP) {
+	/* flag as parsed OK */
+	lcmd->flags |= LOOP_CMD_OK;
     }
 
+    lcmd->opt = cmd->opt;
+
     if (cmd->flags & CMD_CATCH) {
-	loop->cmds[j].flags |= LOOP_CMD_CATCH;
+	lcmd->flags |= LOOP_CMD_CATCH;
     }
+
+#if LOOP_DEBUG    
+    fprintf(stderr, " loop-flagged: nosub %d, catch %d\n",
+	    loop_cmd_nosub(loop, j)? 1 : 0,
+	    loop_cmd_catch(loop, j)? 1 : 0);
+#endif	 
 }
 
 static int loop_delete_object (CMD *cmd, PRN *prn)
@@ -2931,7 +2933,8 @@ static int loop_report_error (LOOPSET *loop, int err,
 
 static int conditional_line (LOOPSET *loop, int j)
 {
-    return loop->cmds[j].ci == IF || loop->cmds[j].ci == ELIF;
+    return loop->cmds[j].ci == IF || loop->cmds[j].ci == ELIF ||
+	loop->cmds[j].ci == ELSE || loop->cmds[j].ci == ENDIF;
 }
 
 static int do_compile_conditional (LOOPSET *loop, int j)
@@ -2944,6 +2947,95 @@ static int do_compile_conditional (LOOPSET *loop, int j)
     }
 }
 
+static int model_command_post_process (ExecState *s,
+				       DATASET *dset,
+				       LOOPSET *loop,
+				       int j)
+{
+    int prog = loop_is_progressive(loop);
+    int moderr = check_gretl_errno();
+    int err = 0;
+
+    if (moderr) {
+	if (prog || model_print_deferred(s->cmd->opt)) {
+	    err = moderr;
+	} else {
+	    errmsg(moderr, s->prn);
+	}
+    } else if (prog && !(s->cmd->opt & OPT_Q)) {
+	LOOP_MODEL *lmod = get_loop_model_by_line(loop, j, &err);
+
+	if (!err) {
+	    err = loop_model_update(lmod, s->model);
+	    set_as_last_model(s->model, GRETL_OBJ_EQN);
+	}
+    } else if (model_print_deferred(s->cmd->opt)) {
+	MODEL *pmod = get_model_record_by_line(loop, j, &err);
+
+	if (!err) {
+	    swap_models(s->model, pmod);
+	    pmod->ID = j + 1;
+	    set_as_last_model(pmod, GRETL_OBJ_EQN);
+	    model_count_minus();
+	}
+    } else {
+	loop_print_save_model(s->model, dset, s->prn, s);
+    }
+
+    return err;
+}
+
+#if LOOPSAVE
+
+/* When re-executing a loop that has been saved onto its
+   calling function, the loop index variable may have been
+   destroyed, in which case it has to be recreated.
+*/
+
+static int loop_reattach_index_var (LOOPSET *loop, DATASET *dset)
+{
+    int err = 0;
+
+    if (!gretl_is_scalar(loop->idxname)) {
+	char genline[64];
+	
+	if (na(loop->init.val)) {
+	    sprintf(genline, "scalar %s = NA", loop->idxname);
+	} else {
+	    gretl_push_c_numeric_locale();
+	    sprintf(genline, "scalar %s = %g", loop->idxname, loop->init.val);
+	    gretl_pop_c_numeric_locale();
+	}
+	err = generate(genline, dset, OPT_Q, NULL);
+    } 
+
+    return err;
+}
+
+static int maybe_preserve_loop (LOOPSET *loop)
+{
+    if (!loop_is_attached(loop) && gretl_function_depth() > 0) {
+	if (gretl_iteration_depth() > 0 || gretl_looping()) {
+	    int err = attach_loop_to_function(loop);
+
+	    if (!err) {
+		loop_set_attached(loop);
+	    }
+	}
+    }
+
+    return loop_is_attached(loop);
+}
+
+#endif
+
+static void abort_loop_execution (ExecState *s)
+{
+    *s->cmd->savename = '\0';
+    gretl_cmd_destroy_context(s->cmd);
+    errmsg(E_STOP, s->prn);
+}
+
 static int block_model (CMD *cmd)
 {
     return cmd->ci == END && 
@@ -2957,22 +3049,27 @@ static int block_model (CMD *cmd)
 			       c == MLE ||  \
 			       c == GMM)
 
-int gretl_loop_exec (ExecState *s, DATASET *dset) 
+#define LTRACE 0
+
+int gretl_loop_exec (ExecState *s, DATASET *dset, LOOPSET *loop) 
 {
-    LOOPSET *loop = currloop;
     char *line = s->line;
     CMD *cmd = s->cmd;
     PRN *prn = s->prn;
-    MODEL *pmod;
-    LOOP_MODEL *lmod;
     char errline[MAXLINE];
     int indent0;
     int progressive;
     int show_activity = 0;
     int j, err = 0;
 
+    if (loop == NULL) {
+	loop = currloop;
+    } else {
+	currloop = loop;
+    }
+
     /* for the benefit of the caller: register the fact that execution
-       of this loop is now under way */
+       of this loop is already under way */
     loop_execute = 0;
 
     if (loop == NULL) {
@@ -2981,9 +3078,14 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
 	return 1;
     }
 
+#if LOOPSAVE
+    if (loop_is_attached(loop) && *loop->idxname != '\0') {
+	loop_reattach_index_var(loop, dset);
+    }
+#endif
+    
     indent0 = gretl_if_state_record();
     progressive = loop_is_progressive(loop);
-
     set_loop_on(loop_is_quiet(loop), progressive);
 
 #if LOOP_DEBUG
@@ -3003,41 +3105,83 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
 #endif
 	j = -1;
 
-	pmod = NULL;
-	lmod = NULL;
-
 	if (gretl_echo_on() && indexed_loop(loop) && !loop_is_quiet(loop)) {
 	    print_loop_progress(loop, dset, prn);
 	}
 
+#if 1 /* experimental, 2014-12-05 */
+	if (gretl_in_gui_mode() && 
+	    loop->iter % 10 == 0 && 
+	    check_for_stop()) {
+	    /* the GUI user clicked the "Stop" button */
+	    abort_loop_execution(s);
+	    err = E_STOP;
+	    break;
+	}
+#endif	
+
 	while (!err && loop_next_command(line, loop, &j)) {
+	    int ci = loop->cmds[j].ci;
+	    int parse = 1;
 	    int subst = 0;
 
-#if LOOP_DEBUG
-	    fprintf(stderr, " j=%d, line='%s', compiled = %d\n", j, line,
-	            genr_compiled(loop, j));
+#if LTRACE || LOOP_DEBUG
+	    fprintf(stderr, "iter=%d, j=%d, line='%s', ci=%s, compiled=%d\n",
+		    loop->iter, j, line, gretl_command_word(ci),
+		    genr_compiled(loop, j) || conditional_compiled(loop, j) ||
+		    cmd_checked(loop, j));
 #endif
 	    strcpy(errline, line);
 
-	    if (!gretl_if_state_false() && genr_compiled(loop, j)) {
-		/* If the current line already has "compiled genr" status,
-		   we should be able to skip several steps that are
-		   potentially quite time-consuming.
-		*/
+	    if (gretl_if_state_false()) {
+		/* the only ways out are via ELSE, ELIF or ENDIF */
+		if (ci == ELSE || ci == ENDIF) {
+		    if (cmd_checked(loop, j)) {
+			cmd->ci = ci;
+			cmd->err = 0;
+			flow_control(NULL, NULL, cmd, NULL);
+			if (cmd->err) {
+			    err = cmd->err;
+			    goto handle_err;
+			} else {
+			    continue;
+			}
+		    } else {
+			/* this line not checked yet */
+			goto do_parsing;
+		    }
+		} else if (ci == IF || ci == ELIF) {
+		    goto cond_next;
+		} else {
+		    continue;
+		}
+	    }
+
+	    if (cmd_checked(loop, j)) {
+		/* no parsing needed */
+		cmd->ci = ci;
+		if (ci == BREAK) {
+		    loop->brk = 1;
+		    break;
+		} else if (ci == LOOP) {
+		    goto child_loop;
+		}
+	    }
+
+	    if (genr_compiled(loop, j)) {
 		if (gretl_echo_on() && !loop_is_quiet(loop)) {
 		    pprintf(prn, "? %s\n", line);
 		}
 		err = execute_genr(loop->cmds[j].genr, dset, prn);
 		if (err) {
-		    err = loop_process_error(loop, j, err, prn);
-		}
-		if (err) {
-		    break;
+		    goto handle_err;
 		} else {
 		    continue;
 		}
 	    }
 
+	cond_next:    
+
 	    if (!loop_cmd_nodol(loop, j)) {
 		if (strchr(line, '$')) {
 		    /* handle loop-specific $-string substitution */
@@ -3053,15 +3197,19 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
 		}
 	    }
 
-	    /* transcribe loop -> cmd */
+	    /* transcribe saved loop info -> cmd */
 	    loop_info_to_cmd(loop, j, cmd);
 
-	    /* call the full command parser, with special treatment
-	       for "if" or "elif" conditions that may be already
-	       compiled, or that should now be compiled
-	    */
 	    if (conditional_compiled(loop, j)) {
-		err = parse_command_line(line, cmd, dset, &loop->cmds[j].genr);
+		cmd->ci = ci;
+		flow_control(line, dset, cmd, &loop->cmds[j].genr);
+		if (cmd->err) {
+		    /* we hit an error evaluating the if state */
+		    err = cmd->err;
+		} else {
+		    cmd->ci = CMD_MASKED;
+		}
+		parse = 0;
 	    } else if (do_compile_conditional(loop, j)) {
 		GENERATOR *ifgen = NULL;
 
@@ -3070,18 +3218,30 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
 		    loop->cmds[j].genr = ifgen;
 		    loop->cmds[j].flags |= LOOP_CMD_COND;
 		}
+		parse = 0;
 	    } else if (cmd_preparsed(loop, j)) {
-		cmd->ci = loop->cmds[j].ci;
-	    } else {
+		cmd->ci = ci;
+		parse = 0;
+	    }
+
+	do_parsing:
+
+#if LTRACE
+	    fprintf(stderr, "HERE, err = %d, parse = %d\n", err, parse);
+#endif	    
+
+	    if (parse && !err) {
 		err = parse_command_line(line, cmd, dset, NULL);
 	    }
 
 #if LOOP_DEBUG
-	    fprintf(stderr, "    after: '%s'\n", line);
+	    fprintf(stderr, "    after: '%s', ci=%d\n", line, cmd->ci);
 	    fprintf(stderr, "    cmd->savename = '%s'\n", cmd->savename);
 	    fprintf(stderr, "    err from parse_command_line: %d\n", err);
 #endif
 
+	handle_err:
+
 	    if (err) {
 		err = loop_process_error(loop, j, err, prn);
 		if (err) {
@@ -3090,6 +3250,7 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
 		    continue;
 		}
 	    } else if (cmd->ci < 0) {
+		/* blocked */
 		if (conditional_line(loop, j)) {
 		    cmd_info_to_loop(loop, j, cmd, &subst);
 		}
@@ -3105,7 +3266,7 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
 			pputc(prn, '\n');
 		    }
 		} else if (!loop_is_quiet(loop)) {
-		    echo_command(cmd, dset, line, prn);
+		    gretl_echo_command(cmd, line, prn);
 		}
 	    }
 
@@ -3113,6 +3274,8 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
 	       require special treatment in loop context
 	    */
 
+	child_loop:
+
 	    if (cmd->ci == LOOP) {
 		currloop = get_child_loop_by_line(loop, j);
 		if (currloop == NULL) {
@@ -3120,52 +3283,23 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
 		    fprintf(stderr, "Got a LOOP command, don't know what to do!\n");
 		    err = 1;
 		} else {
-		    err = gretl_loop_exec(s, dset);
+		    if (loop_is_attached(loop)) {
+			loop_set_attached(currloop);
+		    }
+		    err = gretl_loop_exec(s, dset, NULL);
 		}
 	    } else if (cmd->ci == BREAK) {
 		loop->brk = 1;
 		break;
 	    } else if (cmd->ci == ENDLOOP) {
 		; /* implicit break */
-	    } else if (plain_model_ci(cmd->ci)) {
-		/* model may need special handling */
-		if (progressive && !(cmd->opt & OPT_Q)) {
-		    lmod = get_loop_model_by_line(loop, j, &err);
-		} else if (model_print_deferred(cmd->opt)) {
-		    pmod = get_model_record_by_line(loop, j, &err);
-		}
-		/* estimate the model called for */
-		if (!err) {
-		    err = gretl_cmd_exec(s, dset);
-		}
-		if (!err) {
-		    int moderr = check_gretl_errno();
-
-		    if (moderr) {
-			if (progressive || model_print_deferred(cmd->opt)) {
-			    err = moderr;
-			} else {
-			    errmsg(moderr, prn);
-			}
-		    } else if (progressive && !(cmd->opt & OPT_Q)) {
-			err = loop_model_update(lmod, s->model);
-			set_as_last_model(s->model, GRETL_OBJ_EQN);
-		    } else if (model_print_deferred(cmd->opt)) {
-			swap_models(s->model, pmod);
-			pmod->ID = j + 1;
-			set_as_last_model(pmod, GRETL_OBJ_EQN);
-			model_count_minus();
-		    } else {
-			loop_print_save_model(s->model, dset, prn, s);
-		    }
-		}
-	    } else if (cmd->ci == PRINT && progressive && !loop_literal(loop, j)) {
+	    } else if (progressive && cmd->ci == PRINT && !loop_literal(loop, j)) {
 		if (cmd_preparsed(loop, j)) {
 		    err = loop_print_update(loop, j, NULL);
 		} else {
 		    err = loop_print_update(loop, j, cmd->parm2);
 		}
-	    } else if (cmd->ci == STORE && progressive) {
+	    } else if (progressive && cmd->ci == STORE) {
 		if (cmd_preparsed(loop, j)) {
 		    err = loop_store_update(loop, j, NULL, NULL, 0);
 		} else {
@@ -3174,7 +3308,7 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
 		}
 	    } else if (progressive && not_ok_in_progloop(cmd->ci)) {
 		gretl_errmsg_sprintf(_("%s: not implemented in 'progressive' loops"),
-				     cmd->word);
+				     gretl_command_word(cmd->ci));
 		err = 1;
 	    } else if (cmd->ci == GENR) {
 		if (subst || (cmd->opt & OPT_O)) {
@@ -3185,9 +3319,9 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
 		    if (!loop_is_verbose(loop)) {
 			cmd->opt |= OPT_Q;
 		    }
-		    err = generate(line, dset, cmd->opt, prn);
+		    err = generate(cmd->vstart, dset, cmd->opt, prn);
 		} else {
-		    err = add_loop_genr(loop, j, line, dset);
+		    err = add_loop_genr(loop, j, cmd->vstart, dset);
 		    if (!err) {
 			err = execute_genr(loop->cmds[j].genr, dset, prn);
 		    }
@@ -3195,8 +3329,17 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
 	    } else if (cmd->ci == DELEET && !(cmd->opt & (OPT_F | OPT_T))) {
 		err = loop_delete_object(cmd, prn);
 	    } else {
+		/* send command to the regular processor */
+		int catch = cmd->flags & CMD_CATCH;
+		
 		err = gretl_cmd_exec(s, dset);
-		if (!err && !check_gretl_errno() && block_model(cmd)) {
+		if (catch) {
+		    /* ensure "catch" hasn't been scrubbed */
+		    cmd->flags |= CMD_CATCH;
+		}
+		if (!err && plain_model_ci(cmd->ci)) {
+		    err = model_command_post_process(s, dset, loop, j);
+		} else if (!err && !check_gretl_errno() && block_model(cmd)) {
 		    /* NLS, etc. */
 		    loop_print_save_model(s->model, dset, prn, s);
 		}
@@ -3210,22 +3353,11 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
 	} /* end execution of commands within loop */
 
 	if (err) {
-	    if (!libset_get_bool(HALT_ON_ERR)) {
-		/* print error message but keep going */
-		errmsg(err, prn);
-		err = 0;
-		/* should we clear the "if state" here? */
-	    } else {
-		gretl_if_state_clear();
-	    }
-	} 
-
-	if (!err) {
-	    if (loop->brk) {
-		gretl_if_state_reset(indent0);
-	    } else {
-		err = gretl_if_state_check(indent0);
-	    }
+	    gretl_if_state_clear();
+	} else if (loop->brk) {
+	    gretl_if_state_reset(indent0);
+	} else {
+	    err = gretl_if_state_check(indent0);
 	}
 
 	if (!err && !loop->brk) {
@@ -3238,7 +3370,6 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
     } /* end iterations of loop */
 
     cmd->flags &= ~CMD_NOSUB;
-    cmd->flags &= ~CMD_NOOPT;
 
     if (loop->brk) {
 	/* turn off break flag */
@@ -3279,9 +3410,15 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
 
     if (loop->parent == NULL) {
 	/* reached top of stack: clean up */
-	gretl_loop_destroy(loop);
 	currloop = NULL;
 	set_loop_off();
+#if LOOPSAVE
+	if (maybe_preserve_loop(loop)) {
+	    /* prevent destruction of saved loop */
+	    loop = NULL;
+	}
+#endif
+	gretl_loop_destroy(loop);
     }
 
     return process_command_error(cmd, err);
