@@ -26,6 +26,7 @@
 #include "usermat.h"
 #include "forecast.h"
 #include "kalman.h"
+#include "bootstrap.h"
 
 #define ODEBUG 0
 
@@ -921,20 +922,23 @@ int maybe_stack_var (GRETL_VAR *var, CMD *cmd)
 MODEL *maybe_stack_model (MODEL *pmod, CMD *cmd, PRN *prn, int *err)
 {
     const char *name = gretl_cmd_get_savename(cmd);
+    gretlopt opt = gretl_cmd_get_opt(cmd);
     MODEL *smod = NULL;
 
-    if (*name != '\0') {
+    if (*name != '\0' || (opt & OPT_W)) {
 	MODEL *cpy = gretl_model_copy(pmod);
 
 	if (cpy == NULL) {
 	    *err = E_ALLOC;
-	} else {
+	} else if (*name != '\0') {
 	    *err = real_stack_object(cpy, GRETL_OBJ_EQN, name, NULL);
 	}
 
 	if (!*err) {
 	    set_as_last_model(cpy, GRETL_OBJ_EQN);
-	    pprintf(prn, _("%s saved\n"), name);
+	    if (*name != '\0') {
+		pprintf(prn, _("%s saved\n"), name);
+	    }
 	} else {
 	    errmsg(*err, prn);
 	}
@@ -1157,6 +1161,17 @@ void *get_genr_model (GretlObjType *type)
     return genr_model.ptr;
 }
 
+int get_genr_model_ID (void)
+{
+    if (genr_model.ptr != NULL && genr_model.type == GRETL_OBJ_EQN) {
+	MODEL *pmod = genr_model.ptr;
+
+	return pmod->ID;
+    } else {
+	return 0;
+    }
+}
+
 static stacker *find_smatch (const char *oname)
 {
     stacker *smatch = NULL;
@@ -1180,6 +1195,15 @@ static stacker *find_smatch (const char *oname)
 	}
     }
 
+    if (smatch != NULL && smatch->type == GRETL_OBJ_EQN) {
+	MODEL *pmod = smatch->ptr;
+
+	if (pmod == NULL || pmod->errcode) {
+	    fprintf(stderr, "find_smatch: duff model data!\n");
+	    smatch = NULL;
+	}
+    }
+
     return smatch;
 }
 
@@ -1359,58 +1383,47 @@ last_model_get_irf_matrix (int targ, int shock, double alpha,
     return M;
 }
 
-gretl_bundle *last_model_get_irf_bundle (int targ, int shock, double alpha, 
-					 const DATASET *dset, int *err)
+gretl_matrix *last_model_get_boot_ci (int cnum,
+				      const DATASET *dset,
+				      int B,
+				      double alpha,
+				      int method,
+				      int studentize,
+				      int *err)
 {
     stacker *smatch = find_smatch(NULL);
-    GRETL_VAR *var = NULL;
-    gretl_matrix *m = NULL;
-    gretl_bundle *b = NULL;
+    gretl_matrix *ret = NULL;
 
-    if (smatch == NULL || smatch->type != GRETL_OBJ_VAR) {
-	*err = E_BADSTAT;
+    if (smatch == NULL || smatch->type != GRETL_OBJ_EQN) {
+	*err = E_DATA;
     } else {
-	var = smatch->ptr;
-	m = gretl_VAR_get_impulse_response(var, targ, shock, 0,
-					   alpha, dset, err);
+	MODEL *pmod = smatch->ptr;
+
+	ret = bootstrap_ci_matrix(pmod, dset, cnum, B, alpha,
+				  method, studentize, err);
     }
 
-    if (!*err) {
-	b = gretl_bundle_new();
-	if (b == NULL) {
-	    *err = E_ALLOC;
-	} else {
-	    int vtarg = gretl_VAR_get_variable_number(var, targ);
-	    int vshock = gretl_VAR_get_variable_number(var, shock);
-	    const char *tname = dset->varname[vtarg];
-	    const char *sname = dset->varname[vshock];
-	    const char *label = dataset_period_label(dset);
-	    int i, errs[6];
+    return ret;
+}
 
-	    errs[0] = gretl_bundle_set_payload_matrix(b, m);
-	    errs[1] = gretl_bundle_set_string(b, "targname", tname);
-	    errs[2] = gretl_bundle_set_string(b, "shockname", sname);
-	    errs[3] = gretl_bundle_set_string(b, "period_label", label);
-	    errs[4] = gretl_bundle_set_scalar(b, "alpha", alpha);
-	    errs[5] = gretl_bundle_set_creator(b, "gretl::irf");
+double last_model_get_boot_pval (int cnum,
+				 const DATASET *dset,
+				 int B,
+				 int method,
+				 int *err)
+{
+    stacker *smatch = find_smatch(NULL);
+    double ret = NADBL;
 
-	    for (i=0; i<6; i++) {
-		if (errs[i]) {
-		    *err = errs[i];
-		    break;
-		}
-	    }
+    if (smatch == NULL || smatch->type != GRETL_OBJ_EQN) {
+	*err = E_DATA;
+    } else {
+	MODEL *pmod = smatch->ptr;
 
-	    if (*err) {
-		gretl_bundle_destroy(b);
-		b = NULL;
-	    }
-	}
+	ret = bootstrap_pvalue(pmod, dset, cnum, B, method, err);
     }
 
-    gretl_matrix_free(m);
-
-    return b;
+    return ret;
 }
 
 void *last_model_get_data (const char *key, GretlType *type, 
@@ -1593,8 +1606,10 @@ int last_model_test_ok (int ci, gretlopt opt, const DATASET *dset,
 
     if (type == GRETL_OBJ_EQN) {
 	MODEL *pmod = (MODEL *) ptr;
-  
-	if (!model_test_ok(ci, opt, pmod, dset)) {
+
+	if (pmod->errcode) {
+	    err = E_DATA;
+	} else if (!model_test_ok(ci, opt, pmod, dset)) {
 	    err = E_NOTIMP;
 	}
 	if (model_sample_problem(pmod, dset)) {
@@ -1724,7 +1739,12 @@ void gretl_saved_objects_cleanup (void)
 	       so we drop the associated refcount and nullify the
 	       last model pointer to guard against double-freeing
 	    */
+#if ODEBUG
+	    fprintf(stderr, " ostack[%d] = last model, unrefing\n", i);
+#endif
 	    gretl_object_unref(lmp, lmt);
+	    last_model.ptr = NULL;
+	    last_model.type = 0;
 	    lmp = NULL;
 	}
 #if ODEBUG
@@ -1753,4 +1773,3 @@ void gretl_saved_objects_cleanup (void)
 	last_model.type = 0;
     }
 }
-
