@@ -74,6 +74,8 @@ GretlObjType get_last_model_type (void);
 
 void *get_genr_model (GretlObjType *type);
 
+int get_genr_model_ID (void);
+
 MODEL *get_model_by_name (const char *mname);
 
 MODEL *get_model_by_ID (int ID);
@@ -132,8 +134,15 @@ gretl_matrix *
 last_model_get_irf_matrix (int targ, int shock, double alpha, 
 			   const DATASET *dset, int *err);
 
-gretl_bundle *last_model_get_irf_bundle (int targ, int shock, double alpha, 
-					 const DATASET *dset, int *err);
+gretl_matrix *last_model_get_boot_ci (int cnum, const DATASET *dset,
+				      int B, double alpha,
+				      int method, int studentize,
+				      int *err);
+
+double last_model_get_boot_pval (int cnum,
+				 const DATASET *dset,
+				 int B, int method,
+				 int *err);
 
 void *last_model_get_data (const char *key, GretlType *type, 
 			   int *size, int *err);
