@@ -88,6 +88,7 @@ GPT_SPEC *plotspec_new (void)
     spec->b_cub = NULL;
     spec->b_inv = NULL;
     spec->b_log = NULL;
+    spec->b_linlog = NULL;
 
     spec->code = PLOT_REGULAR;
     spec->flags = 0;
@@ -188,6 +189,7 @@ void plotspec_destroy (GPT_SPEC *spec)
     gretl_matrix_free(spec->b_cub);
     gretl_matrix_free(spec->b_inv);
     gretl_matrix_free(spec->b_log);
+    gretl_matrix_free(spec->b_linlog);
 
     free(spec);
 }
@@ -264,7 +266,9 @@ int gp_style_index_from_name (const char *s)
 	return GP_STYLE_LINESPOINTS;
     } else if (!strcmp(s, "i")) {
 	return GP_STYLE_IMPULSES;
-    } 
+    } else if (!strcmp(s, "b")) {
+	return GP_STYLE_BOXES;
+    }
 
     /* fallback */
     return GP_STYLE_LINES;
@@ -865,6 +869,8 @@ void print_auto_fit_string (FitType fit, FILE *fp)
 	fputs("# plot includes automatic fit: loess\n", fp);
     } else if (fit == PLOT_FIT_LOGLIN) {
 	fputs("# plot includes automatic fit: semilog\n", fp);
+    } else if (fit == PLOT_FIT_LINLOG) {
+	fputs("# plot includes automatic fit: linlog\n", fp);
     }
 }
 
@@ -1068,7 +1074,8 @@ static int print_point_type (GPT_LINE *line)
 		     s->fit == PLOT_FIT_CUBIC ||     \
                      s->fit == PLOT_FIT_INVERSE || \
                      s->fit == PLOT_FIT_LOESS || \
-		     s->fit == PLOT_FIT_LOGLIN)
+		     s->fit == PLOT_FIT_LOGLIN || \
+		     s->fit == PLOT_FIT_LINLOG)
 
 int gp_line_data_columns (GPT_SPEC *spec, int i)
 {
@@ -1190,7 +1197,7 @@ int plotspec_print (GPT_SPEC *spec, FILE *fp)
 	} else {
 	    fputs("set xdata time\n", fp);
 	}	
-	fprintf(fp, "set timefmt x \"%s\"\n", spec->timefmt);
+	fprintf(fp, "set timefmt \"%s\"\n", spec->timefmt);
     }
 
     /* special x and/or y format? */
@@ -1562,65 +1569,46 @@ static int set_loess_fit (GPT_SPEC *spec, int d, double q, gretl_matrix *x,
 }
 
 static void set_plotfit_formula (char *formula, FitType f, const double *b,
-				 double t0, double pd)
+				 double x0, double pd)
 {
+    char x[32];
+    
     gretl_push_c_numeric_locale();
 
-    /* if t0 is not NA that indicates there we're doing a fitted 
-       plot against time, and we have to transform the
-       coefficients
+    /* If x0 is not NA that indicates that we've calculated a fit
+       against time (t = 1,2,3,...,T), and we have to transform the
+       equation so that it produces correct fitted values given
+       x-data in the form year[.fraction].
     */
 
+    if (!na(x0)) {
+	if (pd > 1) {
+	    sprintf(x, "(%.1f*(x-%.10g)+1)", pd, x0);
+	} else {
+	    sprintf(x, "(x-%.10g+1)", x0);
+	}
+    } else {
+	strcpy(x, "x");
+    }
+
     if (f == PLOT_FIT_OLS) {
-	if (!na(t0)) {
-	    double c = b[1] * pd;
-
-	    sprintf(formula, "%.10g + %.10g*x", b[0] - c*t0, c);
-	} else {
-	    sprintf(formula, "%.10g + %.10g*x", b[0], b[1]);
-	}
+	sprintf(formula, "%.10g + %.10g*%s", b[0], b[1], x);
     } else if (f == PLOT_FIT_QUADRATIC) {
-	if (!na(t0)) {
-	    double c = b[1] * pd;
-	    double g = b[2] * pd * pd;
-
-	    sprintf(formula, "%.10g + %.10g*x + %.10g*x**2", 
-		    b[0] - c*t0 + g*t0*t0, c - 2*g*t0, g);
-	} else {
-	    sprintf(formula, "%.10g + %.10g*x + %.10g*x**2", b[0], b[1], b[2]);
-	}
+	sprintf(formula, "%.10g + %.10g*%s + %.10g*%s**2", b[0], b[1], x, b[2], x);
     } else if (f == PLOT_FIT_CUBIC) {	
-	if (!na(t0)) {
-	    double c = b[1] * pd;
-	    double g = b[2] * pd * pd;
-	    double h = b[3] * pd * pd * pd;
-
-	    sprintf(formula, "%.13g + %.10g*x + %.10g*x**2 + %.10g*x**3", 
-		    b[0] - c*t0 + g*t0*t0 - h*t0*t0*t0, 
-		    c - 2*g*t0 + 3*h*t0*t0, 
-		    g - 3*h*t0, h);
-	} else {
-	    sprintf(formula, "%.10g + %.10g*x + %.10g*x**2 + %.10g*x**3", 
-		    b[0], b[1], b[2], b[3]);
-	}	
+	sprintf(formula, "%.10g + %.10g*%s + %.10g*%s**2 + %.10g*%s**3", 
+		b[0], b[1], x, b[2], x, b[3], x);
     } else if (f == PLOT_FIT_INVERSE) {
-	if (!na(t0)) {
-	    double c = t0 * pd;
-
-	    sprintf(formula, "%.10g + %.10g/(%g*x - %.10g)", b[0], b[1], 
-		    pd, c);
-	} else {
-	    sprintf(formula, "%.10g + %.10g/x", b[0], b[1]);
-	}
+	sprintf(formula, "%.10g + %.10g/%s", b[0], b[1], x);
     } else if (f == PLOT_FIT_LOGLIN) {
-	if (!na(t0)) {
-	    double c = b[1] * pd;
-
-	    sprintf(formula, "exp(%.10g + %.10g*x)", b[0] - c*t0, c);
+	sprintf(formula, "exp(%.10g + %.10g*%s)", b[0], b[1], x);
+    } else if (f == PLOT_FIT_LINLOG) {
+	if (!na(x0)) {
+	    sprintf(formula, "%.10g + %.10g*log%s", b[0], b[1], x);
 	} else {
-	    sprintf(formula, "exp(%.10g + %.10g*x)", b[0], b[1]);
+	    sprintf(formula, "%.10g + %.10g*log(x)", b[0], b[1]);
 	}
-    }	
+    }
 
     gretl_pop_c_numeric_locale();
 }
@@ -1659,6 +1647,9 @@ void set_plotfit_line (char *title, char *formula,
 	    sprintf(gstr, "\\n(%s %.2f%%)", _("annual growth"), g);
 	    strcat(title, gstr);
 	}
+    } else if (f == PLOT_FIT_LINLOG) {
+	sprintf(title, "Y = %#.3g %c %#.3glog(%c)", b[0],
+		(b[1] > 0)? '+' : '-', fabs(b[1]), xc);
     }
 
     /* then set the formula itself */
@@ -1684,6 +1675,8 @@ static void plotspec_set_fitted_line (GPT_SPEC *spec, FitType f,
 	b = spec->b_inv->val;
     } else if (f == PLOT_FIT_LOGLIN) {
 	b = spec->b_log->val;
+    } else if (f == PLOT_FIT_LINLOG) {
+	b = spec->b_linlog->val;
     } else {
 	return;
     }
@@ -1697,11 +1690,6 @@ static void plotspec_set_fitted_line (GPT_SPEC *spec, FitType f,
     spec->lines[1].ncols = 0;
 }
 
-#define polyfit(f) (f == PLOT_FIT_OLS || \
-		    f == PLOT_FIT_QUADRATIC ||	\
-		    f == PLOT_FIT_CUBIC || \
-		    f == PLOT_FIT_LOGLIN)
-
 int plotspec_add_fit (GPT_SPEC *spec, FitType f)
 {
     gretl_matrix *y = NULL;
@@ -1717,8 +1705,14 @@ int plotspec_add_fit (GPT_SPEC *spec, FitType f)
     int i, t, k;
     int err = 0;
 
-    if ((spec->flags & GPT_TS) && polyfit(f)) {
+    if ((spec->flags & GPT_TS) && f != PLOT_FIT_LOESS) {
 	if (spec->pd == 1 || spec->pd == 4 || spec->pd == 12) {
+	    /* Annual, quarterly or monthly data: we'll estimate
+	       the fit against t = 1,2,3,...,T to give a more
+	       comprehensible displayed equation. So we have
+	       to record the initial x-value (e.g. 2008.25) to
+	       get the fitted line right on the plot.
+	    */
 	    x0 = px[0];
 	}
     } 
@@ -1727,14 +1721,15 @@ int plotspec_add_fit (GPT_SPEC *spec, FitType f)
 	(f == PLOT_FIT_QUADRATIC && spec->b_quad != NULL) ||
 	(f == PLOT_FIT_CUBIC && spec->b_cub != NULL) ||
 	(f == PLOT_FIT_INVERSE && spec->b_inv != NULL) ||
-	(f == PLOT_FIT_LOGLIN && spec->b_log != NULL)) {
+	(f == PLOT_FIT_LOGLIN && spec->b_log != NULL) ||
+	(f == PLOT_FIT_LINLOG && spec->b_linlog != NULL)) {
 	/* just activate existing setup */
 	plotspec_set_fitted_line(spec, f, x0);
 	return 0;
     }
 
     if (f == PLOT_FIT_OLS || f == PLOT_FIT_INVERSE ||
-	f == PLOT_FIT_LOGLIN) {
+	f == PLOT_FIT_LOGLIN || f == PLOT_FIT_LINLOG) {
 	k = 2;
     } else if (f == PLOT_FIT_QUADRATIC) {
 	k = 3;
@@ -1767,15 +1762,18 @@ int plotspec_add_fit (GPT_SPEC *spec, FitType f)
     i = 0;
     for (t=0; t<spec->nobs; t++) {
 	if (!na(x0)) {
-	    xt = t;
+	    /* use a 1-based time index */
+	    xt = t + 1;
 	} else {
 	    xt = px[t];
 	}
 	if (!na(py[t]) && !na(xt)) {
 	    if (f == PLOT_FIT_LOGLIN) {
+		/* y must be positive */
 		if (py[t] > 0.0) {
 		    y->val[i] = log(py[t]);
 		} else {
+		    gretl_errmsg_set(_("Non-positive values encountered"));
 		    err = E_DATA;
 		    goto bailout;
 		}
@@ -1786,7 +1784,16 @@ int plotspec_add_fit (GPT_SPEC *spec, FitType f)
 		gretl_matrix_set(X, i, 0, xt);
 	    } else {
 		gretl_matrix_set(X, i, 0, 1.0);
-		if (f == PLOT_FIT_INVERSE) {
+		if (f == PLOT_FIT_LINLOG) {
+		    /* x must be positive */
+		    if (xt > 0.0) {
+			gretl_matrix_set(X, i, 1, log(xt));
+		    } else {
+			gretl_errmsg_set(_("Non-positive values encountered"));
+			err = E_DATA;
+			goto bailout;
+		    }
+		} else if (f == PLOT_FIT_INVERSE) {
 		    gretl_matrix_set(X, i, 1, 1.0 / xt);
 		} else {
 		    gretl_matrix_set(X, i, 1, xt);
@@ -1838,6 +1845,9 @@ int plotspec_add_fit (GPT_SPEC *spec, FitType f)
 	} else if (f == PLOT_FIT_LOGLIN) {
 	    spec->b_log = b;
 	    b = NULL;
+	} else if (f == PLOT_FIT_LINLOG) {
+	    spec->b_linlog = b;
+	    b = NULL;
 	}	    
     }
 
@@ -1856,6 +1866,10 @@ int plotspec_add_fit (GPT_SPEC *spec, FitType f)
     gretl_matrix_free(b);
     gretl_matrix_free(yh);
 
+#if 0
+    fprintf(stderr, "plotspec_add_fit: returning %d\n", err);
+#endif    
+
     return err;
 }
 
@@ -1906,8 +1920,8 @@ static plotbars *parse_bars_file (const char *fname,
     char line[128];
     int ncolon = 0;
     int ndash = 0;
+    int nother = 0;
     int d1, d2, d3, d4;
-    int n = 0;
 
     fp = gretl_fopen(fname, "r");
     if (fp == NULL) {
@@ -1922,26 +1936,30 @@ static plotbars *parse_bars_file (const char *fname,
     /* first count the data lines */
 
     while (fgets(line, sizeof line, fp)) {
-	if (*line == '#') {
+	if (*line == '#' || string_is_blank(line)) {
 	    continue;
 	} else if (sscanf(line, "%d:%d %d:%d", &d1, &d2, &d3, &d4) == 4) {
 	    ncolon++;
 	} else if (sscanf(line, "%d-%d %d-%d", &d1, &d2, &d3, &d4) == 4) {
 	    ndash++;
 	} else {
-	    break;
+	    nother++;
 	}
-	n++;
     }
 
     /* initial check and allocation */
 
-    if (n == 0 || (ncolon == 0 && ndash == 0)) {
+    if (nother > 0) {
+	/* non-comment, non-blank, non-parseable cruft found */
+	*err = E_DATA;
+    } else if (ncolon == 0 && ndash == 0) {
+	/* no parseable data found */
 	*err = E_DATA;
     } else if (ncolon > 0 && ndash > 0) {
+	/* unparseable mash-up found */
 	*err = E_DATA;
     } else {
-	bars = plotbars_new(n);
+	bars = plotbars_new(ncolon + ndash);
 	if (bars == NULL) {
 	    *err = E_ALLOC;
 	}
@@ -1956,7 +1974,7 @@ static plotbars *parse_bars_file (const char *fname,
 	/* now read, check, convert and record the date pairs */
 
 	while (fgets(line, sizeof line, fp) && !*err) {
-	    if (*line == '#') {
+	    if (*line == '#' || string_is_blank(line)) {
 		continue;
 	    }
 	    if (ncolon) {
