@@ -202,6 +202,7 @@ struct plugin_function_info plugin_functions[] = {
     { "IPS_tbar_moments",     P_STATS_TABLES },
     { "IPS_tbar_rho_moments", P_STATS_TABLES },
     { "qlr_asy_pvalue",       P_STATS_TABLES },
+    { "qlr_critval_15_05",    P_STATS_TABLES },
 
     /* SUR, 3SLS, FIML */
     { "system_estimate", P_SYSEST },
@@ -245,10 +246,8 @@ struct plugin_function_info plugin_functions[] = {
 
     /* zip and unzip */
     { "gretl_native_make_zipfile",       P_ZIPFILE},
-    { "gretl_native_unzip_file",         P_ZIPFILE},
-    { "gretl_native_unzip_session_file", P_ZIPFILE},
+    { "gretl_native_unzip",              P_ZIPFILE},
     { "gretl_native_zip_datafile",       P_ZIPFILE},
-    { "gretl_native_unzip_datafile",     P_ZIPFILE},
 
     /* Dynamic panel data estimation */
     { "arbond_estimate",    P_ARBOND},
@@ -470,7 +469,13 @@ void *get_plugin_function (const char *funcname)
 
 	if (handle != NULL) {
 	    funp = get_function_address(handle, funcname);
+	} else {
+	    fprintf(stderr, "%s: get_function_address failed\n",
+		    funcname);
 	}
+    } else {
+	fprintf(stderr, "%s: plugin_index_lookup failed\n",
+		funcname);
     }
 
     if (funp == NULL) {
