@@ -512,7 +512,7 @@ char *gretl_strdup (const char *src)
     if (src != NULL) {
 	size_t n = strlen(src) + 1;
 
-	targ = malloc(n);
+	targ = calloc(n, 1);
 	if (targ != NULL) {
 	    memcpy(targ, src, n);
 	}
@@ -693,6 +693,7 @@ char *gretl_word_strdup (const char *src, const char **ptr,
 		src++;
 	    }
 	    if (*src && !is_word_char(*src)) {
+		gretl_errmsg_sprintf(_("Unexpected symbol '%c'"), *src);
 		*err = E_PARSE;
 		return NULL;
 	    }
@@ -709,6 +710,14 @@ char *gretl_word_strdup (const char *src, const char **ptr,
 	    src++;
 	}
 
+	if (opt & OPT_S) {
+	    if (*src != '\0' && *src != ' ' && *src != ',') {
+		gretl_errmsg_sprintf(_("Unexpected symbol '%c'"), *src);
+		*err = E_PARSE;
+		return NULL;
+	    }
+	}
+
 	if (ptr != NULL) {
 	    *ptr = src;
 	}
@@ -1575,6 +1584,53 @@ int strings_array_add (char ***pS, int *n, const char *p)
     return 0;
 }
 
+/**
+ * strings_array_add_uniq:
+ * @pS: pointer to strings array.
+ * @n: location of present number of strings in array.
+ * @p: string to test for addition to array.
+ *
+ * If the array does not already include a copy of @p,
+ * allocates storage for an extra member of @pS and adds a
+ * copy of string @p in the last position. On successful
+ * addition the content of @n is incremented by 1.
+ * 
+ * Returns: 0 on success, %E_ALLOC on failure.
+ */
+
+int strings_array_add_uniq (char ***pS, int *n, const char *p)
+{
+    char **Tmp, **S = *pS;
+    int m = *n;
+    int i;
+
+    for (i=0; i<m; i++) {
+	if (S[i] != NULL && strcmp(S[i], p) == 0) {
+	    return 0; /* no-op */
+	}
+    }
+
+    Tmp = realloc(*pS, (m + 1) * sizeof *Tmp);
+    if (Tmp == NULL) {
+	return E_ALLOC;
+    }
+
+    *pS = Tmp;
+
+    if (p != NULL) {
+	Tmp[m] = gretl_strdup(p);
+	if (Tmp[m] == NULL) {
+	    return E_ALLOC;
+	}
+    } else {
+	Tmp[m] = NULL;
+    }
+    
+    *n += 1;
+
+    return 0;
+}
+
 /**
  * strings_array_new_with_length:
  * @nstrs: number of strings in array.
