@@ -43,24 +43,25 @@ int attach_subsample_to_model (MODEL *pmod, const DATASET *dset);
 int add_dataset_to_model (MODEL *pmod, const DATASET *dset,
 			  gretlopt opt);
 
-int restrict_sample (const char *line, const int *list,  
+int restrict_sample (const char *param, const int *list,  
 		     DATASET *dset, ExecState *state, 
 		     gretlopt opt, PRN *prn,
 		     int *n_dropped);
 
-int 
-restrict_sample_from_mask (char *mask, DATASET *dset, 
-			   gretlopt opt, int *n_dropped);
+int restrict_sample_from_mask (char *mask, DATASET *dset, 
+			       gretlopt opt);
 
 int complex_subsampled (void);
 
+int dataset_is_complex_subsampled (const DATASET *dset);
+
 int get_full_length_n (void);
 
 void set_dataset_resampled (DATASET *dset);
 
 int dataset_is_resampled (const DATASET *dset);
 
-int set_sample (const char *line, DATASET *dset);
+int set_sample (const char *start, const char *stop, DATASET *dset);
 
 int restore_full_sample (DATASET *dset, ExecState *state);
 
@@ -71,7 +72,7 @@ int count_missing_values (const DATASET *dset, gretlopt opt,
 
 void maybe_free_full_dataset (const DATASET *dset);
 
-int model_sample_problem (MODEL *pmod, const DATASET *dset);
+int model_sample_problem (const MODEL *pmod, const DATASET *dset);
 
 void print_sample_obs (const DATASET *dset, PRN *prn);
 
