@@ -440,10 +440,10 @@ gretl_matrix *matrix_get_submatrix (const gretl_matrix *M,
 				    int prechecked, 
 				    int *err)
 {
-    gretl_matrix *S;
+    gretl_matrix *S = NULL;
     int r, c;
 
-    if (gretl_is_null_matrix(M)) {
+    if (gretl_is_null_matrix(M) || spec == NULL) {
 	*err = E_DATA;
 	return NULL;
     }
@@ -462,7 +462,11 @@ gretl_matrix *matrix_get_submatrix (const gretl_matrix *M,
 	int j = mspec_get_col_index(spec);
 	double x = matrix_get_element(M, i, j, err);
 
-	return (*err)? NULL : gretl_matrix_from_scalar(x);
+	if (!*err) {
+	    S = gretl_matrix_from_scalar(x);
+	}
+
+	return S;
     }
 
     if (spec->rslice == NULL && spec->cslice == NULL) {
