@@ -31,13 +31,15 @@
 #include "uservar.h"
 
 #define UVDEBUG 0
+#define HDEBUG 0
 
 #define LEVEL_AUTO -1
 #define LEV_PRIVATE -1
 
 typedef enum {
     UV_PRIVATE = 1 << 0,
-    UV_SHELL   = 1 << 1
+    UV_SHELL   = 1 << 1,
+    UV_MAIN    = 1 << 2
 } UVFlags;
 
 struct user_var_ {
@@ -98,7 +100,7 @@ static user_var *user_var_new (const char *name, int type,
     } else {
 	u->type = type;
 	u->level = gretl_function_depth();
-	u->flags = 0;
+	u->flags = (u->level == 0)? UV_MAIN : 0;
 	*u->name = '\0';
 	strncat(u->name, name, VNAMELEN - 1);
 	u->ptr = NULL;
@@ -177,10 +179,55 @@ static void uvar_free_value (user_var *u)
     }
 }
 
+static GHashTable *uvh0;       /* for use at "main" exec level */
+static GHashTable *uvh1;       /* for use within functions */
+static GHashTable *uvars_hash; /* pointer to one or other of the above */
+static int previous_d = -1;    /* record of previous "function depth" */
+
+static void uvar_hash_destroy (void)
+{
+#if HDEBUG	
+    fprintf(stderr, "uvar_hash_destroy (uvh0=%p, uvh1=%p)\n",
+	    (void *) uvh0, (void *) uvh1);
+#endif
+    
+    if (uvh0 != NULL) {
+#if HDEBUG	
+	fprintf(stderr, " destroying uvh0\n");
+#endif
+	g_hash_table_destroy(uvh0);
+	uvh0 = NULL;
+    }
+
+    if (uvh1 != NULL) {
+#if HDEBUG	
+	fprintf(stderr, " destroying uvh1\n");
+#endif	
+	g_hash_table_destroy(uvh1);
+	uvh1 = NULL;
+    }
+
+    /* also NULL the convenience pointer */
+    uvars_hash = NULL;
+
+    previous_d = -1;
+}
+
 static void user_var_destroy (user_var *u)
 {
     int free_val = 1;
 
+    if (uvars_hash != NULL) {
+# if HDEBUG
+	if (g_hash_table_remove(uvars_hash, u->name)) {
+	    fprintf(stderr, "removed '%s' from hash table at %p\n",
+		    u->name, (void *) uvars_hash);
+	}
+# else
+	g_hash_table_remove(uvars_hash, u->name);
+# endif	
+    }
+
     if (var_is_shell(u)) {
 	free_val = 0;
     } else if (u->type == GRETL_TYPE_MATRIX) {
@@ -263,6 +310,7 @@ static int real_user_var_add (const char *name,
 		u->flags = UV_SHELL;
 	    }
 	    if (opt & OPT_A) {
+		u->flags &= ~UV_MAIN;
 		u->level += 1;
 	    }
 	    uvars[n_vars] = u;
@@ -334,6 +382,11 @@ int user_var_delete_by_name (const char *name, PRN *prn)
 	return E_UNKVAR;
     }
 
+    if (level > 0 && (targ->flags & UV_MAIN)) {
+	gretl_errmsg_sprintf("%s: cannot be deleted here", targ->name);
+	return E_DATA;
+    }
+
     if (user_var_callback != NULL && level == 0 &&
 	!var_is_private(targ) && 
 	(targ->type == GRETL_TYPE_MATRIX ||
@@ -384,58 +437,183 @@ int user_var_delete (user_var *uvar)
     return err;
 }
 
-user_var *get_user_var_by_name (const char *name)
-{
-    int i, d = gretl_function_depth();
+#if HDEBUG > 1
 
-    if (name == NULL || *name == '\0') {
-	return NULL;
-    }
+static int uvar_index (user_var *u)
+{
+    int i;
 
     for (i=0; i<n_vars; i++) {
-	if (uvars[i]->level == d && !strcmp(uvars[i]->name, name)) {
-	    return uvars[i];
+	if (u == uvars[i]) {
+	    return i;
 	}
     }
 
-    return NULL;
+    return -1;
+}
+
+#endif
+
+/* Try to guess whether the currently-called function is big enough
+   (number of lines of code) to make it worthwhile to construct a hash
+   table for uservars at its level of execution, namely @uvh1, given
+   that we'll have to empty the table on exit from the function.
+
+   The number-of-lines threshold here is obviously kinda arbitrary;
+   some systematic experimentation might be useful.
+*/
+
+static inline int use_uvh1 (void)
+{
+    return current_function_size() > 40;
 }
 
 user_var *get_user_var_of_type_by_name (const char *name,
 					GretlType type)
 {
     int i, imin = 0, d = gretl_function_depth();
+    user_var *u = NULL;
 
     if (name == NULL || *name == '\0') {
 	return NULL;
     }
 
-#if UVDEBUG
-    fprintf(stderr, "get_user_var_of_type_by_name: '%s' (n_vars=%d, level=%d)\n",
-	    name, n_vars, d);
-# if UVDEBUG > 1
-    for (i=0; i<n_vars; i++) {
-	fprintf(stderr, " %d: '%s' type %d, level %d, ptr %p\n", i, 
-		uvars[i]->name, uvars[i]->type, uvars[i]->level,
-		uvars[i]->ptr);
-    }
-# endif
-#endif
-
     if (type == GRETL_TYPE_DOUBLE) {
 	/* support "auxiliary scalars" mechanism */
 	imin = scalar_imin;
+    }    
+
+#if HDEBUG > 1
+    int hfound = 0;
+    
+    fprintf(stderr, "get user var: '%s', %s (n_vars=%d, level=%d, imin=%d)\n",
+	    name, gretl_type_get_name(type), n_vars, d, imin);
+# if HDEBUG > 2    
+    fputs("uvars list:\n", stderr);
+    for (i=0; i<n_vars; i++) {
+	fprintf(stderr, " %d: '%s', %s, level %d, ptr %p\n", i, 
+		uvars[i]->name, gretl_type_get_name(uvars[i]->type),
+		uvars[i]->level, uvars[i]->ptr);
+    }
+# endif    
+#endif    
+
+    if (d != previous_d) {
+	if (d == 0) {
+	    /* "main" level */
+	    if (uvh0 == NULL) {
+		uvh0 = g_hash_table_new(g_str_hash, g_str_equal);
+#if HDEBUG
+		fprintf(stderr, "uvh0: d=0, prev=%d, allocated at %p\n",
+			previous_d, uvh0);
+#endif		
+	    }
+	    if (uvh1 != NULL) {
+#if HDEBUG
+		fprintf(stderr, "d=0, prev=%d: clear uvh1 at %p\n",
+			previous_d, uvh1);
+#endif		
+		g_hash_table_remove_all(uvh1);
+	    }	    
+	    uvars_hash = uvh0;
+	} else if (!use_uvh1()) {
+	    /* exec'ing a function, hash table not wanted */
+	    if (previous_d > 0 && uvh1 != NULL) {
+		g_hash_table_remove_all(uvh1);
+	    }
+	    uvars_hash = NULL;
+	} else {
+	    /* exec'ing a function, hash table wanted */
+	    if (uvh1 == NULL) {
+		uvh1 = g_hash_table_new(g_str_hash, g_str_equal);
+#if HDEBUG
+		fprintf(stderr, "uvh1: d=%d, prev=%d, allocated at %p\n",
+			d, previous_d, uvh1);
+#endif		
+	    } else if (previous_d > 0 && uvh1 != NULL) {
+#if HDEBUG
+		fprintf(stderr, "d=%d, prev=%d: clear uvh1 at %p\n",
+			d, previous_d, uvh1);
+#endif		
+		g_hash_table_remove_all(uvh1);
+	    }
+	    uvars_hash = uvh1;
+	}
+	previous_d = d;
     }
 
-    for (i=imin; i<n_vars; i++) {
-	if (uvars[i]->level == d && 
-	    uvars[i]->type == type &&
-	    !strcmp(uvars[i]->name, name)) {
-	    return uvars[i];
+    if (uvars_hash != NULL) {
+	/* first resort: try a hash look-up */
+	u = g_hash_table_lookup(uvars_hash, name);
+	/* but verify type, if specified */
+	if (u != NULL && type != GRETL_TYPE_ANY && u->type != type) {
+	    u = NULL;
+	}
+#if HDEBUG > 1
+	if (u != NULL) hfound = 1;
+#endif	
+    }
+
+    if (u == NULL) {
+	/* "On demand" hashing: if we're successful in looking
+	   up a variable in the traditional manner, then
+	   insert it into the user vars hash table.
+	*/
+	for (i=imin; i<n_vars; i++) {
+	    if (uvars[i]->level == d && 
+		(type == GRETL_TYPE_ANY || uvars[i]->type == type) &&
+		!strcmp(uvars[i]->name, name)) {
+		u = uvars[i];
+		if (uvars_hash != NULL) {
+		    g_hash_table_insert(uvars_hash, u->name, u);
+		}
+		break;
+	    }
 	}
     }
 
-    return NULL;
+#if HDEBUG > 1
+    if (hfound)
+	fprintf(stderr, "found at pos %d via hash\n\n", uvar_index(u));
+    else if (u != NULL)
+	fprintf(stderr, "found at pos %d via regular search\n\n", uvar_index(u));
+    else 
+	fprintf(stderr, "not found\n\n");
+#endif
+
+    return u;
+}
+
+user_var *get_user_var_by_name (const char *name)
+{
+    return get_user_var_of_type_by_name(name, GRETL_TYPE_ANY);
+}
+
+GretlType user_var_get_type_by_name (const char *name)
+{
+    user_var *u;
+
+    u = get_user_var_of_type_by_name(name, GRETL_TYPE_ANY);
+
+    return u == NULL ? GRETL_TYPE_NONE : u->type;
+}
+
+void *user_var_get_value_and_type (const char *name,
+				   GretlType *type)
+{
+    void *ret = NULL;
+    user_var *u;
+
+    u = get_user_var_of_type_by_name(name, GRETL_TYPE_ANY);
+
+    if (u != NULL) {
+	ret = u->ptr;
+	*type = u->type;
+    } else {
+	*type = GRETL_TYPE_NONE;
+    }
+
+    return ret;
 }
 
 /* note: used in kalman.c */
@@ -460,19 +638,6 @@ int gretl_is_user_var (const char *name)
     return get_user_var_by_name(name) != NULL;
 }
 
-GretlType user_var_get_type_by_name (const char *name)
-{
-    int i, d = gretl_function_depth();
-
-    for (i=0; i<n_vars; i++) {
-	if (uvars[i]->level == d && !strcmp(uvars[i]->name, name)) {
-	    return uvars[i]->type;
-	}
-    }
-
-    return GRETL_TYPE_NONE;
-}
-
 user_var *get_user_var_by_data (const void *data)
 {
     int i, d = gretl_function_depth();
@@ -520,22 +685,6 @@ void *user_var_get_value_by_name (const char *name)
     return (u == NULL)? NULL : u->ptr;
 }
 
-void *user_var_get_value_and_type (const char *name,
-				   GretlType *type)
-{
-    user_var *u = get_user_var_by_name(name);
-    void *ret = NULL;
-
-    if (u != NULL) {
-	ret = u->ptr;
-	*type = u->type;
-    } else {
-	*type = GRETL_TYPE_NONE;
-    }
-
-    return ret;
-}
-
 /* special for scalars since user_var_get_value returns
    a pointer */
 
@@ -901,7 +1050,8 @@ void set_scalar_edit_callback (void (*callback))
     scalar_edit_callback = callback; 
 }
 
-/* used in response to bare declaration of a user variable */
+/* used in response to bare declaration of a user variable
+   in geneval.c */
 
 int create_user_var (const char *name, GretlType type)
 {
@@ -1043,6 +1193,11 @@ void destroy_user_vars (void)
 {
     int i, j;
 
+#if HDEBUG
+    fprintf(stderr, "destroy_user_vars, uvars_hash = %p (uvh0 %p, uvh1 %p)\n",
+	    (void *) uvars_hash, (void *) uvh0, (void *) uvh1);
+#endif
+    
     for (i=0; i<n_vars; i++) {
 	if (uvars[i] == NULL) {
 	    break;
@@ -1055,6 +1210,10 @@ void destroy_user_vars (void)
 	i--;
     }
 
+    if (uvh0 != NULL || uvh1 != NULL) {
+	uvar_hash_destroy();
+    }
+
     set_nvars(0, "destroy_user_vars");
 
     free(uvars);
@@ -1081,6 +1240,11 @@ static int real_destroy_user_vars_at_level (int level, int type,
     int i, j, nv = imin;
     int err = 0;
 
+#if HDEBUG    
+    fprintf(stderr, "real_destroy_user_vars_at_level: level %d, "
+	    "type %d (%s)\n", level, type, gretl_type_get_name(type));
+#endif
+    
     for (i=imin; i<n_vars; i++) {
 	if (uvars[i] == NULL) {
 	    break;
@@ -1360,6 +1524,12 @@ int add_auxiliary_scalar (const char *name, double val)
     double *px = malloc(sizeof *px);
     int err;
 
+    /* Note that unlike gretl_scalar_add() above, this function
+       adds a new scalar variable unconditionally; it never
+       modifies the value of an existing scalar of the same
+       name. 
+    */
+
     if (px == NULL) {
 	err = E_ALLOC;
     } else {
@@ -1380,6 +1550,7 @@ static int real_scalar_set_value (const char *name, double val,
     u = get_user_var_of_type_by_name(name, GRETL_TYPE_DOUBLE);
 
     if (u == NULL) {
+	gretl_errmsg_sprintf("%s: no such scalar", name);
 	err = E_DATA;
     } else if (!authorized && scalar_is_read_only_index(name)) {
 	err = E_DATA;
@@ -1405,6 +1576,8 @@ int gretl_scalar_set_value_authorized (const char *name, double val)
     return real_scalar_set_value(name, val, 1);
 }
 
+/* get the value from a user variable of scalar type */
+
 double gretl_scalar_get_value (const char *name, int *err)
 {
     user_var *u;
@@ -1421,6 +1594,42 @@ double gretl_scalar_get_value (const char *name, int *err)
     return ret;
 }
 
+/* more "permissive" than gretl_scalar_get_value(): allows
+   for @name being the identifier for a 1 x 1 matrix 
+*/
+
+double get_scalar_value_by_name (const char *name, int *err)
+{
+    double ret = NADBL;
+    user_var *u;
+
+    u = get_user_var_by_name(name);
+
+    if (u != NULL) {
+	if (u->type == GRETL_TYPE_DOUBLE) {
+	    ret = *(double *) u->ptr;
+	} else if (u->type == GRETL_TYPE_MATRIX) {
+	    gretl_matrix *m = u->ptr;
+
+	    if (gretl_matrix_is_scalar(m)) {
+		ret = m->val[0];
+	    } else {
+		*err = E_TYPES;
+	    }
+	} else {
+	    *err = E_TYPES;
+	}
+    } else {
+	ret = get_const_by_name(name, err);
+    }
+
+    if (*err) {
+	gretl_errmsg_sprintf(_("'%s': not a scalar"), name);
+    }
+
+    return ret;
+}
+
 int gretl_is_scalar (const char *name)
 {
     int ret = 0;
