@@ -15,7 +15,7 @@
 #define HAVE_MPFR 1
 
 /* openblas? */
-/* #define OPENBLAS_BUILD 1 */
+#define OPENBLAS_BUILD 1
 
 /* openmp? */
 #define OPENMP_BUILD 1
@@ -90,7 +90,8 @@
 /* Use the GNU R shared library */
 #define USE_RLIB 1
 
-#define WINVER 0x0500 /* XP */
+/* require XP */
+#define _WIN32_WINNT 0x0501
 
 /* sapi speech support */
 #define WIN32_SAPI 1
